﻿/*
* Program Name: New Standards and Operating Rules - Payer Side (POR)
* Contractor Name: Harris Corporation
* Contract Number: VA118-11-D-1009
* Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
* 
* Developer: Zach Bertram
* Date: 01.08.2013
* ----------------
* Added missing documentation for methods and classes
* Added in starting the CCC so that we can PING it
*/

using System;
using System.IO;
using System.Data;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using System.Net;
using System.Security;
using System.Security.Cryptography.X509Certificates;
using System.Data.SqlClient;
using AETShared;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;
using OfficeOpenXml;

namespace AETService
{
   /// <summary>
   /// Automated Eligibility Tool service for spawning objects to accept incoming connections
   /// </summary>
   public partial class AETService : ServiceBase
   {
      gDataClass configurationData = new gDataClass(); //Service configuration data
      FileSystemWatcher ZipWatcher = new FileSystemWatcher();  //For watching for new Zip/facility files to import
      FileSystemWatcher LEIEWatcher = new FileSystemWatcher();  //For watching for new LEIE files to import
		FileSystemWatcher NPIWatcher = new FileSystemWatcher();  //For watching for new NPI/TIN files to impor


		TcpListener blcTcpListener;
      TcpListener cccTcpListener;
      System.Collections.Generic.SynchronizedCollection<ConnectedClient> blcClientSocketObjectsCollection = new System.Collections.Generic.SynchronizedCollection<ConnectedClient>();
      System.Collections.Generic.SynchronizedCollection<ConnectedClient> cccClientSocketObjectsCollection = new System.Collections.Generic.SynchronizedCollection<ConnectedClient>();
      Thread listenForBlcClientsThread;
      Thread listenForCccClientsThread;
      X509Certificate serverCertificate;
      X509Certificate mviPkiCertificate;

      public const string NL = "\r\n";  //newline  //ZEB VA118-11-D-1009 1/23/13 made a constant

      
      string sZipUpdate = "";  //Stores timestamp of last Zip/Facility file import
		string sLEIEUpdate = "";  //Stores timestamp of last LEIE file import
		string sNPIUpdate = "";  //Stores timestamp of last NPI file import

		//ZEB VA118-11-D-1009 3/19/13 (+enum) added to allow AET to support various testing modes
		/// <summary>
		/// Enum used to track whether or not to activate various testing features not useful for production
		/// </summary>
		private enum TestingState
      {
         Production,                   //Set it to this before deploying, uses all TCP connections
         Testing,                      //Set it to this to avoid startup lag while testing
         TestingInternalInterfaces,    //Set it to this to test without Emdeon interface, but with VAAFI and ES
         DebuggingFiles,               //Set it to this to have time to attach a debugger during startup, using testing files
         DebuggingConnections,         //Set it to this to have time to attach a debugger during startup, using TCP connections
         DebuggingInternalInterfaces,  //Set it to this to have time to attach a debugger during startup, and to test without Emdeon interface, but with VAAFI and ES
      }

      private TestingState TESTING = TestingState.Production;  //This should be TestingState.Production for deployment  //ZEB VA118-11-D-1009 3/12/13 (+2) added to allow testing with flat files
      //private TestingState TESTING = TestingState.DebuggingConnections;  //This should be TestingState.Production for deployment  //ZEB VA118-11-D-1009 3/12/13 (+2) added to allow testing with flat files
      private const string TEST_INPUT_PATH = @"c:\temp\270.txt";  //Input file used during testing/debugging with files
      private const string TEST_OUTPUT_PATH = @"c:\temp\271.txt";  //Output file used during testing/debugging with files

      #region "Methods for Windows Services"
      /// <summary>
      /// Intializes the components hosted on the service's form
      /// </summary>
      public AETService()
      {
         InitializeComponent();
      }

      /// <summary>
      /// When service is started, set up listeners for connections and watchers for update files
      /// </summary>
      /// <param name="args">Command line arguments</param>
      protected override void OnStart(string[] args)
      {

         string sMsg = string.Empty; // Used as a common message string for the event log...

         // trace=yes;payload=yes;webservice=yes;precheck=yes

         // USe the SSLCheck flag for enable / disable trace / writing payloads to the database and writting the results of the web service calls to the install folder
         // This is because the inbound connection is over a T1 and there is no need to check if the SSL is from Emendon.
         // *** Subject to change if the T1 is no used ***
         // Trace enabled / disabled
         // Write the VA payload to the database
         // Write the WebService to the install folder
         // Run Precheck

         //ZEB VA118-11-D-1009 3/12/13 (+if) added delay if we need one for debugging
         if ((TESTING == TestingState.DebuggingFiles) || (TESTING == TestingState.DebuggingConnections))
         {
            Thread.Sleep(15000);  //give time to attach debugger
         }

         //ZEB VA118-11-D-1009 3/22/13 (+if) added to correct handling of Service Type Codes
         if (TESTING == TestingState.DebuggingInternalInterfaces)
         {
            Thread.Sleep(120000);  //give time for really slow remote debugger
         }

         configurationData.PreCheck = false;
         configurationData.TraceEnabled = false;
         configurationData.LogWEBService = false;
         configurationData.LogPayload = false;

         if (configurationData.SSLCheck == "NO")
         {
            configurationData.PreCheck = true;
            configurationData.TraceEnabled = false;
            configurationData.LogWEBService = true;
         }
         else if (configurationData.SSLCheck == "YES")
         {
            configurationData.PreCheck = true;
            configurationData.TraceEnabled = true;
            configurationData.LogWEBService = true;
         }
         else
         {
            try
            {
               //Load configuration data
               string[] statusFlags = configurationData.SSLCheck.Split(';');
               foreach (string s in statusFlags)
               {
                  string cmd = s.ToUpper();

                  if (cmd.Contains("TRACE"))  // Enable writing the trace info to the application log
                  {
                     string[] sCmd = cmd.Split('=');
                     if (sCmd.Length == 2)
                     {
                        if (sCmd[1].ToUpper() == "YES")
                           configurationData.TraceEnabled = true;
                        else
                           configurationData.TraceEnabled = false;
                     }
                     else
                        configurationData.TraceEnabled = false;
                  }
                  else if (cmd.Contains("PAYLOAD"))   // Enable writing the VA payloads to the database
                  {
                     string[] sCmd = cmd.Split('=');
                     if (sCmd.Length == 2)
                     {
                        if (sCmd[1].ToUpper() == "YES")
                           configurationData.LogPayload = true;
                        else
                           configurationData.LogPayload = false;
                     }
                     else
                        configurationData.LogPayload = false;
                  }
                  else if (cmd.Contains("WEBSERVICE"))    // Write the webservice results to the user folder
                  {
                     string[] sCmd = cmd.Split('=');
                     if (sCmd.Length == 2)
                     {
                        if (sCmd[1].ToUpper() == "YES")
                           configurationData.LogWEBService = true;
                        else
                           configurationData.LogWEBService = false;
                     }
                     else
                        configurationData.LogWEBService = false;
                  }
                  else if (cmd.Contains("PRECHECK"))      // Run pre checks
                  {
                     string[] sCmd = cmd.Split('=');
                     if (sCmd.Length == 2)
                     {
                        if (sCmd[1].ToUpper() == "YES")
                           configurationData.PreCheck = true;
                        else
                           configurationData.PreCheck = false;
                     }
                     else
                        configurationData.PreCheck = false;
                  }
               }
            }
            catch (Exception ex)
            {
               sMsg = String.Format("There was a problem in OnStart() configurationData - {0}.\n{1}", ex.Message);
               Logging.WriteLogFile(sMsg + "\r\n" + ex.StackTrace);
					return;
            }
         }

			///////////////////////////////////////////////////////////////////////////////////////////////////////////////
			// DONE with setting system flags
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////
			string ZipImportFolder = "", ZipExt = "", LEIEImportFolder  = "", LEIEExt = "", NPIImportFolder = "", NPIExt = "";

			try
			{
				Logging.WriteLogFile("OnStart() - configurationData start");
				// If the AET port is set to zero the set the AET port to the CtlPort value,
				// This was done beause there was not a place in the install screen to set the AET port. (can be done in the snap in)
				if (configurationData.AETPort == 0) configurationData.AETPort = configurationData.CtlPort;

				//
				// PreCheck & SSL check were removed because we know the certs work (This was for debugging)
				//

				// Do we need to preform pre checks?
				if (configurationData.PreCheck) PreChecks();

				// Read Zip Code & LEIE watch folder infromation. 
				string[] zipInfo = configurationData.ZipImportFolder.Split(';');
				string[] LEIEInfo = configurationData.LEIEImportFolder.Split(';');
				string[] NPIInfo = configurationData.NPIImportFolder.Split(';');


				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Setup Zip Code & LEIE Watch Folder with watch folder and exe information.
				ZipImportFolder = zipInfo.Any() ? zipInfo[0].Trim() : "";  //ZEB VA118-11-D-1009 4/3/13 (+3) gracefully support setting being blank
				ZipExt = zipInfo.Length > 1 ? zipInfo[1].Trim() : "";

				LEIEImportFolder = LEIEInfo.Any() ? LEIEInfo[0] : "";
				LEIEExt = LEIEInfo.Length > 1 ? LEIEInfo[1] : "";

				NPIImportFolder = NPIInfo.Any() ? NPIInfo[0] : "";
				NPIExt = NPIInfo.Length > 1 ? NPIInfo[1] : "";

				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Check Zip Code Folder & Ext
				if (ZipImportFolder.Length == 0)
				{
					// Set default Zip Code floder location.
				}
				if (ZipExt.Length == 0)
				{
					ZipExt = ".txt";
				}

				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Check LEIE Folder & Ext
				if (LEIEImportFolder.Length == 0)
				{
					// Set default LEIE floder location.
				}
				if (LEIEExt.Length == 0)
				{
					LEIEExt = ".txt";
				}


				///////////////////////////////////////////////////////////////////////////////////////////////////////////////
				// Check LEIE Folder & Ext
				if (NPIImportFolder.Length == 0)
				{
					// Set default NPI floder location.
				}
				if (NPIExt.Length == 0)
				{
					NPIExt = ".xlsx";
				}
				Logging.WriteLogFile("OnStart() - configurationData completed");
			}
			catch (Exception ex)
			{
				sMsg = String.Format("There was a problem in OnStart() configurationData - {0}.\n", ex.Message);
				Logging.WriteLogFile(sMsg + "\r\n" + ex.StackTrace);
				return;
			}



			try { 
         
				if (ZipImportFolder != null && ZipImportFolder.Length > 3 && Directory.Exists(ZipImportFolder))
				{
					ZipWatcher.Path = ZipImportFolder;
					ZipWatcher.NotifyFilter = NotifyFilters.LastAccess | NotifyFilters.LastWrite | NotifyFilters.FileName | NotifyFilters.DirectoryName;

					ZipWatcher.Filter = ZipExt; // Only watch (setup) text files.
														 // Add event handlers.
					ZipWatcher.Created += ZipOnChanged;
					ZipWatcher.Renamed += ZipOnRenamed;
					ZipWatcher.EnableRaisingEvents = true; // Begin watching.
				}


				////////////////////////////////////////////////////////////////////////////////////////////////////////////
				if (LEIEImportFolder != null && LEIEImportFolder.Length > 3 && Directory.Exists(LEIEImportFolder))
				{
					LEIEWatcher.Path = LEIEImportFolder;
					LEIEWatcher.NotifyFilter = NotifyFilters.LastAccess | NotifyFilters.LastWrite
						| NotifyFilters.FileName | NotifyFilters.DirectoryName;
					LEIEWatcher.Filter = LEIEExt; // Only watch (setup) text files.
															// Add event handlers.
					LEIEWatcher.Created += LEIEOnChanged;
					LEIEWatcher.Renamed += LEIEOnRenamed;

					LEIEWatcher.EnableRaisingEvents = true; // Begin watching.
				}

				////////////////////////////////////////////////////////////////////////////////////////////////////////////

				

				if (NPIImportFolder != null && NPIImportFolder.Length > 3 && Directory.Exists(NPIImportFolder))
				{
					NPIWatcher.Path = NPIImportFolder;
					NPIWatcher.NotifyFilter = NotifyFilters.LastAccess | NotifyFilters.LastWrite | NotifyFilters.FileName | NotifyFilters.DirectoryName;
					NPIWatcher.Filter = "*";  
					NPIWatcher.Created += NPIOnChanged;

					NPIWatcher.EnableRaisingEvents = true; // Begin watching.  
					Logging.WriteLogFile("Watching NPIImportFolder=" + NPIImportFolder + "\r\n");

				}


			}
         catch (SecurityException sE)
         {
            StringBuilder sbL = new StringBuilder();
            sbL.Append("The denied permission is: " + (PermissionSet)sE.DenySetInstance);
            sbL.Append(NL);
            sbL.Append("The demanded permission is: " + sE.Demanded);
            sbL.Append(NL);
            sbL.Append("The security action is: " + sE.Action);
            sbL.Append(NL);
            sbL.Append("The method is: " + sE.Method);
            sbL.Append(NL);
            sbL.Append("The permission state at the time of the exception was: " + sE.PermissionState);
            sbL.Append(NL);
            sbL.Append("The permission that failed was: " + (IPermission)sE.FirstPermissionThatFailed);
            sbL.Append(NL);
            sbL.Append("The permission type is: " + sE.PermissionType);

            sMsg = string.Format("There was SecurityException setting up Watch Folders: {0}. ", sbL);
            //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Error, 101, 500);
            Logging.WriteLogFile(sMsg + "\r\n" + sE.StackTrace);
         }
         catch (Exception eX)
         {
            sMsg = string.Format("There was a problem setting up Watch Folders: {0}. ", eX.Message);
            //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Error, 101, 500);
            Logging.WriteLogFile(sMsg + "\r\n" + eX.StackTrace);
         }

         if (!configurationData.LoadSTCData())  //ZEB VA118-11-D-1009 1/23/13 (+if) added to correct handling of Service Type Codes
         {
               sMsg = "There was a problem loading the Service Type Code data.";
               //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Error, 101, 500);
               Logging.WriteLogFile(sMsg);
         }

         ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
         // Start the BLC TCP listening thread that will service the AET client connections.
         //Logging.WriteLogFile("Configuration data wants us to find a SSL certificate named: " + configurationData.SSLName);
         serverCertificate = GetCertificateFromStore(configurationData.SSLName);
         if (serverCertificate != null)
         {
            X500DistinguishedName dname = new X500DistinguishedName(((X509Certificate2)serverCertificate).SubjectName);
            //Logging.WriteLogFile("Found SSL certificate named: " + dname.Name);
         }
         else
            //Logging.WriteLogFile("ERROR: could not find SSL certificate");

         //Logging.WriteLogFile("Configuration data wants us to find a MVI PKI certificate named: " + configurationData.MVIPKI);
         mviPkiCertificate = GetCertificateFromStore(configurationData.MVIPKI);
         if (mviPkiCertificate != null)
         {
            X500DistinguishedName dname = new X500DistinguishedName(((X509Certificate2)mviPkiCertificate).SubjectName);
            //Logging.WriteLogFile("Found MVI PKI certificate named: " + dname.Name);
         }
         else
            //Logging.WriteLogFile("ERROR: could not find MVI PKI certificate");

         //this.BLC.Start(new ConnectThreadStart(BLCacceptConnection), theData.AETPort, useAsync);
         listenForBlcClientsThread = new Thread(ListenForBlcClients);
         listenForBlcClientsThread.Start();

         //ZEB VA118-11-D-1009 1/8/13 (+if) re-enabled CCC to handle pings from monitor
         // Start the CCC TCP listener that will service AET control connections
         if (configurationData.AETPort != configurationData.CtlPort)  //skip if setup wasn't completed properly
         {
            //this.CCC.Start(new ConnectThreadStart(CCCacceptConnection), theData.CtlPort, useAsync);
            listenForCccClientsThread = new Thread(ListenForCccClients);
            listenForCccClientsThread.Start();
         }

         ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
         // Write everything we know about the startup enviroment to the windoes application event log.
         // ** If the service does not have premissions this will fail and the service will not start ***
         ///////////////////////////////////////////////////////////////////////////////////////////////////////////////

         // We need to add a IP UDP brodacast that a debug client can pickup a distress call if we have problems.
         sMsg = String.Format("AETPort:{0}\r\nServer:{1}\r\nOS:{2}\r\nMVI:[{3}] [{4}]\r\nES:{5} [{6}] [{7}]\r\n[{8}] [{9}]\r\nZip Code Import Folder:{10} - {11}\r\nLEIE Import Folder:{12} - {13}\r\nNPI Import Folder:{14} - {15}  "
														, configurationData.AETPort
                                          , configurationData.GetComputerName()
                                          , configurationData.GetProcessor()
                                          , configurationData.MVIURL
                                          , configurationData.MVIPKI
                                          , configurationData.ESURL
                                          , configurationData.ESUserName
                                          , configurationData.ESPassword
                                          , configurationData.PKICheck
                                          , configurationData.SSLCheck
                                          , ZipImportFolder
                                          , ZipExt
                                          , LEIEImportFolder
                                          , LEIEExt
														, NPIImportFolder
														, NPIExt
														);
         //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Information, 100, 100);
         Logging.WriteLogFile(sMsg);

         // At this point the listening thread should be running and we have done all we can....
         //                         END OF SERVICE STARTUP CODE      

         //ZEB VA118-11-D-1009 3/12/13 (+if) branch to testing code if needed
         if ((TESTING == TestingState.Testing) || (TESTING == TestingState.DebuggingFiles) || (TESTING == TestingState.TestingInternalInterfaces))
         {
               StreamReader input = new StreamReader(TEST_INPUT_PATH);
               StreamWriter output = new StreamWriter(TEST_OUTPUT_PATH);
               string message;
               const string messageNum = "000000";
               string messageSize;
               //TestBLCServer blc;

               while (!input.EndOfStream)
               {
                  message = input.ReadLine();
                  if (message.Length > 0)
                  {
                     message = messageNum + message;
                     messageSize = string.Format("{0:D6}", message.Length);
                     message = messageSize + message;

                     //blc = new TestBLCServer();

                     //output.WriteLine(blc.Start(message, configurationData, ((TESTING == TestingState.TestingInternalInterfaces) || (TESTING == TestingState.DebuggingInternalInterfaces))));
                  }
               }
               input.Close();
               output.Close();
         }
         Logging.WriteLogFile("OnStart() completed");
      }

      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      //
      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

      /// <summary>
      /// Called when the system or a user stops the AET service -- makes sure ongoing threads are cleaned up
      /// </summary>
      protected override void OnStop()
      {
         //Logging.WriteLogFile("Starting OnStop()");
         // Stop watching.
         ZipWatcher.EnableRaisingEvents = false;
         LEIEWatcher.EnableRaisingEvents = false;  //ZEB VA118-11-D-1009 1/9/13 (+2) add missing abort for LEIEWatcher and CCC
			NPIWatcher.EnableRaisingEvents = false;
			//this.BLC.Abort();
			//this.CCC.Abort();
			CloseConnections();
         //System.Diagnostics.EventLog.WriteEntry("AETService", "OnStop OK", System.Diagnostics.EventLogEntryType.Information, 10, 100);
         //Logging.WriteLogFile("OnStop() Completed");
      }

      #endregion

      #region "Setup sockets"
      // This subroutine is used as a background listener thread to allow reading incoming messages without lagging the user interface.
      private void ListenForBlcClients()
      {
         try
         {
               // Listen for new connections.
               IPAddress ipAddress = Dns.GetHostEntry("localhost").AddressList[0];
               //IPEndPoint localEndPoint = new IPEndPoint(IPAddress.Parse("127.0.0.1"), configurationData.AETPort);
               IPEndPoint localEndPoint = new IPEndPoint(IPAddress.Any, configurationData.AETPort);

               blcTcpListener = new TcpListener(localEndPoint);
               blcTcpListener.Start();
               do
               {
                  // Create a new user connection using TcpClient returned by
                  // KEB: for some reason I had to break the next two statements into two statements (they were combined into one)
                     TcpClient aNewClient = blcTcpListener.AcceptTcpClient();
                  if (serverCertificate != null)
                  {
                     ConnectedClient myConnectedClient = new ConnectedClient(aNewClient, serverCertificate, configurationData);
                     blcClientSocketObjectsCollection.Add(myConnectedClient);
                     //Logging.WriteLogFile("New BLC Client " + myConnectedClient.remoteEndpointAddress + ";" + myConnectedClient.handle + ";" + myConnectedClient.port + " connected");
                  }
                  else
                  {
                     //Logging.WriteLogFile("ERROR: New BLC Client " + aNewClient.Client.RemoteEndPoint + ";" + aNewClient.Client.Handle  + ";"  + " connection refused because certificate is null");
                     aNewClient.Close();
                  }

                  // Create an event handler to allow the UserConnection to communicate with the window.
                  //myConnectedClient.LineReceived += new LineReceivedEventHandler(MessageReceived);
               } while (!false);
         }
         catch (Exception ex)
         {
               Logging.WriteLogFile(ex.ToString());
         }
      }

      // This subroutine is used as a background listener thread to allow reading incoming messages without lagging the user interface.
      private void ListenForCccClients()
      {
         try
         {
               // Listen for new connections.
               IPAddress ipAddress = Dns.GetHostEntry("localhost").AddressList[0];
               cccTcpListener = new TcpListener(IPAddress.Parse("127.0.0.1"), configurationData.CtlPort);
               cccTcpListener.Start();
               do
               {
                  // Create a new user connection using TcpClient returned by
                  // KEB: for some reason I had to break the next two statements into two statements (they were combined into one)
                  TcpClient aNewClient = cccTcpListener.AcceptTcpClient();
                  ConnectedClient myConnectedClient = new ConnectedClient(aNewClient, serverCertificate, configurationData);
                  cccClientSocketObjectsCollection.Add(myConnectedClient);
                  //Logging.WriteLogFile("New CCC Client " + myConnectedClient.remoteEndpointAddress + ";" + myConnectedClient.handle + ";" + myConnectedClient.port + " connected");

                  // Create an event handler to allow the UserConnection to communicate with the window.
                  //myConnectedClient.LineReceived += new LineReceivedEventHandler(MessageReceived);
               } while (!false);
         }
         catch (Exception ex)
         {
               Logging.WriteLogFile(ex.ToString());
         }
      }

      void CloseConnections()
      {
         //Logging.WriteLogFile("Closing TCP Listeners");
         if (cccTcpListener != null)
         {
               cccTcpListener.Stop();
         }
         if (blcTcpListener != null)
         {
               blcTcpListener.Stop();
         }
      }
      #endregion

      #region "Zip and LEIE and NPI File Watchers"

      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      // Define the event handlers.
      ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

      /// <summary>
      /// Called when a new file is added to the directory watched for Zip Code updates; triggers import
      /// </summary>
      /// <param name="source">The object that raised this event</param>
      /// <param name="e">The arguments that came with the event; especially, contains the name of the file added/changed</param>
      private void ZipOnChanged(object source, FileSystemEventArgs e)
      {
         // Specify what is done when a file is changed, created, or deleted.
         //System.Diagnostics.EventLog.WriteEntry("AETService", "AET ready to Import: " + e.FullPath, System.Diagnostics.EventLogEntryType.Warning, 200, 100);
         //Logging.WriteLogFile("AET ready to import zip file: " + e.FullPath);
         // + e.ChangeType

         string cn = string.Format("Initial Catalog=VAZip;Data Source={0};User Id={1};Password={2};Persist Security Info=False", configurationData.ZipServer, configurationData.ZipUser, configurationData.ZipPassword);
         ImportFile(cn, e.FullPath);

         if (sZipUpdate.Length != 0)
         {
               configurationData.ZipImportTime = sZipUpdate;
               configurationData.SetServiceParameters();
         }
      }

      /// <summary>
      /// Called when a file is renamed in the directory watched for Zip Code updates; logs the event
      /// </summary>
      /// <param name="source">The object that raised the event</param>
      /// <param name="e">The arguments that came with the event; especially, contains the new and old names of the renamed file</param>
      private static void ZipOnRenamed(object source, RenamedEventArgs e)
      {
         // Specify what is done when a file is renamed.
         string sMsg = String.Format("File: {0} renamed to {1}", e.OldFullPath, e.FullPath);
         //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Information, 200, 102);
         Logging.WriteLogFile(sMsg);
      }

      /// <summary>
      /// Called when a new file is added to the directory watched for LEIE updates; triggers update
      /// </summary>
      /// <param name="source">The object that raised this event</param>
      /// <param name="e">The arguments that came with the event; especially, contains the name of the file added/changed</param>
      private void LEIEOnChanged(object source, FileSystemEventArgs e)
      {
         // Specify what is done when a file is changed, created, or deleted.
         //System.Diagnostics.EventLog.WriteEntry("AETService", "AET ready to Import LEIE: " + e.FullPath, System.Diagnostics.EventLogEntryType.Warning, 200, 200);
         //Logging.WriteLogFile("AET ready to import LEIE file: " + e.FullPath);
         // + e.ChangeType

         string cn = string.Format("Initial Catalog=VALEIE;Data Source={0};User Id={1};Password={2};Persist Security Info=False", configurationData.ZipServer, configurationData.ZipUser, configurationData.ZipPassword);
         ImportLEIEFile(cn, e.FullPath);

         if (sLEIEUpdate.Length != 0)
         {
               configurationData.LEIEImportTime = sLEIEUpdate;
               configurationData.SetServiceParameters();
         }
      }


		private void NPIOnChanged(object source, FileSystemEventArgs e)
		{

			try
			{
				Logging.WriteLogFile("NPIOnChanged().  File found =" + e.FullPath + "\r\n");

				SqlConnectionStringBuilder myCdwSqlConnectionStringBuilder;
				myCdwSqlConnectionStringBuilder = new SqlConnectionStringBuilder("Integrated Security=SSPI;");
				myCdwSqlConnectionStringBuilder.DataSource = configurationData.CdwServer;
				myCdwSqlConnectionStringBuilder.InitialCatalog = configurationData.CdwDatabase;
				myCdwSqlConnectionStringBuilder.MultipleActiveResultSets = true;


				ImportNPIFile(myCdwSqlConnectionStringBuilder.ConnectionString, e.FullPath);

				if (sNPIUpdate.Length != 0)
				{
					configurationData.NPIImportTime = sNPIUpdate;
					configurationData.SetServiceParameters();
				}
			}
			catch (Exception eX)
			{
				string sMsg = string.Format("NPIOnChanged(): {0}. ", eX.Message);

				Logging.WriteLogFile(sMsg + "\r\n" + eX.StackTrace);
			}

		}


		/// <summary>
		/// Called when a file is renamed in the directory watched for LEIE updates; logs the event
		/// </summary>
		/// <param name="source">The object that raised the event</param>
		/// <param name="e">The arguments that came with the event; especially, contains the new and old names of the renamed file</param>
		private static void LEIEOnRenamed(object source, RenamedEventArgs e)
      {
         // Specify what is done when a file is renamed.
         string sMsg = String.Format("File: {0} renamed to {1}", e.OldFullPath, e.FullPath);
         //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Information, 300, 200);
         Logging.WriteLogFile(sMsg);
      }


      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      //
      ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

      /// <summary>
      /// Imports an LEIE update from a flat file into the LEIE database
      /// </summary>
      /// <param name="LEIEConnection">The connection parameters for the database</param>
      /// <param name="FileName">The name of the file to be imported</param>
      public void ImportLEIEFile(string LEIEConnection, string FileName)
      {
         Thread.Sleep(1000 * 60);
         SqlCommand cmd = null;
         StreamReader importfile = null;
         try
         {
               cmd = new SqlCommand { Connection = new SqlConnection(LEIEConnection) };
               cmd.Connection.Open();
               cmd.CommandType = CommandType.Text;
               cmd.CommandText = "UPDATE LEIELookup SET Retired = 'Y'";
               cmd.ExecuteNonQuery();

               cmd.CommandText = "INSERT INTO [VALEIE].[dbo].[LEIELookup]([NPI]) VALUES (@NPI)";

               cmd.Parameters.Add("@NPI", SqlDbType.NVarChar);

               bool bLoop = true;
               int recordCount = 0;
               using (importfile = new StreamReader(FileName))
               {
                  string tmpLine = importfile.ReadLine();
                  tmpLine = importfile.ReadLine();
                  char[] splitchar = new char[] { ',' };
                  string[] fields = null;
                  string tmp = "";

                  while (!string.IsNullOrEmpty(tmpLine) && bLoop)
                  {
                     recordCount++;
                     fields = tmpLine.Split(splitchar);
                     if (fields.Length < 18)
                     {
                           cmd.Parameters.Clear();
                           cmd.CommandText = "DELETE FROM LEIELookup WHERE [Retired] IS NULL";
                           cmd.ExecuteNonQuery();
                           cmd.CommandText = "UPDATE LEIELookup SET [Retired] = NULL";
                           cmd.ExecuteNonQuery();
                           string msg = String.Format("LEIE Import FAILED Field Count 18:{0} File :{1}", fields.Length, FileName);
                           //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Error, 110, 500);
                           bLoop = false;
                           continue;
                     }
                     if (recordCount == 1)
                     {
                           string msg = "LEIE Import File Validation Complete!";
                           //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Information, 201, 200);
                           Logging.WriteLogFile(msg);
                     }

                     tmp = Utilities.SafeSubString(fields[8].Trim(), 255);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.

                     tmp = tmp.Replace("\"", "");

                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@NPI"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@NPI"].Value = DBNull.Value;
                     }

                     cmd.ExecuteNonQuery();

                     tmpLine = importfile.ReadLine();
                  }
               }

               if (bLoop)
               {
                  cmd.Parameters.Clear();
                  cmd.CommandText = "DELETE FROM LEIELookup WHERE [Retired] = 'Y'";
                  cmd.ExecuteNonQuery();
               }

               if (bLoop)
               {
                  string sMsg = string.Format("AET Imported {0} LEIE NPI's into the database.", recordCount);
                  //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Information, 142, 200);
                  Logging.WriteLogFile(sMsg);

                  DateTime d1 = DateTime.Now;
                  sLEIEUpdate = String.Format("{0} {1} ({2}) Records", d1.ToShortDateString(), d1.ToShortTimeString(), recordCount);
               }
               else
               {
                  string sMsg = string.Format("AET Imported {0} LEIE NPI's into the database.", recordCount);
                  //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Warning, 142, 200);
                  Logging.WriteLogFile(sMsg);
               }
         }
         catch
         {
               if (cmd != null && cmd.Connection != null && cmd.Connection.State == ConnectionState.Open)
               {
                  cmd.Parameters.Clear();
                  cmd.CommandText = "DELETE FROM LEIELookup WHERE [Retired] IS NULL";
                  cmd.ExecuteNonQuery();
                  cmd.CommandText = "UPDATE LEIELookup SET [Retired] = NULL";
                  cmd.ExecuteNonQuery();
               }

         }
         finally
         {
               if (cmd.Connection != null)
               {
                  if (cmd.Connection.State == ConnectionState.Open)
                  {
                     cmd.Connection.Close();
                  }
               }
               cmd.Dispose();

               string newFileName = "";
               try
               {
                  newFileName = FileName + ".arc";
                  try
                  {
                     File.Delete(newFileName);
                  }
                  catch { };
                  File.Move(FileName, newFileName);
               }
               catch (Exception ex)
               {
                  string sMsg = string.Format("Unable to rename [{0}] to [{1}]: {2}", FileName, newFileName, ex.Message);
                  //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Warning, 142, 200);
                  Logging.WriteLogFile(sMsg);
               }
         }
      }



		public void ImportNPIFile(string NPIConnection, string FileName)
		{
			Thread.Sleep(1000 * 60); 
			string excelPath = FileName;
			
			try
			{
				// load into mirror table 
				FileInfo fileInfo = new FileInfo(excelPath);
				var excelPackage = new ExcelPackage(fileInfo);
				var dt = ToDataTable(excelPackage);
				
				using (var conn = new SqlConnection(NPIConnection))
				{
					conn.Open();

					// set target table
					var targetTable = "App.VA_NPI_TINCrossWalk";

					// mark old records for delete
					using (SqlCommand cmd = new SqlCommand("UPDATE App.VA_NPI_TINCrossWalk SET MarkForDelete = 1", conn))
					{
						cmd.ExecuteNonQuery();
					}

					// load
					var bulkCopy = new SqlBulkCopy(conn);
					bulkCopy.DestinationTableName = targetTable;

					bulkCopy.ColumnMappings.Add("NPI", "NPI");
					bulkCopy.ColumnMappings.Add("TIN", "TIN");

					bulkCopy.WriteToServer(dt);


					// delete MarkForDelete and null records
					using (SqlCommand cmd = new SqlCommand("DELETE FROM App.VA_NPI_TINCrossWalk WHERE (TIN is null OR LEN(TIN) < 3) OR (NPI is null OR LEN(NPI) < 3) OR MarkForDelete = 1", conn))
					{
						cmd.ExecuteNonQuery();
					}


				}
			}
			catch (Exception ex)
			{
				string msg = string.Format("There was a problem importing file to table App.VA_NPI_TINCrossWalk: {0}. \r\n Path was {1}", ex.Message, excelPath);
				Logging.WriteLogFile(msg + "\r\n" + ex.StackTrace);
			}

		}



		private DataTable ToDataTable(ExcelPackage package)
		{
			ExcelWorksheet workSheet = package.Workbook.Worksheets.First();
			DataTable table = new DataTable();
			foreach (var firstRowCell in workSheet.Cells[1, 1, 1, workSheet.Dimension.End.Column])
			{
				table.Columns.Add(firstRowCell.Text);
			}
			for (var rowNumber = 2; rowNumber <= workSheet.Dimension.End.Row; rowNumber++)
			{
				var row = workSheet.Cells[rowNumber, 1, rowNumber, workSheet.Dimension.End.Column];
				var newRow = table.NewRow();
				foreach (var cell in row)
				{
					newRow[cell.Start.Column - 1] = cell.Text;
				}
				table.Rows.Add(newRow);
			}
			return table;
		}



		/// <summary>
		/// Imports a Zip Code update from a flat file into the Zip Code/VA Facility Crosswalk database
		/// </summary>
		/// <param name="ZipConnection">The connection parameters for the database</param>
		/// <param name="FileName">The name of the file to be imported</param>
		public void ImportFile(string ZipConnection, string FileName)
      {
         SqlCommand cmd = null;
         Thread.Sleep(1000 * 60);
         StreamReader importfile = null;
         try
         {
               bool bLoop = true;
               int recordCount = 0;
               using (importfile = new StreamReader(FileName))
               {
                  char[] splitchar = new char[] { '\t' };
                  string[] fields = null;
                  string tmp = "";

                  string tmpLine = importfile.ReadLine();
                  while (bLoop && !string.IsNullOrEmpty(tmpLine))
                  {
                     fields = tmpLine.Split(splitchar);

                     int fc = fields.Length;
                     recordCount++;
                     if (recordCount == 1)
                     {
                           string msg = String.Empty;
                           string hdr = Utilities.SafeSubString(fields[0].Trim(), 15);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                           if (hdr != "Lookup Zip Code")
                           {
                              msg = String.Format("Zip Code Import File Error: " + "Invaliad Header Record.");
                              //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Error, 110, 500);
                              Logging.WriteLogFile(msg);
                              bLoop = false;
                              continue;
                           }
                           if (fc != 10)
                           {
                              msg = String.Format("Zip Code Import File Error: Invaliad Field Count. {0} and should be 10", fc);
                              //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Error, 110, 500);
                              Logging.WriteLogFile(msg);
                              bLoop = false;
                              continue;
                           }

                           msg = "Zip Code Import File Validation Complete!";
                           //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Information, 201, 200);
                           Logging.WriteLogFile(msg);

                           cmd = new SqlCommand { Connection = new SqlConnection(ZipConnection) };
                           cmd.Connection.Open();
                           cmd.CommandType = CommandType.Text;
                           cmd.CommandText = "UPDATE ZipLookup SET Retired = 'Y'";
                           cmd.ExecuteNonQuery();

                           cmd.CommandText = "INSERT INTO [VAZip].[dbo].[ZipLookup]([LookupZip],[FacilityName],[AddressLine1],[AddressLine2],[City],[State],[FacilityZip],[ContactPhone],[ContactFax],[ContactEmail]) VALUES (@ZipLookup,@FacilityName,@AddressLine1,@AddressLine2,@City,@State,@FacilityZip,@ContactPhone,@ContactFax,@ContactEmail)";
                           cmd.Parameters.Add("@ZipLookup", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@FacilityName", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@AddressLine1", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@AddressLine2", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@City", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@State", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@FacilityZip", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@ContactPhone", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@ContactFax", SqlDbType.NVarChar);
                           cmd.Parameters.Add("@ContactEmail", SqlDbType.NVarChar);

                           tmpLine = importfile.ReadLine();
                           continue;
                     }

                     cmd.Parameters["@ZipLookup"].Value = Utilities.SafeSubString(fields[0].Trim(), 5);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     cmd.Parameters["@FacilityName"].Value = Utilities.SafeSubString(fields[1].Trim(), 50);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     tmp = Utilities.SafeSubString(fields[2].Trim(), 255);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@AddressLine1"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@AddressLine1"].Value = DBNull.Value;
                     }

                     tmp = Utilities.SafeSubString(fields[3].Trim(), 255);
                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@AddressLine2"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@AddressLine2"].Value = DBNull.Value;
                     }

                     tmp = Utilities.SafeSubString(fields[4].Trim(), 50);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@City"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@City"].Value = DBNull.Value;
                     }

                     tmp = Utilities.SafeSubString(fields[5].Trim(), 2);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@State"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@State"].Value = DBNull.Value;
                     }

                     tmp = Utilities.SafeSubString(fields[6].Trim(), 10);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@FacilityZip"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@FacilityZip"].Value = DBNull.Value;
                     }

                     tmp = Utilities.SafeSubString(fields[7].Trim(), 20);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@ContactPhone"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@ContactPhone"].Value = DBNull.Value;
                     }

                     tmp = Utilities.SafeSubString(fields[8].Trim(), 20);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@ContactFax"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@ContactFax"].Value = DBNull.Value;
                     }

                     tmp = Utilities.SafeSubString(fields[9].Trim(), 255);  //ZEB VA118-11-D-1009 3/20/13 moved SafeSubString to new Utilities class in AETShared.
                     if (!string.IsNullOrEmpty(tmp))
                     {
                           cmd.Parameters["@ContactEmail"].Value = tmp;
                     }
                     else
                     {
                           cmd.Parameters["@ContactEmail"].Value = DBNull.Value;
                     }

                     cmd.ExecuteNonQuery();

                     tmpLine = importfile.ReadLine();
                  }
               }

               if (bLoop)
               {
                  cmd.Parameters.Clear();
                  cmd.CommandText = "DELETE FROM ZipLookup WHERE [Retired] = 'Y'";
                  cmd.ExecuteNonQuery();

                  string sMsg = string.Format("AET OnLine after Importing {0} Zipcodes", recordCount);
                  //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Information, 140, 200);
                  Logging.WriteLogFile(sMsg);

                  DateTime d1 = DateTime.Now;
                  sZipUpdate = String.Format("{0} {1} ({2}) Records", d1.ToShortDateString(), d1.ToShortTimeString(), recordCount);
               }
               else
               {
                  string sMsg = string.Format("AET Imported {0} Zip Code's into the database.", recordCount - 1);
                  //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Warning, 142, 200);
                  Logging.WriteLogFile(sMsg);
               }
         }
         catch
         {
               if (cmd != null && cmd.Connection != null && cmd.Connection.State == ConnectionState.Open)
               {
                  cmd.Parameters.Clear();
                  cmd.CommandText = "DELETE FROM ZipLookup WHERE [Retired] IS NULL";
                  cmd.ExecuteNonQuery();
                  cmd.CommandText = "UPDATE ZipLookup SET [Retired] = NULL";
                  cmd.ExecuteNonQuery();
               }
         }
         finally
         {
               if (cmd != null)
               {
                  if (cmd.Connection != null)
                  {
                     if (cmd.Connection.State == ConnectionState.Open)
                     {
                           cmd.Connection.Close();
                     }
                  }
                  cmd.Dispose();
               }

               string newFileName = "";
               try
               {
                  newFileName = FileName + ".arc";
                  try { File.Delete(newFileName); }
                  catch { };
                  File.Move(FileName, newFileName);
               }
               catch (Exception ex)
               {
                  string sMsg = string.Format("Unable to rename [{0}] to [{1}]: {2}", FileName, newFileName, ex.Message);
                  //System.Diagnostics.EventLog.WriteEntry("AETService", sMsg, System.Diagnostics.EventLogEntryType.Warning, 142, 200);
                  Logging.WriteLogFile(sMsg);
               }
         }
      }
      #endregion

      /// <summary>
      /// Makes sure we have everything we need before we start listening for connections
      /// Right now just logs time of startup, SSL cert checking is disabled
      /// </summary>
      private void PreChecks()
      {
         bool bPermissions = false;
         string cDri = Environment.CommandLine;
         cDri = cDri.Replace("AETService.exe", "");
         cDri = cDri.Replace("\"", "");
         string logFile = String.Format("{0}AET.txt", cDri);
         string errMsg;
         System.Reflection.Assembly myAssembly = System.Reflection.Assembly.GetExecutingAssembly();
         FileVersionInfo myFileVersionInfo = FileVersionInfo.GetVersionInfo(myAssembly.Location);
         string version = myFileVersionInfo.FileVersion;

         try
         {
            errMsg = "Starting Pre Checks.";
            //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Warning, 110, 100);
            Logging.WriteLogFile(errMsg);
         }
         catch (Exception ex)
         {
            Logging.WriteLogFile("AETService event log write exception :" + ex.Message + "\r\n" + ex.StackTrace);
            return;
         }

         DateTime dateTime = DateTime.Now;
         string creationTime = dateTime.ToString("yyyyMMdd HHmmss");  // Creation Time 

         try
         {
            errMsg = String.Format("AET has Acceptable Permissions To\r\n{0}", cDri);
            //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Information, 120, 100);
            bPermissions = true;
            Logging.WriteLogFile(errMsg);
         }
         catch (Exception ex)
         {
            errMsg = String.Format("AET Read/Write File Permissions FAILED! to \r\n[{0}]\r\n {1} ", cDri, ex.Message);
            //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 110, 300);
            Logging.WriteLogFile(errMsg);
         }

         if (bPermissions)
         {
            ScanSSLCert();
            ScanVAAFICert();
         }

         errMsg = "Completed Pre Check.";
         //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Information, 140, 100);
         Logging.WriteLogFile(errMsg);
      }

      #region "Certificates"
      /// <summary>
      /// Makes sure we have the certificate for connecting to VAAFI specified in the config
      /// Currently unused -- not designed to return anything
      /// </summary>
      private void ScanVAAFICert()
      {
         StringBuilder myStringBuilder = new StringBuilder();
         string errMsg = "Starting ScanVAFFICert.";
         //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Information, 130, 100);
         try
         {
               string cDri = Environment.CommandLine;
               cDri = cDri.Replace("AETService.exe", "");
               cDri = cDri.Replace("\"", "");
               string logFile = String.Format("{0}VAAFICerts.txt", cDri);
               myStringBuilder.AppendLine("-----------------------------------------------------");
               myStringBuilder.AppendLine("----  START->AET (VAAFI x509 Certificate Search)  ---");
               myStringBuilder.AppendLine("-----------------------------------------------------");
               myStringBuilder.AppendLine(string.Format("MVIPKI Configuration Value: [{0}] - ", configurationData.MVIPKI));

               try
               {
                  // PREFIX Checks
                  // SN; Serial Number
                  // SUB; Subject Name
                  // (no prefix)          Issuer
                  // NOTE: multiple values can be entered with comma

                  X509Store store = null;
                  string[] certs = configurationData.MVIPKI.Split(';');
                  string sCert = configurationData.MVIPKI;
                  if (certs.Length == 2)
                  {

                     myStringBuilder.AppendLine(string.Format("II\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", "Personal", certs[1], "LocalMachine"));
                     store = new X509Store(StoreName.My, StoreLocation.LocalMachine);

                     //myStringBuilder.AppendLine("II\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", certs[0], certs[1], "LocalMachine");
                     //store = new X509Store(certs[0], StoreLocation.LocalMachine);
                     sCert = certs[1];
                  }
                  else if (certs.Length == 3)
                  {
                     myStringBuilder.AppendLine(string.Format("III\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", "Personal", certs[1], "LocalMachine"));
                     store = new X509Store(StoreName.My, StoreLocation.LocalMachine);

                     //myStringBuilder.AppendLine("III\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", certs[0], certs[1], certs[2]);
                     //if (certs[2] == "lm")
                     //store = new X509Store(certs[0], StoreLocation.LocalMachine);
                     //else
                     //store = new X509Store(certs[0], StoreLocation.CurrentUser);

                     sCert = certs[1];
                  }
                  else
                  {
                     myStringBuilder.AppendLine(string.Format("I\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", "Personal", certs[0], "LocalMachine"));
                     store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
                     sCert = certs[0];
                  }

                  string[] certNames = sCert.Split(',');
                  if (certNames.Length > 1)
                     sCert = certNames[0];

                  myStringBuilder.AppendLine("       Number of values to search for: " + certNames.Length);

                  store.Open(OpenFlags.ReadOnly);
                  X509Certificate2Collection certificates = store.Certificates;
                  foreach (X509Certificate2 x509 in certificates)
                  {
                     X500DistinguishedName dname = new X500DistinguishedName(x509.SubjectName);

                     string c1 = x509.GetCertHashString();                // Returns the hash value for the X.509v3 certificate as a hexadecimal string.  
                     string c2 = x509.GetEffectiveDateString();           // Returns the effective date of this X.509v3 certificate.  
                     string c3 = x509.GetExpirationDateString();          // Returns the expiration date of this X.509v3 certificate.  
                     // c4=x509.GetFormat();                              // Returns the name of the format of this X.509v3 certificate.  
                     // c5=x509.GetHashCode();                            // Overridden. Returns the hash code for the X.509v3 certificate as an integer.  
                     string c6 = x509.Issuer;                             // Returns the name of the certification authority that issued the X.509v3 certificate.  
                     // c7=x509.GetKeyAlgorithm();                        // Returns the key algorithm information for this X.509v3 certificate.  
                     // c8=x509.GetKeyAlgorithmParameters();              // Returns the key algorithm parameters for the X.509v3 certificate.  
                     string c9 = x509.GetKeyAlgorithmParametersString();  // Returns the key algorithm parameters for the X.509v3 certificate.  
                     string c10 = x509.Subject;                           // Returns the name of the principal to which the certificate was issued.  
                     // c11=x509.GetPublicKey();                          // Returns the public key for the X.509v3 certificate.  
                     string c12 = x509.GetPublicKeyString();              // Returns the public key for the X.509v3 certificate.  
                     // c13=x509.GetRawCertData();                        // Returns the raw data for the entire X.509v3 certificate.  
                     string c14 = x509.GetRawCertDataString();            // Returns the raw data for the entire X.509v3 certificate.  
                     //c15=x509.GetSerialNumber();                        // Returns the serial number of the X.509v3 certificate 
                     string c16 = x509.GetSerialNumberString();

                     myStringBuilder.AppendLine("-----------------------------------------------------");

                     myStringBuilder.AppendLine("Subject name: " + dname.Name);
                     myStringBuilder.AppendLine("Format: " + x509.GetFormat());
                     myStringBuilder.AppendLine("Effective Date: " + c2);
                     myStringBuilder.AppendLine("Expiration Date: " + c3);
                     myStringBuilder.AppendLine("IssuerName: " + c6);
                     myStringBuilder.AppendLine("KeyAlgorithm Parameters: " + c9);
                     myStringBuilder.AppendLine("Name: " + c10);
                     //myStringBuilder.AppendLine("Public Key: {0}", c12);
                     //myStringBuilder.AppendLine("RawCert Data: {0}", c14);
                     myStringBuilder.AppendLine("Serial Number: " + c16);

                     myStringBuilder.AppendLine("-----------------------------------------------------");
                     foreach (string s in certNames)
                     {
                           if (certs.Length >= 2)
                           {
                              // check subject
                              if ((certs[0].IndexOf("sub", StringComparison.OrdinalIgnoreCase) > -1)  && (dname.Name.IndexOf(s, StringComparison.OrdinalIgnoreCase) > -1))
                              {
                                 myStringBuilder.AppendLine("\t----->Found Certificate: " + s);
                              }
                              else if ((certs[0].IndexOf("sn", StringComparison.OrdinalIgnoreCase) > -1) && (x509.GetSerialNumberString().ToLower() == s.ToLower()))
                              {
                                 myStringBuilder.AppendLine("\t----->Found Certificate: " + s);
                              }
                           }
                           else
                           {
                              // check issuer
                              if (x509.Issuer.IndexOf(s, StringComparison.OrdinalIgnoreCase) > -1)
                              {
                                 myStringBuilder.AppendLine("\t----->Found Certificate: " + s);
                              }
                           }
                     }
                     x509.Reset();
                  }
                  store.Close();   // Close the x509 Store 
               }
               catch (Exception ex)
               {
                  myStringBuilder.AppendLine(ex.Message);
               }

               myStringBuilder.AppendLine("-----------------------------------------------------");
               myStringBuilder.AppendLine("----    END->AET (VAAFI x509 Certificate Search)  ---");
               myStringBuilder.AppendLine("-----------------------------------------------------");
               //using (StreamWriter writer = new StreamWriter(logFile, false))
               //{
               //    writer.WriteLine(myStringBuilder.ToString());
               //}
               Logging.WriteLogFile(myStringBuilder.ToString());
         }
         catch (Exception ex)
         {
               errMsg = String.Format("AET:VAAFI {0}", ex.Message);
               //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 112, 500);
               //Logging.WriteLogFile("Exception: " + errMsg + "\r\n" + ex.StackTrace);
         }
      }

      /// <summary>
      /// Makes sure we have the certificate for connecting to Emdeon specified in the config
      /// Currently unused -- not designed to return anything
      /// </summary>
      private void ScanSSLCert()
      {
         StringBuilder myStringBuilder = new StringBuilder();
         string errMsg = "Starting ScanSSLCert.";
         //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Information, 114, 100);

         string cDri = Environment.CommandLine;
         cDri = cDri.Replace("AETService.exe", "");
         cDri = cDri.Replace("\"", "");
         string logFile = String.Format("{0}SSLCerts.txt", cDri);
         try
         {
               myStringBuilder.AppendLine("-----------------------------------------------------");
               myStringBuilder.AppendLine("----  START->AET (SSL x509 Certificate Search)    ---");
               myStringBuilder.AppendLine("-----------------------------------------------------");
               myStringBuilder.AppendLine(string.Format("SSLName Configuration Value: [{0}] - ", configurationData.SSLName));

               X509Store store = null;
               string[] certs = configurationData.SSLName.Split(';');
               string sCert = configurationData.SSLName;
               if (certs.Length == 2)
               {
                  //myStringBuilder.AppendLine("II\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", certs[0], certs[1], "LocalMachine");
                  //store = new X509Store(certs[0], StoreLocation.LocalMachine);

                  myStringBuilder.AppendLine(string.Format("II\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", "Personal", certs[1], "LocalMachine"));
                  store = new X509Store(StoreName.My, StoreLocation.LocalMachine);

                  sCert = certs[1];
               }
               else if (certs.Length == 3)
               {
                  myStringBuilder.AppendLine(string.Format("III\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", "Personal", certs[1], "LocalMachine"));
                  store = new X509Store(StoreName.My, StoreLocation.LocalMachine);

                  //myStringBuilder.AppendLine("III\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", certs[0], certs[1], certs[2]);
                  //if (certs[2] == "lm")
                  //store = new X509Store(certs[0], StoreLocation.LocalMachine);
                  //else
                  //store = new X509Store(certs[0], StoreLocation.CurrentUser);

                  sCert = certs[1];
               }
               else
               {
                  myStringBuilder.AppendLine(string.Format("I\r\n       Store:[{0}]\r\n Certificate:[{1}]\r\n    Location:[{2}]", "Personal", certs[0], "LocalMachine"));
                  store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
                  sCert = certs[0];
               }

               string[] certNames = sCert.Split(',');
               if (certNames.Length > 1)
                  sCert = certNames[0];

               myStringBuilder.AppendLine("       Number of values to search for: " + certNames.Length);

               store.Open(OpenFlags.ReadOnly);
               X509Certificate2Collection certificates = store.Certificates;
               foreach (X509Certificate2 x509 in certificates)
               {
                  X500DistinguishedName dname = new X500DistinguishedName(x509.SubjectName);
                  myStringBuilder.AppendLine("-----------------------------------------------------");
                  foreach (string s in certNames)
                  {
                     if (certs.Length >= 2)
                     {
                           // check subject
                           if ((certs[0].IndexOf("sub", StringComparison.OrdinalIgnoreCase) > -1) && (dname.Name.IndexOf(s, StringComparison.OrdinalIgnoreCase) > -1))
                           {
                              myStringBuilder.AppendLine("\t----->Found Certificate: " + s);
                           }
                           else if ((certs[0].IndexOf("sn", StringComparison.OrdinalIgnoreCase) > -1) && (x509.GetSerialNumberString().ToLower() == s.ToLower()))
                           {
                              myStringBuilder.AppendLine("\t----->Found Certificate: " + s);
                           }
                     }
                     else
                     {
                           // check issuer
                           if (x509.Issuer.IndexOf(s, StringComparison.OrdinalIgnoreCase) > -1)
                           {
                              myStringBuilder.AppendLine("\t----->Found Certificate: " + s);
                           }
                     }
                  }

                  x509.Reset();
               }
               store.Close();   // Close the x509 Store 

               myStringBuilder.AppendLine("-----------------------------------------------------");
               myStringBuilder.AppendLine("----    END->AET (SSL x509 Certificate Search)    ---");
               myStringBuilder.AppendLine("-----------------------------------------------------");
               //using (StreamWriter writer = new StreamWriter(logFile, false))
               //{
               //    writer.WriteLine(myStringBuilder.ToString());
               //}
               Logging.WriteLogFile(myStringBuilder.ToString());
         }
         catch (Exception ex)
         {
               errMsg = String.Format("AET:SSL {0}", ex.Message);
               //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 115, 500);
               Logging.WriteLogFile(errMsg);
         }
      }

      private static X509Certificate2 GetCertificateFromStore(string certName)
      {
         string errMsg;
         X509Store store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
         try
         {
               string[] certs = certName.Split(';');
               string sCert = certName;
               if (certs.Length >= 2)
                  sCert = certs[1];
               else
                  sCert = certs[0];

               string[] certNames = sCert.Split(',');
               if (certNames.Length > 1)
                  sCert = certNames[0];
               store.Open(OpenFlags.ReadOnly);

               X509Certificate2Collection certificates = store.Certificates;
               foreach (X509Certificate2 x509 in certificates)
               {
                  X500DistinguishedName dname = new X500DistinguishedName(x509.SubjectName);
                  //if (dname.Name.IndexOf(certName, StringComparison.OrdinalIgnoreCase) > -1)
                  //{
                  //    return x509;
                  //}
                  foreach (string s in certNames)
                  {
                     if (certs.Length >= 2)
                     {
                           // check subject
                           if ((certs[0].IndexOf("sub", StringComparison.OrdinalIgnoreCase) > -1) && (dname.Name.IndexOf(s, StringComparison.OrdinalIgnoreCase) > -1))
                           {
                              return x509;
                           }
                           else if ((certs[0].IndexOf("sn", StringComparison.OrdinalIgnoreCase) > -1) && (x509.GetSerialNumberString().ToLower() == s.ToLower()))
                           {
                              return x509;
                           }
                     }
                     else
                     {
                           // check issuer
                           if (x509.Issuer.IndexOf(s, StringComparison.OrdinalIgnoreCase) > -1)
                           {
                              return x509;
                           }
                     }
                  }
                  x509.Reset();
               }
               return null;
         }
         catch (Exception ex)
         {
               errMsg = String.Format("AET:SSL {0}", ex.Message);
               //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 115, 500);
               Logging.WriteLogFile(errMsg);
               return null;
         }
         finally
         {
               store.Close();
         }
      }
      #endregion
   }
}
