﻿/*
 * Program Name: New Standards and Operating Rules - Payer Side (POR)
 * Contractor Name: Harris Corporation
 * Contract Number: VA118-11-D-1009
 * Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
 * 
 * Developer: Zach Bertram
 * Date: 01.14.2013
 * ----------------
 * Added missing documentation for methods and classes
 * Enfored CORE compliance in responses
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using VA270Parser;
using VAZip;
using System.Linq;
using System.Diagnostics;
using System.Data.SqlClient;
using AETShared;
using System.Security.Cryptography.X509Certificates;
using X12Engine;

/*
 * 
 * Information Source (Loop 2000A)
Information Receiver (Loop 2000B)
Subscriber (Loop 2000C)
Dependent (Loop 2000D)
Eligibility or Benefit Inquiry
Subscriber (Loop 2000C)
Eligibility or Benefit Inquiry
Dependent (Loop 2000D)
Eligibility or Benefit Inquiry
 * 
 * 
 * */

namespace AETService
{
    /// <summary>
    /// Business Logic Component Server for responding to 270 inquiries
    /// </summary>
    class BLCServer : ServerModel
    {
        /// <summary>
        /// Enum for dealing with the results of date comparisons
        /// </summary>
        private enum DateComparisonResult
        {
            Earlier = -1,
            Later = 1,
            TheSame = 0
        };

        string m_Estatus = string.Empty;  //eligibility status returned by ESMCall
        string m_Edate = string.Empty;  //eligibility date returned by ESMCall

        private string m_line1 = string.Empty;  //first line of address information returned by ESMCall
        private string m_city = string.Empty;  //city address information returned by ESMCall
        private string m_state = string.Empty;  //state address information returned by ESMCall
        private string m_zipcode = string.Empty;  //Zip Code address information returned by ESMCall
        private string m_EmdeonControl = string.Empty;  //Emdeon control number that came in on the 270 request

        string MVIPayload = string.Empty;  //response from MVI returned by MVICall
        const string NL = "\r\n";  //newline
        X509Certificate certificate;

        //StreamWriter writer = null;  //StreamWriter used to write to log file as well as TCP connection

        /// <summary>
        /// Creates a new BLCServer; must call Start() to use the new object
        /// </summary>
        public BLCServer()
        {
        }

        /// <summary>
        /// Creates a new BLCServer that can be used to listen for 270 inquiries
        /// </summary>
        /// <returns>A brand new instance of BLCServer; must call Start() to use the new object</returns>
        public override ServerModel Clone()
        {
            return new BLCServer();
        }

        //ZEB VA118-11-D-1009 1/14/13 (+method) fixed misspelling in method name; removed string concatenation for efficiency
        /// <summary>
        /// Composes a message to return over the connection when a special command is sent instead of a 270 inquiry
        /// </summary>
        /// <param name="msgType">1: message for QUIT command; 2: message for CLEAN command</param>
        /// <returns>A string with the appropriate message</returns>
        private string CreateReturnMessage(int msgType)
        {
            string msg = "";

            switch (msgType)
            {
                case 1:
                    msg = "The AET has responded to a close request by the client.\r\n";
                    break;
                case 2:
                    msg = "The AET has responded to a clean request by the client.\r\n";
                    break;
                default:  //ZEB VA118-11-D-1009 4/11/13 (+default) added default case in the event an unexpected value for msgType is passed in
                    break;
            }
            return msg;
        }

        /// <summary>
        /// Reads 270 inquiry or special command present on the TCP steam and responds accordingly.
        /// This method is synchronous: it executes on this thread until completed.
        /// </summary>
        public void Start(string Message, gDataClass ConfigurationData, ConnectedClient myConnectedClient, out string Response, out Parser p270)
        {
				Utils.LogEvent("BLCServer Start()");

				string auditMsg = "";
            Response = null;
            p270 = null;
            string clientIP = myConnectedClient.remoteEndpointAddress;
            string errorMsg = "";
            int errorId = 0;
            string TRN = "";

            //Logging.WriteLogFile("BLC Server received Message.  ClientIP = " + clientIP);

            this.certificate = myConnectedClient.serverCertificate;

            bool returnAAA57 = false;  //ZEB VA118-11-D-1009 3/1/13 keep track of AAA errors for date of service
            bool returnAAA58 = false;  //ZEB VA118-11-D-1009 3/11/13 keep track of AAA errors for birthdate
            bool returnAAA62 = false;  //ZEB VA118-11-D-1009 2/6/13 keep track of AAA errors for date of service
            bool returnAAA71 = false;  //ZEB VA118-11-D-1009 1/31/13 (+3) keep track of AAA errors for alternate MVI searches
            bool returnAAA72 = false;
            bool returnAAA73 = false;
            bool returnAAA75 = false;

            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Client Authenticate as required
            //bool bAuth = Authenticate(myConnectedClient.clientTcpClient, myConnectedClient.mySslStream, ConfigurationData.SSLName);
            // authentication is already done
            bool bAuth = true;
            //if (!bAuth)
            //{
            //    // Client Security Authenticate failed. The client will be dropped.....
            //    auditMsg = String.Format("SSL/TLS-Authenticate Failed from client IP: {0}.", clientIP);
            //    Logging.WriteLogFile(auditMsg);
            //    ConfigurationData.AuditLog(auditMsg, System.Diagnostics.EventLogEntryType.Error);
            //    System.Threading.Thread.Sleep(1000);
            //    myConnectedClient.clientTcpClient.Close();
            //    return;
            //}
            //else
            //{
            //    auditMsg = String.Format("SSL/TLS - Has Been Successfully Authenticate From Client IP: {0}", clientIP);
            //    Logging.WriteLogFile(auditMsg);
            //    if (ConfigurationData.TraceEnabled)
            //    {
            //        // Client Security Authenticate passed. 
            //        ConfigurationData.AuditLog(auditMsg, System.Diagnostics.EventLogEntryType.Information);
            //    }
            //}


            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Get Cliet Data;
            //using (StreamReader reader = new StreamReader(tcp.GetStream()))
            //{
            //    char[] msgCount = new char[6];
            //    reader.ReadBlock(msgCount, 0, 6);  //The first 6 bytes of the message contain the size of the rest of the message

            string sCount = "";
            int byteCnt = 0;
            try
            {
                sCount = Message.Substring(0, 6);
                byteCnt = int.Parse(sCount);
            }
            catch (Exception ex)
            {
                auditMsg = String.Format("The inbound message size of [{0}] is invalid and cannot be read. The client connection will be terminated.\n{1}", sCount, ex.Message);
                Logging.WriteLogFile(auditMsg + "\r\n" + ex.StackTrace);
                ConfigurationData.AuditLog(auditMsg, System.Diagnostics.EventLogEntryType.Error);
                //reader.Close();
                return;
            }

            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Read inbound message
            //char[] InboundMsg = new char[byteCnt];
            //reader.ReadBlock(InboundMsg, 0, byteCnt);   // Read Emdeon Payer byte count
            string sMsg = Message.Substring(6);
            string sControl = sMsg.Substring(0, 6);     // Read Emdeon Control Number
            string s270 = sMsg.Substring(6);            // Extract the 270 payload
            int logId = 0;

            m_EmdeonControl = sControl;

            //Logging.WriteLogFile("Byte Count = " + byteCnt + ", Control Number = " + sControl + ", Message = " + s270);

            //Audit Log ID
            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Check for and process a Infrastructure command message.
            if (s270.Length == 4 && s270 == "QUIT")
            {
                if (ConfigurationData.TraceEnabled)
                {
                    auditMsg = String.Format("AET was requested to close. Inbound message size ={0}\n{1}", sCount, s270);
                    ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                }

                string sBody1 = sControl.Substring(0, 6);
                sBody1 += CreateReturnMessage(1);
                string sSize1 = sBody1.Length.ToString("000000");
                string sEmdeonMessage1 = String.Format("{0}{1}", sSize1, sBody1);
                Response = sEmdeonMessage1;

                //writer = new StreamWriter(tcp.GetStream());
                //writer.AutoFlush = true;
                //writer.Write(sEmdeonMessage1);

                //System.Threading.Thread.Sleep(500);
                //reader.Close();
                return;
            }
            //////////////////////////////////////////////////////////////////////////////////////////
            if (s270.Length == 5 && s270 == "CLEAN")
            {
                if (ConfigurationData.TraceEnabled)
                {
                    auditMsg = String.Format("AET was requested to clean debug files. Inbound message size ={0}\n{1}", sCount, s270);
                    ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                }

                string sourceDir = Environment.CommandLine;
                sourceDir = sourceDir.Replace("AETService.exe", "");
                sourceDir = sourceDir.Replace("\"", "");

                //string NL = "\r\n";
                //System.Text.StringBuilder sb1 = new System.Text.StringBuilder();

                string[] xmlList = Directory.GetFiles(sourceDir, "*_*.xml");
                foreach (string f in xmlList)
                {
                    string fName = f.Substring(sourceDir.Length);
                    string fullFileNamr = Path.Combine(sourceDir, fName);
                    File.Delete(fullFileNamr);

                    //sb1.Append(fullFileNamr);
                    //sb1.Append(NL);
                }

                //string sBody1 = string.Format("{0:D6}", sControl);
                string sBody1 = sControl;
                sBody1 += CreateReturnMessage(2); //sb1.ToString();
                string sSize1 = string.Format("{0:D6}", sBody1.Length);
                string sEmdeonMessage1 = String.Format("{0}{1}", sSize1, sBody1);
                Response = sEmdeonMessage1;

                //writer = new StreamWriter(tcp.GetStream());
                //writer.AutoFlush = true;
                //writer.Write(sEmdeonMessage1);

                //System.Threading.Thread.Sleep(500);
                //reader.Close();
                return;
            }

            if (s270.Length > 4 && s270.Substring(0, 5) == "RPT01")
            {
                if (ConfigurationData.TraceEnabled)
                {
                    auditMsg = String.Format("AET was requested to RPT1. Inbound message size ={0}\n{1}", sCount, s270);
                    ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                }

                string[] indata = s270.Split('~');
                int cnt = indata.Length;
                string Rtn = string.Empty;
                string NL = "\r\n";

                try
                {
                    string sCWCSConnectionString1 = String.Format("Data Source={0};Initial Catalog={1};User ID={2};Password={3};Timeout=30;", ConfigurationData.LogServer, ConfigurationData.LogDatabase, ConfigurationData.LogUser, ConfigurationData.LogPassword);
                    SqlConnection connectionObj1 = new SqlConnection(sCWCSConnectionString1);
                    connectionObj1.Open();

                    SqlCommand cmd = new SqlCommand("SELECT * FROM BenefitAuditLog WHERE RequestTime >= @Min AND RequestTime <= @Max", connectionObj1);
                    cmd.CommandType = System.Data.CommandType.Text;

                    if (cnt == 3)
                    {
                        cmd.Parameters.AddWithValue("@Min", indata[1]);
                        cmd.Parameters.AddWithValue("@Max", indata[2]);
                    }
                    else
                    {
                        DateTime dispDt = DateTime.Now;
                        string frm = String.Format("{0}-{1}-{2}", dispDt.Year, dispDt.Month, dispDt.Day);
                        string to = frm;

                        cmd.Parameters.AddWithValue("@Min", frm + " 0:00");
                        cmd.Parameters.AddWithValue("@Max", to + " 23:59:59");
                    }

                    SqlDataReader readerB = cmd.ExecuteReader();

                    Rtn = NL;
                    Rtn += "AET ID,Transaction Number,Request Time,Transaction Time,EB Return Value,MVI Time,MVI Status,ES Time,ES Status,ErrorID,ErrorMsg,270Errors\r\n";

                    /*
        0	    [k_pk_id]
        1       [ClearingHouseCtlID]
        2       [RequestTime]
        3       [RequestQualifier]
        4       [ResponseTime]
        5       [ResponseQualifier]
        6       [StatusIndicator]
        7       [EBReturn]
        8       [ErrorID]
        9       [ErrorMessage]
        10      [EXETime]
        11      [ASC270Errors]
        12      [TRNID]
        13      [InboundPayload]
        14      [OutboundPayload]
        15      [ResponseIPAddress]
        16      [MVITime]
        17      [MVIStatus]
        18      [MVIPayload]
        19      [ESRTime]
        20      [ESRStatus]
        21      [ESRPayload]
                     * 
                     * */


                    // k_pk_id, TRNID, RequestTime, EBReturn, MVIStatus, MVITime, ESRStatus, ESRTime
                    while (readerB.Read())
                    {
                        string AETID = string.Empty;
                        string ClearingHouseCtlID = string.Empty;
                        string RequestTime = string.Empty;
                        string RequestQualifier = string.Empty;
                        string ResponseTime = string.Empty;
                        string ResponseQualifier = string.Empty;
                        string StatusIndicator = string.Empty;
                        string EBReturn = string.Empty;
                        string ErrorID = string.Empty;

                        string ErrorMessage = string.Empty;
                        string EXETime = string.Empty;
                        string ASC270Errors = string.Empty;
                        string TRNID = string.Empty;
                        string InboundPayload = string.Empty;
                        string OutboundPayload = string.Empty;
                        string ResponseIPAddress = string.Empty;
                        string MVITime = string.Empty;
                        string MVIStatus = string.Empty;
                        string MVIPayload = string.Empty;
                        string ESRTime = string.Empty;
                        string ESRStatus = string.Empty;
                        string ESRPayload = string.Empty;

                        AETID = readerB.GetValue(0).ToString();

                        ClearingHouseCtlID = readerB.GetValue(1).ToString();
                        RequestTime = readerB.GetValue(2).ToString();
                        RequestQualifier = readerB.GetValue(3).ToString();
                        ResponseTime = readerB.GetValue(4).ToString();
                        ResponseQualifier = readerB.GetValue(5).ToString();
                        StatusIndicator = readerB.GetValue(6).ToString();
                        EBReturn = readerB.GetValue(7).ToString();
                        ErrorID = readerB.GetValue(8).ToString();
                        ErrorMessage = readerB.GetValue(9).ToString();
                        EXETime = readerB.GetValue(10).ToString();
                        ASC270Errors = readerB.GetValue(11).ToString();
                        TRNID = readerB.GetValue(12).ToString();
                        InboundPayload = readerB.GetValue(13).ToString();
                        OutboundPayload = readerB.GetValue(14).ToString();
                        ResponseIPAddress = readerB.GetValue(15).ToString();
                        MVITime = readerB.GetValue(16).ToString();
                        MVIStatus = readerB.GetValue(17).ToString();
                        MVIPayload = readerB.GetValue(18).ToString();
                        ESRTime = readerB.GetValue(19).ToString();
                        ESRStatus = readerB.GetValue(20).ToString();
                        ESRPayload = readerB.GetValue(21).ToString();

                        if (EXETime.Length > 8)
                            EXETime = EXETime.Substring(0, 8);

                        if (MVITime.Length > 8)
                            MVITime = MVITime.Substring(0, 8);

                        if (ESRTime.Length > 8)
                            ESRTime = ESRTime.Substring(0, 8);

                        //string rptData = String.Format("{0},{1},{2},{3},{4},{5},{6},{7}\r\n", AETID, TRNID, RequestTime, EBReturn, MVITime, MVIStatus, ESRTime, ESRStatus);
                        string rptData = String.Format("{0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11}\r\n", AETID, TRNID, RequestTime, EXETime, EBReturn, MVITime, MVIStatus, ESRTime, ESRStatus, ErrorID, ErrorMessage, ASC270Errors);
                        Rtn += rptData;
                    }

                    readerB.Close();
                    cmd.Dispose();
                    connectionObj1.Close();
                }
                catch (Exception ex)
                {
                    string erMsg = String.Format("Log Request failed: {0}\r\nServer: {1}, Database: {2}, UserID: {3}, UserData: {4}\r\n", ex.Message, ConfigurationData.LogServer, ConfigurationData.LogDatabase, ConfigurationData.LogUser, ConfigurationData.LogPassword);
                    Logging.WriteLogFile(erMsg + "\r\n" + ex.StackTrace);

                    Rtn = erMsg;
                }
                System.Text.StringBuilder sbE = new System.Text.StringBuilder();
                sbE.Append(Rtn);
                sbE.Append(NL);

                string sBody1 = sControl.Substring(0, 6);
                sBody1 += sbE.ToString();
                string sSize1 = sBody1.Length.ToString("000000");
                string sEmdeonMessage1 = String.Format("{0}{1}", sSize1, sBody1);
                Response = sEmdeonMessage1;

                //writer = new StreamWriter(tcp.GetStream());
                //writer.AutoFlush = true;
                //writer.Write(sEmdeonMessage1);

                //System.Threading.Thread.Sleep(500);
                //reader.Close();
                return;
            }

            //We have taken care of all special commands; what follows is for processing a real 270
            Stopwatch timer270 = new Stopwatch();
            timer270.Start();

            logId = ConfigurationData.LogRequest(sControl, sMsg, clientIP);

            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Check to insure the transaction Contains a proper x12 ISA - INTERCHANGE CONTROL HEADER.
            if (s270.Length < 3 || s270.Substring(0, 3) != "ISA")
            {
                auditMsg = "The inbound 270 request does not contain a ISA - INTERCHANGE CONTROL HEADER. The client connection will be terminated.";
                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                ConfigurationData.LogMVI(logId, (float)0.0, "MVI Not Called:", "MVI Payload"); //MVIPayload)
                ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");
                ConfigurationData.LogError(logId, 107, "No ISA in the 270");
                // Add not a 270 error to erro message.
                return;
            }

            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Check to insure the transaction Contains a x12 005010X279A1 (270) Eligibility request code.
            if (!s270.Contains("005010X279A1"))
            {
                auditMsg = "The inbound 270 request does not contain the expected 005010X279A1 Version Identifier Code. The client connection will be terminated.";
                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                //reader.Close();
                ConfigurationData.LogMVI(logId, (float)0.0, "MVI Not Called:", "MVI Payload"); //MVIPayload)
                ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");
                ConfigurationData.LogError(logId, 109, "No 005010X279A1 in the 270");
                return;
            }

            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // If the inbound rquest is over 3K it is two large for a 270 request.
            if (byteCnt > 1024 * 3)
            {
                auditMsg = String.Format("Inbound Message size is too big for a 270 request. The connection will be closed.\n byteCnt={0}", byteCnt);
                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 170, 500);
                ConfigurationData.LogMVI(logId, (float)0.0, "MVI Not Called:", "NO MVI Payload"); //MVIPayload)
                ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");
                ConfigurationData.LogError(logId, 111, "270 byte count > (1024*3)");
                return;
            }

            string pLN = "";
            string PFN = "";
            string PMN = "";
            string PSF = "";
            string PDOB = "";
            string PGEN = "";
            string PSS = "";
            string PFZ = "";
            string PST = "";
            string VPIN = "";
            string ES = "";
            string msg = "";

            VAZip.Facility AddressInfo = null;
            string VAURL = "www2.va.gov/directory/guide/home.asp";

            if (ConfigurationData.TraceEnabled)
            {
                auditMsg = String.Format("STARTING 270 Parse for Emdeon Control Number:{0}. 270 Packet size:{1} Bytes", sControl, byteCnt);
                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
            }

            try
            {
                p270 = new Parser(ConfigurationData.AnsiServer, ConfigurationData.AnsiDatabase, ConfigurationData.AnsiUser, ConfigurationData.AnsiPassword, "90");
                p270.Parse270(s270);
                foreach (string error in p270.ErrorList)
                {
                    //Logging.WriteLogFile("P270 parser error: " + error);
                }
            }
            catch (Exception ex)
            {
                auditMsg = String.Format("Catastrophic 270 Parser error: {0}", ex.Message);
                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                Logging.WriteLogFile(auditMsg + "\r\n" + ex.StackTrace);
                //reader.Close();

                ConfigurationData.LogMVI(logId, (float)0.0, "MVI Not Called:", "NO MVI Payload"); //MVIPayload)
                ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");

                return;
            }

            bool BadNPI = false;
            bool HasParseErrors = false;  //ZEB VA118-11-D-1009 1/31/13 allow easier handling of errors
            string InfoReceiverNPI;
            InfoReceiverNPI = p270.InfoReceiver.IdentificationNumber;

            SqlConnection connectionObj = null;
            SqlCommand QueryCommand = null;
            SqlDataReader readerA = null;

            string sQuery = String.Format("SELECT NPI FROM LEIELookup WHERE NPI = '{0}'", InfoReceiverNPI);
            string sCWCSConnectionString = String.Format("Data Source={0};Initial Catalog={1};User ID={2};Password={3};Timeout=60;", ConfigurationData.LEIEServer, ConfigurationData.LEIEDatabase, ConfigurationData.LEIEUser, ConfigurationData.LEIEPassword);

            try
            {
                connectionObj = new SqlConnection(sCWCSConnectionString);
                connectionObj.Open();

                QueryCommand = new SqlCommand(sQuery, connectionObj);
                readerA = QueryCommand.ExecuteReader();

                if (readerA.HasRows)
                {
                    readerA.Read();
                    string rptData = readerA.GetString(0);
                    BadNPI = true;
                    string sMsgA = String.Format("NPI {0} was found on the LEIE Exclusion list. A - ASCx12 AAA*Y**50*N~ Segment will be returned.", InfoReceiverNPI);
                    //System.Diagnostics.EventLog.WriteEntry("AETService", sMsgA, System.Diagnostics.EventLogEntryType.Warning, 118, 200);
                    Logging.WriteLogFile(sMsgA);
                }
                else
                {

                    ////System.Diagnostics.EventLog.WriteEntry("AETService", sQuery, System.Diagnostics.EventLogEntryType.Error);
                }
            }
            catch (Exception ex)
            {
                string erMsg = String.Format("LEIE DB Error:: {0}\r\nServer: {1}, Database: {2}, UserID: {3}, UserData: {4}", ex.Message, ConfigurationData.LEIEServer, ConfigurationData.LEIEDatabase, ConfigurationData.LEIEUser, ConfigurationData.LEIEPassword);
                //System.Diagnostics.EventLog.WriteEntry("AETService", erMsg, System.Diagnostics.EventLogEntryType.Error);
                Logging.WriteLogFile(erMsg + "\r\n" + ex.StackTrace);
            }
            finally
            {
                if (readerA != null)
                {
                    readerA.Close();
                    readerA.Dispose();
                    readerA = null;
                }
                QueryCommand.Dispose();
                connectionObj.Close();
            }

            if (BadNPI)
            {
                p270.PatientData.EligibilityOfBenifitInformationCode = "V";  //ZEB VA118-11-D-1009 6/5/13 return V if we won't process due to LEIE
                p270.InfoReceiver.AAA = "AAA*Y**50*N";
                errorMsg = String.Format("Provider NPI: {0} found in LEIE database", InfoReceiverNPI);
                errorId = 110;

                ConfigurationData.LogMVI(logId, (float)0.0, "MVI Not Called:", "MVI Payload"); //MVIPayload)
                ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");
            }
            else if (p270.ErrorList.Count > 0)  //ZEB VA118-11-D-1009 1/31/13 (+else if) fix handling of parser errors for CORE compliance
            {
                foreach (string s in p270.ErrorList)
                {
                    auditMsg = String.Format("LogId {0} 270 parse error {1}", logId, s);
                    ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);

                    switch (s)
                    {
                        case "DTP03 is required but missing.":
                            //set the subscriber date (Date of Service) to today
                            if (p270.PatientData.DtpSegments.Count < 1)
                            {
                                p270.PatientData.DtpSegments.Add(String.Format("DTP*291*D8*{0:D4}{1:D2}{2:D2}", DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day));  //add a DOS of today if it is missing
                            }
                            else
                            {
                                p270.PatientData.DtpSegments[0] = String.Format("DTP*291*D8*{0:D4}{1:D2}{2:D2}", DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day);  //Make sure DOS is today if existing segment had no date
                            }
                            break;
                        case "DMG01 Pair element relation not met.  Specified elements: 01, 02  ":
                            //Missing birthdate. We can use alternate searches for this.
                            break;
                        case "Invalid Birthdate":
                            p270.PatientData.AAASegments.Add("AAA*N**58*C");  //58 - Missing/invalid DOB
                            returnAAA58 = true;
                            HasParseErrors = true;
                            break;
                        case "NM108 Pair element relation not met.  Specified elements: 08, 09  ":
                            //This is a missing SSN, which we can search with (but will get an error later)
                            break;
                        case "EQ01 Element ID is not a valid ID":
                            //This is an invalid service type code, which we can work with
                            break;
                        case "EQ01 Element ID is not a valid IDElement ID is not a valid ID":
                            //This is an invalid service type code, which we can work with
                            break;
                        default:
                            p270.InfoSource.AAA = "AAA*N**79*C";  //This is what the old code did whenever it found an error
                            HasParseErrors = true;
                            break;
                    }
                }
            }

            if (!BadNPI && (p270.PatientData.LastName.Length == 0) && (p270.PatientData.FirstName.Length == 0) && (p270.PatientData.DateOfBirth == DateTime.MinValue))  //ZEB VA118-11-D-1009 1/31/13 (+if) important identifiers from subscriber are missing, try dependent loop instead
            {
                PSS = p270.PatientData.SocialSecurityNum.Length > 0 ? p270.PatientData.SocialSecurityNum : p270.PatientData.PrimaryIdentifier;  //save SSN since dependent loop has no SSN field
                p270.DependentData.IDCodeQualifier = p270.PatientData.IDCodeQualifier; //ZEB VA118-11-D-1009 6/19/13 make sure NM108 is filled in when copying dependent data to subscriber loop
                p270.PatientData = p270.DependentData;  //copy dependent into subscriber loop so 271 is generated properly
                p270.PatientData.PrimaryIdentifier = PSS;

                //ZEB VA118-11-D-1009 4/3/13 (+10) correctly handle DTPs when using dependent loop
                if (p270.PatientData.DtpSegments.Count < 1)
                {
                    p270.PatientData.DtpSegments.Add(String.Format("DTP*291*D8*{0:D4}{1:D2}{2:D2}", DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day));  //add a DOS of today if it is missing
                }
                else if (p270.PatientData.DtpSegments[0].Length < 12)
                {
                    p270.PatientData.DtpSegments[0] = String.Format("DTP*291*D8*{0:D4}{1:D2}{2:D2}", DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day);  //Make sure DOS is today if existing segment had no date
                }
            }

            if (p270.PatientData.LastName == "")  //ZEB VA118-11-D-1009 2/21/13 (+if) added to return proper error when name is missing
            {
                p270.PatientData.AAASegments.Add("AAA*Y**73*C");  //73 - Invalid/Missing Subscriber/Insured Name
                p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                HasParseErrors = true;
            }

            if (!returnAAA58 && (p270.PatientData.DateOfBirth == DateTime.MinValue))  //ZEB VA118-11-D-1009 2/21/13 (+if) added to return proper error when DOB is missing
            {
                p270.PatientData.AAASegments.Add("AAA*Y**58*C");  //58 - Missing/invalid DOB
                p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                HasParseErrors = true;
            }

            if ((p270.PatientData.SocialSecurityNum == "") && (p270.PatientData.PrimaryIdentifier == ""))  //ZEB VA118-11-D-1009 2/21/13 (+if) added to return proper error when SSN is missing
            {
                p270.PatientData.AAASegments.Add("AAA*Y**72*C");  //72 - Ivalid/Missing Subscriber/Insured ID
                p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                HasParseErrors = true;
            }

            if (HasParseErrors)
            {
                PFZ = p270.InfoReceiver.Zip;
                p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient

                errorMsg = "270 Error";
                errorId = 120;

                ConfigurationData.LogMVI(logId, (float)0.0, "MVI Not Called", "No MVI Payload");
                ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");
            }
            else if (!BadNPI && (p270.PatientData != null))  //ZEB VA118-11-D-1009 2/4/13 (+else if) shift some code around to encapsulate better to work with subscriber or dependent
            {
                TRN = p270.PatientData.TraceNumbers.Count > 0 ? p270.PatientData.TraceNumbers[0].TraceNumber : "";  //ZEB VA118-11-D-1009 5/8/13 prevent crash if no trace number specified

                pLN = p270.PatientData.LastName;
                PFN = p270.PatientData.FirstName;
                //PDOB = p270.PatientData.DateOfBirth.ToShortDateString();  //ZEB VA118-11-D-1009 1/22/13 no need to set this twice
                PDOB = String.Format("{0:D4}{1:D2}{2:D2}", p270.PatientData.DateOfBirth.Year, p270.PatientData.DateOfBirth.Month, p270.PatientData.DateOfBirth.Day);
                if (PDOB == "00010101")
                {
                    PDOB = "";
                }
                PSS = p270.PatientData.SocialSecurityNum.Trim();
                if (PSS != null && PSS.Length == 0)
                {
                    PSS = p270.PatientData.PrimaryIdentifier;
                }

                PMN = p270.PatientData.MiddleName;
                PSF = p270.PatientData.NameSuffix;

                PGEN = p270.PatientData.Gender;
                PST = p270.PatientData.ServiceTypeCode;

                PFZ = p270.InfoReceiver.Zip;
                p270.PatientData.EligibilityOfBenifitInformationCode = "6";

                string[] DTPfields = p270.PatientData.DtpSegments[0].Split('*');  //ZEB VA118-11-D-1009 8/19/13 (+27) factored out date validation code to deal with date ranges

                if (DTPfields.Length < 3)
                {
                    returnAAA57 = true;
                }
                else if (DTPfields[2] == "D8")
                {
                    ParsePlanDate(DTPfields[3], out returnAAA57, out returnAAA62);
                }
                else if (DTPfields[2] == "RD8")
                {
                    string[] DTPrangeDates = DTPfields[3].Split('-');
                    if (DTPrangeDates.Length != 2)
                    {
                        returnAAA57 = true;
                    }
                    else
                    {
                        if (ParsePlanDate(DTPrangeDates[0], out returnAAA57, out returnAAA62))
                        {
                            ParsePlanDate(DTPrangeDates[1], out returnAAA57);
                        }
                    }
                }
                else
                {
                    returnAAA57 = true;
                }

                //ZEB VA118-11-D-1009 3/1/13 (+if) used if to add reporting of DOS-related errors without needing to do rest of query
                if (!(returnAAA62 || returnAAA57))
                {
                    Stopwatch timerMVI = new Stopwatch();
                    timerMVI.Start();

                    System.Threading.Thread.Sleep(1000 * 1);

                    if (ConfigurationData.TraceEnabled)
                    {
                        auditMsg = "STARTING The MVI (VAAFI) Web Service Call.";
                        ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }

                    VPIN = "NF";

                    VPIN = MVICall(pLN, PFN, PMN, PSF, PGEN, PDOB, PSS, ConfigurationData.MVIURL, ConfigurationData.MVIPKI, logId, ConfigurationData.PKICheck, ConfigurationData, out returnAAA71, out returnAAA72, out returnAAA73, out returnAAA75);    //ZEB VA118-11-D-1009 1/31/13 AAA errors needed for alternate MVI searches
                    VPIN.Trim();

                    if (returnAAA71)    //ZEB VA118-11-D-1009 2/21/13 AAA errors needed for alternate MVI searches
                    {
                        p270.PatientData.AAASegments.Add("AAA*Y**71*C");  //71 - Patient Birth Date Does Not Match That for the Patient on the Database
                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                    }

                    if (returnAAA72)    //ZEB VA118-11-D-1009 2/21/13 AAA errors needed for alternate MVI searches
                    {
                        p270.PatientData.AAASegments.Add("AAA*Y**72*C");  //72 - Ivalid/Missing Subscriber/Insured ID
                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                    }

                    if (returnAAA73)    //ZEB VA118-11-D-1009 2/21/13 AAA errors needed for alternate MVI searches
                    {
                        p270.PatientData.AAASegments.Add("AAA*Y**73*C");  //73 - Invalid/Missing Subscriber/Insured Name
                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                    }

                    if (returnAAA75)    //ZEB VA118-11-D-1009 2/21/13 AAA errors needed for alternate MVI searches
                    {
                        p270.PatientData.AAASegments.Add("AAA*Y**75*C");  //75 - Subscriber/Insured Not Found
                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                    }


                    ///////////////////////////////////////////////////////////////////////////////////////
                    // Log the 1306 -- Need to add the 1305 as well

                    timerMVI.Stop();


                    //VPIN = "NX"; // Fake out a MVI error messaGE

                    if (VPIN.Length >= 2 && VPIN == "NX")
                    {
                        errorMsg = "MVI WEB Service Error Server:500";
                        errorId = 130;

                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                        p270.InfoSource.AAA = "AAA*Y**42*R";

                        ConfigurationData.LogMVI(logId, (float)timerMVI.Elapsed.TotalSeconds, "MVI WEB Service ", "MVI Payload"); //MVIPayload)
                        ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");

                        auditMsg = String.Format("MVI (VAAFI) Web Service Call FAILED: {0}", VPIN);
                        ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Warning);

                        if (ConfigurationData.TraceEnabled)
                        {

                            auditMsg = String.Format("MVI (VAAFI) Web Service Call returned: {0}.", VPIN);
                            ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }
                    }
                    else if (VPIN.Length >= 2 && VPIN == "NF")
                    {

                        errorMsg = "MVI Not Found: " + VPIN;
                        errorId = 130;

                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient

                        ConfigurationData.LogMVI(logId, (float)timerMVI.Elapsed.TotalSeconds, "*MVI Not Found", "MVI Payload"); //MVIPayload)
                        ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");

                        if (ConfigurationData.TraceEnabled)
                        {

                            auditMsg = String.Format("MVI (VAAFI) Web Service Call returned: {0}.", VPIN);
                            ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }


                    }
                    else if (VPIN.Length >= 2 && VPIN != "NF" && VPIN.Substring(0, 2) != "AE" && VPIN.Substring(0, 2) != "ER")
                    {
                        //if(theData.LogPayload)
                        ConfigurationData.LogMVI(logId, (float)timerMVI.Elapsed.TotalSeconds, VPIN, "MVI Payload"); //MVIPayload)
                        ///////////////////////////////////////////////////////////////////////////////////////
                        // Log the STARTING The ES Web Service Call
                        if (ConfigurationData.TraceEnabled)
                        {
                            auditMsg = "STARTING The ES Web Service Call.";
                            ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }
                        Stopwatch timerESR = new Stopwatch();
                        timerESR.Start();
                        System.Threading.Thread.Sleep(1000);

                        ES = ESMCall(VPIN, logId, ConfigurationData);

                        timerESR.Stop();
                        ConfigurationData.LogESR(logId, (float)timerESR.Elapsed.TotalSeconds, ES, "ESR Payload");

                        if (ES.Length == 0)
                        {
                            msg = "ES Web Service Call FAILED ES Return data was empty.";
                            ConfigurationData.AuditLog(logId, msg, System.Diagnostics.EventLogEntryType.Warning);

                        }
                        else if (ES.Length >= 2 && ES.Substring(0, 2) == "NF")
                        {
                            p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                            p270.PatientData.BenefitBeginDate = "";
                            if (ConfigurationData.TraceEnabled)
                            {
                                auditMsg = "NO Subscribers Info returned from ES.";
                                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                            }
                        }
                        else if (ES.Length >= 2 && ES.Substring(0, 2) == "NX")
                        {
                            p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                            p270.InfoSource.AAA = "AAA*Y**42*R";
                            p270.PatientData.BenefitBeginDate = "";
                            if (ConfigurationData.TraceEnabled)
                            {
                                auditMsg = "NO Subscribers Info returned from ES. Added a AAA segment";
                                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                            }
                        }
                        else if (ES.Length >= 3 && ES.Substring(0, 3) == "ERR")
                        {
                            // ES Web Service Call FAILED

                            p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                            p270.InfoSource.AAA = "AAA*Y**42*R";
                            p270.PatientData.BenefitBeginDate = "";
                            if (ConfigurationData.TraceEnabled)
                            {
                                auditMsg = string.Format("Error returned from ES WEB Service: {0}\r\n. Added a AAA segment", ES);
                                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                            }
                        }
                        else
                        {
                            m_Estatus = m_Estatus.ToUpper();
                            if (m_Estatus == "VERIFIED" && m_Estatus.Length == 8)
                            {
                                string[] md = m_Edate.Split('T');
                                if (md.Any())
                                    m_Edate = md[0];

                                m_Edate = m_Edate.Replace("-", "");
                                p270.PatientData.EligibilityOfBenifitInformationCode = "1";
                                p270.PatientData.BenefitBeginDate = m_Edate;
                                //p270.PatientData.SubscriberDate = m_Edate;

                                if (m_line1.Length == 0)
                                {
                                    p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                                    p270.PatientData.BenefitBeginDate = "";
                                    if (ConfigurationData.TraceEnabled)
                                    {
                                        auditMsg = "NO Subscribers Address returned from ES.";
                                        ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                                    }
                                }
                                else
                                {
                                    p270.PatientData.AddressLine1 = m_line1;
                                    p270.PatientData.City = m_city;
                                    p270.PatientData.State = m_state;
                                    p270.PatientData.Zip = m_zipcode;
                                }

                                if (ConfigurationData.TraceEnabled)
                                {
                                    auditMsg = String.Format("ESCallCall-1A= {0} {1} {2}", m_Estatus, m_Edate, ES);
                                    ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                                }
                            }
                            else
                            {
                                p270.PatientData.EligibilityOfBenifitInformationCode = "6";
                                p270.PatientData.BenefitBeginDate = "";
                                if (ConfigurationData.TraceEnabled)
                                {
                                    auditMsg = String.Format("ESCallCall-1A= {0} {1} {2}", m_Estatus, m_Edate, ES);
                                    ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                                }
                            }
                        }
                    }
                    else
                    {
                        errorMsg = "MVI Not Found: " + VPIN;
                        errorId = 130;

                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient

                        if (VPIN != "NF")
                        {
                            p270.InfoSource.AAA = "AAA*Y**42*R";
                            auditMsg = String.Format("MVI (VAAFI) Web Service Call FAILED: {0}", VPIN);
                            ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                        }

                        ConfigurationData.LogMVI(logId, (float)timerMVI.Elapsed.TotalSeconds, "MVI Not Found", "MVI Payload"); //MVIPayload)
                        ConfigurationData.LogESR(logId, (float)0.0, "ESR not Called", "NO ESR Payload");

                        if (ConfigurationData.TraceEnabled)
                        {
                            auditMsg = String.Format("MVI (VAAFI) Web Service Call returned: {0}.", VPIN);
                            ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }
                    }
                }
                else  //ZEB VA118-11-D-1009 1/31/13 (+else) return correct error for invalid DOS
                {
                    if (returnAAA62)
                    {
                        p270.PatientData.AAASegments.Add("AAA*N**62*C");  //62 - Date of Service not within Allowable Inquiry Period
                    }
                    if (returnAAA57)
                    {
                        p270.PatientData.AAASegments.Add("AAA*N**57*C");  //57 - Invalid/Missing Date(s) of Service  //ZEB VA118-11-D-1009 8/8/13 correct formatting for error 57
                    }

                    p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                }
            }
            else
            {
                // No patient data returned from 270;
                auditMsg = "IParser(s270) FAILED";
                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Error);

                p270.InfoSource.AAA = "AAA*N**79*C";
            }

            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            //////////////////////////////////////////////////////////////////////////////
            // Setup for the Zip Code call
            if (ConfigurationData.TraceEnabled)
            {
                auditMsg = "STARTING Facility Zip Code Call";
                ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
            }
            AddressInfo = ZipCode(ConfigurationData.ZipServer, ConfigurationData.ZipDatabase, ConfigurationData.ZipUser, ConfigurationData.ZipPassword, PFZ, ConfigurationData, logId);
            if (AddressInfo.hasErrors)
            {
                ConfigurationData.AuditLog(logId, AddressInfo.errorMsg, System.Diagnostics.EventLogEntryType.Error);
                AddressInfo.errorMsg = VAURL;

            }
            //////////////////////////////////////////////////////////////////////////////



            StringBuilder sb = new StringBuilder();
            if (AddressInfo.hasErrors || string.IsNullOrEmpty(AddressInfo.name))
            {
                p270.NearestVAFacility.FacilityName = "UNKNOWN";
                p270.NearestVAFacility.AddressLine1 = "";
                p270.NearestVAFacility.AddressLine2 = "";
                p270.NearestVAFacility.City = "";
                p270.NearestVAFacility.State = "";
                p270.NearestVAFacility.Zip = "";
                p270.NearestVAFacility.Telephone = "";
                p270.NearestVAFacility.URL = VAURL;
            }
            else
            {
                p270.NearestVAFacility.FacilityName = AddressInfo.name;
                p270.NearestVAFacility.AddressLine1 = AddressInfo.address.street;
                p270.NearestVAFacility.AddressLine2 = "";
                p270.NearestVAFacility.City = AddressInfo.address.city;
                p270.NearestVAFacility.State = AddressInfo.address.state;
                p270.NearestVAFacility.Zip = AddressInfo.address.zip;
                if (AddressInfo.address.phones.Count > 0)
                {
                    p270.NearestVAFacility.Telephone = AddressInfo.address.phones[0];
                    p270.NearestVAFacility.Telephone = p270.NearestVAFacility.Telephone.Replace("-", "");
                    p270.NearestVAFacility.Telephone = p270.NearestVAFacility.Telephone.Replace("(", "");
                    p270.NearestVAFacility.Telephone = p270.NearestVAFacility.Telephone.Replace(")", "");

                }
                else
                {
                    p270.NearestVAFacility.Telephone = "";
                }
            }


            string s271 = "";
            string sBody = "";
            string sSize = "";
            string sEmdeonMessage = "";

            //ZEB VA118-11-D-1009 6/5/13 (-if/else) always return EB segments
            List<EBSegment> EBs = ConfigurationData.GenerateEBSegments(PST, p270.PatientData.EligibilityOfBenifitInformationCode);  //ZEB VA118-11-D-1009 5/16/13 (+4) return error 42 if unable to generate EB segments
            if (EBs.Count < 1)
            {
                p270.InfoSource.AAA = "AAA*Y**42*R";
                p270.PatientData.EligibilityOfBenifitInformationCode = "V";  //ZEB VA118-11-D-1009 6/5/13 return V if unable to find STC data
                EBs.Add(new EBSegment("V", "", PST, "VA FEE BASIS PROGRAM", "", "", "", "Y"));    //ZEB VA118-11-D-1009 6/6/13 make sure EB exists if unable to find STC data
            }

            s271 = p270.Build271(EBs, p270.PatientData.AAASegments);


            if (m_EmdeonControl != "000000")
                s271 = s271.Replace("\r\n", "");

            if (p270.PatientData.EligibilityOfBenifitInformationCode == "1")
            {
                int found = s271.IndexOf("DTP*291*D8*");
                if (found > 0)
                {
                    string seg = s271.Substring(found, 19);
                    string newSeg = String.Format("DTP*346*D8*{0}", m_Edate);
                    s271 = s271.Replace(seg, newSeg);
                }
            }

            //sBody = string.Format("{0:D6}", sControl);
            sBody = sControl;
            sBody += s271;
            sSize = string.Format("{0:D6}", sBody.Length);
            sEmdeonMessage = String.Format("{0}{1}", sSize, sBody);
            Response = sEmdeonMessage;

            //writer = new StreamWriter(tcp.GetStream());
            //writer.AutoFlush = true;
            //writer.Write(sEmdeonMessage);



            ConfigurationData.LogResponse(logId, p270.InfoSource.EntityTypeQualifier, p270.InfoReceiver.EntityTypeQualifier, "OK", 0, "", s271);
            // AET server process is done........

            auditMsg = String.Format("The AET had delivered a 271 Eligibility Response back to the server {0} with the results of {1}", clientIP, p270.PatientData.EligibilityOfBenifitInformationCode);
            ConfigurationData.AuditLog(logId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
            System.Threading.Thread.Sleep(500);


            // Look into this for final release

            {
                timer270.Stop();
                string sCWCSConnectionString1 = String.Format("Data Source={0};Initial Catalog={1};User ID={2};Password={3};Timeout=30;", ConfigurationData.LogServer, ConfigurationData.LogDatabase, ConfigurationData.LogUser, ConfigurationData.LogPassword);
                SqlConnection connectionObj1 = new SqlConnection(sCWCSConnectionString1);
                connectionObj1.Open();

                // 
                string benCode = p270.PatientData.EligibilityOfBenifitInformationCode;  //ZEB VA118-11-D-1009 6/5/13 (+9) prevent error since EBReturn column only allows numbers
                string sQuery1;

                if ((benCode == "1") || (benCode == "6"))
                {
                    sQuery1 = String.Format("UPDATE BenefitAuditLog SET EBReturn={0},ErrorId={1}, ErrorMessage='{2}', EXETime= {3}, ASC270Errors={4},TRNID='{5}' WHERE k_pk_id = {6}", benCode, errorId, errorMsg, timer270.Elapsed.TotalSeconds, p270.ErrorList.Count, TRN, logId);
                }
                else
                {
                    sQuery1 = String.Format("UPDATE BenefitAuditLog SET EBReturn=0,ErrorId={0}, ErrorMessage='{1}', EXETime= {2}, ASC270Errors={3},TRNID='{4}' WHERE k_pk_id = {5}", errorId, errorMsg, timer270.Elapsed.TotalSeconds, p270.ErrorList.Count, TRN, logId);
                }
                SqlCommand QueryCommand1 = new SqlCommand(sQuery1, connectionObj1);
                SqlDataReader readerA1 = QueryCommand1.ExecuteReader();

                if (readerA1.HasRows)
                {

                }
                else
                {
                    ////System.Diagnostics.EventLog.WriteEntry("AETService", sQuery, System.Diagnostics.EventLogEntryType.Error);
                }
                readerA1.Close();
                QueryCommand1.Dispose();
                connectionObj1.Close();
            }
            //} // socket is closed by using statement
        }

        //ZEB VA118-11-D-1009 8/19/13 factored out of Start to deal with validating date ranges
        /// <summary>
        /// Parses a DateTime from the date string in a DTP segment, and performs validation for a plan date
        /// Skips future date check -- for use with end date in a query with a date range
        /// </summary>
        /// <param name="PlanDate">The date from the DTP segment</param>
        /// <param name="returnAAA57">Whether or not AAA error 57 - Invalid/Missing Date of Service(</param>
        /// <returns>True if the plan date is valid, False otherwise</returns>
        private bool ParsePlanDate(string PlanDate, out bool returnAAA57)
        {
            bool bitBucket;  //Needed to fill out param, but not used

            ParsePlanDate(PlanDate, out returnAAA57, out bitBucket);

            return !returnAAA57;
        }

        //ZEB VA118-11-D-1009 8/19/13 factored out of Start to deal with validating date ranges
        /// <summary>
        /// Parses a DateTime from the date string in a DTP segment, and performs validation for a plan date
        /// </summary>
        /// <param name="PlanDate">The date from the DTP segment</param>
        /// <param name="returnAAA57">Whether or not AAA error 57 - Invalid/Missing Date of Service(</param>
        /// <param name="returnAAA62">Whether or not AAA error 62 - Date of Service not Within Allowable Inquiry Period</param>
        /// <returns>True if the plan date is valid and not too far in the future, False otherwise</returns>
        private bool ParsePlanDate(string PlanDate, out bool returnAAA57, out bool returnAAA62)
        {
            DateTime DOS = DateTime.MinValue;

            returnAAA57 = false;
            returnAAA62 = false;

            if ((PlanDate.Length > 1) && (PlanDate.Length < 8))  //ZEB VA118-11-D-1009 4/10/13 remove redundant condition
            {
                returnAAA57 = true;
            }
            else
            {
                int year, month, date;
                Int32.TryParse(Utilities.SafeSubStringMid(PlanDate, 0, 4), out year);
                Int32.TryParse(Utilities.SafeSubStringMid(PlanDate, 4, 2), out month);
                Int32.TryParse(Utilities.SafeSubStringMid(PlanDate, 6, 2), out date);

                if (year < 1) returnAAA57 = true;  //ZEB VA118-11-D-1009 6/19/13 (+20) strengthen date format check
                if (month < 1) returnAAA57 = true;
                if (date < 1) returnAAA57 = true;

                if (!returnAAA57)
                {

                    try
                    {
                        DOS = new DateTime(year, month, date);
                    }
                    catch (Exception ex)
                    {
                        returnAAA57 = true;
                        DOS = DateTime.MinValue;
                    }

                    if (!returnAAA57)
                    {
                        returnAAA62 = DOS > DateTime.Today.AddDays(30);
                        returnAAA57 = ((DOS == DateTime.MinValue) || ((PlanDate != "") && (PlanDate != String.Format("{0:D4}{1:D2}{2:D2}", DOS.Year, DOS.Month, DOS.Day))));  //ZEB VA118-11-D-1009 6/17/13 fixed check for invalid plan date format
                    }
                }
            }

            return !(returnAAA57 || returnAAA62);
        }

        /// <summary>
        /// Retrieves the nearest VA facility for a given Zip code
        /// </summary>
        /// <param name="DBServer">The database server that contains the Zip code crosswalk database</param>
        /// <param name="DB">The database on that server</param>
        /// <param name="user">The username for logging into the database</param>
        /// <param name="PWD">The password for logging into the database</param>
        /// <param name="zipcode">The Zip code to use to find a VA facility</param>
        /// <param name="theData">The service configuration</param>
        /// <param name="LogId">The ID of the log to use</param>
        /// <returns>A Facility object containing the information on the nearest VA facility to the given Zip code</returns>
        private Facility ZipCode(string DBServer, string DB, string user, string PWD, string zipcode, gDataClass theData, int LogId)
        {
            string auditMsg = string.Empty;
            Facility AddressInfo = null;
            try
            {
                VAFacilities vaf = new VAFacilities(DBServer, DB, user, PWD);
                AddressInfo = vaf.FindFacility(zipcode);
            }
            catch (Exception ex)
            {
                auditMsg = String.Format("ERR:ZipCode:{0}", ex.Message);
                theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                AddressInfo = new Facility();
                Logging.WriteLogFile(auditMsg + "\r\n" + ex.StackTrace);
            }
            return AddressInfo;
        }

        //ZEB VA118-11-D-1009 1/31/13 add boolean params for error codes needed by alternate searches
        /// <summary>
        /// Performs the call to the MVI web service to find a patient's ICN given their demographic info
        /// </summary>
        /// <param name="lastName">The patient's last name</param>
        /// <param name="firstName">The patient's first name</param>
        /// <param name="middleName">The patient's middle name</param>
        /// <param name="suffix">The patient's name suffix (like "Jr")</param>
        /// <param name="gender">The patient's gender ("M" or "F")</param>
        /// <param name="dob">The patient's date of birth</param>
        /// <param name="ssn">The patient's SSN</param>
        /// <param name="url">The URL of the MVI webservice</param>
        /// <param name="mvipki">The name of the PKI certificate to use</param>
        /// <param name="LogId">The ID of the log to log events</param>
        /// <param name="pkiCheck">Unused</param>
        /// <param name="theData">Service configuration information</param>
        /// <param name="AAA71">Returned as true if a AAA segment with an error code of 71 should be returned by the caller</param>
        /// <param name="AAA72">Returned as true if a AAA segment with an error code of 72 should be returned by the caller</param>
        /// <param name="AAA73">Returned as true if a AAA segment with an error code of 73 should be returned by the caller</param>
        /// <param name="AAA75">Returned as true if a AAA segment with an error code of 75 should be returned by the caller</param>
        /// <returns></returns>
        private string MVICall(string lastName, string firstName, string middleName, string suffix, string gender, string dob, string ssn, string url, string mvipki, int LogId, string pkiCheck, gDataClass theData, out bool AAA71, out bool AAA72, out bool AAA73, out bool AAA75)
        {
            string response = string.Empty;
            string auditMsg = string.Empty;

            AAA71 = false;  //ZEB VA118-11-D-1009 1/31/13 (+7) needed for alternate searches
            AAA72 = false;
            AAA73 = false;
            AAA75 = false;

            bool hasLastName = lastName.Length > 0;
            bool hasFirstName = firstName.Length > 0;
            bool hasDOB = dob.Length > 0;
            bool hasSSN = ssn.Length > 0;

            //ZEB VA118-11-D-1009 1/31/13 (+if/etc) added conditions and extra copies of search code to support alternate searches
            if (hasLastName && hasFirstName && hasDOB && hasSSN)
            {
                //search with all four
                try
                {
                    if (theData.TraceEnabled)
                    {
                        auditMsg = String.Format("MVICall-IN= {0} {1} {2} {3} {4} {5}\n{6}\n{7}", ssn, dob, firstName, lastName, middleName, suffix, url, mvipki);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }

                    PRPA1305 x1305 = new PRPA1305();

                    x1305.LastName = lastName;
                    x1305.FirstName = firstName;
                    x1305.MiddleName = middleName;
                    x1305.Suffix = suffix;

                    x1305.Gender = gender;
                    x1305.DOB = dob;
                    x1305.SSN = ssn;

                    x1305.MVI_ID = "200HAPE";
                    x1305.MviPki = mvipki;
                    x1305.LOGID = LogId;
                    x1305.URL = url;
                    x1305.WriteLog = theData.LogWEBService;

                    response = x1305.GET_VPID(certificate);

                    //ZEB VA118-11-D-1009 1/31/13 (+if/etc) added checks for results that don't match query for CORE compliance
                    if (response != "NF")
                    {
                        if ((lastName.ToUpper() != x1305.LastName.ToUpper()) || (firstName.ToUpper() != x1305.FirstName.ToUpper()))
                        {
                            AAA73 = true;
                            response = "NF";
                        }

                        if (dob != x1305.DOB)
                        {
                            AAA71 = true;
                            response = "NF";
                        }

                        if (ssn != x1305.SSN)
                        {
                            AAA72 = true;
                            response = "NF";
                        }
                    }

                    MVIPayload = x1305.MviPayload;

                    if (theData.TraceEnabled)
                    {
                        auditMsg = String.Format("response.Length= {0} {1}", response.Length, response);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }
                }
                catch (Exception ex)
                {
                    //log exception calling service
                    auditMsg = String.Format("MVI ERROR-1= {0}\n{1}", ex.Message, url);
                    theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 188, 500);
                    Logging.WriteLogFile(auditMsg + "\r\n" + ex.StackTrace);
                    return "";
                }

                //ZEB VA118-11-D-1009 1/31/13 (+if/etc) added checks for results that don't match query for CORE compliance
                if ((response == "NF") && !AAA73)
                {
                    //if failed, try with no first name
                    //if that succeeds, return error 73

                    try
                    {
                        if (theData.TraceEnabled)
                        {
                            auditMsg = String.Format("MVICall-IN= {0} {1} {2} {3} {4} {5}\n{6}\n{7}", ssn, dob, firstName, lastName, middleName, suffix, url, mvipki);
                            theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }

                        PRPA1305 x1305 = new PRPA1305();

                        x1305.LastName = lastName;
                        x1305.FirstName = "";
                        x1305.MiddleName = middleName;
                        x1305.Suffix = suffix;

                        x1305.Gender = gender;
                        x1305.DOB = dob;
                        x1305.SSN = ssn;

                        x1305.MVI_ID = "200HAPE";
                        x1305.MviPki = mvipki;
                        x1305.LOGID = LogId;
                        x1305.URL = url;
                        x1305.WriteLog = theData.LogWEBService;

                        response = x1305.GET_VPID(certificate);

                        //ZEB VA118-11-D-1009 1/31/13 (+if/etc) added checks for results that don't match query for CORE compliance
                        if ((response.Length > 0) && (response != "NF"))
                        {
                            AAA73 = true;
                            response = "NF";
                        }

                        MVIPayload = x1305.MviPayload;

                        if (theData.TraceEnabled)
                        {
                            auditMsg = String.Format("response.Length= {0} {1}", response.Length, response);
                            theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }

                    }
                    catch (Exception ex)
                    {
                        //log exception calling service
                        auditMsg = String.Format("MVI ERROR-1= {0}\n{1}", ex.Message, url);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 188, 500);
                        Logging.WriteLogFile(auditMsg + "\r\n" + ex.StackTrace);
                        return "";
                    }

                    //ZEB VA118-11-D-1009 1/31/13 (+if/etc) added checks for results that don't match query for CORE compliance
                    if ((response == "NF") && !AAA73 && !AAA71)
                    {
                        //if failed, try with no dob
                        //if that succeeds, return error 71

                        try
                        {
                            if (theData.TraceEnabled)
                            {
                                auditMsg = String.Format("MVICall-IN= {0} {1} {2} {3} {4} {5}\n{6}\n{7}", ssn, dob, firstName, lastName, middleName, suffix, url, mvipki);
                                theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                            }

                            PRPA1305 x1305 = new PRPA1305();

                            x1305.LastName = lastName;
                            x1305.FirstName = firstName;
                            x1305.MiddleName = middleName;
                            x1305.Suffix = suffix;

                            x1305.Gender = gender;
                            x1305.DOB = "";
                            x1305.SSN = ssn;

                            x1305.MVI_ID = "200HAPE";
                            x1305.MviPki = mvipki;
                            x1305.LOGID = LogId;
                            x1305.URL = url;
                            x1305.WriteLog = theData.LogWEBService;

                            response = x1305.GET_VPID(certificate);

                            //ZEB VA118-11-D-1009 1/31/13 (+if/etc) added checks for results that don't match query for CORE compliance
                            if (response != "NF")
                            {
                                AAA71 = true;
                                response = "NF";
                            }
                            else
                            {
                                AAA72 = true;  //if they all fail, return errors 72 and 73
                                AAA73 = true;
                            }

                            MVIPayload = x1305.MviPayload;

                            if (theData.TraceEnabled)
                            {
                                auditMsg = String.Format("response.Length= {0} {1}", response.Length, response);
                                theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                            }
                        }
                        catch (Exception ex)
                        {
                            //log exception calling service
                            auditMsg = String.Format("MVI ERROR-1= {0}\n{1}", ex.Message, url);
                            theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 188, 500);
                            Logging.WriteLogFile(auditMsg + "\r\n" + ex.StackTrace);
                            return "";
                        }
                    }
                }
            }
            else
            {
                //search with everything we've got
                //if we fail, return error code 75

                try
                {
                    if (theData.TraceEnabled)
                    {
                        auditMsg = String.Format("MVICall-IN= {0} {1} {2} {3} {4} {5}\n{6}\n{7}", ssn, dob, firstName, lastName, middleName, suffix, url, mvipki);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }


                    PRPA1305 x1305 = new PRPA1305();

                    x1305.LastName = lastName;
                    x1305.FirstName = firstName;
                    x1305.MiddleName = middleName;
                    x1305.Suffix = suffix;

                    x1305.Gender = gender;
                    x1305.DOB = dob;
                    x1305.SSN = ssn;

                    x1305.MVI_ID = "200HAPE";
                    x1305.MviPki = mvipki;
                    x1305.LOGID = LogId;
                    x1305.URL = url;
                    x1305.WriteLog = theData.LogWEBService;

                    response = x1305.GET_VPID(certificate);

                    //ZEB VA118-11-D-1009 1/31/13 (+if/etc) added checks for results that don't match query for CORE compliance
                    if (response != "NF")
                    {
                        if ((lastName.ToUpper() != x1305.LastName.ToUpper()) || (firstName.ToUpper() != x1305.FirstName.ToUpper()))
                        {
                            AAA73 = true;
                            response = "NF";
                        }

                        if (dob != x1305.DOB)
                        {
                            AAA71 = true;
                            response = "NF";
                        }

                        if (ssn != x1305.SSN)
                        {
                            AAA72 = true;
                            response = "NF";
                        }
                    }

                    MVIPayload = x1305.MviPayload;

                    if (theData.TraceEnabled)
                    {
                        auditMsg = String.Format("response.Length= {0} {1}", response.Length, response);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }

                }
                catch (Exception ex)
                {
                    //log exception calling service
                    auditMsg = String.Format("MVI ERROR-1= {0}\n{1}", ex.Message, url);
                    theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 188, 500);
                    Logging.WriteLogFile(auditMsg + "\r\n" + ex.StackTrace);
                    return "";
                }

                //ZEB VA118-11-D-1009 1/31/13 (+if/etc) added checks for results that don't match query for CORE compliance
                if ((response == "NF") && !(AAA71 || AAA72 || AAA73))
                {
                    AAA75 = true;
                }
            }

            return response;
        }

        private string ESMCall(string VPIN, int LogId, gDataClass theData)
        {
            string ES = "";
            try
            {
                if (theData.TraceEnabled)
                {
                    string auditMsg = String.Format("ES-IN= {0} {1}", VPIN, theData.ESURL);
                    theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                }

                if (VPIN != "NF" && VPIN != "AE")
                {
                    ESWebService es = new ESWebService();

                    es.MVI_ID = VPIN;
                    es.LOGID = LogId;

                    es.WriteLog = theData.LogWEBService;
                    es.ESUN = theData.ESUserName;
                    es.ESPW = theData.ESPassword;

                    es.URL = theData.ESURL;
                    ES = es.CALL_ES();

                    m_Estatus = es.Status;
                    m_Edate = es.Date;

                    m_line1 = Utilities.RemoveX12Delimiters(es.Line1);  //ZEB VA118-11-D-1009 10/7/13 (+3) cleaned up returned data in case it contains special chars
                    m_city = Utilities.RemoveX12Delimiters(es.City);
                    m_state = Utilities.RemoveX12Delimiters(es.State);
                    m_zipcode = Utilities.RemoveX12Delimiters(es.ZipCode);


                    //ES = "OK ";
                    //m_Estatus = "VERIFIED";
                    //m_Edate = "19700401";

                    //m_line1 = "1500 Belleau Wood";
                    //m_city = "Tallahassee";
                    //m_state = "FL";
                    //m_zipcode = "32308";
                }
                else
                {
                    string auditMsg = String.Format("ES Was not called due to the VPIN= {0}", VPIN);
                    theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Warning);

                }
            }
            catch (Exception ex)
            {
                ES = String.Format("ERR:ES:{0}", ex.Message);
                Logging.WriteLogFile(ES + "\r\n" + ex.StackTrace);
            }
            return ES;
        }

        public bool CheckAuthorization(string Username, string Password)
        {
            return true;
        }

        public string DecryptPassword(string Password)
        {
            return Password;
        }

        public string GetPassword(string Username)
        {
            try
            {

            }
            catch
            {
            }
            return "";
        }


    }
}
