﻿using System;
using System.Data.SqlClient;
using System.Net;
using System.Text;
using System.Net.Sockets;
using System.Security.Cryptography.X509Certificates;
using System.Net.Security;
using System.Security.Authentication;
using AuthorizationServer;
using System.Diagnostics;
using AETShared;
using X12Engine.BLL.X12._278;
using X12Engine;

namespace AETService
{
   // optional - used to raise an event when data is received
   public delegate void LineReceivedEventHandler(ConnectedClient sender, string data);

   /// <summary>
   /// This class handles connection and communications with clients that connect with this server
   /// </summary>
   public class ConnectedClient
   {
      public TcpClient clientTcpClient;
      static int readBufferSize = 8192;
      byte[] readBuffer = new byte[readBufferSize];
      //StringBuilder sbReceivedData = new StringBuilder();
      public string localEndpointAddress;
      public string remoteEndpointAddress; // used to Identify a specific client
      public long handle; // used to Identify a specific client
      public int port; // should be either AET or CCC (command)
      public SslStream mySslStream;
      gDataClass configurationData;
      CCCServer myCccServer;  // The CCC Server needs to maintain state of Username and Authorized
      object thisLock = new object();
      StringBuilder newMessageStringBuilder = new StringBuilder();
      public X509Certificate serverCertificate;

      public ConnectedClient()
      {

		}

      public ConnectedClient(TcpClient newTcpClient, X509Certificate ServerCertificate, gDataClass ConfigurationData)
      {
			Utils.LogEvent("ConnectedClient()");

			clientTcpClient = newTcpClient;
         configurationData = ConfigurationData;
         serverCertificate = ServerCertificate;
				
         try
         {
               // we use the remote end point and handle to give us a unique identifier for the client
               localEndpointAddress = newTcpClient.Client.LocalEndPoint.ToString();
               remoteEndpointAddress = newTcpClient.Client.RemoteEndPoint.ToString();
               handle = newTcpClient.Client.Handle.ToInt64();
               IPEndPoint localEndPoint = (IPEndPoint)newTcpClient.Client.LocalEndPoint;
               //IPEndPoint remoteEndPoint = (IPEndPoint)newTcpClient.Client.RemoteEndPoint;
               port = localEndPoint.Port;
               if (port == configurationData.CtlPort)
                  myCccServer = new CCCServer();

               X500DistinguishedName dname = new X500DistinguishedName(((X509Certificate2)ServerCertificate).SubjectName);

               mySslStream = new SslStream(clientTcpClient.GetStream(), true);
               mySslStream.AuthenticateAsServer(ServerCertificate, false, SslProtocols.Ssl3 | SslProtocols.Tls | SslProtocols.Tls11 | SslProtocols.Tls12, false);
               //mySslStream.ReadTimeout = 5000;
               //mySslStream.WriteTimeout = 5000;

               // This starts the asynchronous read thread.  The data will be saved into readBuffer.
               //clientTcpClient.GetStream().BeginRead(readBuffer, 0, readBufferSize, StreamReceiver, null);
               mySslStream.BeginRead(readBuffer, 0, readBufferSize, StreamReceiver, null);
         }
         catch (Exception ex)
         {
               string errorMessage = "Exception on Client " + remoteEndpointAddress + ";" + handle + ";" + port + "," + ex.Message + "\r\n" + ex.StackTrace;
               //System.Diagnostics.EventLog.WriteEntry("AETService", errorMessage, System.Diagnostics.EventLogEntryType.Error, 115, 500);
               Logging.WriteLogFile(errorMessage);
         }
      }

      public event LineReceivedEventHandler LineReceived;
      protected virtual void OnLineReceived(ConnectedClient sender, string data)
      {
         // raise line received event
         //LineReceivedEventHandler handler = LineReceived;
         //if (handler != null)
         //{
         //    handler(sender, data);
         //}
      }

      public void SendDataString(string DataToSend)
      {
         try
         {
				// Synclock ensure that no other threads try to use the stream at the same time.
				lock (thisLock)
            {
               System.IO.StreamWriter writer = new System.IO.StreamWriter(mySslStream);
               {
                  writer.Write(DataToSend);
                  // Make sure all data is sent now.
                  writer.Flush();
               }
               CloseClient();
            }
         }
         catch (Exception ex)
         {
            string errorMessage = "Exception on Client " + remoteEndpointAddress + ";" + handle + ";" + port + "," + ex.Message + "\r\n" + ex.StackTrace;
            //System.Diagnostics.EventLog.WriteEntry("AETService", errorMessage, System.Diagnostics.EventLogEntryType.Error, 115, 500);
            Logging.WriteLogFile(errorMessage);
         }
      }

      // This is the callback function for TcpClient.GetStream.Begin. It begins an asynchronous read from a stream.
      private void StreamReceiver(IAsyncResult ar)
      {
         int bytesRead = 0;
         string newMessage = null;

         try
         {
            // Keith 5/12/2016 - I was receiving partial messages.  A delay here should allow time for the whole message to be received
            //System.Threading.Thread.Sleep(25);
            // Ensure that no other threads try to use the stream at the same time.
            lock (thisLock)
            {
               // Finish asynchronous read into readBuffer and get number of bytes read.
               bytesRead = mySslStream.EndRead(ar);
            }

            if (bytesRead > 0)
               newMessageStringBuilder.Append(Encoding.ASCII.GetString(readBuffer, 0, bytesRead));

            if ((newMessageStringBuilder.ToString().IndexOf("IEA*") == -1) && (bytesRead != 0))
            {
               // We are not finished reading.
               // Asynchronously read more message data from  the server.
               mySslStream.BeginRead(readBuffer, 0, readBufferSize, StreamReceiver, null);
            }
            else
            {
               // processing message can take some time.  We don't want to miss any new messages
               // grab message, copy message off, in case new message comes in
               newMessage = newMessageStringBuilder.ToString();
               newMessageStringBuilder.Length = 0;
               if (newMessage.Length > 6)
               {
						//Debug.WriteLine("Message from the server: {0}", newMessage);
						Utils.LogEvent("AET ProcessMessage() newMessage - " + newMessage);
						ProcessMessage(newMessage);
               }
               else
               {
                  // KEB 7/18/2016 - when the production test server started testing this AET Server - it was constantly sending zero length messages
                  // It create HUGE 300 MB log files in one day.  So if the message is zero bytes - do not log it
                  //if ((newMessage.Length > 0) || (configurationData.EnableLogging))
                        //Logging.WriteLogFile("Received message, but message length was too short to be processed as a valid message.  IP Address: " + remoteEndpointAddress + ", Handle: " + handle + ", Port: " + port + ", sent message: " + newMessage + ", Length = " + newMessage.Length);
               }
               // we close the connection after we read a message and then respond.  We do NOT read more than one message per connection
               //mySslStream.BeginRead(readBuffer, 0, readBufferSize, StreamReceiver, null);
            }
         }
         catch (Exception ex)
         {
            string errorMessage = "Exception on Client " + remoteEndpointAddress + ";" + handle + ";" + port + "," + ex.Message + "\r\n" + ex.StackTrace;
            //System.Diagnostics.EventLog.WriteEntry("AETService", errorMessage, System.Diagnostics.EventLogEntryType.Error, 115, 500);
            Logging.WriteLogFile(errorMessage);
         }
      }

      protected void ProcessMessage(string NewMessage)
      {

			string[] messageSegments;
         string messageNumber = null;
         string response = null;
         VA270Parser.Parser p270;
         string strIncomingSize;
         string strOutgoingSize;
         string strControlNumber;
         int intSize;
         int intControlNumber;
         BLCServer myBlcServer;
         AuthorizationServer.RequestProcessor my278Server;

         SqlConnectionStringBuilder myCdwSqlConnectionStringBuilder;

         try
         {
            if (!string.IsNullOrEmpty(NewMessage))
            {
               if (port == configurationData.AETPort)
               {
                  // get info regarding the message 
                  strIncomingSize = NewMessage.Substring(0, 6);
                  strControlNumber = NewMessage.Substring(6, 6);

                  int.TryParse(strIncomingSize, out intSize);
                  int.TryParse(strControlNumber, out intControlNumber);
                  messageSegments = NewMessage.Split('~');

                  ////Logging.WriteLogFile("AET or AAA Client " + remoteEndpointAddress + ";" + handle + ";" + port + ", sent message: " + NewMessage, 2);
                  ////Logging.WriteLogFile("Message Received, Size = " + intSize + ", Control Number = " + intControlNumber + ", Number Message Segments = " + messageSegments.Length);
                  //Logging.WriteLogFile(NewMessage);

                  // DO NOT set a default response.  If the client requests authentication and we send a response - both client and server think everything is okay
                  //responseFileContents = File.ReadAllText(ResponseDirectory + "ServerResponse002.txt");
                  // determine what message the client sent and load the response
                  if (messageSegments.Length > 3)
                  {

							foreach (string mySegment in messageSegments)
                     {
                        if (mySegment.StartsWith("ST"))
                        {
									Utils.LogEvent("AET RequestProcessor(270)");
									messageNumber = mySegment.Substring(3, 3);
                           if (messageNumber == "270")
                           {
                              myBlcServer = new BLCServer();
                              myBlcServer.Start(NewMessage, configurationData, this, out response, out p270);
                              if (!string.IsNullOrEmpty(response))
                              {
                                 response = response.Trim();
                                 if (!string.IsNullOrEmpty(response))
                                 {
                                    ////Logging.WriteLogFile("Response from 270 Server: " + response);
                                    SendDataString(response);
                                 }
                              }
                           }
                           else if (messageNumber == "278")
                           {
										Utils.LogEvent("AET RequestProcessor(278)");
										myCdwSqlConnectionStringBuilder = new SqlConnectionStringBuilder("Integrated Security=SSPI;");
                              myCdwSqlConnectionStringBuilder.DataSource = configurationData.CdwServer;
                              myCdwSqlConnectionStringBuilder.InitialCatalog = configurationData.CdwDatabase;
                              //myCdwSqlConnectionStringBuilder.IntegratedSecurity = true;
                              myCdwSqlConnectionStringBuilder.MultipleActiveResultSets = true;
                              ////Logging.WriteLogFile("CDW Connection string: " + myCdwSqlConnectionStringBuilder.ConnectionString);

                              my278Server = new RequestProcessor(myCdwSqlConnectionStringBuilder.ConnectionString);
                              response = my278Server.ProcessRequest(NewMessage);
                              response = response.Trim();


                              // 278 processor does not put length and cntrol number back on
                              response = strControlNumber + response;
                              strOutgoingSize = string.Format("{0:D6}", response.Length);
                              response = strOutgoingSize + response;
										Utils.LogEvent("AET RequestProcessor End() response - " + response);
										SendDataString(response);
                           }
                           break;
                        }
                     }
                  }
               }
               else if (port == configurationData.CtlPort)
               {
                  myCccServer.ProcessCommand(NewMessage, configurationData, this, out response);
                  if (!string.IsNullOrEmpty(response))
                  {
                     // we do not have string.IsNollOrWhitespace in this .NET Framework -- we have to do it the hard way :-(
                     response = response.Trim();
                     if (!string.IsNullOrEmpty(response))
                     {
                        SendDataString(response);
                     }
                     else
                        response = string.Empty;
                  }
                  else
                     response = string.Empty;

                  ////Logging.WriteLogFile("CCC Client " + remoteEndpointAddress + ";" + handle + ";" + port + ", sent command: " + NewMessage + ", response was: " + response);
               }
            }
         }
         catch (Exception ex)
         {
            string errorMessage = "Exception on Client " + remoteEndpointAddress + ";" + handle + ";" + port + "," + ex.Message + "\r\n" + ex.StackTrace;
				//System.Diagnostics.EventLog.WriteEntry("AETService", errorMessage, System.Diagnostics.EventLogEntryType.Error, 115, 500);
				Utils.LogEvent(errorMessage);
         }
      }

      protected void CloseClient()
      {
         // client disconnected - OR we want to manually close the client - we cleanup here
         ////Logging.WriteLogFile("Closing Client " + remoteEndpointAddress + ";" + handle + ";" + port);
         if (mySslStream != null)
         {
               mySslStream.Close();
         }
         if (clientTcpClient.GetStream() != null)
               clientTcpClient.GetStream().Close();
         clientTcpClient.Close();
      }
   }
}
