﻿/*
 * Program Name: New Standards and Operating Rules - Payer Side (POR)
 * Contractor Name: Harris Corporation
 * Contract Number: VA118-11-D-1009
 * Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
 * 
 * Developer: Zach Bertram
 * Date: 03.05.2013
 * ----------------
 * Expose SSN, DOB, and First and Last Name returned by MVI for validation
 */

using System;
using System.Text;
using System.IO;
using System.Net;
using System.Xml;
using System.Security.Cryptography.X509Certificates;
using System.Security.Policy;
using AETShared;

namespace AETService
{
    class PRPA1305
    {

        private string m_post_data = string.Empty;

        private string m_url = string.Empty;
        private string m_mviID = string.Empty;

        private string m_lname = string.Empty;
        private string m_fname = string.Empty;
        private string m_mname = string.Empty;
        private string m_suffix = string.Empty;
        private string m_gender = string.Empty;
        private string m_dob = string.Empty;
        private string m_ssn = string.Empty;
        private string m_mvipayload = string.Empty;
        private int m_logid = 0;

        private string m_mvipki = string.Empty;
        private bool m_WriteLog = false;

        X509Certificate certificate;

        public string MviPayload
        {
            get { return m_mvipayload; }
            set { m_mvipayload = value; }
        }

        public string MviPki
        {
            get { return m_mvipki; }
            set { m_mvipki = value; }
        }

        public bool WriteLog
        {
            get { return m_WriteLog; }
            set { m_WriteLog = value; }
        }

        public int LOGID
        {
            get { return m_logid; }
            set { m_logid = value; }
        }

        public string URL
        {
            get { return m_url; }
            set { m_url = value; }
        }
        public string MVI_ID
        {
            get { return m_mviID; }
            set { m_mviID = value; }
        }
        public string FirstName
        {
            get { return m_fname; }
            set { m_fname = value; }
        }
        public string LastName
        {
            get { return m_lname; }
            set { m_lname = value; }
        }
        public string MiddleName
        {
            get { return m_mname; }
            set { m_mname = value; }
        }
        public string Suffix
        {
            get { return m_suffix; }
            set { m_suffix = value; }
        }
        public string Gender
        {
            get { return m_gender; }
            set { m_gender = value; }
        }
        public string DOB
        {
            get { return m_dob; }
            set { m_dob = value; }
        }
        public string SSN
        {
            get { return m_ssn; }
            set { m_ssn = value; }
        }
        //////////////////////////////////////////////////////////////////////////
        /// <summary>
        /// ////////////////////
        /// Read the file pointed to by fileName
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public string GetFileData(string fileName)
        {
            string RtnData = "";
            try
            {
                using (StreamReader sr = new StreamReader(fileName))
                {
                    //This allows you to do one Read operation.
                    RtnData = sr.ReadToEnd();
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("The process failed: {0}", e);
                //Logging.WriteLogFile("Exception in GetFileData() for filename " + fileName + ", =" + e.Message);
            }

            return RtnData;
        }
        /// <summary>
        /// ////////////////////////////////////////////////////////////////////////
        /// </summary>
        /// <returns></returns>
        public string GET_VPID(X509Certificate Certificate)
        {
            string vpid = string.Empty;
            bool MVIError = false;
            bool bLogMVIPostXML = m_WriteLog;

            certificate = Certificate;

            //if (m_WriteLog != "YES")
            //bLogMVIPostXML = false;

            if (m_url.Trim().Length == 0)
            {
                MVIError = true;
                vpid = "ERR:" + "NO URL";
                //Logging.WriteLogFile("Error in GET_VPID() = " + vpid);
                return vpid;
            }
            /////////////////////////////////////////////////////////////
            // Get next mesage control number

            int nCtl = m_logid;
            /////////////////////////////////////////////////////////////


            DateTime dateTime = DateTime.Now;
            string creationTime_value = dateTime.ToString("yyyyMMdd HHmmss");  // Creation Time 
            creationTime_value = creationTime_value.Replace(" ", "");

            string receiver_device_classCode = "DEV";
            string sender_device_id_extension = receiver_device_classCode;
            string HAPE_ID = m_mviID;                                           // Our MVI Id = "200HAPE";

            string livingSubjectAdministrativeGender_value_code = m_gender;     // "M";
            string livingSubjectBirthTime_value_value1 = m_dob;                 // "19780512";
            string livingSubjectId_value_extension = m_ssn;                     // "11111-111";

            string livingSubjectName_value_family = m_lname;                    // "LOPEZ";
            string livingSubjectName_value_given = m_fname;                     // "Daniel";
            string livingSubjectName_value_middle = m_mname;                    // "";
            string livingSubjectName_value_suffix = m_suffix;                   // "";

            string soapHEADER = "<?xml version=\"1.0\" encoding=\"utf-8\"?><soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"><soap:Body><PRPA_IN201305UV02 xsi:type=\"xsd:string\" xmlns=\"http://URL         .DNS\">";
            string soapTRAILER = "</PRPA_IN201305UV02></soap:Body></soap:Envelope>";

            string outXML = string.Empty;

            try
            {

                PRPA_IN201305UV02 PRPA_1305 = new PRPA_IN201305UV02();
                PRPA_1305.controlActProcess = new PRPA_IN201305UV02ControlActProcess();
                PRPA_1305.creationTime = new PRPA_IN201305UV02CreationTime();

                PRPA_1305.id = new id();
                PRPA_1305.id.root = "1.2.840.114350.1.13.0.1.7.1.1";
                PRPA_1305.id.extension = String.Format("VISTA-5010-{0}", nCtl);

                PRPA_1305.interactionId = new PRPA_IN201305UV02InteractionId();
                PRPA_1305.interactionId.root = "2.16.840.1.113883.1.6";
                PRPA_1305.interactionId.extension = "PRPA_IN201305UV02";
                PRPA_1305.ITSVersion = "XML_1.0";
                PRPA_1305.creationTime.value = creationTime_value;

                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                PRPA_1305.acceptAckCode = new PRPA_IN201305UV02AcceptAckCode();
                PRPA_1305.controlActProcess = new PRPA_IN201305UV02ControlActProcess();
                PRPA_1305.controlActProcess.classCode = "CACT";
                PRPA_1305.controlActProcess.moodCode = "EVN";
                PRPA_1305.controlActProcess.code = new PRPA_IN201305UV02ControlActProcessCode();
                PRPA_1305.controlActProcess.code.code = "PRPA_TE201305UV02";
                PRPA_1305.controlActProcess.code.codeSystem = "2.16.840.1.113883.1.6";
                PRPA_1305.controlActProcess.queryByParameter = new PRPA_IN201305UV02ControlActProcessQueryByParameter();
                PRPA_1305.controlActProcess.queryByParameter.queryId = new PRPA_IN201305UV02ControlActProcessQueryByParameterQueryId();
                PRPA_1305.controlActProcess.queryByParameter.queryId.root = "1.2.840.114350.1.13.28.1.18.5.999";
                PRPA_1305.controlActProcess.queryByParameter.queryId.extension = "18204";
                PRPA_1305.controlActProcess.queryByParameter.initialQuantity = new PRPA_IN201305UV02ControlActProcessQueryByParameterInitialQuantity();
                PRPA_1305.controlActProcess.queryByParameter.statusCode = new PRPA_IN201305UV02ControlActProcessQueryByParameterStatusCode();
                PRPA_1305.controlActProcess.queryByParameter.statusCode.code = "new";
                PRPA_1305.controlActProcess.queryByParameter.initialQuantity.value = "1";
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                PRPA_1305.processingCode = new PRPA_IN201305UV02ProcessingCode();
                PRPA_1305.processingCode.code = "T";
                PRPA_1305.processingModeCode = new PRPA_IN201305UV02ProcessingModeCode();
                PRPA_1305.processingModeCode.code = "I";
                PRPA_1305.acceptAckCode = new PRPA_IN201305UV02AcceptAckCode();
                PRPA_1305.acceptAckCode.code = "AL";
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                PRPA_1305.receiver = new PRPA_IN201305UV02Receiver();
                PRPA_1305.receiver.device = new device();
                PRPA_1305.receiver.device.classCode = receiver_device_classCode;
                PRPA_1305.receiver.device.determinerCode = "INSTANCE";
                PRPA_1305.receiver.device.id = new id();
                PRPA_1305.receiver.device.telecom = new deviceTelecom();
                PRPA_1305.receiver.device.telecom.value = "http://servicelocation/PDQuery";
                PRPA_1305.receiver.device.id.root = "1.2.840.114350.1.13.999.234";
                PRPA_1305.receiver.typeCode = "RCV";
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                PRPA_1305.sender = new PRPA_IN201305UV02Sender();
                PRPA_1305.sender.device = new device();
                PRPA_1305.sender.typeCode = "SND";
                PRPA_1305.sender.device.classCode = sender_device_id_extension;
                PRPA_1305.sender.device.determinerCode = "INSTANCE";
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                //                                            MVI Internal ID
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                PRPA_1305.sender.device.id = new id();
                PRPA_1305.sender.device.id.extension = HAPE_ID;
                PRPA_1305.sender.device.id.root = "2.16.840.1.113883.4.349";
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                // START FILL OF parameterList (Patient Gender,Patient Date of Birth,Patient SSN,Patient Name(Last, First, Middle, Suffex)
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                PRPA_1305.controlActProcess.queryByParameter.parameterList = new PRPA_IN201305UV02ControlActProcessQueryByParameterParameterList();

                ////////////////////////////////////////////////[ Gender ]//////////////////////////////////////////////////////////////////////////////
                if (livingSubjectAdministrativeGender_value_code.Trim().Length > 0)
                {
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectAdministrativeGender = new PRPA_IN201305UV02ControlActProcessQueryByParameterParameterListLivingSubjectAdministrativeGender();
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectAdministrativeGender.semanticsText = "LivingSubject.administrativeGender";
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectAdministrativeGender.value = new value();
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectAdministrativeGender.value.code = livingSubjectAdministrativeGender_value_code;
                }
                ////////////////////////////////////////////////[ Date of Birth ]//////////////////////////////////////////////////////////////////////
                if (livingSubjectBirthTime_value_value1.Trim().Length > 0)
                {
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectBirthTime = new PRPA_IN201305UV02ControlActProcessQueryByParameterParameterListLivingSubjectBirthTime();
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectBirthTime.semanticsText = "LivingSubject.birthTime";
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectBirthTime.value = new value();
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectBirthTime.value.value1 = livingSubjectBirthTime_value_value1;
                }
                ////////////////////////////////////////////////[ SSN ]////////////////////////////////////////////////////////////////////////////////
                if (livingSubjectId_value_extension.Trim().Length > 0)
                {
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectId = new PRPA_IN201305UV02ControlActProcessQueryByParameterParameterListLivingSubjectId();
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectId.value = new value();
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectId.value.root = "2.16.840.1.113883.4.1";
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectId.value.extension = livingSubjectId_value_extension;
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectId.semanticsText = "SSN";
                }
                ////////////////////////////////////////////////[ Name ]///////////////////////////////////////////////////////////////////////////////
                PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectName = new PRPA_IN201305UV02ControlActProcessQueryByParameterParameterListLivingSubjectName();
                PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectName.semanticsText = "LivingSubject.name";
                PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectName.value = new value();
                PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectName.value.use = "L";

                if (livingSubjectName_value_family.Trim().Length > 0)
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectName.value.family = livingSubjectName_value_family.Trim();
                if (livingSubjectName_value_given.Trim().Length > 0)
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectName.value.given = livingSubjectName_value_given.Trim();
                if (livingSubjectName_value_suffix.Trim().Length > 0)
                    PRPA_1305.controlActProcess.queryByParameter.parameterList.livingSubjectName.value.suffix = livingSubjectName_value_suffix.Trim();

                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                //                                            END FILL OF parameterList
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

                System.Xml.Serialization.XmlSerializer prpaser = new System.Xml.Serialization.XmlSerializer(PRPA_1305.GetType());
                System.IO.MemoryStream ms = new System.IO.MemoryStream();
                prpaser.Serialize(ms, PRPA_1305);
                byte[] bytexml = ms.GetBuffer();

                for (int i = 0; i < bytexml.GetLength(0); i++)
                {
                    if ((char)bytexml[i] != 0)
                        outXML += (char)bytexml[i];
                }
                outXML = outXML.Replace("<?xml version=\"1.0\"?>", "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
                outXML = outXML.Replace("PRPA_IN201305UV02 xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" ITSVersion=\"XML_1.0\" xmlns=\"urn:hl7-org:v3\"", "PRPA_IN201305UV02 xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"urn:hl7-org:v3 ../../schema/hl7V3/NE2008/multicacheschemas/PRPA_IN201305UV02.xsd\" xmlns=\"urn:hl7-org:v3\" ITSVersion=\"XML_1.0\"");

                if (livingSubjectName_value_middle.Trim().Length > 0)
                {
                    string taga = String.Format("<given>{0}</given>", livingSubjectName_value_given.Trim());
                    string tagb = String.Format("<given>{0}</given>\r\n            <given>{1}</given>", livingSubjectName_value_given.Trim(), livingSubjectName_value_middle.Trim());
                    outXML = outXML.Replace(taga, tagb);
                }

                //outXML=GetFileData(@"C:\MVI_POST.txt");ES_POST.txt
                if (bLogMVIPostXML)
                {
                    string cDri = Environment.CommandLine;
                    cDri = cDri.Replace("AETService.exe", "");
                    cDri = cDri.Replace("\"", "");
                    string MVIPostFile = String.Format("{0}{1}_1305.xml", cDri, m_logid);
                    using (System.IO.StreamWriter writer = new System.IO.StreamWriter(MVIPostFile, false))
                    {
                        writer.Write(outXML);
                        writer.Close();
                    }
                }
                outXML = outXML.Replace("<", "&lt;");
                outXML = outXML.Replace(">", "&gt;");
            }
            catch (Exception ex)
            {
                MVIError = true;
                vpid = "ERR:" + ex.Message;
                //Logging.WriteLogFile("Exception in GET_VPID() = " + ex.Message);
            }
            finally
            {
                if (!MVIError)
                {
                    m_post_data = soapHEADER + outXML + soapTRAILER;
                    vpid = MakeSOAPCall(bLogMVIPostXML);
                }
            }

            return vpid;
        }

        /// <summary>
        /// //////////////////////////////////////////////////////////////////////////
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="certificate"></param>
        /// <param name="chain"></param>
        /// <param name="sslPolicyErrors"></param>
        /// <returns></returns>
        public static bool MyCertValidationCb(object sender, X509Certificate certificate, X509Chain chain, System.Net.Security.SslPolicyErrors sslPolicyErrors)
        {
            if (sslPolicyErrors == System.Net.Security.SslPolicyErrors.None)
                //Logging.WriteLogFile("Certificate Validation Callback: No SSL policy errors");
            if (sslPolicyErrors == System.Net.Security.SslPolicyErrors.RemoteCertificateNotAvailable)
                //Logging.WriteLogFile("Certificate Validation Callback: Remote Certificate Not Available");
            if (sslPolicyErrors == System.Net.Security.SslPolicyErrors.RemoteCertificateNameMismatch)
                //Logging.WriteLogFile("Certificate Validation Callback: Remote Certificate Name Mismatch");
            if (sslPolicyErrors == System.Net.Security.SslPolicyErrors.RemoteCertificateChainErrors)
                //Logging.WriteLogFile("Certificate Validation Callback: Remote Certificate Chain Errors");

            if ((sslPolicyErrors & System.Net.Security.SslPolicyErrors.RemoteCertificateChainErrors)
                      == System.Net.Security.SslPolicyErrors.RemoteCertificateChainErrors)
            {
                return true;
            }
            else if ((sslPolicyErrors & System.Net.Security.SslPolicyErrors.RemoteCertificateNameMismatch)
                            == System.Net.Security.SslPolicyErrors.RemoteCertificateNameMismatch)
            {
                Zone z;
                z = Zone.CreateFromUrl(((HttpWebRequest)sender).RequestUri.ToString());
                if (z.SecurityZone == System.Security.SecurityZone.Intranet
                  || z.SecurityZone == System.Security.SecurityZone.MyComputer)
                {
                    return true;
                }
                return false;
            }
            return true;
        }
        /// <summary>
        /// //////////////////////////////////////////////////////////////////////////
        /// </summary>
        /// <returns></returns>

        private string MakeSOAPCall(bool bLogMVIPostXML)
        {
            bool bSOAPError = false;
            bool bFoundCert = false;
            string sRTN = string.Empty;

            string m_sCertName = m_mvipki;
            int m_timeOut = 60;

            m_lname = "";  //ZEB VA118-11-D-1009 3/5/13 (+3) make sure values searched for aren't still in output
            m_fname = "";
            m_dob = "";
            m_ssn = "";
            //////////////////////////////////////////////////////////////////////////////
            // Make sure we have some data to send.
            if (m_post_data.Length == 0)
            {
                bSOAPError = true;
                sRTN = "ERR: POST data was empty.";
                m_mvipayload = sRTN;
                //Logging.WriteLogFile("Error in MakeSOAPCall() = " + sRTN);
                return sRTN;
            }

            //////////////////////////////////////////////////////////////////////////////
            // Install out SSL CertValidation call back function
            ServicePointManager.ServerCertificateValidationCallback = MyCertValidationCb;
            //ServicePointManager.Expect100Continue = true;
            // Enable all protocols supported by .NET Framework 4.5.  Let the service decide which protocol to use
            //ServicePointManager.SecurityProtocol = SecurityProtocolType.Ssl3 | SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
            byte[] postBytes = Encoding.ASCII.GetBytes(m_post_data);    // turn our request string into a byte stream
            //////////////////////////////////////////////////////////////////////////////
            // Create out http(s) request & load the URL
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(m_url);

            //////////////////////////////////////////////////////////////////////////////
            // Open the x509 Store and retreive out (LOCAL PKI CERT)
            try
            {
                if (certificate != null)
                {
                    request.ClientCertificates.Add(certificate);
                    bFoundCert = true;
                }
                else
                    bFoundCert = false;

                //string msg = string.Empty;
                //string[] certs = m_sCertName.Split(';');
                //string sCert = m_sCertName;

                //X509Store store = null;
                //if (certs.Length == 2)
                //{
                //    //msg = String.Format("Part: 1=[{0}] 2=[{1}] [{2}]", certs[0], certs[1], m_sCertName);
                //    ////System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Information);

                //    store = new X509Store(StoreName.My, StoreLocation.LocalMachine);

                //    //store = new X509Store(certs[0], StoreLocation.LocalMachine);
                //    sCert = certs[1];
                //}
                //else if (certs.Length == 3)
                //{
                //    //msg = String.Format("FULL: 1=[{0}] 2=[{1}] 3=[{2}] [{3}]", certs[0], certs[1], certs[2], m_sCertName);
                //    ////System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Information);

                //    //if (certs[2] == "lm")
                //    //store = new X509Store(certs[0], StoreLocation.LocalMachine);
                //    //else
                //    //store = new X509Store(certs[0], StoreLocation.CurrentUser);

                //    store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
                //    sCert = certs[1];
                //}
                //else
                //{
                //    //msg = String.Format("Default: TrustedPublisher:LocalMachine: [{0}]", m_sCertName);
                //    ////System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Information);
                //    //store = new X509Store(StoreName.TrustedPublisher, StoreLocation.LocalMachine);
                //    store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
                //}

                //store.Open(OpenFlags.ReadOnly);
                //X509Certificate2Collection certificates = store.Certificates;

                //foreach (X509Certificate2 x509 in certificates)
                //{
                //    X500DistinguishedName dname = new X500DistinguishedName(x509.SubjectName);
                //    //request.ClientCertificates.Add(x509);   // We found what we were looking for.
                //    //bFoundCert = true;
                //    ////Logging.WriteLogFile("PRPA1305 is using certficate with name: " + dname.Name);

                //    if (dname.Name.Contains(sCert))
                //    {
                //        msg = String.Format("FOUND {0}", m_sCertName);
                //        //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Information);
                //        request.ClientCertificates.Add(x509);   // We found what we were looking for.
                //        bFoundCert = true;
                //        //Logging.WriteLogFile("PRPA1305 is using certficate with name: " + dname.Name);
                //        break;
                //    }
                //    //else
                //    //{
                //    //    request.ClientCertificates.Add(x509);   // We found what we were looking for.
                //    //    x509.Reset();
                //    //}
                //}
                //store.Close();

                // Set the flag that we loaded our certificate
                if (!bFoundCert)                                // Check to make sure we found & loaded the certificate
                {
                    bSOAPError = true;
                    sRTN = "ERR:" + String.Format("Unable to locate Cert {0}", m_sCertName);
                    //Logging.WriteLogFile("Error in MakeSOAPCall() = " + sRTN);
                }
                else
                {
                    // The certificate was loaded so populate (ONLY THE ONES NEEDED) the rest of the Http(s) request values.
                    request.Method = "POST";
                    request.KeepAlive = true;
                    request.ProtocolVersion = HttpVersion.Version10;

                    request.ContentType = "text/xml; charset=utf-8";
                    request.ContentLength = postBytes.Length; // this is important - make sure you specify type this way

                    request.Timeout = 1000 * m_timeOut;
                    request.AllowAutoRedirect = true;
                }
            }
            catch (Exception ex)
            {
                bSOAPError = true;

                string errMsg = String.Format("[{0}]-MVI-3 Exception: {1}", m_logid, ex.Message);
                m_mvipayload = errMsg;
                //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 400, 500);
                //Logging.WriteLogFile("Exception in MakeSOAPCall() = " + errMsg);
            }
            ///////////////////////////////////////////////////////////////
            // All the pre setup should be done when we get here
            if (bSOAPError)
                return sRTN;

            //ServicePointManager.Expect100Continue = true;
            //ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;

            // Keith Baldwin 5/18/2016 - MVI call was blowing up so I logged a lot of information
            //Logging.WriteLogFile("");
            //Logging.WriteLogFile("MVI m_url: " + m_url);
            //Logging.WriteLogFile("MVI m_sCertName: " + m_sCertName);
            //Logging.WriteLogFile("MVI Request.Uri: " + request.RequestUri.ToString());
            //Logging.WriteLogFile("MVI Request.Method: " + request.Method);
            //Logging.WriteLogFile("MVI Request.ContentType: " + request.ContentType);
            //Logging.WriteLogFile("MVI Request.ContentLength: " + request.ContentLength);
            ////Logging.WriteLogFile("Request.Method: " + request.Method);
            //ServicePointManager.SecurityProtocol = SecurityProtocolType.Ssl3 | SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
            if ((ServicePointManager.SecurityProtocol & SecurityProtocolType.Ssl3) == SecurityProtocolType.Ssl3)
                //Logging.WriteLogFile("MVI: SSL 3 is available for security protocol");
            if ((ServicePointManager.SecurityProtocol & SecurityProtocolType.Tls) == SecurityProtocolType.Tls)
                //Logging.WriteLogFile("MVI: TLS 1.0 is available for security protocol");
            if ((ServicePointManager.SecurityProtocol & SecurityProtocolType.Tls11) == SecurityProtocolType.Tls11)
                //Logging.WriteLogFile("MVI: TLS 1.1 is available for security protocol");
            if ((ServicePointManager.SecurityProtocol & SecurityProtocolType.Tls12) == SecurityProtocolType.Tls12)
                //Logging.WriteLogFile("MVI: TLS 1.2 is available for security protocol");
            ////Logging.WriteLogFile("MVI m_post_data: " + m_post_data);
            ////Logging.WriteLogFile("MVI m_mvipayload: " + m_mvipayload);
            ////Logging.WriteLogFile("MVI bLogMVIPostXML: " + bLogMVIPostXML);
            //Logging.WriteLogFile("");

            try
            {
                Stream requestStream = request.GetRequestStream();   // Creat a Request Stream.
                requestStream.Write(postBytes, 0, postBytes.Length); // Now send it.
                requestStream.Close();
            }
            catch (ProtocolViolationException ex)
            {
                //Logging.WriteLogFile("Protocol Violation Exception: " + ex.Message);
                throw ex;  // we rethrow exception because the exception handling for this is done at a higher level.  This exception handler was added to give us more detailed information about exceptions.
            }
            catch (NotSupportedException ex)
            {
                //Logging.WriteLogFile("Not Supported Exception: " + ex.Message);
                throw ex;  // we rethrow exception because the exception handling for this is done at a higher level.  This exception handler was added to give us more detailed information about exceptions.
            }
            catch (WebException ex)
            {
                //Logging.WriteLogFile("Web Exception: " + ex.Message);
                if (ex.InnerException != null)
                {
                    //Logging.WriteLogFile("Inner Web Exception: " + ex.InnerException.Message);
                    if (ex.InnerException.InnerException != null)
                    {
                        //Logging.WriteLogFile("Inner Inner Web Exception: " + ex.InnerException.InnerException.Message);
                    }
                }
                if (ex.Response != null)
                {
                    //Logging.WriteLogFile("ex.Response.ResponseUri: " + ex.Response.ResponseUri);
                }
                throw ex;  // we rethrow exception because the exception handling for this is done at a higher level.  This exception handler was added to give us more detailed information about exceptions.
            }
            catch (ObjectDisposedException ex)
            {
                //Logging.WriteLogFile("Object Disposed Exception: " + ex.Message);
                throw ex;  // we rethrow exception because the exception handling for this is done at a higher level.  This exception handler was added to give us more detailed information about exceptions.
            }
            catch (InvalidOperationException ex)
            {
                //Logging.WriteLogFile("Invalid Operation Exception: " + ex.Message);
                throw ex;  // we rethrow exception because the exception handling for this is done at a higher level.  This exception handler was added to give us more detailed information about exceptions.
            }
            catch (Exception ex)
            {
                //Logging.WriteLogFile("Exception: " + ex.Message);
                throw ex;  // we rethrow exception because the exception handling for this is done at a higher level.  This exception handler was added to give us more detailed information about exceptions.
            }

            string sc = string.Empty;
            string xmlData = string.Empty;

            try
            {
                // Grab te response and print it out to the console along with the status code
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                sc = response.StatusCode.ToString();
                using (StreamReader sr = new StreamReader(response.GetResponseStream()))
                {
                    xmlData = sr.ReadToEnd();
                }

                if (bLogMVIPostXML)
                {
                    string cDri = Environment.CommandLine;
                    cDri = cDri.Replace("AETService.exe", "");
                    cDri = cDri.Replace("\"", "");
                    string MVIPostFile = String.Format("{0}{1}_1306.xml", cDri, m_logid);
                    using (System.IO.StreamWriter writer = new System.IO.StreamWriter(MVIPostFile, false))
                    {
                        writer.Write(xmlData);
                        writer.Close();
                    }
                }
                sRTN = "OK";

                ///////////////////////////////////////////////////////////////////////////////////////////
                try
                {
                    m_mvipayload = xmlData;
                    // Load XML string into an System.Xml.XmlDocument
                    XmlDocument xd = new XmlDocument();
                    xd.LoadXml(xmlData);
                    // Apply necessary XML namespaces specified in the XML
                    XmlNamespaceManager nsMan = new XmlNamespaceManager(xd.NameTable);
                    nsMan.AddNamespace("idm", "http://URL         .DNS");
                    nsMan.AddNamespace("env", "http://schemas.xmlsoap.org/soap/envelope/");
                    // Get Code
                    XmlElement objectNode = (XmlElement)xd.SelectSingleNode("/env:Envelope/env:Body/idm:PRPA_IN201306UV02", nsMan);
                    XmlElement controlActProcess = (XmlElement)objectNode.GetElementsByTagName("controlActProcess")[0];
                    XmlElement queryAck = (XmlElement)controlActProcess.GetElementsByTagName("queryAck")[0];
                    XmlNode queryResponseCode = queryAck.GetElementsByTagName("queryResponseCode")[0];
                    string code = queryResponseCode.Attributes["code"].Value;
                    //string sa = String.Format("Code: '{0}'", code);
                    if (code != "OK")
                    {
                        sRTN = code;
                    }
                    else
                    {
                        // Get extension
                        XmlElement subject = (XmlElement)controlActProcess.GetElementsByTagName("subject")[0];
                        XmlElement registrationEvent = (XmlElement)subject.GetElementsByTagName("registrationEvent")[0];
                        XmlElement subject1 = (XmlElement)registrationEvent.GetElementsByTagName("subject1")[0];
                        XmlElement patient = (XmlElement)subject1.GetElementsByTagName("patient")[0];
                        XmlNodeList idNodes = patient.GetElementsByTagName("id");
                        XmlElement person = (XmlElement)patient.GetElementsByTagName("patientPerson")[0];  //ZEB VA118-11-D-1009 3/5/13 allow checking against returned info

                        foreach (XmlElement name in person.GetElementsByTagName("name"))  //ZEB VA118-11-D-1009 3/5/13 (+foreach) allow checking against returned info
                        {
                            if (name.Attributes["use"].Value == "L")
                            {
                                m_lname = name.GetElementsByTagName("family")[0].InnerText;
                                m_fname = name.GetElementsByTagName("given")[0].InnerText;
                            }
                        }

                        m_dob = person.GetElementsByTagName("birthTime")[0].Attributes["value"].Value;  //ZEB VA118-11-D-1009 3/5/13 allow checking against returned info

                        foreach (XmlNode idNode in idNodes)
                        {
                            string root = idNode.Attributes["root"].Value;
                            if (root == "2.16.840.1.113883.4.1")  //ZEB VA118-11-D-1009 3/5/13 (+if) allow checking against returned info
                            {
                                m_ssn = idNode.Attributes["extension"].Value.Split('^')[0];
                                continue;
                            }
                            if (root == null || !root.Equals("2.16.840.1.113883.4.349")) continue;
                            string extension = idNode.Attributes["extension"].Value;
                            if (extension == null) continue;
                            if (!extension.Contains("NI^200M^USVHA")) continue;

                            string[] VPID = extension.Split('^');
                            sRTN = VPID[0];
                            //break;   //ZEB VA118-11-D-1009 4/1/13 allow finding SSN and ICN
                        }
                    }
                }
                catch (Exception ex)
                {
                    string errMsg = String.Format("[{0}]-MVI-1 Exception: {1}", m_logid, ex.Message);
                    m_mvipayload = errMsg;
                    //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 402, 500);
                    //Logging.WriteLogFile("Exception in MakeSOAPCall() = " + errMsg);
                }
                ///////////////////////////////////////////////////////////////////////////////////////////
            }
            catch (System.Net.WebException ex)
            {
                // Write any exceptions to a log file.
                // Write any exceptions to a log file.
                string errMsg = String.Format("[{0}]-MVI-2 Exception: {1}", m_logid, ex.Message);
                m_mvipayload = errMsg;
                //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 404, 500);
                //Logging.WriteLogFile("WebException in MakeSOAPCall() = " + errMsg);

                string cDri = Environment.CommandLine;
                cDri = cDri.Replace("AETService.exe", "");
                cDri = cDri.Replace("\"", "");
                string ESPostFile = String.Format("{0}{1}_MVIResponse.xml", cDri, m_logid);

                HttpWebResponse response1 = (HttpWebResponse)ex.Response;
                using (StreamReader sr1 = new StreamReader(response1.GetResponseStream()))
                {
                    xmlData = sr1.ReadToEnd();
                }
                if (response1.StatusCode == HttpStatusCode.InternalServerError)
                {
                    errMsg = String.Format("[{0}]-MVI-3 Exception-Data: {1}", m_logid, xmlData);
                    //System.Diagnostics.EventLog.WriteEntry("AETService", errMsg, System.Diagnostics.EventLogEntryType.Error, 201, 500);
                    //Logging.WriteLogFile("Error in MakeSOAPCall() = " + errMsg);
                    using (System.IO.StreamWriter writer = new System.IO.StreamWriter(ESPostFile, false))
                    {
                        writer.Write(xmlData);
                        writer.Close();
                        sRTN = "NX";
                    }
                }
                else
                {
                    using (System.IO.StreamWriter writer = new System.IO.StreamWriter(ESPostFile, false))
                    {
                        //Logging.WriteLogFile("Exception in MakeSOAPCall() = " + errMsg);
                        sRTN = "ERR:" + ex.Message;
                        writer.Write(ex.Message);
                        writer.Close();
                    }
                }
            }
            return sRTN;
        }
        //////////////////////////////////////////////////////////////////////////
    }
}
