﻿using AETShared;
using System;
using System.Net.Security;
using System.Net.Sockets;
using System.Security.Cryptography.X509Certificates;

namespace AETService
{
    /// <summary>
    /// Base class used for executing a server-side protocol on an accepted socket.
    /// </summary>
    public abstract class ServerModel
    {
        string sServerKeyList = string.Empty;

        protected ServerModel()
        {
        }


        protected bool Authenticate(TcpClient tcp, SslStream ClientSslStream, string SSName)
        {
            sServerKeyList = SSName;

            string msg = string.Empty;
            string sCMD = string.Empty;
            string sCert = sServerKeyList;
            string clientIP = tcp.Client.RemoteEndPoint.ToString();
            string[] certs = sServerKeyList.Split(';');
            bool bReturn = false;

            try
            {
                //Logging.WriteLogFile("Authenticating client with IP Address " + clientIP + ", for SSName = " + SSName);

                if (Certificate == null)
                {
                    if (certs.Length == 2)
                    {
                        sCMD = certs[0].ToUpper();
                        sCert = certs[1];
                    }
                    else if (certs.Length == 3)
                    {
                        sCMD = certs[0].ToUpper();
                        sCert = certs[1];
                    }
                    else
                    {
                        sCert = certs[0];
                    }

                    X509Store store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
                    store.Open(OpenFlags.ReadOnly);
                    try
                    {
                        if (sCMD == "SN")
                        {
                            X509Certificate2Collection collection = (X509Certificate2Collection)store.Certificates;
                            X509Certificate2Collection fcollection = (X509Certificate2Collection)collection.Find(X509FindType.FindBySerialNumber, sCert, false);
                            if (fcollection.Count > 0)
                            {
                                Certificate = fcollection[0];
                                if (fcollection.Count > 1)
                                {
                                    X500DistinguishedName dname = new X500DistinguishedName(Certificate.SubjectName);
                                    msg = String.Format("The X509Certificate store has {0} Certificate that match Serial Number {1}\n The AET will use the first one found {2} to authentication the client.", fcollection.Count, sCert, dname.Name);
                                    //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Warning);
                                    Logging.WriteLogFile(msg);
                                }
                            }
                        }
                        if (sCMD == "SUB")
                        {
                            X509Certificate2Collection certificates = store.Certificates;
                            foreach (X509Certificate2 x509 in certificates)
                            {
                                X500DistinguishedName dname = new X500DistinguishedName(x509.SubjectName);
                                //if (string.Compare(dname.Name, sCert, true) == 0)
                                if (dname.Name.IndexOf(sCert, StringComparison.OrdinalIgnoreCase) > -1)
                                {
                                    Certificate = x509;
                                    //Logging.WriteLogFile("Certificate found");
                                    break;
                                }
                                else
                                {
                                    x509.Reset();
                                }
                            }
                        }
                        else
                        {
                            X509Certificate2Collection certificates = store.Certificates;
                            foreach (X509Certificate2 x509 in certificates)
                            {
                                string c6 = x509.Issuer;
                                //if (string.Compare(c6, sCert, true) == 0)
                                if (c6.IndexOf(sCert, StringComparison.OrdinalIgnoreCase) > -1)
                                {
                                    Certificate = x509;
                                    //Logging.WriteLogFile("Certificate found");
                                    break;
                                }
                                else
                                {
                                    x509.Reset();
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        msg = String.Format("Error with the Personal x509 Store on the Local Machine. {0}\n{1}.", sServerKeyList, ex.Message);
                        //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Error, 160, 500);
                        Logging.WriteLogFile(msg + "\r\n" + ex.StackTrace);
                        return bReturn;
                    }

                    // Close the x509 Store 
                    store.Close();
                }

                //////////////////////////////////////////////////////////////////////////////////////////
                if (Certificate != null)
                {
                    //ServerSecurity security = new ServerSecurity();
                    //security.Certificate = Certificate;
                    //security.RequireClientCertificate = true;
                    //security.Protocols = System.Security.Authentication.SslProtocols.Tls;
                    //security.ValidationCallback = RemoteCertificateValidation;
                    try
                    {
                        //Logging.WriteLogFile("Certificate was found - now calling AuthenticateAsServer");
                        ClientSslStream.AuthenticateAsServer(Certificate, true, System.Security.Authentication.SslProtocols.Ssl3 | System.Security.Authentication.SslProtocols.Tls | System.Security.Authentication.SslProtocols.Tls11 | System.Security.Authentication.SslProtocols.Tls12, false);
                        bReturn = true;
                    }
                    catch (Exception ex)
                    {
                        //If an error is encountered, disconnect the client. Otherwise the connection will remain open.
                        bReturn = false;
                        msg = String.Format("Certificate Authentication ERROR=clientIP={0}\nLocation:Personal x509 Store on the Local Machine.\nCertificate={1}\nCertificate.Subject={2}\nCertificate.SerialNumber={3}\nException={4}.", clientIP, sCert, Certificate.Subject.ToString(), Certificate.SerialNumber.ToString(), ex.ToString());
                        Logging.WriteLogFile(msg + "\r\n" + ex.StackTrace);
                        //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Error, 162, 500);
                    }
                }
                else
                {
                    if (sCMD == "SN")
                        msg = String.Format("The AET was unable to locate the Certificate with the SerialNumber:\n{0}\n in the Personal Store on the Local Machine.", sCert);
                    else
                        msg = String.Format("The AET was unable to locate the Certificate Name:\n{0}\n in the Personal x509 Store on the Local Machine.", sCert);

                    //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Error, 164, 500);
                    Logging.WriteLogFile(msg);
                    bReturn = false;
                }
            }
            catch (Exception ex)
            {
                msg = String.Format("The AET incountered an exception in accessing or retreiving the SSL cerificate from the x509 store.\nPlease insure the AET has the correct premissions and the cerificate {0} exist in the Personal x509 Store on the Local Machine.\n(1)", sServerKeyList, ex.Message.ToString());
                //System.Diagnostics.EventLog.WriteEntry("AETService", msg, System.Diagnostics.EventLogEntryType.Error, 166, 500);
                Logging.WriteLogFile(msg + "\r\n" + ex.StackTrace);
                bReturn = false;
            }

            return bReturn;
        }

        /// <summary>
        /// Used for server authentication;
        /// </summary>
        public static X509Certificate2 Certificate = null;

        /// <summary>
        /// The "well-known" port number for the protocol used.
        /// </summary>
        //abstract public int DefaultPort { get; }

        /// <summary>
        /// Clone a new copy.
        /// </summary>
        /// <returns></returns>
        public abstract ServerModel Clone();

        /// <summary>
        /// Call this method on worker thread to implement a server protocol using blocking communications.
        /// </summary>
        //abstract public void Start(TcpBase tcp, gDataClass theData);

        /// <summary>
        /// Call this method on worker thread to implement a server protocol using asynchronous communications.
        /// </summary>
        //abstract public void StartAsync(TcpBase tcp);

        /// <summary>
        /// Reports value of client certificate.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="certificate"></param>
        /// <param name="chain"></param>
        /// <param name="sslPolicyErrors"></param>
        /// <returns></returns>
        protected bool RemoteCertificateValidation(Object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
        {
            bool bReturn = true;

            return bReturn;
        }
    }
}
