﻿/*
 * Program Name: New Standards and Operating Rules - Payer Side (POR)
 * Contractor Name: Harris Corporation
 * Contract Number: VA118-11-D-1009
 * Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
 * 
 * Developer: Zach Bertram
 * Date: 01.14.2013
 * ----------------
 * Clone of BLCServer for testing via flat files instead of interfaces.
 */

using System;
using System.Collections.Generic;
using System.Text;
//using Dart.Sockets;
using System.Reflection;
using System.IO;
using System.Xml;
using VA270Parser;
using VAZip;
using System.Linq;
using System.Diagnostics;
using System.Data;
using System.Data.SqlClient;
using AETShared;

/*
 * 
 * Information Source (Loop 2000A)
Information Receiver (Loop 2000B)
Subscriber (Loop 2000C)
Dependent (Loop 2000D)
Eligibility or Benefit Inquiry
Subscriber (Loop 2000C)
Eligibility or Benefit Inquiry
Dependent (Loop 2000D)
Eligibility or Benefit Inquiry
 * 
 * 
 * */

namespace AETService
{
    /// <summary>
    /// Business Logic Component Server for responding to 270 inquiries
    /// </summary>
    class TestBLCServer : ServerModel
    {
        /// <summary>
        /// Enum for dealing with the results of date comparisons
        /// </summary>
        private enum DateComparisonResult
        {
            Earlier = -1,
            Later = 1,
            TheSame = 0
        };

        string m_Estatus = string.Empty;  //eligibility status returned by ESMCall
        string m_Edate = string.Empty;  //eligibility date returned by ESMCall

        private string m_line1 = string.Empty;  //first line of address information returned by ESMCall
        private string m_city = string.Empty;  //city address information returned by ESMCall
        private string m_state = string.Empty;  //state address information returned by ESMCall
        private string m_zipcode = string.Empty;  //Zip Code address information returned by ESMCall
        private string m_EmdeonControl = string.Empty;  //Emdeon control number that came in on the 270 request

        string MVIPayload = string.Empty;  //response from MVI returned by MVICall
        const string NL = "\r\n";  //newline

        private bool _useInternalInterfaces = false;

        //StreamWriter writer = null;  //StreamWriter used to write to log file as well as TCP connection

        /// <summary>
        /// Creates a new BLCServer; must call Start() to use the new object
        /// </summary>
        public TestBLCServer()
        {
        }

        /// <summary>
        /// Creates a new BLCServer that can be used to listen for 270 inquiries
        /// </summary>
        /// <returns>A brand new instance of BLCServer; must call Start() to use the new object</returns>
        public override ServerModel Clone()
        {
            return new TestBLCServer();
        }

        //ZEB VA118-11-D-1009 1/14/13 fixed misspelling in method name; removed string concatenation for efficiency
        /// <summary>
        /// Composes a message to return over the connection when a special command is sent instead of a 270 inquiry
        /// </summary>
        /// <param name="msgType">1: message for QUIT command; 2: message for CLEAN command</param>
        /// <returns>A string with the appropriate message</returns>
        private string CreateReturnMessage(int msgType)
        {
            string msg = "";

            switch (msgType)
            {
                case 1:
                    msg = "The AET has responded to a close request by the client.\r\n";
                    break;
                case 2:
                    msg = "The AET has responded to a clean request by the client.\r\n";
                    break;
                default:  //ZEB VA118-11-D-1009 4/11/13 (+default) added default case in the event an unexpected value for msgType is passed in
                    break;
            }

            return msg;

        }

        /// <summary>
        /// Reads 270 inquiry or special command present on the TCP steam and responds accordingly.
        /// This method is synchronous; it executes on this thread until completed.
        /// </summary>
        /// <param name="tcp">TCP stream to read and write with</param>
        /// <param name="theData">Service configuration information</param>
        public override void Start(TcpBase tcp, gDataClass theData)
        {
        }

        /// <summary>
        /// Processes 270 inquiry or special command sent in from test application and responds accordingly.
        /// This method is synchronous; it executes on this thread until completed.
        /// </summary>
        /// <param name="Message">The message to process</param>
        /// <param name="theData">Service configuration information</param>
        /// <param name="UseInternalInterfaces">Whether or not to use the MVI and ES interfaces instead of SQL for testing</param>
        /// <returns>A string with the resulting 271 message (if not blank), or text report (if 270 is actually the report command)</returns>
        public string Start(string Message, gDataClass theData, bool UseInternalInterfaces)
        {
            _useInternalInterfaces = UseInternalInterfaces;
            string auditMsg = "";
            //string clientIP = tcp.Socket.RemoteEndPoint.ToString();  //Warren: for testing, hardcode this 


            string ErrorMsg = "";
            int ErrorId = 0;

            string TRN = "";

            bool returnAAA57 = false;  //ZEB VA118-11-D-1009 3/1/13 keep track of AAA errors for date of service
            bool returnAAA58 = false;  //ZEB VA118-11-D-1009 3/11/13 keep track of AAA errors for birthdate
            bool returnAAA62 = false;  //ZEB VA118-11-D-1009 2/6/13 keep track of AAA errors for date of service
            bool returnAAA71 = false;  //ZEB VA118-11-D-1009 1/31/13 keep track of AAA errors for alternate MVI searches
            bool returnAAA72 = false;
            bool returnAAA73 = false;
            bool returnAAA75 = false;

            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Client Authenticate as required
            //bool bAuth = Authenticate(tcp, theData.SSLName);  //Warren: START BLOCK <<< disable SSL authentication for testing
            //if (!bAuth)
            //{
            //    // Client Security Authenticate failed. The client will be dropped.....
            //    auditMsg = String.Format("SSL/TLS-Authenticate Failed from client IP: {0}.", clientIP);
            //    theData.AuditLog(auditMsg, System.Diagnostics.EventLogEntryType.Error);
            //    System.Threading.Thread.Sleep(1000);
            //    tcp.Close();
            //    return;
            //}
            //else
            //{
            //    if (theData.TraceEnabled)
            //    {
            //        // Client Security Authenticate passed. 
            //        auditMsg = String.Format("SSL/TLS - Has Been Successfully Authenticate From Client IP: {0}", clientIP);
            //        theData.AuditLog(auditMsg, System.Diagnostics.EventLogEntryType.Information);
            //    }
            //}  //Warren: END BLOCK >>> disable SSL authentication for testing


            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Get Cliet Data;
            //using (StreamReader reader = new StreamReader(tcp.GetStream()))  //Warren: START BLOCK <<< replace code that reads message for testing
            //{
                //char[] msgCount = new char[6];
                //reader.ReadBlock(msgCount, 0, 6);  //The first 6 bytes of the message contain the size of the rest of the message
            string sCount = Message.Substring(0, 6);

            int byteCnt = 0;
            try
            {
                //sCount = new string(msgCount);
                byteCnt = int.Parse(sCount);
            }
            catch (Exception ex)
            {
                auditMsg = String.Format("The inbound message size of [{0}] is invalid and cannot be read. The client connection will be terminated.\n{1}", sCount, ex.Message);
                theData.AuditLog(auditMsg, System.Diagnostics.EventLogEntryType.Error);
                //reader.Close();
                return "";
            }



            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Read inbound message
            //char[] InboundMsg = new char[byteCnt];
            //reader.ReadBlock(InboundMsg, 0, byteCnt);   // Read Emdeon Payer byte count
            //string sMsg = new string(InboundMsg);  //Warren: END BLOCK >>> replace code that reads message for testing
            string sMsg = Message.Substring(6, byteCnt);
            string sControl = sMsg.Substring(0, 6);     // Read Emdeon Control Number
            string s270 = sMsg.Substring(6);            // Extract the 270 payload
            int LogID = 0;

            m_EmdeonControl = sControl;

            //Audit Log ID
            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Check for and process a Infrastructure command message.
            if (s270.Length == 4 && s270 == "QUIT")  //Warren: no need to test this, as this is currently unused
            {
                if (theData.TraceEnabled)
                {
                    auditMsg = String.Format("AET was requested to close. Inbound message size ={0}\n{1}", sCount, s270);
                    theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                }


                string sBody1 = string.Format("{0:D6}", sControl);
                sBody1 += CreateReturnMessage(1);
                string sSize1 = string.Format("{0:D6}", sBody1.Length);
                string sEmdeonMessage1 = String.Format("{0}{1}", sSize1, sBody1);

                //writer = new StreamWriter(tcp.GetStream());
                //writer.AutoFlush = true;


                //writer.Write(sEmdeonMessage1);

                //System.Threading.Thread.Sleep(500);
                //reader.Close();
                return sEmdeonMessage1;
            }
            //////////////////////////////////////////////////////////////////////////////////////////
            if (s270.Length == 5 && s270 == "CLEAN")  //Warren: no need to test this as it isn't relevant
            {
                if (theData.TraceEnabled)
                {
                    auditMsg = String.Format("AET was requested to clean debug files. Inbound message size ={0}\n{1}", sCount, s270);
                    theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                }


                string sourceDir = Environment.CommandLine;
                sourceDir = sourceDir.Replace("AETService.exe", "");
                sourceDir = sourceDir.Replace("\"", "");


                //string NL = "\r\n";
                //System.Text.StringBuilder sb1 = new System.Text.StringBuilder();

                string[] xmlList = Directory.GetFiles(sourceDir, "*_*.xml");
                foreach (string f in xmlList)
                {


                    string fName = f.Substring(sourceDir.Length);
                    string fullFileNamr = Path.Combine(sourceDir, fName);
                    File.Delete(fullFileNamr);


                    //sb1.Append(fullFileNamr);
                    //sb1.Append(NL);
                }


                string sBody1 = string.Format("{0:D6}", sControl);
                sBody1 += CreateReturnMessage(2); //sb1.ToString();
                string sSize1 = string.Format("{0:D6}", sBody1.Length);
                string sEmdeonMessage1 = String.Format("{0}{1}", sSize1, sBody1);

                //writer = new StreamWriter(tcp.GetStream());
                //writer.AutoFlush = true;


                //writer.Write(sEmdeonMessage1);

                //System.Threading.Thread.Sleep(500);
                //reader.Close();
                return sEmdeonMessage1;
            }

            if (s270.Length > 4 && s270.Substring(0, 5) == "RPT01")  //Warren: no need to test this part as it is not currently in use
            {
                if (theData.TraceEnabled)
                {
                    auditMsg = String.Format("AET was requested to RPT1. Inbound message size ={0}\n{1}", sCount, s270);
                    theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                }


                string[] indata = s270.Split('~');
                int cnt = indata.Count();

                string Rtn = string.Empty;
                string NL = "\r\n";


                try
                {


                    string sCWCSConnectionString1 = String.Format("Data Source={0};Initial Catalog={1};User ID={2};Password={3};Timeout=30;", theData.LogServer, theData.LogDatabase, theData.LogUser, theData.LogPassword);
                    SqlConnection connectionObj1 = new SqlConnection(sCWCSConnectionString1);
                    connectionObj1.Open();


                    SqlCommand cmd = new SqlCommand("SELECT * FROM BenefitAuditLog WHERE RequestTime >= @Min AND RequestTime <= @Max", connectionObj1);
                    cmd.CommandType = System.Data.CommandType.Text;



                    if (cnt == 3)
                    {
                        cmd.Parameters.AddWithValue("@Min", indata[1]);
                        cmd.Parameters.AddWithValue("@Max", indata[2]);
                    }
                    else
                    {
                        DateTime dispDt = DateTime.Now;
                        string frm = String.Format("{0}-{1}-{2}", dispDt.Year, dispDt.Month, dispDt.Day);
                        string to = frm;

                        cmd.Parameters.AddWithValue("@Min", frm + " 0:00");
                        cmd.Parameters.AddWithValue("@Max", to + " 23:59:59");

                    }

                    SqlDataReader readerB = cmd.ExecuteReader();

                    Rtn = NL;
                    Rtn += "AET ID,Transaction Number,Request Time,Transaction Time,EB Return Value,MVI Time,MVI Status,ES Time,ES Status,ErrorID,ErrorMsg,270Errors\r\n";

                    /*
        0	    [k_pk_id]
        1       [ClearingHouseCtlID]
        2       [RequestTime]
        3       [RequestQualifier]
        4       [ResponseTime]
        5       [ResponseQualifier]
        6       [StatusIndicator]
        7       [EBReturn]
        8       [ErrorID]
        9       [ErrorMessage]
        10      [EXETime]
        11      [ASC270Errors]
        12      [TRNID]
        13      [InboundPayload]
        14      [OutboundPayload]
        15      [ResponseIPAddress]
        16      [MVITime]
        17      [MVIStatus]
        18      [MVIPayload]
        19      [ESRTime]
        20      [ESRStatus]
        21      [ESRPayload]
                     * 
                     * */


                    // k_pk_id, TRNID, RequestTime, EBReturn, MVIStatus, MVITime, ESRStatus, ESRTime
                    while (readerB.Read())
                    {
                        string AETID = string.Empty;
                        string ClearingHouseCtlID = string.Empty;
                        string RequestTime = string.Empty;
                        string RequestQualifier = string.Empty;
                        string ResponseTime = string.Empty;
                        string ResponseQualifier = string.Empty;
                        string StatusIndicator = string.Empty;
                        string EBReturn = string.Empty;
                        string ErrorID = string.Empty;

                        string ErrorMessage = string.Empty;
                        string EXETime = string.Empty;
                        string ASC270Errors = string.Empty;
                        string TRNID = string.Empty;
                        string InboundPayload = string.Empty;
                        string OutboundPayload = string.Empty;
                        string ResponseIPAddress = string.Empty;
                        string MVITime = string.Empty;
                        string MVIStatus = string.Empty;
                        string MVIPayload = string.Empty;
                        string ESRTime = string.Empty;
                        string ESRStatus = string.Empty;
                        string ESRPayload = string.Empty;




                        AETID = readerB.GetValue(0).ToString();

                        ClearingHouseCtlID = readerB.GetValue(1).ToString();
                        RequestTime = readerB.GetValue(2).ToString();
                        RequestQualifier = readerB.GetValue(3).ToString();
                        ResponseTime = readerB.GetValue(4).ToString();
                        ResponseQualifier = readerB.GetValue(5).ToString();
                        StatusIndicator = readerB.GetValue(6).ToString();
                        EBReturn = readerB.GetValue(7).ToString();
                        ErrorID = readerB.GetValue(8).ToString();
                        ErrorMessage = readerB.GetValue(9).ToString();
                        EXETime = readerB.GetValue(10).ToString();
                        ASC270Errors = readerB.GetValue(11).ToString();
                        TRNID = readerB.GetValue(12).ToString();
                        InboundPayload = readerB.GetValue(13).ToString();
                        OutboundPayload = readerB.GetValue(14).ToString();
                        ResponseIPAddress = readerB.GetValue(15).ToString();
                        MVITime = readerB.GetValue(16).ToString();
                        MVIStatus = readerB.GetValue(17).ToString();
                        MVIPayload = readerB.GetValue(18).ToString();
                        ESRTime = readerB.GetValue(19).ToString();
                        ESRStatus = readerB.GetValue(20).ToString();
                        ESRPayload = readerB.GetValue(21).ToString();

                        if (EXETime.Length > 8)
                            EXETime = EXETime.Substring(0, 8);

                        if (MVITime.Length > 8)
                            MVITime = MVITime.Substring(0, 8);

                        if (ESRTime.Length > 8)
                            ESRTime = ESRTime.Substring(0, 8);

                        //string rptData = String.Format("{0},{1},{2},{3},{4},{5},{6},{7}\r\n", AETID, TRNID, RequestTime, EBReturn, MVITime, MVIStatus, ESRTime, ESRStatus);
                        string rptData = String.Format("{0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11}\r\n", AETID, TRNID, RequestTime, EXETime, EBReturn, MVITime, MVIStatus, ESRTime, ESRStatus, ErrorID, ErrorMessage, ASC270Errors);
                        Rtn += rptData;
                    }

                    readerB.Close();
                    cmd.Dispose();
                    connectionObj1.Close();
                }
                catch (Exception ex)
                {
                    string erMsg = String.Format("Log Request failed: {0}\r\nServer: {1}, Database: {2}, UserID: {3}, UserData: {4}\r\n", ex.Message.ToString(), theData.LogServer, theData.LogDatabase, theData.LogUser, theData.LogPassword);

                    Rtn = erMsg;

                }
                System.Text.StringBuilder sbE = new System.Text.StringBuilder();
                sbE.Append(Rtn);
                sbE.Append(NL);

                string sBody1 = string.Format("{0:D6}", sControl);
                sBody1 += sbE.ToString();
                string sSize1 = string.Format("{0:D6}", sBody1.Length);
                string sEmdeonMessage1 = String.Format("{0}{1}", sSize1, sBody1);

                //writer = new StreamWriter(tcp.GetStream());
                //writer.AutoFlush = true;


                //writer.Write(sEmdeonMessage1);

                //System.Threading.Thread.Sleep(500);
                //reader.Close();
                return sEmdeonMessage1;
            }

            //We have taken care of all special commands; what follows is for processing a real 270
            Stopwatch timer270 = new Stopwatch();
            timer270.Start();

            LogID = theData.LogRequest(sControl, sMsg, "");  //Warren: disable this logging for testing [Zach: maybe better for debugging?]

            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Check to insure the transaction Contains a proper x12 ISA - INTERCHANGE CONTROL HEADER.  //Warren: ignore this for testing [Zach: this seems like a useful test case, but is unchanged]
            if (s270.Length < 3 || s270.Substring(0, 3) != "ISA")
            {
                auditMsg = String.Format("The inbound 270 request does not contain a ISA - INTERCHANGE CONTROL HEADER. The client connection will be terminated.");
                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);




                theData.LogMVI(LogID, (float)0.0, "MVI Not Called:", "MVI Payload"); //MVIPayload)
                theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");
                theData.LogError(LogID, 107, "No ISA in the 270");

                // Add not a 270 error to erro message.

                return "";

            }

            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Check to insure the transaction Contains a x12 005010X279A1 (270) Eligibility request code.  //Warren: ignore this for testing [Zach: this seems like a useful test case, but is unchanged]
            if (!s270.Contains("005010X279A1"))
            {
                auditMsg = String.Format("The inbound 270 request does not contain the expected 005010X279A1 Version Identifier Code. The client connection will be terminated.");
                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                //reader.Close();



                theData.LogMVI(LogID, (float)0.0, "MVI Not Called:", "MVI Payload"); //MVIPayload)
                theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");

                theData.LogError(LogID, 109, "No 005010X279A1 in the 270");

                return "";

            }

            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // If the inbound rquest is over 3K it is two large for a 270 request.  //Warren: ignore this for testing [Zach: this seems like a useful test case, but is unchanged]
            if (byteCnt > (1024 * 3))
            {
                auditMsg = String.Format("Inbound Message size is too big for a 270 request. The connection will be closed.\n byteCnt={0}", byteCnt);
                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error, 170, 500);



                theData.LogMVI(LogID, (float)0.0, "MVI Not Called:", "NO MVI Payload"); //MVIPayload)
                theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");

                theData.LogError(LogID, 111, "270 byte count > (1024*3)");


                return "";

            }


            string pLN = "";
            string PFN = "";
            string PMN = "";
            string PSF = "";
            string PDOB = "";
            string PGEN = "";
            string PSS = "";
            string PFZ = "";
            string PST = "";
            string VPIN = "";
            string ES = "";
            string msg = "";

            VAZip.Facility AddressInfo = null;
            string VAURL = "www2.va.gov/directory/guide/home.asp";

            Parser p270 = null;




            if (theData.TraceEnabled)
            {
                auditMsg = String.Format("STARTING 270 Parse for Emdeon Control Number:{0}. 270 Packet size:{1} Bytes", sControl, byteCnt);
                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
            }


            try
            {

                p270 = new Parser(theData.AnsiServer, theData.AnsiDatabase, theData.AnsiUser, theData.AnsiPassword, "90");
                p270.Parse270(s270);


            }
            catch (Exception ex)
            {
                auditMsg = String.Format("Catastrophic 270 Parser error: {0}", ex.Message.ToString());
                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                //reader.Close();


                theData.LogMVI(LogID, (float)0.0, "MVI Not Called:", "NO MVI Payload"); //MVIPayload)
                theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");


                return "";

            }

            bool BadNPI = false;
            bool HasParseErrors = false;
            string InfoReceiverNPI;
            InfoReceiverNPI = p270.InfoReceiver.IdentificationNumber;

            SqlConnection connectionObj = null;
            SqlCommand QueryCommand = null;
            SqlDataReader readerA = null;

            string sQuery = String.Format("SELECT NPI FROM LEIELookup WHERE NPI = '{0}'", InfoReceiverNPI);
            string sCWCSConnectionString = String.Format("Data Source={0};Initial Catalog={1};User ID={2};Password={3};Timeout=30;", theData.LEIEServer, theData.LEIEDatabase, theData.LEIEUser, theData.LEIEPassword);  //Warren: hard-code this for testing

            try
            {
                connectionObj = new SqlConnection(sCWCSConnectionString);
                connectionObj.Open();


                QueryCommand = new SqlCommand(sQuery, connectionObj);
                readerA = QueryCommand.ExecuteReader();

                if (readerA.HasRows)
                {
                    readerA.Read();
                    string rptData = readerA.GetString(0);
                    BadNPI = true;
                    string sMsgA = String.Format("NPI {0} was found on the LEIE Exclusion list. A - ASCx12 AAA*Y**50*N~ Segment will be returned.", InfoReceiverNPI);
                    System.Diagnostics.EventLog.WriteEntry("AETService", sMsgA, System.Diagnostics.EventLogEntryType.Warning, 118, 200);  //Warren: disable this for testing [Zach: this might be useful for debugging]
                }
                else
                {

                    //System.Diagnostics.EventLog.WriteEntry("AETService", sQuery, System.Diagnostics.EventLogEntryType.Error);
                }


            }
            catch (Exception ex)
            {

                string erMsg = String.Format("LEIE DB Error:: {0}\r\nServer: {1}, Database: {2}, UserID: {3}, UserData: {4}", ex.Message.ToString(), theData.LEIEServer, theData.LEIEDatabase, theData.LEIEUser, theData.LEIEPassword);
                System.Diagnostics.EventLog.WriteEntry("AETService", erMsg, System.Diagnostics.EventLogEntryType.Error);  //Warren: disable this for testing [Zach: this might be useful for debugging]

            }
            finally
            {
                if (readerA != null)
                {
                    readerA.Close();
                    readerA.Dispose();
                    readerA = null;
                }
                QueryCommand.Dispose();
                connectionObj.Close();

            }

            if (BadNPI)
            {


                p270.PatientData.EligibilityOfBenifitInformationCode = "V";  //ZEB VA118-11-D-1009 6/5/13 return V if we won't process due to LEIE
                p270.InfoReceiver.AAA = "AAA*Y**50*N";
                ErrorMsg = String.Format("Provider NPI: {0} found in LEIE database", InfoReceiverNPI);
                ErrorId = 110;

                theData.LogMVI(LogID, (float)0.0, "MVI Not Called:", "MVI Payload"); //MVIPayload)
                theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");

            }


            else if (p270.ErrorList.Count > 0)
            {                       
                foreach (string s in p270.ErrorList)
                {
                    auditMsg = String.Format("LogId {0} 270 parse error {1}", LogID, s);
                    theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);

                    switch (s)
                    {
                        case "DTP03 is required but missing.":
                            //set the subscriber date (Date of Service) to today
                            if (p270.PatientData.DtpSegments.Count < 1)
                            {
                                p270.PatientData.DtpSegments.Add(String.Format("DTP*291*D8*{0:D4}{1:D2}{2:D2}", DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day));  //add a DOS of today if it is missing
                            }
                            else
                            {
                                p270.PatientData.DtpSegments[0] = String.Format("DTP*291*D8*{0:D4}{1:D2}{2:D2}", DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day);  //Make sure DOS is today if existing segment had no date
                            }
                            break;
                        case "DMG01 Pair element relation not met.  Specified elements: 01, 02  ":
                            //Missing birthdate. We can use alternate searches for this.
                            break;
                        case "Invalid Birthdate":
                            p270.PatientData.AAASegments.Add("AAA*N**58*C");  //58 - Missing/invalid DOB
                            returnAAA58 = true;
                            HasParseErrors = true;
                            break;
                        case "NM108 Pair element relation not met.  Specified elements: 08, 09  ":
                            //This is a missing SSN, which we can search with (but will get an error later)
                            break;
                        case "EQ01 Element ID is not a valid ID":
                            //This is an invalid service type code, which we can work with
                            break;
                        case "EQ01 Element ID is not a valid IDElement ID is not a valid ID":
                            //This is an invalid service type code, which we can work with
                            break;
                        default:
                            p270.InfoSource.AAA = "AAA*N**79*C";  //This is what the old code did whenever it found an error
                            HasParseErrors = true;
                            break;
                    }
                }
            }

            if (!BadNPI && (p270.PatientData.LastName.Length == 0) && (p270.PatientData.FirstName.Length == 0) && (p270.PatientData.DateOfBirth == DateTime.MinValue))  //important identifiers from subscriber are missing, try dependent loop instead
            {
                PSS = (p270.PatientData.SocialSecurityNum.Length > 0) ? p270.PatientData.SocialSecurityNum : p270.PatientData.PrimaryIdentifier;
                p270.DependentData.IDCodeQualifier = p270.PatientData.IDCodeQualifier; //ZEB VA118-11-D-1009 6/19/13 make sure NM108 is filled in when copying dependent data to subscriber loop
                p270.PatientData = p270.DependentData;  //copy dependent into subscriber loop so 271 is generated properly
                p270.PatientData.PrimaryIdentifier = PSS;

                //ZEB VA118-11-D-1009 4/3/13 correctly handle DTPs when using dependent loop
                if (p270.PatientData.DtpSegments.Count < 1)
                {
                    p270.PatientData.DtpSegments.Add(String.Format("DTP*291*D8*{0:D4}{1:D2}{2:D2}", DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day));  //add a DOS of today if it is missing
                }
                else if (p270.PatientData.DtpSegments[0].Length < 12)
                {
                    p270.PatientData.DtpSegments[0] = String.Format("DTP*291*D8*{0:D4}{1:D2}{2:D2}", DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day);  //Make sure DOS is today if existing segment had no date
                }
            }

            if ((p270.PatientData.LastName == ""))
            {
                p270.PatientData.AAASegments.Add("AAA*Y**73*C");  //73 - Invalid/Missing Subscriber/Insured Name
                p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                HasParseErrors = true;
            }

            if (!returnAAA58 && (p270.PatientData.DateOfBirth == DateTime.MinValue))
            {
                p270.PatientData.AAASegments.Add("AAA*Y**58*C");  //58 - Missing/invalid DOB
                p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                HasParseErrors = true;
            }

            if ((p270.PatientData.SocialSecurityNum == "") && (p270.PatientData.PrimaryIdentifier == ""))
            {
                p270.PatientData.AAASegments.Add("AAA*Y**72*C");  //72 - Ivalid/Missing Subscriber/Insured ID
                p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                HasParseErrors = true;
            }

            if (HasParseErrors)
            {
                PFZ = p270.InfoReceiver.Zip;
                p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient

                ErrorMsg = "270 Error";
                ErrorId = 120;

                theData.LogMVI(LogID, (float)0.0, "MVI Not Called", "No MVI Payload");
                theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");
            }
            else if (!BadNPI && (p270.PatientData != null))
            {
                TRN = (p270.PatientData.TraceNumbers.Count > 0) ? p270.PatientData.TraceNumbers[0].TraceNumber : "";  //ZEB VA118-11-D-1009 5/8/13 prevent crash if no trace number specified

                pLN = p270.PatientData.LastName;
                PFN = p270.PatientData.FirstName;
                //PDOB = p270.PatientData.DateOfBirth.ToShortDateString();  //ZEB VA118-11-D-1009 1/22/13 no need to set this twice
                PDOB = String.Format("{0:D4}{1:D2}{2:D2}", p270.PatientData.DateOfBirth.Year, p270.PatientData.DateOfBirth.Month, p270.PatientData.DateOfBirth.Day);
                if (PDOB == "00010101")
                {
                    PDOB = "";
                }
                PSS = p270.PatientData.SocialSecurityNum.Trim();
                if (PSS != null && PSS.Length == 0)
                {
                    PSS = p270.PatientData.PrimaryIdentifier;
                }

                PMN = p270.PatientData.MiddleName;
                PSF = p270.PatientData.NameSuffix;

                PGEN = p270.PatientData.Gender;
                PST = p270.PatientData.ServiceTypeCode;

                PFZ = p270.InfoReceiver.Zip;
                p270.PatientData.EligibilityOfBenifitInformationCode = "6";

                string[] DTPfields = p270.PatientData.DtpSegments[0].Split('*');  //ZEB VA118-11-D-1009 8/19/13 (+27) factored out date validation code to deal with date ranges

                if (DTPfields.Length < 3)
                {
                    returnAAA57 = true;
                }
                else if (DTPfields[2] == "D8")
                {
                    ParsePlanDate(DTPfields[3], out returnAAA57, out returnAAA62);
                }
                else if (DTPfields[2] == "RD8")
                {
                    string[] DTPrangeDates = DTPfields[3].Split('-');
                    if (DTPrangeDates.Length != 2)
                    {
                        returnAAA57 = true;
                    }
                    else
                    {
                        if (ParsePlanDate(DTPrangeDates[0], out returnAAA57, out returnAAA62))
                        {
                            ParsePlanDate(DTPrangeDates[1], out returnAAA57);
                        }
                    }
                }
                else
                {
                    returnAAA57 = true;
                }
                
                if (!(returnAAA62 || returnAAA57))
                {

                    Stopwatch timerMVI = new Stopwatch();
                    timerMVI.Start();

                    System.Threading.Thread.Sleep(1000 * 1);


                    if (theData.TraceEnabled)
                    {

                        auditMsg = String.Format("STARTING The MVI (VAAFI) Web Service Call.");
                        theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }

                    VPIN = "NF";

                    VPIN = MVICall(pLN, PFN, PMN, PSF, PGEN, PDOB, PSS, theData.MVIURL, theData.MVIPKI, LogID, theData.PKICheck, theData, out returnAAA71, out returnAAA72, out returnAAA73, out returnAAA75);    //ZEB VA118-11-D-1009 1/31/13 AAA errors needed for alternate MVI searches  //Warren: replace with testing call (MVICall_SQL?) for tesing with no MVI
                    VPIN.Trim();

                    if (returnAAA71)    //ZEB VA118-11-D-1009 2/21/13 AAA errors needed for alternate MVI searches
                    {
                        p270.PatientData.AAASegments.Add("AAA*Y**71*C");  //71 - Patient Birth Date Does Not Match That for the Patient on the Database
                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                    }

                    if (returnAAA72)    //ZEB VA118-11-D-1009 2/21/13 AAA errors needed for alternate MVI searches
                    {
                        p270.PatientData.AAASegments.Add("AAA*Y**72*C");  //72 - Ivalid/Missing Subscriber/Insured ID
                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                    }

                    if (returnAAA73)    //ZEB VA118-11-D-1009 2/21/13 AAA errors needed for alternate MVI searches
                    {
                        p270.PatientData.AAASegments.Add("AAA*Y**73*C");  //73 - Invalid/Missing Subscriber/Insured Name
                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                    }

                    if (returnAAA75)    //ZEB VA118-11-D-1009 2/21/13 AAA errors needed for alternate MVI searches
                    {
                        p270.PatientData.AAASegments.Add("AAA*Y**75*C");  //75 - Subscriber/Insured Not Found
                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                    }


                    ///////////////////////////////////////////////////////////////////////////////////////
                    // Log the 1306 -- Need to add the 1305 as well

                    timerMVI.Stop();


                    //VPIN = "NX"; // Fake out a MVI error messaGE

                    if (VPIN.Length >= 2 && VPIN == "NX")
                    {


                        ErrorMsg = "MVI WEB Service Error Server:500";
                        ErrorId = 130;

                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                        p270.InfoSource.AAA = "AAA*Y**42*R";


                        theData.LogMVI(LogID, (float)timerMVI.Elapsed.TotalSeconds, "MVI WEB Service ", "MVI Payload"); //MVIPayload)
                        theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");


                        auditMsg = String.Format("MVI (VAAFI) Web Service Call FAILED: {0}", VPIN);
                        theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Warning);

                        if (theData.TraceEnabled)
                        {

                            auditMsg = String.Format("MVI (VAAFI) Web Service Call returned: {0}.", VPIN);
                            theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }




                    }
                    else if (VPIN.Length >= 2 && VPIN == "NF")
                    {

                        ErrorMsg = "MVI Not Found: " + VPIN;
                        ErrorId = 130;

                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient



                        theData.LogMVI(LogID, (float)timerMVI.Elapsed.TotalSeconds, "*MVI Not Found", "MVI Payload"); //MVIPayload)
                        theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");

                        if (theData.TraceEnabled)
                        {

                            auditMsg = String.Format("MVI (VAAFI) Web Service Call returned: {0}.", VPIN);
                            theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }


                    }
                    else if (VPIN.Length >= 2 && VPIN != "NF" && VPIN.Substring(0, 2) != "AE" && VPIN.Substring(0, 2) != "ER")
                    {
                        //if(theData.LogPayload)
                        theData.LogMVI(LogID, (float)timerMVI.Elapsed.TotalSeconds, VPIN, "MVI Payload"); //MVIPayload)
                        ///////////////////////////////////////////////////////////////////////////////////////
                        // Log the STARTING The ES Web Service Call
                        if (theData.TraceEnabled)
                        {
                            auditMsg = String.Format("STARTING The ES Web Service Call.");
                            theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }
                        Stopwatch timerESR = new Stopwatch();
                        timerESR.Start();
                        System.Threading.Thread.Sleep(1000);

                        ES = ESMCall(VPIN, LogID, theData);  //Warren: Need to replace with a testing call (ESMCall_SQL?) to test without ES

                        timerESR.Stop();
                        theData.LogESR(LogID, (float)timerESR.Elapsed.TotalSeconds, ES, "ESR Payload");

                        if (ES.Length == 0)
                        {
                            msg = String.Format("ES Web Service Call FAILED ES Return data was empty.");
                            theData.AuditLog(LogID, msg, System.Diagnostics.EventLogEntryType.Warning);

                        }
                        else if (ES.Length >= 2 && ES.Substring(0, 2) == "NF")
                        {
                            p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                            p270.PatientData.BenefitBeginDate = "";
                            if (theData.TraceEnabled)
                            {
                                auditMsg = String.Format("NO Subscribers Info returned from ES.");
                                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                            }

                        }
                        else if (ES.Length >= 2 && ES.Substring(0, 2) == "NX")
                        {

                            p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                            p270.InfoSource.AAA = "AAA*Y**42*R";
                            p270.PatientData.BenefitBeginDate = "";
                            if (theData.TraceEnabled)
                            {
                                auditMsg = String.Format("NO Subscribers Info returned from ES. Added a AAA segment");
                                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                            }

                        }
                        else if (ES.Length >= 3 && ES.Substring(0, 3) == "ERR")
                        {
                            // ES Web Service Call FAILED

                            p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                            p270.InfoSource.AAA = "AAA*Y**42*R";
                            p270.PatientData.BenefitBeginDate = "";
                            if (theData.TraceEnabled)
                            {
                                auditMsg = String.Format("Error returned from ES WEB Service: {0}\r\n. Added a AAA segment", ES);
                                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                            }

                        }
                        else
                        {

                            m_Estatus = m_Estatus.ToUpper();
                            
                            if (m_Estatus == "VERIFIED" && m_Estatus.Length == 8)
                            {
                                string[] md = m_Edate.Split('T');
                                if (md.Count() > 0)
                                    m_Edate = md[0];

                                m_Edate = m_Edate.Replace("-", "");
                                p270.PatientData.EligibilityOfBenifitInformationCode = "1";
                                p270.PatientData.BenefitBeginDate = m_Edate;
                                //p270.PatientData.SubscriberDate = m_Edate;

                                if (m_line1.Length == 0)
                                {
                                    p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                                    p270.PatientData.BenefitBeginDate = "";
                                    if (theData.TraceEnabled)
                                    {
                                        auditMsg = String.Format("NO Subscribers Address returned from ES.");
                                        theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                                    }


                                }
                                else
                                {
                                    p270.PatientData.AddressLine1 = m_line1;
                                    p270.PatientData.City = m_city;
                                    p270.PatientData.State = m_state;
                                    p270.PatientData.Zip = m_zipcode;
                                }



                                if (theData.TraceEnabled)
                                {
                                    auditMsg = String.Format("ESCallCall-1A= {0} {1} {2}", m_Estatus, m_Edate, ES);
                                    theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                                }


                            }
                            else
                            {
                                p270.PatientData.EligibilityOfBenifitInformationCode = "6";
                                p270.PatientData.BenefitBeginDate = "";
                                if (theData.TraceEnabled)
                                {
                                    auditMsg = String.Format("ESCallCall-1A= {0} {1} {2}", m_Estatus, m_Edate, ES);
                                    theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                                }
                            }

                        }
                    }
                    else
                    {

                        ErrorMsg = "MVI Not Found: " + VPIN;
                        ErrorId = 130;

                        p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient

                        if (VPIN != "NF")
                        {
                            p270.InfoSource.AAA = "AAA*Y**42*R";
                            auditMsg = String.Format("MVI (VAAFI) Web Service Call FAILED: {0}", VPIN);
                            theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Warning);
                        }


                        theData.LogMVI(LogID, (float)timerMVI.Elapsed.TotalSeconds, "MVI Not Found", "MVI Payload"); //MVIPayload)
                        theData.LogESR(LogID, (float)0.0, "ESR not Called", "NO ESR Payload");

                        if (theData.TraceEnabled)
                        {

                            auditMsg = String.Format("MVI (VAAFI) Web Service Call returned: {0}.", VPIN);
                            theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }


                    }



                }
                else  //ZEB VA118-11-D-1009 1/31/13 return correct error for invalid DOS
                {
                    if (returnAAA62)
                    {
                        p270.PatientData.AAASegments.Add("AAA*N**62*C");  //62 - Date of Service not within Allowable Inquiry Period             
                    }

                    if (returnAAA57)
                    {
                        p270.PatientData.AAASegments.Add("AAA*N**57*C");  //57 - Invalid/Missing Date(s) of Service  //ZEB VA118-11-D-1009 8/8/13 correct formatting for error 57
                    }
					
					p270.PatientData.EligibilityOfBenifitInformationCode = "U";  //ZEB VA118-11-D-1009 6/5/13 return U if no matched patient
                }
            }
            else
            {
                // No patient data returned from 270;
                auditMsg = String.Format("IParser(s270) FAILED");
                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                
                p270.InfoSource.AAA = "AAA*N**79*C";
            }

            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            //////////////////////////////////////////////////////////////////////////////
            // Setup for the Zip Code call
            if (theData.TraceEnabled)
            {
                auditMsg = String.Format("STARTING Facility Zip Code Call");
                theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
            }
            AddressInfo = ZipCode(theData.ZipServer, theData.ZipDatabase, theData.ZipUser, theData.ZipPassword, PFZ, theData, LogID);
            if (AddressInfo.hasErrors)
            {
                theData.AuditLog(LogID, AddressInfo.errorMsg, System.Diagnostics.EventLogEntryType.Error);
                AddressInfo.errorMsg = VAURL;

            }
            //////////////////////////////////////////////////////////////////////////////



            //StringBuilder sb = new StringBuilder();
            if (AddressInfo.hasErrors || string.IsNullOrEmpty(AddressInfo.name))
            {
                p270.NearestVAFacility.FacilityName = "UNKNOWN";
                p270.NearestVAFacility.AddressLine1 = "";
                p270.NearestVAFacility.AddressLine2 = "";
                p270.NearestVAFacility.City = "";
                p270.NearestVAFacility.State = "";
                p270.NearestVAFacility.Zip = "";
                p270.NearestVAFacility.Telephone = "";
                p270.NearestVAFacility.URL = VAURL;
            }
            else
            {
                p270.NearestVAFacility.FacilityName = AddressInfo.name;
                p270.NearestVAFacility.AddressLine1 = AddressInfo.address.street;
                p270.NearestVAFacility.AddressLine2 = "";
                p270.NearestVAFacility.City = AddressInfo.address.city;
                p270.NearestVAFacility.State = AddressInfo.address.state;
                p270.NearestVAFacility.Zip = AddressInfo.address.zip;
                if (AddressInfo.address.phones.Count > 0)
                {
                    p270.NearestVAFacility.Telephone = AddressInfo.address.phones[0];
                    p270.NearestVAFacility.Telephone = p270.NearestVAFacility.Telephone.Replace("-", "");
                    p270.NearestVAFacility.Telephone = p270.NearestVAFacility.Telephone.Replace("(", "");
                    p270.NearestVAFacility.Telephone = p270.NearestVAFacility.Telephone.Replace(")", "");

                }
                else
                {
                    p270.NearestVAFacility.Telephone = "";
                }
            }


            string s271 = "";
            string sBody = "";
            string sSize = "";
            string sEmdeonMessage = "";

			//ZEB VA118-11-D-1009 6/5/13 (-if/else) always return EB segments
            List<EBSegment> EBs = theData.GenerateEBSegments(PST, p270.PatientData.EligibilityOfBenifitInformationCode);  //ZEB VA118-11-D-1009 5/16/13 (+5) return error 42 if unable to generate EB segments
			if (EBs.Count < 1)
			{
				p270.InfoSource.AAA = "AAA*Y**42*R";
                p270.PatientData.EligibilityOfBenifitInformationCode = "V";  //ZEB VA118-11-D-1009 6/5/13 return V if unable to find STC data
                EBs.Add(new EBSegment("V", "", PST, "VA FEE BASIS PROGRAM", "", "", "", "Y"));    //ZEB VA118-11-D-1009 6/6/13 make sure EB exists if unable to find STC data
			}
			
			s271 = p270.Build271(EBs, p270.PatientData.AAASegments);

            if (m_EmdeonControl != "000000")
                s271 = s271.Replace("\r\n", "");

            if (p270.PatientData.EligibilityOfBenifitInformationCode == "1")
            {
                int found = s271.IndexOf("DTP*291*D8*");
                if (found > 0)
                {
                    string seg = s271.Substring(found, 19);
                    string newSeg = String.Format("DTP*346*D8*{0}", m_Edate);
                    s271 = s271.Replace(seg, newSeg);
                }
            }

            sBody = string.Format("{0:D6}", sControl);
            sBody += s271;
            sSize = string.Format("{0:D6}", sBody.Length);
            sEmdeonMessage = String.Format("{0}{1}", sSize, sBody);


            //writer = new StreamWriter(tcp.GetStream()); //Warren: BEGIN BLOCK <<<<< replace with writing 271 to text file for testing
            //writer.AutoFlush = true;
            //writer.Write(sEmdeonMessage);  //Warren: END BLOCK >>>>> replace with writing 271 to text file for testing



            theData.LogResponse(LogID, p270.InfoSource.EntityTypeQualifier, p270.InfoReceiver.EntityTypeQualifier, "OK", 0, "", s271);
            // AET server process is done........

            auditMsg = String.Format("The AET had delivered a 271 Eligibility Response back to the server {0} with the results of {1}", "", p270.PatientData.EligibilityOfBenifitInformationCode);
            theData.AuditLog(LogID, auditMsg, System.Diagnostics.EventLogEntryType.Information);
            System.Threading.Thread.Sleep(500);


            // Look into this for final release

            {
                timer270.Stop();
                string sCWCSConnectionString1 = String.Format("Data Source={0};Initial Catalog={1};User ID={2};Password={3};Timeout=30;", theData.LogServer, theData.LogDatabase, theData.LogUser, theData.LogPassword);
                SqlConnection connectionObj1 = new SqlConnection(sCWCSConnectionString1);
                connectionObj1.Open();

                // 
                string benCode = p270.PatientData.EligibilityOfBenifitInformationCode;  //ZEB VA118-11-D-1009 6/5/13 (+9) prevent error since EBReturn column only allows numbers
                string sQuery1;

                if ((benCode == "1") || (benCode == "6"))
                {
                    sQuery1 = String.Format("UPDATE BenefitAuditLog SET EBReturn={0},ErrorId={1}, ErrorMessage='{2}', EXETime= {3}, ASC270Errors={4},TRNID='{5}' WHERE k_pk_id = {6}", benCode, ErrorId, ErrorMsg, timer270.Elapsed.TotalSeconds, p270.ErrorList.Count, TRN, LogID);
                }
                else
                {
                    sQuery1 = String.Format("UPDATE BenefitAuditLog SET EBReturn=0,ErrorId={0}, ErrorMessage='{1}', EXETime= {2}, ASC270Errors={3},TRNID='{4}' WHERE k_pk_id = {5}", ErrorId, ErrorMsg, timer270.Elapsed.TotalSeconds, p270.ErrorList.Count, TRN, LogID);
                }
                SqlCommand QueryCommand1 = new SqlCommand(sQuery1, connectionObj1);
                SqlDataReader readerA1 = QueryCommand1.ExecuteReader();

                if (readerA1.HasRows)
                {

                }
                else
                {
                    //System.Diagnostics.EventLog.WriteEntry("AETService", sQuery, System.Diagnostics.EventLogEntryType.Error);
                }
                readerA1.Close();
                QueryCommand1.Dispose();
                connectionObj1.Close();



            }


            //} // socket is closed by using statement

            return sEmdeonMessage;
        }

        //ZEB VA118-11-D-1009 8/19/13 factored out of Start to deal with validating date ranges
        /// <summary>
        /// Parses a DateTime from the date string in a DTP segment, and performs validation for a plan date
        /// Skips future date check -- for use with end date in a query with a date range
        /// </summary>
        /// <param name="PlanDate">The date from the DTP segment</param>
        /// <param name="returnAAA57">Whether or not AAA error 57 - Invalid/Missing Date of Service(</param>
        /// <param name="returnAAA62">Whether or not AAA error 62 - Date of Service not Within Allowable Inquiry Period</param>
        /// <returns>True if the plan date is valid, False otherwise</returns>
        private bool ParsePlanDate(string PlanDate, out bool returnAAA57)
        {
            bool bitBucket;  //Needed to fill out param, but not used

            ParsePlanDate(PlanDate, out returnAAA57, out bitBucket);

            return !returnAAA57;
        }

        //ZEB VA118-11-D-1009 8/19/13 factored out of Start to deal with validating date ranges
        /// <summary>
        /// Parses a DateTime from the date string in a DTP segment, and performs validation for a plan date
        /// </summary>
        /// <param name="PlanDate">The date from the DTP segment</param>
        /// <param name="returnAAA57">Whether or not AAA error 57 - Invalid/Missing Date of Service(</param>
        /// <param name="returnAAA62">Whether or not AAA error 62 - Date of Service not Within Allowable Inquiry Period</param>
        /// <returns>True if the plan date is valid and not too far in the future, False otherwise</returns>
        private bool ParsePlanDate(string PlanDate, out bool returnAAA57, out bool returnAAA62)
        {
            DateTime DOS = DateTime.MinValue;
            
            returnAAA57 = false;
            returnAAA62 = false;

            if ((PlanDate.Length > 1) && (PlanDate.Length < 8))  //ZEB VA118-11-D-1009 4/10/13 remove redundant condition
            {
                returnAAA57 = true;
            }
            else
            {
                int year, month, date;
                Int32.TryParse(Utilities.SafeSubStringMid(PlanDate, 0, 4), out year);
                Int32.TryParse(Utilities.SafeSubStringMid(PlanDate, 4, 2), out month);
                Int32.TryParse(Utilities.SafeSubStringMid(PlanDate, 6, 2), out date);

                if (year < 1) returnAAA57 = true;  //ZEB VA118-11-D-1009 6/19/13 (+20) strengthen date format check
                if (month < 1) returnAAA57 = true;
                if (date < 1) returnAAA57 = true;

                if (!returnAAA57)
                {

                    try
                    {
                        DOS = new DateTime(year, month, date);
                    }
                    catch (Exception ex)
                    {
                        returnAAA57 = true;
                        DOS = DateTime.MinValue;
                    }

                    if (!returnAAA57)
                    {

                        returnAAA62 = (DOS > DateTime.Today.AddDays(30));
                        returnAAA57 = ((DOS == DateTime.MinValue) || ((PlanDate != "") && (PlanDate != String.Format("{0:D4}{1:D2}{2:D2}", DOS.Year, DOS.Month, DOS.Day))));  //ZEB VA118-11-D-1009 6/17/13 fixed check for invalid plan date format
                    }
                }
            }

            return !(returnAAA57 || returnAAA62);
        }
        
        private Facility ZipCode(string DBServer, string DB, string user, string PWD, string zipcode, gDataClass theData, int LogId)
        {
            string auditMsg = string.Empty;
            Facility AddressInfo = null;
            try
            {
                VAFacilities vaf = new VAFacilities(DBServer, DB, user, PWD);
                AddressInfo = vaf.FindFacility(zipcode);
            }
            catch (Exception ex)
            {

                auditMsg = String.Format("ERR:ZipCode:{0}", ex.Message.ToString());
                theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error);
                AddressInfo = new Facility();
            }

            return AddressInfo;

        }

        //Warren: Need to write an alternate version (MVICall_SQL?) to be used during testing
        //ZEB VA118-11-D-1009 1/31/13 add boolean params for error codes needed by alternate searches
        /// <summary>
        /// Performs the call to the MVI web service to find a patient's ICN given their demographic info
        /// </summary>
        /// <param name="lastName">The patient's last name</param>
        /// <param name="firstName">The patient's first name</param>
        /// <param name="middleName">The patient's middle name</param>
        /// <param name="suffix">The patient's name suffix (like "Jr")</param>
        /// <param name="gender">The patient's gender ("M" or "F")</param>
        /// <param name="dob">The patient's date of birth</param>
        /// <param name="ssn">The patient's SSN</param>
        /// <param name="url">The URL of the MVI webservice</param>
        /// <param name="mvipki">The name of the PKI certificate to use</param>
        /// <param name="LogId">The ID of the log to log events</param>
        /// <param name="pkiCheck">Unused</param>
        /// <param name="theData">Service configuration information</param>
        /// <param name="AAA71">Returned as true if a AAA segment with an error code of 71 should be returned by the caller</param>
        /// <param name="AAA72">Returned as true if a AAA segment with an error code of 72 should be returned by the caller</param>
        /// <param name="AAA73">Returned as true if a AAA segment with an error code of 73 should be returned by the caller</param>
        /// <param name="AAA75">Returned as true if a AAA segment with an error code of 75 should be returned by the caller</param>
        /// <returns></returns>
        private string MVICall(string lastName, string firstName, string middleName, string suffix, string gender, string dob, string ssn, string url, string mvipki, int LogId, string pkiCheck, gDataClass theData, out bool AAA71, out bool AAA72, out bool AAA73, out bool AAA75)
        {
            string response = string.Empty;
            string auditMsg = string.Empty;

            AAA71 = false;
            AAA72 = false;
            AAA73 = false;
            AAA75 = false;

            bool hasLastName = (lastName.Length > 0);  //ZEB VA118-11-D-1009 1/31/13 needed for alternate searches
            bool hasFirstName = (firstName.Length > 0);
            bool hasDOB = (dob.Length > 0);
            bool hasSSN = (ssn.Length > 0);

            string mviLastName = "", mviFirstName = "", mviDOB = "", mviSSN = "";
            XmlDocument doc = new XmlDocument();

            if (hasLastName && hasFirstName && hasDOB && hasSSN)
            {
                //search with all four
                try
                {
                    if (theData.TraceEnabled)
                    {
                        auditMsg = String.Format("MVICall-IN= {0} {1} {2} {3} {4} {5}\n{6}\n{7}", ssn, dob, firstName, lastName, middleName, suffix, url, mvipki);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }

                    PRPA1305 x1305 = new PRPA1305();

                    x1305.LastName = lastName;
                    x1305.FirstName = firstName;
                    x1305.MiddleName = middleName;
                    x1305.Suffix = suffix;

                    x1305.Gender = gender;
                    x1305.DOB = dob;
                    x1305.SSN = ssn;

                    x1305.MVI_ID = "200HAPE";
                    x1305.MviPki = mvipki;
                    x1305.LOGID = LogId;
                    x1305.URL = url;
                    x1305.WriteLog = theData.LogWEBService;

                    if (_useInternalInterfaces)
                    {
                        response = x1305.GET_VPID();

                        mviLastName = x1305.LastName;
                        mviFirstName = x1305.FirstName;
                        mviDOB = x1305.DOB;
                        mviSSN = x1305.SSN;
                    }
                    else
                    {
                        response = MVI_Lookup(lastName, firstName, middleName, suffix, gender, dob, ssn, 1, out mviLastName, out mviFirstName, out mviDOB, out mviSSN);
                    }

                    if (response != "NF")
                    {
                        if ((lastName.ToUpper() != mviLastName.ToUpper()) || (firstName.ToUpper() != mviFirstName.ToUpper()))
                        {
                            AAA73 = true;
                            response = "NF";
                        }

                        if (dob != mviDOB)
                        {
                            AAA71 = true;
                            response = "NF";
                        }

                        if (ssn != mviSSN)
                        {
                            AAA72 = true;
                            response = "NF";
                        }
                    }

                    MVIPayload = x1305.MviPayload;

                    if (theData.TraceEnabled)
                    {
                        auditMsg = String.Format("response.Length= {0} {1}", response.Length, response);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }

                }
                catch (Exception ex)
                {
                    //log exception calling service
                    auditMsg = String.Format("MVI ERROR-1= {0}\n{1}", ex.Message, url);
                    theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 188, 500);
                    return "";
                }

                if ((response == "NF") && !AAA73)
                {
                    //if failed, try with no first name
                    //if that succeeds, return error 73

                    try
                    {
                        if (theData.TraceEnabled)
                        {
                            auditMsg = String.Format("MVICall-IN= {0} {1} {2} {3} {4} {5}\n{6}\n{7}", ssn, dob, firstName, lastName, middleName, suffix, url, mvipki);
                            theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }

                        PRPA1305 x1305 = new PRPA1305();

                        x1305.LastName = lastName;
                        x1305.FirstName = "";
                        x1305.MiddleName = middleName;
                        x1305.Suffix = suffix;

                        x1305.Gender = gender;
                        x1305.DOB = dob;
                        x1305.SSN = ssn;

                        x1305.MVI_ID = "200HAPE";
                        x1305.MviPki = mvipki;
                        x1305.LOGID = LogId;
                        x1305.URL = url;
                        x1305.WriteLog = theData.LogWEBService;

                        if (_useInternalInterfaces)
                        {
                            response = x1305.GET_VPID();

                            mviLastName = x1305.LastName;
                            mviFirstName = x1305.FirstName;
                            mviDOB = x1305.DOB;
                            mviSSN = x1305.SSN;
                        }
                        else
                        {
                            response = MVI_Lookup(lastName, firstName, middleName, suffix, gender, dob, ssn, 1, out mviLastName, out mviFirstName, out mviDOB, out mviSSN);
                        }
                        if ((response.Length > 0) && (response != "NF"))
                        {
                            AAA73 = true;
                            response = "NF";
                        }

                        MVIPayload = x1305.MviPayload;

                        if (theData.TraceEnabled)
                        {
                            auditMsg = String.Format("response.Length= {0} {1}", response.Length, response);
                            theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                        }

                    }
                    catch (Exception ex)
                    {
                        //log exception calling service
                        auditMsg = String.Format("MVI ERROR-1= {0}\n{1}", ex.Message, url);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 188, 500);
                        return "";
                    }

                    if ((response == "NF") && !AAA73 && !AAA71)
                    {
                        //if failed, try with no dob
                        //if that succeeds, return error 71

                        try
                        {
                            if (theData.TraceEnabled)
                            {
                                auditMsg = String.Format("MVICall-IN= {0} {1} {2} {3} {4} {5}\n{6}\n{7}", ssn, dob, firstName, lastName, middleName, suffix, url, mvipki);
                                theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                            }

                            PRPA1305 x1305 = new PRPA1305();

                            x1305.LastName = lastName;
                            x1305.FirstName = firstName;
                            x1305.MiddleName = middleName;
                            x1305.Suffix = suffix;

                            x1305.Gender = gender;
                            x1305.DOB = "";
                            x1305.SSN = ssn;

                            x1305.MVI_ID = "200HAPE";
                            x1305.MviPki = mvipki;
                            x1305.LOGID = LogId;
                            x1305.URL = url;
                            x1305.WriteLog = theData.LogWEBService;

                            if (_useInternalInterfaces)
                            {
                                response = x1305.GET_VPID();

                                mviLastName = x1305.LastName;
                                mviFirstName = x1305.FirstName;
                                mviDOB = x1305.DOB;
                                mviSSN = x1305.SSN;
                            }
                            else
                            {
                                response = MVI_Lookup(lastName, firstName, middleName, suffix, gender, dob, ssn, 1, out mviLastName, out mviFirstName, out mviDOB, out mviSSN);
                            }
                            if (response != "NF")
                            {
                                AAA71 = true;
                                response = "NF";
                            }
                            else
                            {
                                AAA72 = true;  //if they all fail, return errors 72 and 73
                                AAA73 = true;
                            }

                            MVIPayload = x1305.MviPayload;

                            if (theData.TraceEnabled)
                            {
                                auditMsg = String.Format("response.Length= {0} {1}", response.Length, response);
                                theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                            }

                        }
                        catch (Exception ex)
                        {
                            //log exception calling service
                            auditMsg = String.Format("MVI ERROR-1= {0}\n{1}", ex.Message, url);
                            theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 188, 500);
                            return "";
                        }
                    }
                }
            }
            else
            {
                //search with everything we've got
                //if we fail, return error code 75

                try
                {
                    if (theData.TraceEnabled)
                    {
                        auditMsg = String.Format("MVICall-IN= {0} {1} {2} {3} {4} {5}\n{6}\n{7}", ssn, dob, firstName, lastName, middleName, suffix, url, mvipki);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }


                    PRPA1305 x1305 = new PRPA1305();

                    x1305.LastName = lastName;
                    x1305.FirstName = firstName;
                    x1305.MiddleName = middleName;
                    x1305.Suffix = suffix;

                    x1305.Gender = gender;
                    x1305.DOB = dob;
                    x1305.SSN = ssn;

                    x1305.MVI_ID = "200HAPE";
                    x1305.MviPki = mvipki;
                    x1305.LOGID = LogId;
                    x1305.URL = url;
                    x1305.WriteLog = theData.LogWEBService;

                    if (_useInternalInterfaces)
                    {
                        response = x1305.GET_VPID();

                        mviLastName = x1305.LastName;
                        mviFirstName = x1305.FirstName;
                        mviDOB = x1305.DOB;
                        mviSSN = x1305.SSN;
                    }
                    else
                    {
                        response = MVI_Lookup(lastName, firstName, middleName, suffix, gender, dob, ssn, 1, out mviLastName, out mviFirstName, out mviDOB, out mviSSN);
                    }

                    if (response != "NF")
                    {
                        if ((lastName.ToUpper() != mviLastName.ToUpper()) || (firstName.ToUpper() != mviFirstName.ToUpper()))
                        {
                            AAA73 = true;
                            response = "NF";
                        }

                        if (dob != x1305.DOB)
                        {
                            AAA71 = true;
                            response = "NF";
                        }

                        if (ssn != x1305.SSN)
                        {
                            AAA72 = true;
                            response = "NF";
                        }
                    }

                    MVIPayload = x1305.MviPayload;

                    if (theData.TraceEnabled)
                    {
                        auditMsg = String.Format("response.Length= {0} {1}", response.Length, response);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                    }

                }
                catch (Exception ex)
                {
                    //log exception calling service
                    auditMsg = String.Format("MVI ERROR-1= {0}\n{1}", ex.Message, url);
                    theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Error, 188, 500);
                    return "";
                }

                if ((response == "NF")  && !(AAA71 || AAA72 || AAA73))
                {
                    AAA75 = true;
                }
            }

            return response;
        }

        public string MVI_Lookup(string lastName, string firstName, string middleName, string suffix, string gender, string dob, string ssn, int max_responses, out string ResultLastName, out string ResultFirstName, out string ResultDOB, out string ResultSSN)
        {
            string ICN = string.Empty;

            int Number_of_ICNs = 0;
            string[] ICNs = new string[10];

            ResultLastName = "";
            ResultFirstName = "";
            ResultDOB = "";
            ResultSSN = "";

            using (System.Data.SqlClient.SqlConnection Server_Connection = new System.Data.SqlClient.SqlConnection())
            {
                Server_Connection.ConnectionString = SQL_Server_Access();
                using (System.Data.SqlClient.SqlCommand SQL_Command = new System.Data.SqlClient.SqlCommand())
                {
                    Server_Connection.Open();
                    SQL_Command.Connection = Server_Connection;

                    var sql_select = "SELECT * FROM MVI WHERE ";
                    if (lastName != "") sql_select = sql_select + "Last_Name='" + lastName + "' AND";
                    if (firstName != "") sql_select = sql_select + " First_Name='" + firstName + "' AND";
                    if (gender != "") sql_select = sql_select + " Gender='" + gender + "' AND";
                    if (dob != "") sql_select = sql_select + " Date_of_Birth='" + dob + "' AND";
                    if (ssn != "") sql_select = sql_select + " Social_Security_Number='" + ssn + "'";
                    if (sql_select.EndsWith("AND") == true) sql_select = sql_select.Substring(0, sql_select.Length - 3);
                    sql_select = sql_select + "ORDER BY ICN";

                    SQL_Command.CommandText = sql_select;
                    // Response.Write(sql_select);
                    using (System.Data.SqlClient.SqlDataReader table_entries = SQL_Command.ExecuteReader())
                    {
                        if (table_entries.HasRows)
                        {
                            while (table_entries.Read())
                            {
                                Number_of_ICNs++;
                                if (Number_of_ICNs <= 10) ICNs[Number_of_ICNs] = table_entries["ICN"].ToString();
                                ResultLastName = table_entries["Last_Name"].ToString();
                                ResultFirstName = table_entries["First_Name"].ToString();
                                ResultDOB = table_entries["Date_of_Birth"].ToString();
                                ResultSSN = table_entries["Social_Security_Number"].ToString();
                            }
                        }
                        table_entries.Close();
                    }
                    Server_Connection.Close();
                }
            }
            if (Number_of_ICNs == 1) ICN = ICNs[Number_of_ICNs];
            if (Number_of_ICNs > max_responses) ICN = "NF";
            if ((Number_of_ICNs > 1) && (Number_of_ICNs <= max_responses))
            {
                for (var i = 1; i <= Number_of_ICNs; i++) ICN = ICN + ICNs[i] + ",";
                ICN = ICN.Substring(0, ICN.Length - 1);
            }

            if (Number_of_ICNs == 0) ICN = "NF";

            return ICN;
        }

        //Warren: Need to write alternate version (ESMCall_SQL?) to be used during testing
        private string ESMCall(string VPIN, int LogId, gDataClass theData)
        {
            string ES = "";
            try
            {
                if (theData.TraceEnabled)
                {
                    string auditMsg = String.Format("ES-IN= {0} {1}", VPIN, theData.ESURL);
                    theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Information);
                }

                if (_useInternalInterfaces)
                {
                    if (VPIN != "NF" && VPIN != "AE")
                    {

                        ESWebService es = new ESWebService();

                        es.MVI_ID = VPIN;
                        es.LOGID = LogId;

                        es.WriteLog = theData.LogWEBService;
                        es.ESUN = theData.ESUserName;
                        es.ESPW = theData.ESPassword;

                        es.URL = theData.ESURL;
                        ES = es.CALL_ES();

                        m_Estatus = es.Status;
                        m_Edate = es.Date;



                        m_line1 = Utilities.RemoveX12Delimiters(es.Line1);  //ZEB VA118-11-D-1009 10/7/13 (+3) cleaned up returned data in case it contains special chars
                        m_city = Utilities.RemoveX12Delimiters(es.City);
                        m_state = Utilities.RemoveX12Delimiters(es.State);
                        m_zipcode = Utilities.RemoveX12Delimiters(es.ZipCode);


                        //ES = "OK ";
                        //m_Estatus = "VERIFIED";
                        //m_Edate = "19700401";

                        //m_line1 = "1500 Belleau Wood";
                        //m_city = "Tallahassee";
                        //m_state = "FL";
                        //m_zipcode = "32308";





                    }
                    else
                    {


                        string auditMsg = String.Format("ES Was not called due to the VPIN= {0}", VPIN);
                        theData.AuditLog(LogId, auditMsg, System.Diagnostics.EventLogEntryType.Warning);

                    }
                }
                else
                {
                    string address_line1, city, state, zipcode;

                    string enrollment_date = ES_Lookup(VPIN, out address_line1, out city, out state, out zipcode);

                    if (enrollment_date.Length > 0)
                    {
                        ES = "OK ";
                        m_Estatus = "VERIFIED";
                        m_Edate = enrollment_date;

                        m_line1 = Utilities.RemoveX12Delimiters(address_line1);  //ZEB VA118-11-D-1009 10/7/13 (+3) cleaned up returned data in case it contains special chars
                        m_city = Utilities.RemoveX12Delimiters(city);
                        m_state = Utilities.RemoveX12Delimiters(state);
                        m_zipcode = Utilities.RemoveX12Delimiters(zipcode);

                    }
                    else
                    {
                        //  Not sure what to return/set if no entry exists in the ES database.  wtc 2.18.2013
                        ES = "NF";  //ZEB VA118-11-D-1009 9/9/13 return NF if Veteran not in test ES
                        m_Estatus = "";
                        m_Edate = "";

                        m_line1 = "";
                        m_city = "";
                        m_state = "";
                        m_zipcode = "";
                    }
                }

            }
            catch (Exception ex)
            {
                ES = String.Format("ERR:ES:{0}", ex.Message.ToString());


            }
            return ES;

        }

        public string ES_Lookup(string ICN, out string address_line1, out string city, out string state, out string zipcode)
        {
            string ES = "";
            address_line1 = "";
            city = "";
            state = "";
            zipcode = "";

            using (System.Data.SqlClient.SqlConnection Server_Connection = new System.Data.SqlClient.SqlConnection())
            {
                Server_Connection.ConnectionString = SQL_Server_Access();
                using (System.Data.SqlClient.SqlCommand SQL_Command = new System.Data.SqlClient.SqlCommand())
                {
                    Server_Connection.Open();
                    SQL_Command.Connection = Server_Connection;

                    var sql_select = "SELECT * FROM ES WHERE ICN='" + ICN + "'";
                    // Response.Write(sql_select);
                    SQL_Command.CommandText = sql_select;

                    using (System.Data.SqlClient.SqlDataReader table_entries = SQL_Command.ExecuteReader())
                    {
                        if (table_entries.HasRows)
                        {
                            table_entries.Read();
                            ES = table_entries["Enrollment_Date"].ToString();
                            address_line1 = table_entries["address_line1"].ToString();
                            city = table_entries["city"].ToString();
                            state = table_entries["state"].ToString();
                            zipcode = table_entries["zip_code"].ToString();
                        }
                        table_entries.Close();
                    }
                    Server_Connection.Close();
                }
            }

            return ES;
        }

        public bool CheckAuthorization(string Username, string Password)
        {
            return true;
        }

        public string DecryptPassword(string Password)
        {
            return Password;
        }

        public string GetPassword(string Username)
        {
            try
            {

            }
            catch
            {
            }
            return "";
        }

        public string SQL_Server_Access()
        {
            string data_source = "vhahacfbpaet";
            string catalog = "AET_DEV";
            string userid = "AET_MONITOR";
            string password = "66br4490wrgbr!A";
            string security_flag = "false";

            string x = "Data Source=" + data_source + ";Initial Catalog=" + catalog + ";Integrated Security=" + security_flag
                + ";user id=" + userid + ";password=" + password;
            return x;
        }


    }
}
