﻿/*
 * Program Name: New Standards and Operating Rules - Payer Side (POR)
 * Contractor Name: Harris Corporation
 * Contract Number: VA118-11-D-1009
 * Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
 * 
 * Developer: Zach Bertram
 * Date: 04.2.2013
 * ----------------
 * Added registry values needed for snap-in
 */

using Microsoft.ManagementConsole;
using System.ComponentModel;
using System;
using System.Security.Permissions;
using Microsoft.Win32;

[assembly: PermissionSetAttribute(SecurityAction.RequestMinimum, Unrestricted = true)]
namespace AETServiceSettings {
    /// <summary>
    /// The RunInstaller attribute allows the .Net framework to install the assembly.
    /// </summary>
    [RunInstaller(true)]
    public class InstallUtilSupport : SnapInInstaller {

        //ZEB VA118-11-D-1009 4/2/13 (+method) added to add snap-in to registry during install
		public override void Install(System.Collections.IDictionary stateSaver)
        {
            base.Install(stateSaver);

            RegistryKey SnapIns;
            RegistryKey AETSnapIn;
            RegistryKey NodeTypes;
            RegistryKey Standalone;

            SnapIns = Registry.LocalMachine.OpenSubKey(@"Software\Wow6432Node\Microsoft\MMC\SnapIns", true);

            AETSnapIn = SnapIns.OpenSubKey("FX:{b8ea823b-345c-4573-bf64-a38620568667}", true);
            if (AETSnapIn == null)
            {
                SnapIns.CreateSubKey("FX:{b8ea823b-345c-4573-bf64-a38620568667}");
                AETSnapIn = SnapIns.OpenSubKey("FX:{b8ea823b-345c-4573-bf64-a38620568667}", true);
            }

            AETSnapIn.SetValue("UseCustomHelp", 0x00000000, RegistryValueKind.DWord);
            AETSnapIn.SetValue("Type", "AETServiceSettings.AETServiceSettings, AETServiceSettings, Version=3.0.0.0, Culture=neutral, PublicKeyToken=null", RegistryValueKind.String);
            AETSnapIn.SetValue("ApplicationBase", "C:\\Program Files\\VistAFee5010\\AETService", RegistryValueKind.String);
            AETSnapIn.SetValue("NameString", "AET Service Settings", RegistryValueKind.String);
            AETSnapIn.SetValue("Description", "AET Service Settings SnapIn", RegistryValueKind.String);
            AETSnapIn.SetValue("ModuleName", "AETServiceSettings.dll", RegistryValueKind.String);
            AETSnapIn.SetValue("AssemblyName", "AETServiceSettings", RegistryValueKind.String);
            AETSnapIn.SetValue("RuntimeVersion", "v2.0.50727", RegistryValueKind.String);
            AETSnapIn.SetValue("FxVersion", "3.0.0.0", RegistryValueKind.String);
            AETSnapIn.SetValue("About", "{00000000-0000-0000-0000-000000000000}", RegistryValueKind.String);

            NodeTypes = AETSnapIn.OpenSubKey("NodeTypes", false);
            if (NodeTypes == null)
            {
                AETSnapIn.CreateSubKey("NodeTypes");
                NodeTypes = AETSnapIn.OpenSubKey("NodeTypes", false);
            }

            Standalone = AETSnapIn.OpenSubKey("Standalone", false);
            if (Standalone == null)
            {
                AETSnapIn.CreateSubKey("Standalone");
                Standalone = AETSnapIn.OpenSubKey("Standalone", false);
            }

            Standalone.Close();
            NodeTypes.Close();
            AETSnapIn.Close();
            SnapIns.Close();
        }

		//ZEB VA118-11-D-1009 4/2/13 (+method) added to remove snap-in from registry on uninstall
        public override void Uninstall(System.Collections.IDictionary savedState)
        {
            base.Uninstall(savedState);

            RegistryKey SnapIns = Registry.LocalMachine.OpenSubKey(@"Software\Wow6432Node\Microsoft\MMC\SnapIns", true);

            try
            {
                SnapIns.DeleteSubKeyTree("FX:{b8ea823b-345c-4573-bf64-a38620568667}");
            }
            catch
            {
                //silently deal if keys have already been deleted
            }
        }
    }

    /// <summary>
    /// The main entry point for the creation of the snap-in.
    /// </summary>
    [SnapInSettings("{b8ea823b-345c-4573-bf64-a38620568667}",
         DisplayName = "AET Service Settings",
         Description = "AET Service Settings SnapIn")]  //ZEB VA118-11-D-1009 4/2/13 (-2) changed GUID
    public class AETServiceSettings : SnapIn {
        /// <summary>
        /// The constructor.
        /// </summary>
        public AETServiceSettings() {
            // Create the root node.
            this.RootNode = new ScopeNode();
            this.RootNode.DisplayName = "AET Service Settings";

            // Create a form view for the root node.
            FormViewDescription fvd = new FormViewDescription();
            fvd.DisplayName = "AET Service Settings (FormView)";
            fvd.ViewType = typeof(AETFormView);
            fvd.ControlType = typeof(AETControl);

            // Attach the view to the root node.
            this.RootNode.ViewDescriptions.Add(fvd);
            this.RootNode.ViewDescriptions.DefaultIndex = 0;

        }
    } // class
} // namespace