﻿/*
 * Program Name: New Standards and Operating Rules - Payer Side (POR)
 * Contractor Name: Harris Corporation
 * Contract Number: VA118-11-D-1009
 * Project: POR - CORE Compliance - Automated Eligibility Tool (AET)
 * 
 * Developer: Zach Bertram
 * Date: 03.20.2013
 * ----------------
 * Moved SafeSubString from AETService so it could be used in other classes
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AETShared
{
    /// <summary>
    /// Utility functions for use in the AET
    /// </summary>
    public static class Utilities
    {
        /// <summary>
        /// Returns the first len characters of Input; safely stops at Input if len is more than Input's length
        /// </summary>
        /// <param name="Input">The string to get the first len characters of</param>
        /// <param name="len">The number of characters to take from the beginning of Input</param>
        /// <returns>The first len characters of Input, or Input itself if len is greater than Input's length</returns>
        public static string SafeSubString(string Input, int len)
        {
            string rtn = string.Empty;
            if (!string.IsNullOrEmpty(Input))
            {
                if (Input.Length >= len)
                {
                    rtn = Input.Substring(0, len);
                }
                else
                {
                    rtn = Input;
                }
            }
            return rtn;
        }

        /// <summary>
        /// Returns the characters of Input starting with Start; safely returns nothing if Start is more than Input's length
        /// </summary>
        /// <param name="Input">The string to chop off the first Start-1 characters of</param>
        /// <param name="Start">The position to start returning characters from Input</param>
        /// <returns>All the characters starting at Start in the Input string, or null if there aren't that many characters</returns>
        public static string SafeSubStringToEnd(string Input, int Start)
        {
            if (string.IsNullOrEmpty(Input))
            {
                return "";
            }

            if (Start > Input.Length)
            {
                return "";
            }
            else
            {
                return Input.Substring(Start);
            }
        }

        /// <summary>
        /// Returns Length characters from Input starting at Start; safely returns nothing, or less than Length, if the string isn't long enough
        /// </summary>
        /// <param name="Input">The string to get the substring from</param>
        /// <param name="Start">The position to start returning characters from Input</param>
        /// <param name="Length">The number of characters to return</param>
        /// <returns>Length characters from Input starting at Start</returns>
        public static string SafeSubStringMid(string Input, int Start, int Length)
        {
            if (string.IsNullOrEmpty(Input))
            {
                return "";
            }

            if (Start > Input.Length)
            {
                return "";
            }

            if ((Start + Length) > Input.Length)
            {
                return Input.Substring(Start);
            }

            return Input.Substring(Start, Length);
        }

        //ZEB VA118-11-D-1009 10/7/13 (+function) needed to strip special characters out of returned fields
        /// <summary>
        /// Strip X12 delimiters out of a string for inclusion in the 271 message
        /// </summary>
        /// <param name="Input">The string to manipulate</param>
        /// <returns>The input strings with any *, ^, ~, or : stripped out</returns>
        public static string RemoveX12Delimiters(string Input)
        {
            return Input.Replace("*", "").Replace("^", "").Replace("~", "").Replace(":","");
        }
    }
}
