CREATE PROCEDURE App.usp_GetVendorByIDNumber (@IDNumber varchar(50))
AS
BEGIN
	SET NOCOUNT ON;

	SELECT * FROM CDWWork.SStaff.SFeeVendor WHERE IDNumber = @IDNumber;
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetVendorByIDNumber'
GO



CREATE PROCEDURE App.usp_GetInstitutionById (@Sta3n int, @InstitutionCode varchar(50))
AS
BEGIN
	SET NOCOUNT ON;

	SELECT * FROM CDWWork.Dim.Institution WHERE InactiveFlag is null AND Sta3n = @Sta3n AND InstitutionCode = @InstitutionCode;
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetInstitutionById'
GO


CREATE PROCEDURE App.usp_GetPatient (@PatientSSN varchar(50), @Sta3n int)
AS
BEGIN
	SET NOCOUNT ON;

	SELECT PatientSID,PatientIEN,Sta3n,PatientFirstName,PatientLastName,PatientSSN,StreetAddress1,StreetAddress2,StreetAddress3,City,Province,Zip,PostalCode,Country,DateOfBirth,Gender,EnteredIntoFileDate 
	 FROM CDWWork.SPatient.SPatient WHERE PatientSSN = @PatientSSN AND Sta3n = @Sta3n;
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetPatient'
GO


CREATE PROCEDURE App.usp_GetAuthorizationByPatient (@IDNumber varchar(50), @PatientIEN varchar(50))
AS
BEGIN

	SET NOCOUNT ON;

	SELECT a.FeeAuthorizationSID, a.FeeAuthorizationIEN, p.PatientSID, p.PatientIEN,
	a.Sta3n,a.FromDate,a.ToDate,a.FeeVendorSID,
	ISNULL(r.Status,'') AS FeeAuthorizationStatus 
	FROM CDWWork.Fee.FeeAuthorization a WITH (NOLOCK) 
	JOIN CDWWork.SPatient.SPatient p WITH (NOLOCK) on p.PatientIEN = a.PatientIEN AND p.Sta3n = a.Sta3n 
	JOIN CDWWork.SStaff.SFeeVendor v WITH (NOLOCK) on v.FeeVendorSID = a.FeeVendorSID 
	LEFT JOIN CDWWork.Fee.FeeAuthorizationRequest r WITH (NOLOCK) ON a.PatientSID = r.PatientSID AND a.Sta3n = r.Sta3n 
	AND a.FeeVendorSID = r.FeeVendorSID 
	WHERE a.PatientIEN = @PatientIEN AND v.IDNumber = @IDNumber
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetAuthorizationByPatient'
GO


CREATE PROCEDURE App.usp_GetAuthorizationByPatientAndDate (@IDNumber varchar(50), @PatientIEN varchar(50), @FromDate datetime, @ToDate datetime)
AS
BEGIN

	SET NOCOUNT ON;

	SELECT a.FeeAuthorizationSID, a.FeeAuthorizationIEN, p.PatientSID, p.PatientIEN,
	a.Sta3n,a.FromDate,a.ToDate,a.FeeVendorSID,
	ISNULL(r.Status,'') AS FeeAuthorizationStatus 
	FROM CDWWork.Fee.FeeAuthorization a WITH (NOLOCK) 
	JOIN CDWWork.SPatient.SPatient p WITH (NOLOCK) on p.PatientIEN = a.PatientIEN AND p.Sta3n = a.Sta3n 
	JOIN CDWWork.SStaff.SFeeVendor v WITH (NOLOCK) on v.FeeVendorSID = a.FeeVendorSID 
	LEFT JOIN CDWWork.Fee.FeeAuthorizationRequest r WITH (NOLOCK) ON a.PatientSID = r.PatientSID AND a.Sta3n = r.Sta3n 
	AND a.FeeVendorSID = r.FeeVendorSID 
	WHERE a.PatientIEN = @PatientIEN AND v.IDNumber = @IDNumber 
	AND a.FromDate = @FromDate AND a.ToDate = @ToDate 
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetAuthorizationByPatientAndDate'
GO



CREATE PROCEDURE App.usp_GetAuthorizationRemarks (@PatientIEN varchar(50), @FeeAuthorizationIEN varchar(50), @Sta3n int)
AS
BEGIN
	SET NOCOUNT ON;

	SELECT ISNULL(FeeAuthorizationRemarks,'') AS FeeAuthoriationRemark FROM CDWWork.Fee.FeeAuthorizationRemarks WHERE PatientIEN = @PatientIEN AND FeeAuthorizationIEN = @FeeAuthorizationIEN AND Sta3n = @Sta3n
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetAuthorizationRemarks'
GO

GO

CREATE TABLE [Dflt].[Request](
	[RequestId] [int] IDENTITY(1,1) NOT NULL,
	[RequestReceived] [datetime] NOT NULL DEFAULT (getdate()),
	[UAK] [varchar](100) NULL,
	[MessageText] [varchar](max) NULL,
	[PatientIEN] [varchar](50) NULL,
	[FeeVendorIDNumber] [varchar](50) NULL,
PRIMARY KEY CLUSTERED 
(
	[RequestId] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [DefFG]
) ON [DefFG] TEXTIMAGE_ON [DefFG]

GO


CREATE TABLE [Dflt].[Response](
	[ResponseId] [int] IDENTITY(1,1) NOT NULL,
	[RequestId] [int] NOT NULL DEFAULT ((0)),
	[ResponseSent] [datetime] NOT NULL DEFAULT (getdate()),
	[Sta3n] [int] NOT NULL DEFAULT ((0)),
	[FeeAuthorizationIEN] [varchar](250) NOT NULL,
	[MessageText] [varchar](max) NULL,
	[FeeAuthorizationStatus] [varchar](250) NULL,
	[LogEvent] [varchar](max) NULL,
PRIMARY KEY CLUSTERED 
(
	[ResponseId] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [DefFG]
) ON [DefFG] TEXTIMAGE_ON [DefFG]

GO


CREATE PROCEDURE [App].[usp_SaveResponse] (@RequestID int, @Sta3n int, @FeeAuthorizationIEN varchar(50), @MessageText varchar(max), @FeeAuthorizationStatus varchar(250), @LogEvent varchar(max)) 
AS
BEGIN
	SET NOCOUNT ON;

	INSERT INTO [Dflt].[Response] (RequestID, Sta3n, FeeAuthorizationIEN, MessageText, FeeAuthorizationStatus, LogEvent) 
	VALUES (@RequestID, @Sta3n, @FeeAuthorizationIEN, @MessageText, @FeeAuthorizationStatus, @LogEvent)

END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_SaveResponse'
GO




CREATE PROCEDURE [App].[usp_SaveRequest] (@RequestID bigint = 0, @RequestReceived datetime, @UAK varchar(100), @MessageText varchar(max), @PatientIEN varchar(50), @FeeVendorIDNumber varchar(50))
AS
BEGIN

	SET NOCOUNT ON;

	IF ISNULL(@RequestID,0) = 0
		BEGIN
			INSERT INTO [Dflt].[Request] (RequestReceived, UAK, MessageText, PatientIEN, FeeVendorIDNumber)
			VALUES (@RequestReceived, @UAK, @MessageText, @PatientIEN, @FeeVendorIDNumber);
			SELECT CAST(scope_identity() AS int);
		END
	ELSE
		BEGIN
			UPDATE [Dflt].[Request] 
			SET
				  [RequestReceived] = @RequestReceived
				, [UAK] = @UAK
				, [MessageText] = @MessageText
				, [PatientIEN] = @PatientIEN
				, [FeeVendorIDNumber] = @FeeVendorIDNumber
			WHERE
				[RequestID] = @RequestID;
				SELECT CAST(@RequestID AS int);
		END
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_SaveRequest'
GO




/*

DROP PROCEDURE App.usp_GetVendorByIDNumber

*/

USE [PD_EDI]
GO


CREATE PROCEDURE App.usp_GetVendorByIDNumber (@IDNumber varchar(50))
AS
BEGIN
       SET NOCOUNT ON;

       SELECT * FROM CDWWork.SStaff.SFeeVendor WHERE IDNumber = @IDNumber;
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetVendorByIDNumber'
GO


CREATE PROCEDURE App.usp_GetVendorByMedicareIDNumber (@MedicareIDNumber varchar(50))
AS
BEGIN
       SET NOCOUNT ON;

       SELECT * FROM CDWWork.SStaff.SFeeVendor WHERE MedicareIDNumber = @MedicareIDNumber;
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetVendorByMedicareIDNumber'
GO



CREATE PROCEDURE App.usp_GetVendorByNPI (@NPI varchar(50))
AS
BEGIN
       SET NOCOUNT ON;

       SELECT * FROM CDWWork.SStaff.SFeeVendor WHERE NPI = @NPI;
END

GO
EXEC [dbo].[sp_SignAppObject] 'usp_GetVendorByNPI'
GO












