﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using AnsiParser5010;
using System.Data;

namespace VA270Parser
{
    public class Parser
    {
        private string FileData;

        public Patient PatientData;
        public InformationSource InfoSource;
        public InformationReceiver InfoReceiver;
        public RelatedEntity NearestVAFacility;

        public List<string> ErrorList = new List<string>();

        public string ISASender;
        public string ISAReceiver;
        public string ISAControlNumber;
        public string ISATestProductionFlag;
        public string GSSender;
        public string GSReceiver;
        public string GSControlNumber;
        public string SubmitterTransactionIdentifier;

        private char InputElementSeparator;
        private char InputCompositeSeparator;
        private char InputRepetitionSeparator;
        private char InputSegmnetSeparator;

        private string Server;
        private string Database;
        private string Uid;
        private string Pwd;
        private string Timeout;

        public Parser(string server, string database, string uid, string pwd, string timeout)
        {
            Server = server;
            Database = database;
            Uid = uid;
            Pwd = pwd;
            Timeout = timeout;
        }

        public void Parse270(string fileData)
        {
            FileData = fileData;
            string ConnectionString = String.Format("Data Source={0};Initial Catalog={1};Integrated Security=False;User ID={2};Password={3};Timeout={4}", new string[] { Server, Database, Uid, Pwd, Timeout });
            ANSIReader reader = new ANSIReader(ConnectionString);
            reader.ParseFile(fileData, false);

            reader.GetSeparators(out InputElementSeparator, out InputCompositeSeparator, out InputRepetitionSeparator, out InputSegmnetSeparator);

            PatientData = new Patient();
            InfoSource = new InformationSource();
            InfoReceiver = new InformationReceiver();
            NearestVAFacility = new RelatedEntity();

            ErrorList = reader.errorList;
            //if (ErrorList.Count == 0)
            //{

                reader.ReadNext("000003");
                SubmitterTransactionIdentifier = reader.Get("Submitter.TransactionIdentifier");
                reader.ReadNext("2000A");

                InfoSource.EntityIdentifierCode = reader.Get("InformationSource.EntityIdentifierCode");
                InfoSource.EntityTypeQualifier = reader.Get("InformationSource.EntityTypeQualifier");
                InfoSource.LastName = reader.Get("InformationSource.LastName");
                InfoSource.FirstName = reader.Get("InformationSource.FirstName");
                InfoSource.IdentificationCodeQualifier = reader.Get("InformationSource.IdentificationCodeQualifier");
                InfoSource.PrimaryIdentifier = reader.Get("InformationSource.PrimaryIdentifier");

                reader.ReadNext("2000B");

                InfoReceiver.EntityIdentifierCode = reader.Get("InformationReceiver.EntityIdentifierCode");
                InfoReceiver.EntityTypeQualifier = reader.Get("InformationReceiver.EntityTypeQualifier");
                InfoReceiver.LastName = reader.Get("InformationReceiver.LastName");
                InfoReceiver.FirstName = reader.Get("InformationReceiver.FirstName");
                InfoReceiver.IdentificationCodeQualifier = reader.Get("InformationReceiver.IdentificationCodeQualifier");
                InfoReceiver.IdentificationNumber = reader.Get("InformationReceiver.IdentificationNumber");
                for (int i = 1; i < 10; ++i)
                {
                    string refseg = reader.Get("InformationReceiver.REF" + i.ToString());
                    if (refseg != "")
                        InfoReceiver.RefSegments.Add(refseg);
                    else
                        break;
                }
                InfoReceiver.AddressLine1 = reader.Get("InformationReceiver.AddressLine1");
                InfoReceiver.AddressLine2 = reader.Get("InformationReceiver.AddressLine2");
                InfoReceiver.City = reader.Get("InformationReceiver.City");
                InfoReceiver.State = reader.Get("InformationReceiver.State");
                InfoReceiver.Zip = reader.Get("InformationReceiver.Zip");


                //PatientData.FacilityZip = reader.Get("Facility.ZipCode");
                reader.ReadNext("2000C");

                string trace = reader.Get("Subscriber.TraceNumber1");
                if (!string.IsNullOrEmpty(trace))
                {
                    TRN t = new TRN();
                    t.TraceNumber = trace;
                    t.TraceAssigningEntityIdentifier = reader.Get("Subscriber.TraceAssigningEntityIdentifier1");
                    t.TraceAssigningEntityAdditionalIdentifier = reader.Get("Subscriber.TraceAssigningEntityAdditionalIdentifier1");
                    t.TraceSegment = reader.Get("Subscriber.TraceSegment1");
                    PatientData.TraceNumbers.Add(t);
                }
                trace = reader.Get("Subscriber.TraceNumber2");
                if (!string.IsNullOrEmpty(trace))
                {
                    TRN t = new TRN();
                    t.TraceNumber = trace;
                    t.TraceAssigningEntityIdentifier = reader.Get("Subscriber.TraceAssigningEntityIdentifier2");
                    t.TraceAssigningEntityAdditionalIdentifier = reader.Get("Subscriber.TraceAssigningEntityAdditionalIdentifier2");
                    t.TraceSegment = reader.Get("Subscriber.TraceSegment2");
                    PatientData.TraceNumbers.Add(t);
                }

                PatientData.LastName = reader.Get("Subscriber.LastName");
                PatientData.FirstName = reader.Get("Subscriber.FirstName");
                PatientData.MiddleName = reader.Get("Subscriber.MiddleName");
                PatientData.NameSuffix = reader.Get("Subscriber.NameSuffix");
                PatientData.IDCodeQualifier = reader.Get("Subscriber.IDCodeQualifier");
                PatientData.PrimaryIdentifier = reader.Get("Subscriber.PrimaryIdentifier");
                DateTime.TryParseExact(reader.Get("Subscriber.BirthDate"), "yyyyMMdd", null, System.Globalization.DateTimeStyles.None, out PatientData.DateOfBirth);
                PatientData.SocialSecurityNum = reader.Get("SocialSecurityNumber");
                for (int i = 1; i < 10; ++i)
                {
                    string refseg = reader.Get("Subscriber.REF" + i.ToString());
                    if (refseg != "")
                        PatientData.RefSegments.Add(refseg);
                    else
                        break;
                }
                PatientData.AddressLine1 = reader.Get("Subscriber.AddressLine1");
                PatientData.AddressLine2 = reader.Get("Subscriber.AddressLine2");
                PatientData.City = reader.Get("Subscriber.City");
                PatientData.State = reader.Get("Subscriber.State");
                PatientData.Zip = reader.Get("Subscriber.Zip");
                PatientData.PRV = reader.Get("Subscriber.PRV");
                PatientData.Gender = reader.Get("Subscriber.Gender");
                PatientData.MultipleBirthSeqNum = reader.Get("Subscriber.MultipleBirthSequenceNumber");
                PatientData.HI = reader.Get("Subscriber.HI");
                string dtp = reader.Get("Subscriber.DTP1");
                if (dtp != "")
                    PatientData.DtpSegments.Add(dtp);
                dtp = reader.Get("Subscriber.DTP2");
                if (dtp != "")
                    PatientData.DtpSegments.Add(dtp);


                reader.ReadNext("2110C");
                PatientData.ServiceTypeCode = reader.Get("SubscriberInquiry.ServiceTypeCode");
            //}

            ISASender = reader.ISASenderID;
            ISAReceiver = reader.ISAReceiverID;
            ISAControlNumber = reader.ISAControlNumber;
            ISATestProductionFlag = reader.ISATestProductionFlag;
            GSSender = reader.GSSenderID;
            GSReceiver = reader.GSReceiverID;
            GSControlNumber = reader.GSControlNumber;
        }

        public string Build271()
        {
            string ConnectionString = String.Format("Data Source={0};Initial Catalog={1};Integrated Security=False;User ID={2};Password={3};Timeout={4}", new string[] { Server, Database, Uid, Pwd, Timeout });
            ANSIWriter writer = new ANSIWriter(ConnectionString);
            writer.SetFileType("005010X279A1", "HB");
            writer.SetSeparators('*', ':', '^', '~');
            writer.StartLoop("000003");
            writer.StartLoop("2000A");
            //writer.Set("InformationSource.ValidRequestIndicator", "N");
            if (!string.IsNullOrEmpty(InfoSource.AAA))
            {
                writer.Set("InformationSource.AAA", InfoSource.AAA);
            }
            writer.Set("InformationSource.EntityIdentifierCode", InfoSource.EntityIdentifierCode);
            writer.Set("InformationSource.EntityTypeQualifier", InfoSource.EntityTypeQualifier);
            writer.Set("InformationSource.LastName", InfoSource.LastName);
            writer.Set("InformationSource.FirstName", InfoSource.FirstName);
            writer.Set("InformationSource.IdentificationCodeQualifier", InfoSource.IdentificationCodeQualifier);
            writer.Set("InformationSource.PrimaryIdentifier", InfoSource.PrimaryIdentifier);
            writer.StartLoop("2000B");
            if (!string.IsNullOrEmpty(InfoReceiver.AAA))
            {
                writer.Set("InformationReceiver.AAA", InfoReceiver.AAA);
            }
            writer.Set("InformationReceiver.EntityIdentifierCode", InfoReceiver.EntityIdentifierCode);
            writer.Set("InformationReceiver.EntityTypeQualifier", InfoReceiver.EntityTypeQualifier);
            writer.Set("InformationReceiver.LastName", InfoReceiver.LastName);
            writer.Set("InformationReceiver.FirstName", InfoReceiver.FirstName);
            writer.Set("InformationReceiver.IdentificationCodeQualifier", InfoReceiver.IdentificationCodeQualifier);
            writer.Set("InformationReceiver.IdentificationNumber", InfoReceiver.IdentificationNumber);
            int ct = 1;
            foreach (string s in InfoReceiver.RefSegments)
            {
                string newRef = s.Replace(InputElementSeparator, '*').Replace(InputCompositeSeparator, ':').Replace(InputRepetitionSeparator, '^');
                writer.Set("InformationReceiver.REF" + ct.ToString(), newRef);
                ++ct;
            }
            writer.Set("InformationReceiver.AddressLine1", InfoReceiver.AddressLine1);
            writer.Set("InformationReceiver.AddressLine2", InfoReceiver.AddressLine2);
            writer.Set("InformationReceiver.City", InfoReceiver.City);
            writer.Set("InformationReceiver.State", InfoReceiver.State);
            writer.Set("InformationReceiver.Zip", InfoReceiver.Zip);
            writer.Set("InformationReceiver.ProviderCode", InfoReceiver.ProviderCode);
            writer.Set("InformationReceiver.ReferenceIdentificationQualifier", InfoReceiver.ReferenceIdentificationQualifier);
            writer.Set("InformationReceiver.ProviderTaxonomyCode", InfoReceiver.ProviderTaxonomyCode);

            writer.StartLoop("2000C");
            writer.Set("Subscriber.EntityType", "1");
            if (PatientData.TraceNumbers.Count > 0)
                writer.Set("Subscriber.TraceSegment1", PatientData.TraceNumbers[0].TraceSegment);
            if (PatientData.TraceNumbers.Count > 1)
                writer.Set("Subscriber.TraceSegment2", PatientData.TraceNumbers[1].TraceSegment);
            writer.Set("Subscriber.LastName", PatientData.LastName);
            writer.Set("Subscriber.FirstName", PatientData.FirstName);
            writer.Set("Subscriber.MiddleName", PatientData.MiddleName);
            writer.Set("Subscriber.NameSuffix", PatientData.NameSuffix);
            writer.Set("Subscriber.IDCodeQualifier", PatientData.IDCodeQualifier);
            writer.Set("Subscriber.PrimaryIdentifier", PatientData.PrimaryIdentifier);
            if (PatientData.DateOfBirth != DateTime.MinValue)
                writer.Set("Subscriber.BirthDate", PatientData.DateOfBirth.ToString("yyyyMMdd"));
            //writer.Set("Subscriber.SocialSecurityNumber", PatientData.SocialSecurityNum);
            ct = 1;
            foreach (string s in PatientData.RefSegments)
            {
                string newref = s.Replace(InputElementSeparator, '*').Replace(InputCompositeSeparator, ':').Replace(InputRepetitionSeparator, '^');
                writer.Set("Subscriber.REF" + ct.ToString(), newref);
                ++ct;
            }
            writer.Set("Subscriber.AddressLine1", PatientData.AddressLine1);
            writer.Set("Subscriber.AddressLine2", PatientData.AddressLine2);
            writer.Set("Subscriber.City", PatientData.City);
            writer.Set("Subscriber.State", PatientData.State);
            writer.Set("Subscriber.Zip", PatientData.Zip);
            if (!string.IsNullOrEmpty(PatientData.PRV))
                writer.Set("Subscriber.PRV", PatientData.PRV.Replace(InputElementSeparator, '*').Replace(InputCompositeSeparator, ':').Replace(InputRepetitionSeparator, '^'));
            writer.Set("Subscriber.Gender", PatientData.Gender);
            writer.Set("Subscriber.MultipleBirthSequenceNumber", PatientData.MultipleBirthSeqNum);
            if (!string.IsNullOrEmpty(PatientData.HI))
                writer.Set("Subscriber.HI", PatientData.HI.Replace(InputElementSeparator, '*').Replace(InputCompositeSeparator, ':').Replace(InputRepetitionSeparator, '^'));

            ct = 1;
            foreach (string s in PatientData.DtpSegments)
            {
                string newdtp = s.Replace(InputElementSeparator, '*').Replace(InputCompositeSeparator, ':').Replace(InputRepetitionSeparator, '^');
                writer.Set("Subscriber.DTP" + ct.ToString(), newdtp);
                ++ct;
            }

            writer.StartLoop("2110C");
            writer.Set("EligibilityInformation.Code", PatientData.EligibilityOfBenifitInformationCode);
            writer.Set("EligibilityInformation.ServiceTypeCode", "1^33^35^47^86^88^98^AL^MH^UC");
            writer.Set("EligibilityInformation.AuthorizationIndicator", "Y");
            if (!string.IsNullOrEmpty(PatientData.BenefitBeginDate))
            {
                writer.Set("EligibilityInformation.BenefitBegin", PatientData.BenefitBeginDate);
                writer.Set("EligibilityInformation.BenefitBeginQual", "D8");
            }
            writer.StartLoop("2120C");
            writer.Set("RelatedEntity.EntityIdentifierCode", "FA");
            writer.Set("RelatedEntity.EntityTypeQualifier", "2");
            writer.Set("RelatedEntity.LastName", NearestVAFacility.FacilityName);
            writer.Set("RelatedEntity.NPI", NearestVAFacility.NPI);
            writer.Set("RelatedEntity.AddressLine1", NearestVAFacility.AddressLine1);
            writer.Set("RelatedEntity.AddressLine2", NearestVAFacility.AddressLine2);
            writer.Set("RelatedEntity.City", NearestVAFacility.City);
            writer.Set("RelatedEntity.State", NearestVAFacility.State);
            writer.Set("RelatedEntity.Zip", NearestVAFacility.Zip);
            // easier to manually build the PER segments
            string PER = "PER*IC*";
            int eleCt = 2;
            int PERCt = 1;
            if (!string.IsNullOrEmpty(NearestVAFacility.Telephone))
            {
                eleCt += 2;
                PER += "*TE*" + NearestVAFacility.Telephone;
            }
            if (!string.IsNullOrEmpty(NearestVAFacility.TelephoneExtension))
            {
                eleCt += 2;
                PER += "*EX*" + NearestVAFacility.TelephoneExtension;
            }
            if (!string.IsNullOrEmpty(NearestVAFacility.Fax))
            {
                eleCt += 2;
                PER += "*FX*" + NearestVAFacility.Fax;
            }
            if (eleCt == 8)
            {
                writer.Set("RelatedEntity.PER" + PERCt.ToString(), PER);
                ++PERCt;
                PER = "PER*IC*";
                eleCt = 2;
            }
            if (!string.IsNullOrEmpty(NearestVAFacility.Email))
            {
                eleCt += 2;
                PER += "*EM*" + NearestVAFacility.Email;
            }
            if (eleCt == 8)
            {
                writer.Set("RelatedEntity.PER" + PERCt.ToString(), PER);
                ++PERCt;
                PER = "PER*IC**";
                eleCt = 2;
            }
            if (!string.IsNullOrEmpty(NearestVAFacility.URL))
            {
                eleCt += 2;
                PER += "*UR*" + NearestVAFacility.URL;
            }
            if (eleCt != 2)
                writer.Set("RelatedEntity.PER" + PERCt.ToString(), PER);
            writer.EndLoop();
            writer.EndLoop();

            writer.EndLoop();
            writer.EndLoop();
            writer.EndLoop();
            writer.EndLoop();
            writer.SetISAFields(ISAReceiver, ISASender, ISAControlNumber, ISATestProductionFlag);
            writer.SetGSFields(GSReceiver, GSSender, GSControlNumber);
            writer.SetSubmitterTransactionIdentifier(SubmitterTransactionIdentifier);
            return writer.GetCompletedFile();
        }

        public string Build271InStagingTables(string connectionString)
        {
            int ResponseID;
            int Loop2100CID;

            SqlConnection sqlCon = new SqlConnection(connectionString);
            sqlCon.Open();

            SqlCommand cmd = new SqlCommand("spCreateResponse", sqlCon);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.AddWithValue("@Name", "test");
            ResponseID = Convert.ToInt32(cmd.ExecuteScalar());
            cmd.Dispose();

            cmd = new SqlCommand("spLoop2100CInsert", sqlCon);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.AddWithValue("@ResponseID", ResponseID);
            Loop2100CID = Convert.ToInt32(cmd.ExecuteScalar());
            cmd.Dispose();

            cmd = new SqlCommand("spSegmentNM1Insert", sqlCon);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
            cmd.Parameters.AddWithValue("@fk2120C", DBNull.Value);
            cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
            cmd.Parameters.AddWithValue("@fk2120D", DBNull.Value);
            cmd.Parameters.AddWithValue("@cNM101", "IL");
            cmd.Parameters.AddWithValue("@cNM102", "1");
            cmd.Parameters.AddWithValue("@cNM103", PatientData.LastName);
            cmd.Parameters.AddWithValue("@cNM104", PatientData.FirstName);
            cmd.Parameters.AddWithValue("@cNM105", PatientData.MiddleName);
            cmd.Parameters.AddWithValue("@cNM106", DBNull.Value);
            cmd.Parameters.AddWithValue("@cNM107", PatientData.NameSuffix);
            cmd.Parameters.AddWithValue("@cNM108", PatientData.IDCodeQualifier);
            cmd.Parameters.AddWithValue("@cNM109", PatientData.PrimaryIdentifier);
            cmd.Parameters.AddWithValue("@cNM110", DBNull.Value);
            cmd.Parameters.AddWithValue("@cNM111", DBNull.Value);
            cmd.Parameters.AddWithValue("@cNM112", DBNull.Value);
            cmd.ExecuteScalar();
            cmd.Dispose();

            foreach (string s in PatientData.RefSegments)
            {
                string[] eles = s.Split(InputElementSeparator);
                cmd = new SqlCommand("spSegmentREFInsert", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
                cmd.Parameters.AddWithValue("@fk2110C", DBNull.Value);
                cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
                cmd.Parameters.AddWithValue("@fk2110D", DBNull.Value);
                cmd.Parameters.AddWithValue("@cREF01", eles[1]);
                cmd.Parameters.AddWithValue("@cREF02", eles[2]);
                if (eles.Length > 3)
                    cmd.Parameters.AddWithValue("@cREF03", eles[3]);
                else
                    cmd.Parameters.AddWithValue("@cREF03", DBNull.Value);
                cmd.ExecuteScalar();
                cmd.Dispose();
            }

            cmd = new SqlCommand("spSegmentN3Insert", sqlCon);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
            cmd.Parameters.AddWithValue("@fk2110C", DBNull.Value);
            cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
            cmd.Parameters.AddWithValue("@fk2110D", DBNull.Value);
            cmd.Parameters.AddWithValue("@cN301", PatientData.AddressLine1);
            cmd.Parameters.AddWithValue("@cN302", PatientData.AddressLine2);
            cmd.ExecuteScalar();
            cmd.Dispose();

            cmd = new SqlCommand("spSegmentN4Insert", sqlCon);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
            cmd.Parameters.AddWithValue("@fk2110C", DBNull.Value);
            cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
            cmd.Parameters.AddWithValue("@fk2110D", DBNull.Value);
            cmd.Parameters.AddWithValue("@cN401", PatientData.City);
            cmd.Parameters.AddWithValue("@cN402", PatientData.State);
            cmd.Parameters.AddWithValue("@cN403", PatientData.Zip);
            cmd.Parameters.AddWithValue("@cN404", DBNull.Value);
            cmd.Parameters.AddWithValue("@cN405", DBNull.Value);
            cmd.Parameters.AddWithValue("@cN406", DBNull.Value);
            cmd.Parameters.AddWithValue("@cN407", DBNull.Value);
            cmd.ExecuteScalar();
            cmd.Dispose();

            if (PatientData.PRV != null)
            {
                string[] elesPRV = PatientData.PRV.Split(InputElementSeparator);
                cmd = new SqlCommand("spSegmentPRVInsert", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
                cmd.Parameters.AddWithValue("@fk2120C", DBNull.Value);
                cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
                cmd.Parameters.AddWithValue("@fk2120D", DBNull.Value);
                if (elesPRV.Length > 1)
                    cmd.Parameters.AddWithValue("@cPRV01", elesPRV[1]);
                else
                    cmd.Parameters.AddWithValue("@cPRV01", DBNull.Value);
                if (elesPRV.Length > 2)
                    cmd.Parameters.AddWithValue("@cPRV02", elesPRV[2]);
                else
                    cmd.Parameters.AddWithValue("@cPRV02", DBNull.Value);
                if (elesPRV.Length > 3)
                    cmd.Parameters.AddWithValue("@cPRV03", elesPRV[3]);
                else
                    cmd.Parameters.AddWithValue("@cPRV03", DBNull.Value);
                if (elesPRV.Length > 4)
                    cmd.Parameters.AddWithValue("@cPRV04", elesPRV[4]);
                else
                    cmd.Parameters.AddWithValue("@cPRV04", DBNull.Value);
                if (elesPRV.Length > 6)
                    cmd.Parameters.AddWithValue("@cPRV06", elesPRV[6]);
                else
                    cmd.Parameters.AddWithValue("@cPRV06", DBNull.Value);
                cmd.ExecuteScalar();
                cmd.Dispose();
            }

            cmd = new SqlCommand("spSegmentDMGInsert", sqlCon);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
            cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
            cmd.Parameters.AddWithValue("@cDMG01", "D8");
            cmd.Parameters.AddWithValue("@cDMG02", PatientData.DateOfBirth.ToString("yyyyMMdd"));
            cmd.Parameters.AddWithValue("@cDMG03", PatientData.Gender);
            cmd.Parameters.AddWithValue("@cDMG04", DBNull.Value);
            cmd.Parameters.AddWithValue("@cDMG06", DBNull.Value);
            cmd.Parameters.AddWithValue("@cDMG07", DBNull.Value);
            cmd.Parameters.AddWithValue("@cDMG08", DBNull.Value);
            cmd.Parameters.AddWithValue("@cDMG09", DBNull.Value);
            cmd.Parameters.AddWithValue("@cDMG10", DBNull.Value);
            cmd.Parameters.AddWithValue("@cDMG11", DBNull.Value);
            cmd.ExecuteScalar();
            cmd.Dispose();

            cmd = new SqlCommand("spSegmentINSInsert", sqlCon);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
            cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS01", "Y");
            cmd.Parameters.AddWithValue("@cINS02", "18");
            cmd.Parameters.AddWithValue("@cINS03", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS04", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS05", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS06", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS07", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS08", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS09", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS10", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS11", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS12", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS13", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS14", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS15", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS16", DBNull.Value);
            cmd.Parameters.AddWithValue("@cINS17", PatientData.MultipleBirthSeqNum);
            cmd.ExecuteScalar();
            cmd.Dispose();

            if (PatientData.HI != "")
            {
                string[] HIEles = PatientData.HI.Split(InputElementSeparator);
                cmd = new SqlCommand("spSegmentHIInsert", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
                cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
                int hinum = 1;
                foreach (string HIEle in HIEles)
                {
                    string[] HISubEles = HIEle.Split(InputCompositeSeparator);
                    if (HISubEles.Length > 0)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[0]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    if (HISubEles.Length > 1)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[1]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    if (HISubEles.Length > 2)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[2]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    if (HISubEles.Length > 3)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[3]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    if (HISubEles.Length > 4)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[4]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    if (HISubEles.Length > 5)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[5]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    if (HISubEles.Length > 6)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[6]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    if (HISubEles.Length > 7)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[7]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    if (HISubEles.Length > 8)
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", HISubEles[8]);
                    else
                        cmd.Parameters.AddWithValue("@cHI" + hinum.ToString("00") + "_1", DBNull.Value);
                    ++hinum;
                }
                cmd.ExecuteScalar();
                cmd.Dispose();
            }

            foreach (string s in PatientData.DtpSegments)
            {
                string[] eles = s.Split(InputElementSeparator);
                cmd = new SqlCommand("spSegmentDTPInsert", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@fk2100C", Loop2100CID);
                cmd.Parameters.AddWithValue("@fk2110C", DBNull.Value);
                cmd.Parameters.AddWithValue("@fk2100D", DBNull.Value);
                cmd.Parameters.AddWithValue("@fk2110D", DBNull.Value);
                cmd.Parameters.AddWithValue("@cDTP01", eles[1]);
                cmd.Parameters.AddWithValue("@cDTP02", eles[2]);
                cmd.Parameters.AddWithValue("@cDTP03", eles[3]);
                cmd.ExecuteScalar();
                cmd.Dispose();
            }


            /////////////////////////////////////////////////////////////////////////////////////////
            StringBuilder sb = new StringBuilder();
            int HL = 3;


            cmd = new SqlCommand("spLoop2100C", sqlCon);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.AddWithValue("@fkResponse", ResponseID);
            SqlDataReader dr = cmd.ExecuteReader();
            List<int> Loop2100C = new List<int>();
            while (dr.Read())
                Loop2100C.Add(dr.GetInt32(0));
            dr.Close();
            dr.Dispose();
            cmd.Dispose();

            foreach (int loop in Loop2100C)
            {
                sb.AppendLine("HL*" + HL.ToString() + "*2*22*0~");
                cmd = new SqlCommand("spSegmentTRN", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("TRN*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" + (dr.IsDBNull(5) ? "" : dr.GetString(5))
                        + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" + (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentNM1", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("NM1*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                        (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                        (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "*" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "*" +
                        (dr.IsDBNull(13) ? "" : dr.GetString(13)) + "*" + (dr.IsDBNull(14) ? "" : dr.GetString(14)) + "*" +
                        (dr.IsDBNull(15) ? "" : dr.GetString(15)) + "*" + (dr.IsDBNull(16) ? "" : dr.GetString(16)) + "*" +
                        (dr.IsDBNull(17) ? "" : dr.GetString(17)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentREF", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("REF*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentN3", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("N3*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" + (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "~");

                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentN4", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("N4*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                        (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                        (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "*" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentAAA", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("AAA*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                        (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentPRV", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("PRV*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                        (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "**" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentDMG", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("DMG*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" +
                        (dr.IsDBNull(5) ? "" : dr.GetString(5)) + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "**" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                        (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                        (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "*" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "*" +
                        (dr.IsDBNull(13) ? "" : dr.GetString(13)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentINS", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("INS*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" +
                        (dr.IsDBNull(5) ? "" : dr.GetString(5)) + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "**" +
                        (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                        (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "*" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "*" +
                        (dr.IsDBNull(13) ? "" : dr.GetString(13)) + "*" + (dr.IsDBNull(14) ? "" : dr.GetString(14)) + "*" +
                        (dr.IsDBNull(15) ? "" : dr.GetString(15)) + "*" + (dr.IsDBNull(16) ? "" : dr.GetString(16)) + "*" +
                        (dr.IsDBNull(17) ? "" : dr.GetString(17)) + "*" + (dr.IsDBNull(18) ? "" : dr.GetString(18)) + "*" +
                        (dr.IsDBNull(19) ? "" : dr.GetString(19)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentHI", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("HI:" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + ":" +
                        (dr.IsDBNull(5) ? "" : dr.GetString(5)) + ":" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + ":" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + ":" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + ":" +
                        (dr.IsDBNull(9) ? "" : dr.GetString(9)) + ":" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + ":" +
                        (dr.IsDBNull(11) ? "" : dr.GetString(11)) + ":" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "*" +
                        (dr.IsDBNull(13) ? "" : dr.GetString(13)) + ":" + (dr.IsDBNull(14) ? "" : dr.GetString(14)) + ":" +
                        (dr.IsDBNull(15) ? "" : dr.GetString(15)) + ":" + (dr.IsDBNull(16) ? "" : dr.GetString(16)) + ":" +
                        (dr.IsDBNull(17) ? "" : dr.GetString(17)) + ":" + (dr.IsDBNull(18) ? "" : dr.GetString(18)) + ":" +
                        (dr.IsDBNull(19) ? "" : dr.GetString(19)) + ":" + (dr.IsDBNull(20) ? "" : dr.GetString(20)) + ":" +
                        (dr.IsDBNull(21) ? "" : dr.GetString(21)) + "*" + (dr.IsDBNull(22) ? "" : dr.GetString(22)) + ":" +
                        (dr.IsDBNull(23) ? "" : dr.GetString(23)) + ":" + (dr.IsDBNull(24) ? "" : dr.GetString(24)) + ":" +
                        (dr.IsDBNull(25) ? "" : dr.GetString(25)) + ":" + (dr.IsDBNull(26) ? "" : dr.GetString(26)) + ":" +
                        (dr.IsDBNull(27) ? "" : dr.GetString(27)) + ":" + (dr.IsDBNull(28) ? "" : dr.GetString(28)) + ":" +
                        (dr.IsDBNull(29) ? "" : dr.GetString(29)) + ":" + (dr.IsDBNull(30) ? "" : dr.GetString(30)) + "*" +
                        (dr.IsDBNull(31) ? "" : dr.GetString(31)) + ":" + (dr.IsDBNull(32) ? "" : dr.GetString(32)) + ":" +
                        (dr.IsDBNull(33) ? "" : dr.GetString(33)) + ":" + (dr.IsDBNull(34) ? "" : dr.GetString(34)) + ":" +
                        (dr.IsDBNull(35) ? "" : dr.GetString(35)) + ":" + (dr.IsDBNull(36) ? "" : dr.GetString(36)) + ":" +
                        (dr.IsDBNull(37) ? "" : dr.GetString(37)) + ":" + (dr.IsDBNull(38) ? "" : dr.GetString(38)) + ":" +
                        (dr.IsDBNull(39) ? "" : dr.GetString(39)) + "*" + (dr.IsDBNull(40) ? "" : dr.GetString(40)) + ":" +
                        (dr.IsDBNull(41) ? "" : dr.GetString(41)) + ":" + (dr.IsDBNull(42) ? "" : dr.GetString(42)) + ":" +
                        (dr.IsDBNull(43) ? "" : dr.GetString(43)) + ":" + (dr.IsDBNull(44) ? "" : dr.GetString(44)) + ":" +
                        (dr.IsDBNull(45) ? "" : dr.GetString(45)) + ":" + (dr.IsDBNull(46) ? "" : dr.GetString(46)) + ":" +
                        (dr.IsDBNull(47) ? "" : dr.GetString(47)) + ":" + (dr.IsDBNull(48) ? "" : dr.GetString(48)) + "*" +
                        (dr.IsDBNull(49) ? "" : dr.GetString(49)) + ":" + (dr.IsDBNull(50) ? "" : dr.GetString(50)) + ":" +
                        (dr.IsDBNull(51) ? "" : dr.GetString(51)) + ":" + (dr.IsDBNull(52) ? "" : dr.GetString(52)) + ":" +
                        (dr.IsDBNull(53) ? "" : dr.GetString(53)) + ":" + (dr.IsDBNull(54) ? "" : dr.GetString(54)) + ":" +
                        (dr.IsDBNull(55) ? "" : dr.GetString(55)) + ":" + (dr.IsDBNull(56) ? "" : dr.GetString(56)) + ":" +
                        (dr.IsDBNull(57) ? "" : dr.GetString(57)) + "*" + (dr.IsDBNull(58) ? "" : dr.GetString(58)) + ":" +
                        (dr.IsDBNull(59) ? "" : dr.GetString(59)) + ":" + (dr.IsDBNull(60) ? "" : dr.GetString(60)) + ":" +
                        (dr.IsDBNull(61) ? "" : dr.GetString(61)) + ":" + (dr.IsDBNull(62) ? "" : dr.GetString(62)) + ":" +
                        (dr.IsDBNull(63) ? "" : dr.GetString(63)) + ":" + (dr.IsDBNull(64) ? "" : dr.GetString(64)) + ":" +
                        (dr.IsDBNull(65) ? "" : dr.GetString(65)) + ":" + (dr.IsDBNull(66) ? "" : dr.GetString(66)) + "*" +
                        (dr.IsDBNull(67) ? "" : dr.GetString(67)) + ":" + (dr.IsDBNull(68) ? "" : dr.GetString(68)) + ":" +
                        (dr.IsDBNull(69) ? "" : dr.GetString(69)) + ":" + (dr.IsDBNull(70) ? "" : dr.GetString(70)) + ":" +
                        (dr.IsDBNull(71) ? "" : dr.GetString(71)) + ":" + (dr.IsDBNull(72) ? "" : dr.GetString(72)) + ":" +
                        (dr.IsDBNull(73) ? "" : dr.GetString(73)) + ":" + (dr.IsDBNull(74) ? "" : dr.GetString(74)) + ":" +
                        (dr.IsDBNull(75) ? "" : dr.GetString(75)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentDTP", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("DTP*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                cmd = new SqlCommand("spSegmentMPI", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@SubscriberID", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    sb.AppendLine("MPI*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" +
                        (dr.IsDBNull(5) ? "" : dr.GetString(5)) + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                        (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                        (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "~");
                }
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                List<int> Loop2110C = new List<int>();

                cmd = new SqlCommand("spLoop2110C", sqlCon);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("@fk2100C", loop);
                dr = cmd.ExecuteReader();
                while (dr.Read())
                    Loop2110C.Add(dr.GetInt32(0));
                dr.Close();
                dr.Dispose();
                cmd.Dispose();

                foreach (int loop2110 in Loop2110C)
                {
                    cmd = new SqlCommand("spSegmentEB", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@SubscriberResponseID", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                    {
                        sb.AppendLine("EB*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" +
                            (dr.IsDBNull(5) ? "" : dr.GetString(5)) + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                            (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                            (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                            (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "*" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "*" +
                            (dr.IsDBNull(13) ? "" : dr.GetString(13)) + "*" + (dr.IsDBNull(14) ? "" : dr.GetString(14)) + "*" +
                            (dr.IsDBNull(15) ? "" : dr.GetString(15)) + "*" + (dr.IsDBNull(16) ? "" : dr.GetString(16)) + ":" +
                            (dr.IsDBNull(17) ? "" : dr.GetString(17)) + ":" + (dr.IsDBNull(18) ? "" : dr.GetString(18)) + ":" +
                            (dr.IsDBNull(19) ? "" : dr.GetString(19)) + ":" + (dr.IsDBNull(20) ? "" : dr.GetString(20)) + ":" +
                            (dr.IsDBNull(21) ? "" : dr.GetString(21)) + ":" + (dr.IsDBNull(22) ? "" : dr.GetString(22)) + ":" +
                            (dr.IsDBNull(23) ? "" : dr.GetString(23)) + "*" + (dr.IsDBNull(24) ? "" : dr.GetString(24)) + ":" +
                            (dr.IsDBNull(25) ? "" : dr.GetString(25)) + ":" + (dr.IsDBNull(26) ? "" : dr.GetString(26)) + ":" +
                            (dr.IsDBNull(27) ? "" : dr.GetString(27)) + "~");
                    }
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    cmd = new SqlCommand("spSegmentHSD", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@SubscriberResponseID", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                    {
                        sb.AppendLine("HSD*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" +
                            (dr.IsDBNull(5) ? "" : dr.GetString(5)) + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                            (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                            (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                            (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "~");
                    }
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    cmd = new SqlCommand("spSegmentREF", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@SubscriberResponseID", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                    {
                        sb.AppendLine("REF*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                            (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "~");
                    }
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    cmd = new SqlCommand("spSegmentDTP", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@SubscriberResponseID", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                    {
                        sb.AppendLine("DTP*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                            (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "~");
                    }
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    cmd = new SqlCommand("spSegmentAAA", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@SubscriberResponseID", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                    {
                        sb.AppendLine("AAA*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                            (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                            (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "~");
                    }
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    cmd = new SqlCommand("spSegmentMSG", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@SubscriberResponseID", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                    {
                        sb.AppendLine("MSG*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" +
                            (dr.IsDBNull(5) ? "" : dr.GetString(5)) + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "~");
                    }
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    List<int> Loop2115C = new List<int>();

                    cmd = new SqlCommand("spLoop2115C", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@fk2110C", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                        Loop2115C.Add(dr.GetInt32(0));
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    foreach (int loop2115 in Loop2115C)
                    {
                        cmd = new SqlCommand("spSegmentIII", sqlCon);
                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.Parameters.AddWithValue("@SubscriberResponseAdditionalID", loop2115);
                        dr = cmd.ExecuteReader();
                        while (dr.Read())
                        {
                            sb.AppendLine("III*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" +
                                (dr.IsDBNull(5) ? "" : dr.GetString(5)) + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                                (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "**" +
                                (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                                (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "~");
                        }
                        dr.Close();
                        dr.Dispose();
                        cmd.Dispose();
                    }

                    cmd = new SqlCommand("spSegmentLS", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@SubscriberResponseID", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                    {
                        sb.AppendLine("LS*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "~");
                    }
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    List<int> Loop2120C = new List<int>();

                    cmd = new SqlCommand("spLoop2120C", sqlCon);
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@fk2110C", loop2110);
                    dr = cmd.ExecuteReader();
                    while (dr.Read())
                        Loop2120C.Add(dr.GetInt32(0));
                    dr.Close();
                    dr.Dispose();
                    cmd.Dispose();

                    foreach (int loop2120 in Loop2120C)
                    {
                        cmd = new SqlCommand("spSegmentNM1", sqlCon);
                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.Parameters.AddWithValue("@SubscriberRelatedEntityID", loop2120);
                        dr = cmd.ExecuteReader();
                        while (dr.Read())
                        {
                            sb.AppendLine("NM1*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                                (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                                (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                                (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "*" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "*" +
                                (dr.IsDBNull(13) ? "" : dr.GetString(13)) + "*" + (dr.IsDBNull(14) ? "" : dr.GetString(14)) + "*" +
                                (dr.IsDBNull(15) ? "" : dr.GetString(15)) + "*" + (dr.IsDBNull(16) ? "" : dr.GetString(16)) + "*" +
                                (dr.IsDBNull(17) ? "" : dr.GetString(17)) + "~");
                        }
                        dr.Close();
                        dr.Dispose();
                        cmd.Dispose();

                        cmd = new SqlCommand("spSegmentN3", sqlCon);
                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.Parameters.AddWithValue("@SubscriberRelatedEntityID", loop2120);
                        dr = cmd.ExecuteReader();
                        while (dr.Read())
                        {
                            sb.AppendLine("N3*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" + (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "~");

                        }
                        dr.Close();
                        dr.Dispose();
                        cmd.Dispose();

                        cmd = new SqlCommand("spSegmentN4", sqlCon);
                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.Parameters.AddWithValue("@SubscriberRelatedEntityID", loop2120);
                        dr = cmd.ExecuteReader();
                        while (dr.Read())
                        {
                            sb.AppendLine("N4*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                                (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                                (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                                (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "*" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "~");
                        }
                        dr.Close();
                        dr.Dispose();
                        cmd.Dispose();

                        cmd = new SqlCommand("spSegmentPER", sqlCon);
                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.Parameters.AddWithValue("@SubscriberRelatedEntityID", loop2120);
                        dr = cmd.ExecuteReader();
                        while (dr.Read())
                        {
                            sb.AppendLine("PER*" + (dr.IsDBNull(4) ? "" : dr.GetString(4)) + "*" +
                                (dr.IsDBNull(5) ? "" : dr.GetString(5)) + "*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                                (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                                (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "*" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "*" +
                                (dr.IsDBNull(11) ? "" : dr.GetString(11)) + "*" + (dr.IsDBNull(12) ? "" : dr.GetString(12)) + "~");
                        }
                        dr.Close();
                        dr.Dispose();
                        cmd.Dispose();

                        cmd = new SqlCommand("spSegmentPRV", sqlCon);
                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.Parameters.AddWithValue("@SubscriberRelatedEntityID", loop2120);
                        dr = cmd.ExecuteReader();
                        while (dr.Read())
                        {
                            sb.AppendLine("PRV*" + (dr.IsDBNull(6) ? "" : dr.GetString(6)) + "*" +
                                (dr.IsDBNull(7) ? "" : dr.GetString(7)) + "*" + (dr.IsDBNull(8) ? "" : dr.GetString(8)) + "*" +
                                (dr.IsDBNull(9) ? "" : dr.GetString(9)) + "**" + (dr.IsDBNull(10) ? "" : dr.GetString(10)) + "~");
                        }
                        dr.Close();
                        dr.Dispose();
                        cmd.Dispose();


                    }
                }
                ++HL;
            }

            sqlCon.Close();

            return sb.ToString();
        }
    }

    public enum PatientTypes { Subscriber, Dependent };

    public class TRN
    {
        public string TraceNumber;
        public string TraceAssigningEntityIdentifier;
        public string TraceAssigningEntityAdditionalIdentifier;
        public string TraceSegment;
    }

    public class Patient
    {
        public string FirstName;
        public string LastName;
        public string MiddleName;
        public string NameSuffix;
        public string IDCodeQualifier;
        public string PrimaryIdentifier;
        public DateTime DateOfBirth;
        public string SocialSecurityNum;
        public PatientTypes PatientType;
        //public string FacilityZip;
        public string ServiceTypeCode;
        public List<string> RefSegments = new List<string>();
        public string AddressLine1;
        public string AddressLine2;
        public string City;
        public string State;
        public string Zip;
        public string PRV;
        public string Gender;
        public string MultipleBirthSeqNum;
        public string HI;
        public List<string> DtpSegments = new List<string>();
        public string EligibilityOfBenifitInformationCode;
        public string BenefitBeginDate;
        public List<TRN> TraceNumbers = new List<TRN>();
    }

    public class InformationSource
    {
        public string EntityIdentifierCode;
        public string EntityTypeQualifier;
        public string LastName;
        public string FirstName;
        public string IdentificationCodeQualifier;
        public string PrimaryIdentifier;
        public string AAA;
    }

    public class InformationReceiver
    {
        public string EntityIdentifierCode;
        public string EntityTypeQualifier;
        public string LastName;
        public string FirstName;
        public string IdentificationCodeQualifier;
        public string IdentificationNumber;
        public List<string> RefSegments = new List<string>();
        public string AddressLine1;
        public string AddressLine2;
        public string City;
        public string State;
        public string Zip;
        public string ProviderCode;
        public string ReferenceIdentificationQualifier;
        public string ProviderTaxonomyCode;
        public string AAA;
    }

    public class RelatedEntity
    {
        public string FacilityName;
        public string NPI;
        public string AddressLine1;
        public string AddressLine2;
        public string City;
        public string State;
        public string Zip;
        public string Email;
        public string Telephone;
        public string TelephoneExtension;
        public string Fax;
        public string URL;
    }
}
