﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;

namespace VAZip
{

    public class Address
    {
        public string street { get; set; }
        public string city { get; set; }
        public string state { get; set; }
        public string zip { get; set; }
        public string zipEx { get; set; }
        public List<string> phones { get; set; }

        public Address()
        {

            street = string.Empty;
            city = string.Empty;
            state = string.Empty;
            zip = string.Empty;
            zipEx = string.Empty;
            phones = new List<string>();
        }
    }

    public class Facility
    {
        public string name { get; set; }
        public string facilityType { get; set; }
        public Address address  { get; set;}
        public bool hasErrors { get; set; }
        public string errorMsg { get; set; }

        public Facility()
        {
            name = string.Empty;
            facilityType = string.Empty;
            address = new Address();
            errorMsg = string.Empty;
            hasErrors = false;
        }
    }

    public class VAFacilities
    {
        protected string _Server;
        protected string _DB;
        protected string _USR;
        protected string _PWD;

        public VAFacilities(string Server, string DB, string USR, string PWD)
        {
            _Server = Server;
            _DB = DB;
            _USR = USR;
            _PWD = PWD;
        }

        public VAFacilities()
        {


            //_Server = "M1-DEV-M99";
            //_DB = "VAZip";
            //_USR = "sa";
            //_PWD = "Meg@s$QL";

            //_Server = "vm-dev-m01";
            //_DB = "VAZip";
            //_USR = "sa";
            //_PWD = "megas_2006";
        }

        public string Server
        {
            set { _Server = value; }
        }

        public string Database
        {
            set { _DB = value; }
        }

        public string User
        {
            set { _USR = value; }
        }

        public string Password
        {
            set { _PWD = value; }
        }

        public Facility FindFacility(string Zip)
        {
            Facility Rtn = new Facility();

            SqlConnectionStringBuilder bldr = new SqlConnectionStringBuilder();
            bldr.DataSource = _Server;
            bldr.InitialCatalog = _DB;
            bldr.UserID = _USR;
            bldr.Password = _PWD;

            string FixedZip = "";

            if (Zip.Length > 5)
            {
                FixedZip = Zip.Substring(0, 5);
            }
            else
            {
                FixedZip = Zip;
            }

            SqlCommand cmd = new SqlCommand("usp_GetFacility", new SqlConnection(bldr.ConnectionString));
            cmd.CommandType = CommandType.StoredProcedure;
            SqlDataReader rdr = null;

            try
            {
                cmd.Connection.Open();
                cmd.Parameters.AddWithValue("@ZIP", FixedZip);
                rdr = cmd.ExecuteReader();
                if (rdr.HasRows)
                {
                    rdr.Read();
                    Rtn.name = rdr["Facility"].ToString();
                    Rtn.facilityType = rdr["Type"].ToString();
                    Rtn.address.street = rdr["Address"].ToString();
                    Rtn.address.city = rdr["City"].ToString();
                    Rtn.address.state = rdr["State"].ToString();
                    Rtn.address.zip = rdr["Zip"].ToString();
                    if (rdr["ZipEx"] != DBNull.Value)
                    {
                        Rtn.address.zipEx = rdr["ZipEx"].ToString();
                    }
                    if (rdr["Phone"] != DBNull.Value)
                    {

                        string tmp = rdr["Phone"].ToString();
                        if (tmp.ToLower().Contains("or"))
                        {
                            string[] tmparry = tmp.ToLower().Split(new string[1]{"or"}, StringSplitOptions.RemoveEmptyEntries);
                            foreach (string tmpphone in tmparry)
                            {
                                Rtn.address.phones.Add(tmpphone.Trim());
                            }
                        }
                        else
                        {
                            Rtn.address.phones.Add(rdr["Phone"].ToString().Trim());
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Rtn.hasErrors = true;
                Rtn.errorMsg = "Error getting facility: " + ex.Message;
            }
            finally
            {
                if (rdr != null)
                {
                    rdr.Close();
                    rdr.Dispose();
                    rdr = null;
                }
                if (cmd.Connection != null)
                {
                    if (cmd.Connection.State != ConnectionState.Closed)
                    {
                        cmd.Connection.Close();
                    }
                    cmd.Dispose();
                    cmd = null;
                }
            }
            return Rtn;
        }
    }
}
