﻿using AETShared;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using X12Engine.BLL.X12._278.XParse;
using X12Engine.BO.X12._278;
using X12Engine.BO.X12._278.XModel;

namespace X12Engine.BLL.X12._278
{
	public class AuthorizationManager
	{

		public string LogPath = "";

		public string ProcessRequest(string message)
		{
			WriteLogFile("AuthorizationManager.ProcessRequest() - Start");

			Settings settings = new Settings();

			// AppVersion
			Globals.AppVersion = "Version.20170525";


			//Log TransactionLog 
			TransactionLog transactionLog = new TransactionLog();
			transactionLog.RequestDate = DateTime.Now;
			transactionLog.RequestMessageText = message;

			string feeAuthorizationSIDs = "";
			string messageIn = string.Empty;

			int requestId = 0, stationId = 0;
			int messageSize = 0;
			int controlNumber = 0;
			MessageBase messageBase = new MessageBase();

			char[] sizeBuffer = new char[6];
			string sizeIn = new string(sizeBuffer);
			char[] messageBuffer = new char[messageSize];
			string controlIn = string.Empty;
			string messageResponse = "";
			int authorizationCount = 0;
			string errorResponse = "";

			//tryA
				
			
	
			messageIn = message;

			//The second 6 bytes of the message contain the control number
			if (messageIn.Length >= 6)
			{
				controlIn = messageIn.Substring(0, 6);
				int.TryParse(controlIn, out controlNumber);

				//Remove control number from request
				messageIn = messageIn.Substring(6);
			}

			messageIn = messageIn.Substring(6);


			//********************************************************* Parse the request - prevalidate segments  *******************************

			MessageBaseManager messageBaseManager = new MessageBaseManager();
			messageBase = messageBaseManager.ParseSegments(ref settings, messageIn);

			errorResponse = GetErrorResponse(settings, message);

			// check db connection
			if (!messageBase.ValidConnection)
			{
				string errorMessage = String.Format("SQL Exception in AuthorizationServer.RequestProcessor(): Unable to connect database.");
				////System.Diagnostics.EventLog.WriteEntry("AETService", errorMessage, System.Diagnostics.EventLogEntryType.Error, 166, 500);
				WriteLogFile("Unable to connect database. - " + Globals.DBConnectionString);
				return errorResponse;
			}

			List<PatientEventDetail> patientEventDetails = new List<PatientEventDetail>();
			patientEventDetails = messageBase.PatientEventDetails;
			if (messageBase.ErrorFound || !messageBase.ValidateMessage())
			{
			}
			else
			{

				// set critical entities
				SPatient sPatient = new SPatient();
				List<SFeeVendor> vendors = new List<SFeeVendor>();
					
				List<SFeeVendor> validVendors = new List<SFeeVendor>();
				List<FeeAuthorization> feeAuthorizations = new List<FeeAuthorization>();

				// get authorization by patient ssn   , DateTime issueDate, DateTime fromDate, DateTime toDate   
				sPatient = messageBase.Patient;
				vendors = messageBase.FeeVendors;
					


				if (sPatient != null && Utils.GetIntValue(sPatient.PatientSID) > 0)
				{
					foreach (var vendor in vendors)
					{
						foreach (var patientEventDetail in patientEventDetails)
						{
							if (!patientEventDetail.ErrorFound && patientEventDetail.RequestValidation.Count == 0)
							{

								DateTime? fromDate = null;
								DateTime? toDate = null;

								if (patientEventDetail.FromDate != null)
								{
									fromDate = patientEventDetail.FromDate;
									toDate = patientEventDetail.ToDate;
								}

								if (patientEventDetail.IssueDateFrom != null)
								{
									fromDate = patientEventDetail.IssueDateFrom;
									toDate = patientEventDetail.IssueDateTo;
								}

								feeAuthorizations = GetFeeAuthorization(ref settings, vendor.FeeVendorSID, Utils.GetIntValue(sPatient.PatientSID), Utils.GetIntValue(vendor.Sta3n), fromDate, toDate, patientEventDetail.AdmissionDateFrom, patientEventDetail.AdmissionDateTo);

								foreach (var feeAuthorization in feeAuthorizations)
								{
									// Set authorizations in patientEventDetail
									patientEventDetail.FeeAuthorizations.Add(feeAuthorization);
									feeAuthorizationSIDs += feeAuthorization.FeeAuthorizationSID + ", ";
								}

								// Set authorization count
								authorizationCount += feeAuthorizations.Count();
								if (feeAuthorizations.Count() > 0)
								{
									validVendors.Add(vendor);
								}
							}
						}
					}
				}

				settings.AuthorizationCount = authorizationCount;

				if (!messageBase.TransErrorFound)
				{
					if (authorizationCount == 0)
					{
						messageBase.AddErrorToPatientEvent(ref settings, "N", "AA", "C", "Authorization Number Not Found.");
					}
					else
					{

						// log feeAuthorizationSIDs
						if (feeAuthorizationSIDs.Length > 2)
						{
							feeAuthorizationSIDs = feeAuthorizationSIDs.Substring(0, feeAuthorizationSIDs.Length - 2);
						}


						// Format authorization

						// UpdateUmoDetail
						foreach (var validVendor in validVendors)
						{
							stationId = X12Engine.Utils.GetIntValue(validVendor.Sta3n);
							Institution feeInstitution = GetInstitutionById(stationId);
							if (feeInstitution != null)
							{
								messageBase.UpdateUmoDetail(feeInstitution);
							}
						}

						// update message
						messageBase.UpdateRequesterDetail();
						messageBase.UpdateSubscriberDetail(settings, sPatient);
						messageBase.UpdateDependentDetail();
						messageBase.UpdateServiceDetail();
					}
				}

			}

			if (messageBase == null)
			{
				return errorResponse;
			}

			if (patientEventDetails.Count > 0 && !messageBase.TransErrorFound)
			{
				messageBase.UpdatePatientEventDetail(settings, patientEventDetails);
			}
				
			// Format response
			messageBase.FormatForResponse(settings);
			messageResponse = messageBase.ToString();


			// Log TransactionLog 
			settings.LogMessages.AppendLine("AuthorizationCount = " + authorizationCount);
			transactionLog.QueueId = 0;
				
			transactionLog.Sta3n = stationId;


			if (authorizationCount > 0)
			{
				// Set RequestStatus
				if (settings.DataRoutingCode.Count == 0)
				{
					transactionLog.RequestStatus = "RETURNED";
					settings.LogMessages.Append("RequestStatus: RETURNED");
				}
				else
				{
					SetRoutingQueue(ref settings, ref transactionLog);
				}
			}
			else if (settings.AAACount == 1 && messageResponse.Contains("AAA*N**AA*C~".Replace("*",settings.Data_Element_Separator.ToString())) && settings.DataRoutingCode.Count > 0)
			{
				SetRoutingQueue(ref settings, ref transactionLog);
			}
			else if (messageResponse.Contains("AAA"))
			{
				transactionLog.RequestStatus = "REJECTED";
				settings.LogMessages.Append("RequestStatus: REJECTED");
			}
			else if (settings.DataRoutingCode.Count > 0)
			{
				SetRoutingQueue(ref settings, ref transactionLog);
			} else
			{
				transactionLog.RequestStatus = "RETURNED";
				settings.LogMessages.Append("RequestStatus: RETURNED");
			}

			transactionLog.EventTraceNumber = messageBase.EventTraceNumber;
			transactionLog.AdminReferenceNumber = "";
			transactionLog.FeeAuthorizationSIDs = feeAuthorizationSIDs;
			transactionLog.FeeVendorID = messageBase.FeeVendorID;
			transactionLog.PatientFullName = messageBase.NameFirst + " " + messageBase.NameLast;
			transactionLog.GroupControlNumber = messageBase.GroupControlNumber;
			transactionLog.LogEvent = settings.LogMessages.ToString();
			transactionLog.ResponseMessageText = messageResponse;
			transactionLog.AuthorizationCount = authorizationCount;
			transactionLog.UAK = "";


			requestId = SaveTransactionLog(transactionLog);

			messageResponse = messageResponse.Replace("[ReferenceIdentification]", requestId.ToString());


			// save segments
			SaveMessageSegments(settings, requestId);


			WriteLogFile("ProcessRequest End");
			Globals.ClientMessage = settings.LogMessages.ToString();

			return messageResponse;

			//}

			//catch (Exception ex)
			//{
			//	var st = new StackTrace(ex, true);
			//	var frame = st.GetFrame(0);
			//	var line = frame.GetFileLineNumber();

			//	string errorMessage = String.Format("General Exception in AuthorizationServer.RequestProcessor(): " + ex.Message + "\r\n Line number: " + line);
			//	WriteLogFile(errorMessage + "\r\n" + ex.StackTrace);
			//	return errorResponse;
			//}

		}



		public void SetRoutingQueue(ref Settings settings, ref TransactionLog transactionLog)
		{
			// Request is queued, get the first sequence rule
			var firstOne = settings.DataRoutingCode.OrderBy(o => o.RuleNumber).ThenBy(t => t.SequenceNumber).FirstOrDefault();
			transactionLog.QueueId = firstOne.QueueId;
			transactionLog.RequestStatus = "QUEUED";
			settings.LogMessages.Append("RequestStatus: QUEUED to QueueId: " + firstOne.QueueId + "  RuleNumber: " + firstOne.RuleNumber + "   SequenceNumber: " + firstOne.SequenceNumber + " Description: " + firstOne.Description);


			// Write to AuditLog
			string sql = "INSERT INTO App.AuditLog(RequestId,DateTimeStamp,AuditAction,UserId,Queue,ReasonCode) ";
			sql += "VALUES([RequestId], getdate(), 'Authorization Queue', 'Authorization Queue', '" + firstOne.QueueId + "', @ReasonCode)";
			SqlCommand cmd = new SqlCommand(sql);

			cmd.Parameters.AddWithValue("@ReasonCode", firstOne.RuleNumber + "-" + firstOne.SequenceNumber + " - " + firstOne.Description);

			settings.SqlCommandsResponse.Add(cmd);
		}

		public string GetErrorResponse(Settings settings, string inMsg)
		{
			string returnAAA = "";
			try
			{
				string inputMessage = inMsg.Substring(12);
				returnAAA = inputMessage;
				string ds = char.ToString(settings.Data_Element_Separator);
				string st = char.ToString(settings.Segment_Terminator);
				string hlLine = "HL" + ds + "1" + ds + ds + "20" + ds + "1" + st;
				string seLine = "SE" + ds;


				if (inputMessage.Contains(hlLine))
				{
					string[] ray = inputMessage.Split(new string[] { hlLine }, StringSplitOptions.None);
					returnAAA = ray[0] + hlLine + "AAA" + ds + "N" + ds + ds + "42" + ds + "C" + st;
				}

				if (inputMessage.Contains(seLine))
				{
					string[] ray = inputMessage.Split(new string[] { seLine }, StringSplitOptions.None);
					returnAAA = returnAAA + seLine + ds + ray[1];
				}


			}
			catch (Exception)
			{
				return "Message request is empty";
			}

			return returnAAA;
		}

		public void WriteLogFile(string logContent)
		{
			return;
			//try
			//{
			//	TimeSpan timeSpan = new TimeSpan();
			//	//timeSpan = DateTime.Now - AAAManager.ProcessDateTime;

			//	string LogPath = "C:\\Program Files\\VistAFee5010\\AETService\\Logs\\AETLog_" + DateTime.Now.Year + DateTime.Now.Month + DateTime.Now.Day + ".txt";

			//	if (File.Exists(LogPath))
			//	{
			//		using (StreamWriter outfile = File.AppendText(LogPath))
			//		{
			//			outfile.WriteLine("\r\n-----------------------------------------------------------------------");
			//			outfile.WriteLine(DateTime.Now);
			//			outfile.WriteLine(logContent + " - " + timeSpan.TotalMilliseconds + " milliseconds");
			//		}
			//	}
			//	else
			//	{
			//		using (StreamWriter outfile = new StreamWriter(LogPath))
			//		{
			//			outfile.WriteLine("\r\n-----------------------------------------------------------------------");
			//			outfile.WriteLine(DateTime.Now);
			//			outfile.WriteLine(logContent + " - " + timeSpan.TotalMilliseconds + " milliseconds");
			//		}
			//	}

			//	//AAAManager.ProcessDateTime = DateTime.Now;
			//}
			//catch (Exception)
			//{

			//}

		}


		public List<FeeAuthorization> GetFeeAuthorization(ref Settings settings, int feeVendorSID, int patientSID, int sta3n, DateTime? fromDate, DateTime? toDate, DateTime? admissionDateFrom, DateTime? admissionDateTo)
		{
			List<FeeAuthorization> feeAuthorizations = new List<FeeAuthorization>();
			try
			{
				using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
				{
					conn.Open();

					using (SqlCommand cmd = new SqlCommand("App.usp_GetAuthorizations", conn))
					{
						cmd.CommandTimeout = 300;
						cmd.CommandType = System.Data.CommandType.StoredProcedure;

						cmd.Parameters.AddWithValue("@FeeVendorSID", feeVendorSID);
						cmd.Parameters.AddWithValue("@PatientSID", patientSID);
						cmd.Parameters.AddWithValue("@Sta3n", sta3n);
						cmd.Parameters.AddWithValue("@FromDate", fromDate);
						cmd.Parameters.AddWithValue("@ToDate", toDate);
						cmd.Parameters.AddWithValue("@AdmissionDateFrom", admissionDateFrom);
						cmd.Parameters.AddWithValue("@AdmissionDateTo", admissionDateTo);
						using (SqlDataReader reader = cmd.ExecuteReader())
						{

							if (reader.HasRows)
							{
								settings.LogMessages.Append("GetFeeAuthorization() Found.\r\n");
							} else
							{
								settings.LogMessages.Append("GetFeeAuthorization() Not Found.\r\n");
							}
							settings.LogMessages.Append("GetFeeAuthorization() feeVendorSID: " + feeVendorSID + " patientSID: " + patientSID + "   sta3n: " + sta3n + "   fromDate: " + fromDate + "   toDate: " + toDate + "   admissionDateFrom: " + admissionDateFrom + "   admissionDateTo: " + admissionDateTo + "\r\n");

							while (reader.Read())
							{

								

															StringBuilder sbRemark = new StringBuilder();
								sbRemark.Append("");
								string feeAuthorizationStatus = (reader["FeeAuthorizationStatus"] != null) ? reader["FeeAuthorizationStatus"].ToString() : "";
								if (feeAuthorizationStatus != "DC")
								{
									// get remarks
									using (SqlCommand cmdRemark = new SqlCommand("App.usp_GetAuthorizationRemarks", conn))
									{
										cmdRemark.CommandType = System.Data.CommandType.StoredProcedure;
										cmdRemark.CommandTimeout = 300;
										cmdRemark.Parameters.AddWithValue("@FeeAuthorizationSID", Convert.ToInt64(reader["FeeAuthorizationSID"]));
										cmdRemark.Parameters.AddWithValue("@PatientSID", patientSID);
										cmdRemark.Parameters.AddWithValue("@Sta3n", Convert.ToInt64(reader["Sta3n"]));

										using (SqlDataReader readerRemark = cmdRemark.ExecuteReader())
										{
											while (readerRemark.Read())
											{
												sbRemark.AppendLine(readerRemark["FeeAuthoriationRemark"].ToString().Trim());
											}
										}
									}

									feeAuthorizations.Add(ParseFeeAuthorization(reader, sbRemark.ToString()));
								}
							}
						}
					}
				}

			}
			catch (Exception)
			{

			}


			return feeAuthorizations;
		}

		public FeeAuthorization ParseFeeAuthorization(SqlDataReader dr, string feeAuthorizationRemark)
		{
			FeeAuthorization feeAuthorization = new FeeAuthorization();
			feeAuthorization.FeeAuthorizationSID = Utils.GetLongValue(dr["FeeAuthorizationSID"]);
			feeAuthorization.FeeAuthorizationIEN = (dr["FeeAuthorizationIEN"] == null) ? "" : dr["FeeAuthorizationIEN"].ToString();
			feeAuthorization.PatientSID = Utils.GetIntValue(dr["PatientSID"]);
			feeAuthorization.PatientIEN = Utils.GetStringValue(dr["PatientIEN"]);
			feeAuthorization.FeeVendorSID = Utils.GetIntValue(dr["FeeVendorSID"]);
			feeAuthorization.Sta3n = Utils.GetIntValue(dr["Sta3n"]);
			feeAuthorization.FromDate = Utils.GetDateValue(dr["FromDate"]);
			feeAuthorization.ToDate = Utils.GetDateValue(dr["ToDate"]);
			feeAuthorization.AdmissionDate = Utils.GetDateValue(dr["AdmissionDate"]);
			feeAuthorization.FeeAuthorizationRemark = feeAuthorizationRemark;
			feeAuthorization.FeeAuthorizationStatus = Utils.GetStringValue(dr["FeeAuthorizationStatus"]);
			return feeAuthorization;
		}

		public Institution GetInstitutionById(int sta3n)
		{
			using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
			{
				conn.Open();

				using (SqlCommand cmd = new SqlCommand("App.usp_GetInstitutionById", conn))
				{
					cmd.CommandTimeout = 300;
					cmd.CommandType = System.Data.CommandType.StoredProcedure;
					cmd.Parameters.AddWithValue("@Sta3n", sta3n);
					cmd.Parameters.AddWithValue("@InstitutionCode", sta3n.ToString());
					using (SqlDataReader reader = cmd.ExecuteReader())
					{
						if (reader.Read())
						{
							return ParseInstitution(reader);
						}
					}
				}
			}

			return null;
		}

		public List<PatientVendor> GetPatientVendor(string patientId, DateTime? fromDate, DateTime? toDate, DateTime? admissionDateFrom, DateTime? admissionDateTo, string idNumber, string npi, string medicareIDNumber)
		{

			List<PatientVendor> patientVendors = new List<PatientVendor>();
			using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
			{
				conn.Open();
				
				using (SqlCommand cmd = new SqlCommand("App.usp_GetPatientVendor", conn))
				{
					cmd.CommandTimeout = 300;
					cmd.CommandType = System.Data.CommandType.StoredProcedure;
					cmd.Parameters.AddWithValue("@PatientId", patientId);
					cmd.Parameters.AddWithValue("@FromDate", fromDate);
					cmd.Parameters.AddWithValue("@ToDate", toDate);
					cmd.Parameters.AddWithValue("@AdmissionDateFrom", admissionDateFrom);
					cmd.Parameters.AddWithValue("@AdmissionDateTo", admissionDateTo);
					cmd.Parameters.AddWithValue("@IDNumber", idNumber);
					cmd.Parameters.AddWithValue("@NPI", npi);
					cmd.Parameters.AddWithValue("@MedicareIDNumber", medicareIDNumber);
					using (SqlDataReader reader = cmd.ExecuteReader())
					{
						while (reader.Read())
						{
							PatientVendor patientVendor = new PatientVendor();
							patientVendor.PatientSID = Utils.ParseInt(reader["PatientSID"].ToString());
							patientVendor.FeeVendorSID = Utils.ParseInt(reader["FeeVendorSID"].ToString());
							patientVendors.Add(patientVendor);
						}
					}
				}
			}

			return patientVendors;
		}

		public SPatient GetPatientBySID(int patientSID)
		{
			using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
			{
				conn.Open();

				using (SqlCommand cmd = new SqlCommand("App.usp_GetPatientBySID", conn))
				{
					cmd.CommandTimeout = 300;
					cmd.CommandType = System.Data.CommandType.StoredProcedure;
					cmd.Parameters.AddWithValue("@PatientSID", patientSID);

					using (SqlDataReader reader = cmd.ExecuteReader())
					{
						if (reader.Read())
						{
							return ParseSPatient(reader, conn);
						}
					}
				}
			}

			return null;
		}

		public SFeeVendor GetVendorBySID(int feeVendorSID)
		{
			using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
			{
				conn.Open();

				using (SqlCommand cmd = new SqlCommand("App.usp_GetVendorBySID", conn))
				{
					cmd.CommandTimeout = 300;
					cmd.CommandType = System.Data.CommandType.StoredProcedure;
					cmd.Parameters.AddWithValue("@FeeVendorSID", feeVendorSID);

					using (SqlDataReader reader = cmd.ExecuteReader())
					{
						if (reader.Read())
						{
							return ParseSFeeVendor(reader);
						}
					}
				}
			}

			return null;
		}

		public List<SPatient> GetPatient(string ssn)
		{

			List<SPatient> patients = new List<SPatient>();
			using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
			{
				conn.Open();

				using (SqlCommand cmd = new SqlCommand("App.usp_GetPatient", conn))
				{
					cmd.CommandTimeout = 300;
					cmd.CommandType = System.Data.CommandType.StoredProcedure;
					cmd.Parameters.AddWithValue("@PatientSSN", ssn);
					cmd.Parameters.AddWithValue("@Sta3n", 0);
					using (SqlDataReader reader = cmd.ExecuteReader())
					{
						while (reader.Read())
						{
							patients.Add(ParseSPatient(reader, conn));
						}
					}
				}
			}

			return patients;
		}

		public List<SFeeVendor> GetVendorByIDNumber(string idNumber, ref MessageBase messageBase)
		{
			return GetVendor("App.usp_GetVendorByIDNumber", "@IDNumber", idNumber, ref messageBase);
		}

		public List<SFeeVendor> GetVendorByMedicareIDNumber(string medicareIDNumber, ref MessageBase messageBase)
		{
			return GetVendor("App.usp_GetVendorByMedicareIDNumber", "@MedicareIDNumber", medicareIDNumber, ref messageBase);
		}

		public List<SFeeVendor> GetVendorByNPI(string npi, ref MessageBase messageBase)
		{
			return GetVendor("App.usp_GetVendorByNPI", "@NPI", npi, ref messageBase);
		}

		public List<SFeeVendor> GetVendor(string proc, string parameter, string idValue, ref MessageBase messageBase)
		{
			try
			{
				List<SFeeVendor> vendors = new List<SFeeVendor>();

				using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
				{
					conn.Open();

					using (SqlCommand cmd = new SqlCommand(proc, conn))
					{
						cmd.CommandTimeout = 300;
						cmd.CommandType = System.Data.CommandType.StoredProcedure;
						cmd.Parameters.AddWithValue(parameter, idValue);

						using (SqlDataReader reader = cmd.ExecuteReader())
						{
							while (reader.Read())
							{
								vendors.Add(ParseSFeeVendor(reader));
							}
						}
					}
				}

				return vendors;
			}
			catch (Exception ex)
			{
				messageBase.ValidConnection = false;
				Utils.LogEvent(string.Format("GetVendor(): {0}. ", ex.Message) + "\r\n" + ex.StackTrace);
			}
			return null;
		}

		public SPatient ParseSPatient(SqlDataReader dr, SqlConnection conn)
		{
			SPatient sPatient = new SPatient();

			// set country code, state 
			sPatient.State = "";
			sPatient.Country = "";
			using (SqlCommand cmd = new SqlCommand("App.usp_GetPatientCountryCode", conn))
			{
				cmd.CommandTimeout = 300;
				cmd.CommandType = System.Data.CommandType.StoredProcedure;
				cmd.Parameters.AddWithValue("@CountrySID", Utils.GetIntValue(dr["CountrySID"]));
				using (SqlDataReader reader = cmd.ExecuteReader())
				{
					if (reader.Read())
					{
						sPatient.Country = Utils.GetStringValue(reader["FIPSCode"]);
					}
				}
			}

			// if US, set state code
			if (sPatient.Country == "US")
			{
				sPatient.Country = "";
				using (SqlCommand cmd = new SqlCommand("App.usp_GetPatientStateCode", conn))
				{
					cmd.CommandType = System.Data.CommandType.StoredProcedure;
					cmd.Parameters.AddWithValue("@StateSID", Utils.GetIntValue(dr["StateSID"]));
					using (SqlDataReader reader = cmd.ExecuteReader())
					{
						if (reader.Read())
						{
							sPatient.State = Utils.GetStringValue(reader["StateAbbrev"]);
						}
					}
				}
			}


			sPatient.PatientSID = Utils.GetIntValue(dr["PatientSID"]);
			sPatient.Sta3n = Utils.GetIntValue(dr["Sta3n"]);
			sPatient.PatientFirstName = Utils.GetStringValue(dr["PatientFirstName"]);
			sPatient.PatientLastName = Utils.GetStringValue(dr["PatientLastName"]);
			sPatient.PatientSSN = Utils.GetStringValue(dr["PatientSSN"]);
			sPatient.StreetAddress1 = Utils.GetStringValue(dr["StreetAddress1"]);
			sPatient.StreetAddress2 = Utils.GetStringValue(dr["StreetAddress2"]);

			sPatient.City = Utils.GetStringValue(dr["City"]);


			string zip = Utils.GetStringValue(dr["Zip"]);
			if (zip.Length == 0)
			{
				zip = Utils.GetStringValue(dr["PostalCode"]);
			}
			sPatient.Zip = zip;

			sPatient.DateOfBirth = Utils.GetDateValue(dr["DateOfBirth"]);
			sPatient.Gender = Utils.GetStringValue(dr["Gender"]);


			return sPatient;
		}

		public SFeeVendor ParseSFeeVendor(SqlDataReader dr)
		{
			SFeeVendor sFeeVendor = new SFeeVendor();

			sFeeVendor.FeeVendorSID = Utils.GetIntValue(dr["FeeVendorSID"]);
			sFeeVendor.Sta3n = Utils.GetIntValue(dr["Sta3n"]);

			return sFeeVendor;
		}

		public Institution ParseInstitution(SqlDataReader dr)
		{
			Institution institution = new Institution();
			institution.Sta3n = Utils.GetIntValue(dr["Sta3n"]);
			institution.InstitutionName = Utils.GetStringValue(dr["InstitutionName"]);
			return institution;
		}


		public int SaveTransactionLog(TransactionLog transactionLog)
		{
			try
			{
				int id = 0;
				using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
				{
					conn.Open();

					string sql = "INSERT INTO App.TransactionLog(ResponseDate, Sta3n, QueueId, RequestStatus, EventTraceNumber, AdminReferenceNumber, FeeAuthorizationSIDs, VendorQualifier, FeeVendorID, PatientFullName, GroupControlNumber, LogEvent, ResponseMessageText, AuthorizationCount, UAK, Version, RequestMessageText, RequestDate, TransactionType) VALUES(";
					sql += "@ResponseDate, @Sta3n, @QueueId, @RequestStatus, @EventTraceNumber, @AdminReferenceNumber, @FeeAuthorizationSIDs, @VendorQualifier, @FeeVendorID, @PatientFullName, @GroupControlNumber, @LogEvent, @ResponseMessageText, @AuthorizationCount, @UAK, @Version, @RequestMessageText, @RequestDate, @TransactionType); SELECT CAST(scope_identity() AS int);";
					using (SqlCommand cmd = new SqlCommand(sql, conn))
					{
						cmd.CommandTimeout = 300;

						cmd.Parameters.AddWithValue("@ResponseDate", DateTime.Now);
						cmd.Parameters.AddWithValue("@Sta3n", Utils.GetIntValue(transactionLog.Sta3n));
						cmd.Parameters.AddWithValue("@QueueId", Utils.GetIntValue(transactionLog.QueueId));
						cmd.Parameters.AddWithValue("@RequestStatus", Utils.GetStringValue(transactionLog.RequestStatus));
						cmd.Parameters.AddWithValue("@EventTraceNumber", Utils.GetStringValue(transactionLog.EventTraceNumber));
						cmd.Parameters.AddWithValue("@AdminReferenceNumber", Utils.GetStringValue(transactionLog.AdminReferenceNumber));
						cmd.Parameters.AddWithValue("@FeeAuthorizationSIDs", Utils.GetStringValue(transactionLog.FeeAuthorizationSIDs));
						cmd.Parameters.AddWithValue("@VendorQualifier", Utils.GetStringValue(transactionLog.VendorQualifier));
						cmd.Parameters.AddWithValue("@FeeVendorID", Utils.GetStringValue(transactionLog.FeeVendorID));
						cmd.Parameters.AddWithValue("@PatientFullName", Utils.GetStringValue(transactionLog.PatientFullName));
						cmd.Parameters.AddWithValue("@GroupControlNumber", Utils.GetStringValue(transactionLog.GroupControlNumber));
						cmd.Parameters.AddWithValue("@LogEvent", Utils.GetStringValue(transactionLog.LogEvent));
						cmd.Parameters.AddWithValue("@ResponseMessageText", Utils.GetStringValue(transactionLog.ResponseMessageText));
						cmd.Parameters.AddWithValue("@AuthorizationCount", Utils.GetIntValue(transactionLog.AuthorizationCount));
						cmd.Parameters.AddWithValue("@UAK", Utils.GetStringValue(transactionLog.UAK));
						cmd.Parameters.AddWithValue("@Version", Globals.AppVersion);
						cmd.Parameters.AddWithValue("@RequestMessageText", Utils.GetStringValue(transactionLog.RequestMessageText));
						cmd.Parameters.AddWithValue("@RequestDate", transactionLog.RequestDate);
						cmd.Parameters.AddWithValue("@TransactionType", "");

						id = (int)cmd.ExecuteScalar();
					}
				}
				return id;
			}
			catch (Exception)
			{
				return 0;
			}

		}

		public void SaveMessageSegments(Settings settings, int requestId)
		{
			try
			{
				using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
				{
					conn.Open();

					foreach (var cmd in settings.SqlCommands)
					{
						cmd.Connection = conn;
						cmd.CommandText = cmd.CommandText.Replace("[RequestId]", requestId.ToString()).Replace("[IsResponse]", "0");

						cmd.ExecuteNonQuery();
					}


					foreach (var cmd in settings.SqlCommandsResponse)
					{
						cmd.Connection = conn;
						cmd.CommandText = cmd.CommandText.Replace("[RequestId]", requestId.ToString()).Replace("[IsResponse]", "1");

						cmd.ExecuteNonQuery();
					}


				}
			}
			catch (Exception ex)
			{
				settings.LogMessages.Append("Error SaveMessageSegments() - " + ex.Message);
			}

}


	}
}
