﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;
using X12Engine.BO.X12._278.IdentifierCodes;

namespace X12Engine.BLL.X12._278
{
	public static class AAAManager
	{
		public static AAA GetAAA(ref Settings settings, string loopCode, int loopNumber, string YesNoConditionOrResponseCode, string RejectReasonCode, string FollowUpActionCode, string ErrorMessage)
		{
			AAA aaa = new AAA(settings);
			aaa.YesNoConditionOrResponseCode = new IDENTIFIER(YesNoConditionOrResponseCode);
			aaa.RejectReasonCode = new IDENTIFIER(RejectReasonCode);
			aaa.FollowUpActionCode = new IDENTIFIER(FollowUpActionCode);

			settings.AAACount++;

			// Add SqlCommand
			string sql = "INSERT INTO App.AAA(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, YesNo, RejectCode, FollowUpCode) ";
			sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @YesNo, @RejectCode, @FollowUpCode)";
			SqlCommand cmd = new SqlCommand(sql);

			cmd.Parameters.AddWithValue("@LoopCode", loopCode);
			cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
			cmd.Parameters.AddWithValue("@YesNo", (YesNoConditionOrResponseCode != null) ? YesNoConditionOrResponseCode : "");
			cmd.Parameters.AddWithValue("@RejectCode", (RejectReasonCode != null) ? RejectReasonCode : "");
			cmd.Parameters.AddWithValue("@FollowUpCode", (FollowUpActionCode != null) ? FollowUpActionCode : "");

			settings.SqlCommandsResponse.Add(cmd);

			return aaa;
		}

	}

	public static class HLManager
	{
		public static HL Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			HL objReturn = new HL(settings);
			loopCode = loopCode.Replace("10","00");
			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.HierarchicalIdNumber = elements[1];
				if (elements.Length > 2) objReturn.HierarchicalParentIdNumber = elements[2];
				if (elements.Length > 3) objReturn.HierarchicalLevelCode = new IDENTIFIER(elements[3]);
				if (!CodeLookup.Validate(loopCode, "HL03", objReturn.HierarchicalLevelCode))
				{ 
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.HLManager - HL03 - Invalid HierarchicalLevelCode=" + objReturn.HierarchicalLevelCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 4) objReturn.HierarchicalChildCode = new IDENTIFIER(elements[4]);
				if (!CodeLookup.Validate(loopCode, "HL04", objReturn.HierarchicalChildCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.HLManager - HL04 - Invalid HierarchicalChildCode=" + objReturn.HierarchicalChildCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
			}

			return objReturn;
		}
	}

	public static class TRNManager
	{
		public static TRN Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			TRN objReturn = new TRN(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.TraceTypeCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "TRN01", objReturn.TraceTypeCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.TRNManager - TRN01 - Invalid TraceTypeCode=" + objReturn.TraceTypeCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2) objReturn.ReferenceIdentification1 = elements[2];
				if (string.IsNullOrEmpty(objReturn.ReferenceIdentification1))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.TRNManager - ReferenceIdentification1 is required   Loop = " + loopCode + "  Segment = " + segment));
				}
				if (elements.Length > 3) objReturn.OriginatingCompanyIdentifier = elements[3];
				if (string.IsNullOrEmpty(objReturn.OriginatingCompanyIdentifier))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.TRNManager - OriginatingCompanyIdentifier is required   Loop = " + loopCode + "  Segment = " + segment));
				}
				if (elements.Length > 4) objReturn.ReferenceIdentification2 = elements[4];
			}

			return objReturn;
		}
	}


	public static class UMManager
	{
		public static UM Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			UM objReturn = new UM(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.RequestCategoryCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "UM01", objReturn.RequestCategoryCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "33", "C", "CommonManager.UMManager - UM01 - Invalid RequestCategoryCode=" + objReturn.RequestCategoryCode + " Loop=" + loopCode + "  Segment=" + segment));
				}

				if (elements.Length > 2) objReturn.CertificationTypeCode = new IDENTIFIER(elements[2]);
				if (!CodeLookup.Validate(loopCode, "UM02", objReturn.CertificationTypeCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "33", "C", "CommonManager.UMManager - UM02 - Invalid CertificationTypeCode=" + objReturn.CertificationTypeCode + " Loop=" + loopCode + "  Segment=" + segment));
				}

				if (elements.Length > 3 && !string.IsNullOrEmpty(elements[3]))
				{
					objReturn.ServiceTypeCode = new IDENTIFIER(elements[3]);
					if (!CodeLookup.Validate(loopCode, "UM03", objReturn.ServiceTypeCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "33", "C", "CommonManager.UMManager - UM03 - Invalid ServiceTypeCode=" + objReturn.ServiceTypeCode + " Loop=" + loopCode + "  Segment=" + segment));
					}

					CodeLookup.ValidateRoute(ref settings, loopCode, "UM03", elements[3]);

				}

				if (elements.Length > 4 && !string.IsNullOrEmpty(elements[4])) {
					objReturn.HealthCareServiceLocationInformation = new HEALTH_CARE_SERVICE_LOCATION_INFORMATION(settings, elements[4]);
					CodeLookup.ValidateRoute(ref settings, loopCode, "UM04", elements[4]);
				}
				


				if (elements.Length > 5) objReturn.RelatedCausesInformation = new RELATED_CAUSES_INFORMATION(elements[5]);


				if (elements.Length > 6 && !string.IsNullOrEmpty(elements[6]))
				{
					objReturn.LevelOfServiceCode = new IDENTIFIER(elements[6]);
					if (!CodeLookup.Validate(loopCode, "UM06", objReturn.LevelOfServiceCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "33", "C", "CommonManager.UMManager - UM06 - Invalid LevelOfServiceCode=" + objReturn.LevelOfServiceCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 7 && !string.IsNullOrEmpty(elements[7]))
				{
					objReturn.CurrentHealthConditionCode = new IDENTIFIER(elements[7]);
					if (!CodeLookup.Validate(loopCode, "UM07", objReturn.CurrentHealthConditionCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "33", "C", "CommonManager.UMManager - UM07 - Invalid CurrentHealthConditionCode=" + objReturn.CurrentHealthConditionCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 8 && !string.IsNullOrEmpty(elements[8]))
				{
					objReturn.PrognosisCode = new IDENTIFIER(elements[8]);
					if (!CodeLookup.Validate(loopCode, "UM08", objReturn.PrognosisCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "33", "C", "CommonManager.UMManager - UM08 - Invalid PrognosisCode=" + objReturn.PrognosisCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 9 && !string.IsNullOrEmpty(elements[9]))
				{
					objReturn.ReleaseOfInformationCode = new IDENTIFIER(elements[9]);
					if (!CodeLookup.Validate(loopCode, "UM09", objReturn.ReleaseOfInformationCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "33", "C", "CommonManager.UMManager - UM09 - Invalid ReleaseOfInformationCode=" + objReturn.ReleaseOfInformationCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 10 && !string.IsNullOrEmpty(elements[10]))
				{
					objReturn.DelayReasonCode = new IDENTIFIER(elements[10]);
					if (!CodeLookup.Validate(loopCode, "UM10", objReturn.DelayReasonCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "33", "C", "CommonManager.UMManager - UM10 -Invalid DelayReasonCode=" + objReturn.DelayReasonCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


				// Add SqlCommand
				string sql = "INSERT INTO App.HealthServicesReview(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, RequestCategCode, CertificationType, ServicesTypeCode, Facility, FacilityQualifier, RelatedCauseCode1, RelatedCauseCode2, RelatedCauseCode3, StateProvCode, CountryCode, LevelOfServiceCode, CurrentHealthConditionCode, PrognosisCode, ReleaseOfInformationCode, DelayReasonCode) ";
				sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @RequestCategCode, @CertificationType, @ServicesTypeCode, @Facility, @FacilityQualifier, @RelatedCauseCode1, @RelatedCauseCode2, @RelatedCauseCode3, @StateProvCode, @CountryCode, @LevelOfServiceCode, @CurrentHealthConditionCode, @PrognosisCode, @ReleaseOfInformationCode, @DelayReasonCode)";
				SqlCommand cmd = new SqlCommand(sql);

				cmd.Parameters.AddWithValue("@LoopCode", loopCode);
				cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
				cmd.Parameters.AddWithValue("@RequestCategCode", (elements.Length > 1 && elements[1] != null) ? elements[1] : "");
				cmd.Parameters.AddWithValue("@CertificationType", (elements.Length > 2 && elements[2] != null) ? elements[2] : "");
				cmd.Parameters.AddWithValue("@ServicesTypeCode", (elements.Length > 3 && elements[3] != null) ? elements[3] : "");
				cmd.Parameters.AddWithValue("@Facility", (elements.Length > 4 && elements[4] != null) ? elements[4] : "");
				cmd.Parameters.AddWithValue("@FacilityQualifier", "");
				cmd.Parameters.AddWithValue("@RelatedCauseCode1", (elements.Length > 5 && elements[5] != null) ? elements[5] : "");
				cmd.Parameters.AddWithValue("@RelatedCauseCode2", "");
				cmd.Parameters.AddWithValue("@RelatedCauseCode3", "");
				cmd.Parameters.AddWithValue("@StateProvCode", "");
				cmd.Parameters.AddWithValue("@CountryCode", "");
				cmd.Parameters.AddWithValue("@LevelOfServiceCode", (elements.Length > 6 && elements[6] != null) ? elements[6] : "");
				cmd.Parameters.AddWithValue("@CurrentHealthConditionCode", (elements.Length > 7 && elements[7] != null) ? elements[7] : "");
				cmd.Parameters.AddWithValue("@PrognosisCode", (elements.Length > 8 && elements[8] != null) ? elements[8] : "");
				cmd.Parameters.AddWithValue("@ReleaseOfInformationCode", (elements.Length > 9 && elements[9] != null) ? elements[9] : "");
				cmd.Parameters.AddWithValue("@DelayReasonCode", (elements.Length > 10 && elements[10] != null) ? elements[10] : "");
				settings.SqlCommands.Add(cmd);

				settings.SqlCommandsResponse.Add(cmd);

			}

			return objReturn;
		}
	}

	public static class HCRManager
	{
		public static HCR Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			HCR objReturn = new HCR(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ActionCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "HCR01", objReturn.ActionCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.HCRManager - HCR01 - Invalid ActionCode=" + objReturn.ActionCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
			}

			return objReturn;
		}
	}

	public static class REFManager
	{
		public static REF Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			REF objReturn = new REF(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ReferenceIdentificationQualifier = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "REF01", objReturn.ReferenceIdentificationQualifier))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.REFManager - REF01 - Invalid ReferenceIdentificationQualifier=" + objReturn.ReferenceIdentificationQualifier + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2) objReturn.ReferenceIdentification = elements[2];
				if (string.IsNullOrEmpty(objReturn.ReferenceIdentification))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.REFManager - ReferenceIdentification is required    Loop = " + loopCode + "  Segment = " + segment));
				}
			}

			return objReturn;
		}
	}

	public static class DTPManager
	{
		public static DTP Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			DTP objReturn = new DTP(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.DateTimeQualifier = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "DTP01", objReturn.DateTimeQualifier))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.DTPManager - DTP01 - Invalid DateTimeQualifier=" + objReturn.DateTimeQualifier + " Loop=" + loopCode + "  Segment=" + segment));
				}

				if (elements.Length > 2)
				{
					objReturn.DateTimePeriodFormatQualifier = new IDENTIFIER(elements[2]);
					if (!CodeLookup.Validate(loopCode, "DTP02", objReturn.DateTimePeriodFormatQualifier))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.DTPManager - DTP02 - Invalid DateTimePeriodFormatQualifier=" + objReturn.DateTimePeriodFormatQualifier + " Loop=" + loopCode + "  Segment=" + segment));
					}
					else
					{
						if (elements.Length > 3)
						{
							objReturn.DateTimePeriod = elements[3];
						}
						else
						{
							AddDTPValidation(settings, loopCode, objReturn.DateTimeQualifier, segment, ref requestValidation);
						} 
					}
				}
				else
				{
					AddDTPValidation(settings, loopCode, objReturn.DateTimeQualifier, segment, ref requestValidation);
				}


				// Add SqlCommand
				string sql = "INSERT INTO App.DateInformation(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, DateTimeQualifier, DateTimePeriodFormat, DateTimePeriod) ";
				sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @DateTimeQualifier, @DateTimePeriodFormat, @DateTimePeriod)";
				SqlCommand cmd = new SqlCommand(sql);

				cmd.Parameters.AddWithValue("@LoopCode", loopCode);
				cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
				cmd.Parameters.AddWithValue("@DateTimeQualifier", (elements.Length > 1 && elements[1] != null) ? elements[1] : "");
				cmd.Parameters.AddWithValue("@DateTimePeriodFormat", (elements.Length > 2 && elements[2] != null) ? elements[2] : "");
				cmd.Parameters.AddWithValue("@DateTimePeriod", (elements.Length > 3 && elements[3] != null) ? elements[3] : "");

				settings.SqlCommands.Add(cmd);

				settings.SqlCommandsResponse.Add(cmd);
			}

			return objReturn;
		}

		private static void AddDTPValidation(Settings settings, string loopCode, IDENTIFIER dateTimeQualifier, string segment, ref List<AAA> requestValidation)
		{
			var dateTimeQualifier_ = (dateTimeQualifier != null) ? dateTimeQualifier.Code : "";
			switch (dateTimeQualifier_)
			{
				case "439":
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "AI", "C", "CommonManager.DTPManager - Invalid/Missing Accident Date\r\nSegment=" + segment));
					break;
				case "435":
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "AM", "C", "CommonManager.DTPManager - Invalid/Missing Admission Date\r\nSegment=" + segment));
					break;
				case "096":
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "AN", "C", "CommonManager.DTPManager - Invalid/Missing Discharge Date\r\nSegment=" + segment));
					break;
				case "AAH":
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "57", "C", "CommonManager.DTPManager - Invalid/Missing Date(s) of Servicex\r\nSegment=" + segment));
					break;
				default:
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.DTPManager - Invalid/Missing Date(s) of DateTimePeriodFormatQualifier\r\nSegment=" + segment));
					break;
			}
		}

	}

	public static class HIManager
	{
		public static HI Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			HI objReturn = new HI(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1)
				{
					settings.HIIndex++;

					

					switch (settings.HIIndex)
					{
						case 1:
							objReturn.HealthCareCodeInformation1 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 2:
							objReturn.HealthCareCodeInformation2 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 3:
							objReturn.HealthCareCodeInformation3 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 4:
							objReturn.HealthCareCodeInformation4 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 5:
							objReturn.HealthCareCodeInformation5 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 6:
							objReturn.HealthCareCodeInformation6 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 7:
							objReturn.HealthCareCodeInformation7 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 8:
							objReturn.HealthCareCodeInformation8 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 9:
							objReturn.HealthCareCodeInformation9 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 10:
							objReturn.HealthCareCodeInformation10 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 11:
							objReturn.HealthCareCodeInformation11 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;
						case 12:
							objReturn.HealthCareCodeInformation12 = new HEALTH_CARE_CODE_INFORMATION(settings, elements[1]);
							break;

						default:
							break;
					}
					

					string[] comps = elements[1].Split(settings.Component_Element_Separator);



					// validate

					if (comps.Length <= 1)
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "AF", "C", "CommonManager.HIManager - Invalid/Missing Diagnosis Code(s) - Industry Code required.    Loop = " + loopCode + "  Segment = " + segment));
					}

					if (comps.Length > 0)
					{
						CodeLookup.ValidateRoute(ref settings, loopCode, "HI0" + settings.HIIndex + "-2", comps[1]);
					}

					if (comps.Length > 2)
					{

						if (comps[2] != "D8")
						{
							requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "AF", "C", "CommonManager.HIManager - Invalid/Missing Diagnosis Code(s) - DateTimePeriodFormatQualifier should be D8    Loop = " + loopCode + "  Segment = " + segment));
						}
						else
						{
							if (comps.Length > 3)
							{
								string[] formats = { "yyyyMMdd" };
								DateTime dTemp = new DateTime();
								if (!DateTime.TryParseExact(comps[3], formats, new System.Globalization.CultureInfo("en-US"), System.Globalization.DateTimeStyles.None, out dTemp))
								{
									requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "AF", "C", "CommonManager.HIManager - Invalid/Missing Diagnosis Code(s) - Invalid date format yyyymmdd    Loop = " + loopCode + "  Segment = " + segment));
								}
							}
							else
							{
								requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "AF", "C", "CommonManager.HIManager - Invalid/Missing Diagnosis Code(s) - DateTimePeriod is missing    Loop = " + loopCode + "  Segment = " + segment));
							}
						}
					}

				}
				
			}

			return objReturn;
		}
	}

	public static class SV1Manager
	{
		public static SV1 Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			SV1 objReturn = new SV1(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				CodeLookup.ValidateRoute(ref settings, loopCode, "SV1", "");

				if (elements.Length > 1)
				{
					CodeLookup.ValidateRoute(ref settings, loopCode, "SV101", elements[1]);
					objReturn.CompositeMedicalProcedureIdentifier = new COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER(settings, elements[1]);
				}

				
				if (elements.Length > 2) objReturn.MonetaryAmount1 = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3 && !string.IsNullOrEmpty(elements[3]))
				{
					objReturn.UnitOrBasisForMeasurementCode = new IDENTIFIER(elements[3]);
					if (!CodeLookup.Validate(loopCode, "SV103", objReturn.UnitOrBasisForMeasurementCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.SV1Manager - SV103 - Invalid UnitOrBasisForMeasurementCode=" + objReturn.UnitOrBasisForMeasurementCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 4) objReturn.Quantity = Utils.ParseDecimal(elements[4]);


				if (elements.Length > 7) objReturn.CompositeDiagnosisCodePointer = new COMPOSITE_DIAGNOSIS_CODE_POINTER(settings, elements[7]);



				if (elements.Length > 11 && !string.IsNullOrEmpty(elements[11]))
				{
					objReturn.YesNoConditionOrResponseCode2 = new IDENTIFIER(elements[11]);
					if (!CodeLookup.Validate(loopCode, "SV111", objReturn.YesNoConditionOrResponseCode2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.SV1Manager - SV111 - Invalid YesNoConditionOrResponseCode2=" + objReturn.YesNoConditionOrResponseCode2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


				if (elements.Length > 20 && !string.IsNullOrEmpty(elements[20]))
				{
					objReturn.LevelOfCareCode = new IDENTIFIER(elements[20]);
					if (!CodeLookup.Validate(loopCode, "SV120", objReturn.LevelOfCareCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.SV1Manager - SV120 - Invalid LevelOfCareCode=" + objReturn.LevelOfCareCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


			}

			return objReturn;
		}
	}

	public static class SV2Manager
	{
		public static SV2 Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			SV2 objReturn = new SV2(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				CodeLookup.ValidateRoute(ref settings, loopCode, "SV2", "");

				if (elements.Length > 1)
				{
					CodeLookup.ValidateRoute(ref settings, loopCode, "SV201", elements[1]);
					objReturn.ProductServiceId = elements[1];
				}

				if (elements.Length > 2)
				{
					CodeLookup.ValidateRoute(ref settings, loopCode, "SV202", elements[2]);
					objReturn.CompositeMedicalProcedureIdentifier = new COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER(settings, elements[2]);
				}
				
				if (elements.Length > 3) objReturn.MonetaryAmount1 = Utils.ParseDecimal(elements[3]);
				if (elements.Length > 4 && !string.IsNullOrEmpty(elements[4]))
				{
					objReturn.UnitOrBasisForMeasurementCode = new IDENTIFIER(elements[4]);
					if (!CodeLookup.Validate(loopCode, "SV204", objReturn.UnitOrBasisForMeasurementCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.SV2Manager - SV204 - Invalid UnitOrBasisForMeasurementCode=" + objReturn.UnitOrBasisForMeasurementCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 5) objReturn.Quantity = Utils.ParseDecimal(elements[5]);
				if (elements.Length > 6) objReturn.UnitRate = Utils.ParseDecimal(elements[6]);


				if (elements.Length > 9 && !string.IsNullOrEmpty(elements[9]))
				{
					objReturn.NursingHomeResidentialStatusCode = new IDENTIFIER(elements[9]);
					if (!CodeLookup.Validate(loopCode, "SV209", objReturn.NursingHomeResidentialStatusCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.SV2Manager - SV209 - Invalid NursingHomeResidentialStatusCode=" + objReturn.NursingHomeResidentialStatusCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 10 && !string.IsNullOrEmpty(elements[10]))
				{
					objReturn.LevelOfCareCode = new IDENTIFIER(elements[10]);
					if (!CodeLookup.Validate(loopCode, "SV210", objReturn.LevelOfCareCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.SV2Manager - SV210 - Invalid LevelOfCareCode=" + objReturn.LevelOfCareCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

			}

			return objReturn;
		}
	}

	public static class SV3Manager
	{
		public static SV3 Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			SV3 objReturn = new SV3(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				CodeLookup.ValidateRoute(ref settings, loopCode, "SV3", "");

				if (elements.Length > 1)
				{
					CodeLookup.ValidateRoute(ref settings, loopCode, "SV301", elements[1]);
					objReturn.CompositeMedicalProcedureIdentifier = new COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER(settings, elements[1]);
				}
				
				if (elements.Length > 2) objReturn.MonetaryAmount = Utils.ParseDecimal(elements[2]);

				if (elements.Length > 4) objReturn.OralCavityDesignation = new ORAL_CAVITY_DESIGNATION(settings, elements[4]);
				if (elements.Length > 5 && !string.IsNullOrEmpty(elements[5]))
				{
					objReturn.ProsthesisCrownOrInlayCode = new IDENTIFIER(elements[5]);
					if (!CodeLookup.Validate(loopCode, "SV305", objReturn.ProsthesisCrownOrInlayCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.SV3Manager - SV305 - Invalid ProsthesisCrownOrInlayCode=" + objReturn.ProsthesisCrownOrInlayCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 6) objReturn.Quantity = Utils.ParseDecimal(elements[6]);
				if (elements.Length > 7) objReturn.Description = elements[7];
			}

			return objReturn;
		}
	}

	public static class TOOManager
	{
		public static TOO Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			TOO objReturn = new TOO(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.CodeListQualifierCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "TOO01", objReturn.CodeListQualifierCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.TOOManager - TOO01 - Invalid CodeListQualifierCode=" + objReturn.CodeListQualifierCode + "  Loop = " + loopCode + "  Segment = " + segment));
				}
				if (elements.Length > 2) objReturn.IndustryCode = elements[2];
				if (string.IsNullOrEmpty(objReturn.IndustryCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.TOOManager - IndustryCode is reqired  Loop = " + loopCode + "  Segment = " + segment));
				}
				if (elements.Length > 3) objReturn.ToothSurface = new TOOTH_SURFACE(settings, elements[3]);
			}

			return objReturn;
		}
	}

	public static class HSDManager
	{
		public static HSD Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			HSD objReturn = new HSD(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1 && !string.IsNullOrEmpty(elements[1]))
				{
					objReturn.QuantityQualifier = new IDENTIFIER(elements[1]);
					if (!CodeLookup.Validate(loopCode, "HSD01", objReturn.QuantityQualifier))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.HSDManager - HSD01 - Invalid QuantityQualifier([1])=" + objReturn.QuantityQualifier + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 2) objReturn.Quantity = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3 && !string.IsNullOrEmpty(elements[3]))
				{
					objReturn.UnitOrBasisForMeasurementCode = new IDENTIFIER(elements[3]);
					if (!CodeLookup.Validate(loopCode, "HSD03", objReturn.UnitOrBasisForMeasurementCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.HSDManager - HSD03 - Invalid UnitOrBasisForMeasurementCode([3])=" + objReturn.UnitOrBasisForMeasurementCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 4) objReturn.SampleSelectionModulus = Utils.ParseDecimal(elements[4]);

				if (elements.Length > 5 && !string.IsNullOrEmpty(elements[5]))
				{
					objReturn.TimePeriodQualifier = new IDENTIFIER(elements[5]);
					if (!CodeLookup.Validate(loopCode, "HSD05", objReturn.TimePeriodQualifier))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.HSDManager - HSD05 - Invalid TimePeriodQualifier([5])=" + objReturn.TimePeriodQualifier + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 6) objReturn.NumberOfPeriods = Utils.ParseInt(elements[6]);

				if (elements.Length > 7 && !string.IsNullOrEmpty(elements[7]))
				{
					objReturn.ShipDeliveryOrCalendarPatternCode = new IDENTIFIER(elements[7]);
					if (!CodeLookup.Validate(loopCode, "HSD07", objReturn.ShipDeliveryOrCalendarPatternCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.HSDManager - HSD07 - Invalid ShipDeliveryOrCalendarPatternCode([7])=" + objReturn.ShipDeliveryOrCalendarPatternCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 8 && !string.IsNullOrEmpty(elements[8]))
				{
					objReturn.ShipDeliveryPatternTimeCode = new IDENTIFIER(elements[8]);
					if (!CodeLookup.Validate(loopCode, "HSD08", objReturn.ShipDeliveryPatternTimeCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.HSDManager - HSD08 - Invalid ShipDeliveryPatternTimeCode([8])=" + objReturn.ShipDeliveryPatternTimeCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

			}

			return objReturn;
		}
	}

	public static class CRCManager
	{
		public static CRC Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			CRC objReturn = new CRC(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.CodeCategory = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "CRC01", objReturn.CodeCategory))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CRCManager - CRC01 - Invalid CodeCategory=" + objReturn.CodeCategory + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2) objReturn.YesNoConditionOrResponseCode = new IDENTIFIER(elements[2]);
				if (!CodeLookup.Validate(loopCode, "CRC02", objReturn.YesNoConditionOrResponseCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CRCManager - CRC02 - Invalid YesNoConditionOrResponseCode=" + objReturn.YesNoConditionOrResponseCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 3) objReturn.ConditionIndicator1 = new IDENTIFIER(elements[3]);
				if (!CodeLookup.Validate(loopCode, "CRC03", objReturn.ConditionIndicator1))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CRCManager - CRC03 - Invalid ConditionIndicator1=" + objReturn.ConditionIndicator1 + " Loop=" + loopCode + "  Segment=" + segment));
				}

				if (elements.Length > 4 && !string.IsNullOrEmpty(elements[4]))
				{
					objReturn.ConditionIndicator2 = new IDENTIFIER(elements[4]);
					if (!CodeLookup.Validate(loopCode, "CRC04", objReturn.ConditionIndicator2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CRCManager - CRC04 - Invalid ConditionIndicator2=" + objReturn.ConditionIndicator2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 5 && !string.IsNullOrEmpty(elements[5]))
				{
					objReturn.ConditionIndicator3 = new IDENTIFIER(elements[5]);
					if (!CodeLookup.Validate(loopCode, "CRC05", objReturn.ConditionIndicator3))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CRCManager - CRC05 - Invalid ConditionIndicator3=" + objReturn.ConditionIndicator3 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 6 && !string.IsNullOrEmpty(elements[6]))
				{
					objReturn.ConditionIndicator4 = new IDENTIFIER(elements[6]);
					if (!CodeLookup.Validate(loopCode, "CRC06", objReturn.ConditionIndicator4))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CRCManager - CRC06 - Invalid ConditionIndicator4=" + objReturn.ConditionIndicator4 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


				if (elements.Length > 7 && !string.IsNullOrEmpty(elements[7]))
				{
					objReturn.ConditionIndicator5 = new IDENTIFIER(elements[7]);
					if (!CodeLookup.Validate(loopCode, "CRC07", objReturn.ConditionIndicator5))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CRCManager - CRC07 - Invalid ConditionIndicator5=" + objReturn.ConditionIndicator5 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

			}

			return objReturn;
		}
	}

	public static class CL1Manager
	{
		public static CL1 Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			CL1 objReturn = new CL1(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1 && !string.IsNullOrEmpty(elements[1]))
				{
					objReturn.AdmissionTypeCode = new IDENTIFIER(elements[1]);
					if (!CodeLookup.Validate(loopCode, "CL101", objReturn.AdmissionTypeCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CL1Manager - CL101 - Invalid AdmissionTypeCode=" + objReturn.AdmissionTypeCode + " Loop=" + loopCode + "  Segment=" + segment));
					}

					CodeLookup.ValidateRoute(ref settings, loopCode, "CL101", elements[1]);
				}

				if (elements.Length > 2 && !string.IsNullOrEmpty(elements[2]))
				{
					objReturn.AdmissionSourceCode = new IDENTIFIER(elements[2]);
					if (!CodeLookup.Validate(loopCode, "CL102", objReturn.AdmissionSourceCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CL1Manager - CL102 - Invalid AdmissionSourceCode=" + objReturn.AdmissionSourceCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
					CodeLookup.ValidateRoute(ref settings, loopCode, "CL102", elements[2]);
				}

				if (elements.Length > 3 && !string.IsNullOrEmpty(elements[3]))
				{
					objReturn.PatientStatusCode = new IDENTIFIER(elements[3]);
					if (!CodeLookup.Validate(loopCode, "CL103", objReturn.PatientStatusCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CL1Manager - CL103 - Invalid PatientStatusCode=" + objReturn.PatientStatusCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 4 && !string.IsNullOrEmpty(elements[4]))
				{
					objReturn.NursingHomeResidentialStatusCode = new IDENTIFIER(elements[4]);
					if (!CodeLookup.Validate(loopCode, "CL104", objReturn.NursingHomeResidentialStatusCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CL1Manager - CL104 - Invalid NursingHomeResidentialStatusCode=" + objReturn.NursingHomeResidentialStatusCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}




				// Add SqlCommand
				string sql = "INSERT INTO App.InstitutionClaim(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, AddmissionTypeCode, AddmissionSourceCode, PatientStatusCode, ResidentialStatusCode) ";
				sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @AddmissionTypeCode, @AddmissionSourceCode, @PatientStatusCode, @ResidentialStatusCode)";
				SqlCommand cmd = new SqlCommand(sql);

				cmd.Parameters.AddWithValue("@LoopCode", loopCode);
				cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
				cmd.Parameters.AddWithValue("@AddmissionTypeCode", (elements.Length > 1 && elements[1] != null) ? elements[1] : "");
				cmd.Parameters.AddWithValue("@AddmissionSourceCode", (elements.Length > 2 && elements[2] != null) ? elements[2] : "");
				cmd.Parameters.AddWithValue("@PatientStatusCode", (elements.Length > 3 && elements[3] != null) ? elements[3] : "");
				cmd.Parameters.AddWithValue("@ResidentialStatusCode", (elements.Length > 4 && elements[4] != null) ? elements[4] : "");

				settings.SqlCommands.Add(cmd);

				settings.SqlCommandsResponse.Add(cmd);
			}

			return objReturn;
		}
	}

	public static class CR1Manager
	{
		public static CR1 Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			CR1 objReturn = new CR1(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1 && !string.IsNullOrEmpty(elements[1]))
				{
					objReturn.UnitOrBasisForMeasurementCode1 = new IDENTIFIER(elements[1]);
					if (!CodeLookup.Validate(loopCode, "CR101", objReturn.UnitOrBasisForMeasurementCode1))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR1Manager - CR101 - Invalid UnitOrBasisForMeasurementCode1=" + objReturn.UnitOrBasisForMeasurementCode1 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 2) objReturn.Weight = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3) objReturn.AmbulanceTransportCode = new IDENTIFIER(elements[3]);
				if (!CodeLookup.Validate(loopCode, "CR103", objReturn.AmbulanceTransportCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR1Manager - CR103 - Invalid AmbulanceTransportCode=" + objReturn.AmbulanceTransportCode + " Loop=" + loopCode + "  Segment=" + segment));
				}


				if (elements.Length > 4 && !string.IsNullOrEmpty(elements[4]))
				{
					objReturn.AmbulanceTransportReasonCode = new IDENTIFIER(elements[4]);
					if (!CodeLookup.Validate(loopCode, "CR104", objReturn.AmbulanceTransportReasonCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR1Manager - CR104 - Invalid AmbulanceTransportReasonCode=" + objReturn.AmbulanceTransportReasonCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 5 && !string.IsNullOrEmpty(elements[5]))
				{
					objReturn.UnitOrBasisForMeasurementCode2 = new IDENTIFIER(elements[5]);
					if (!CodeLookup.Validate(loopCode, "CR105", objReturn.UnitOrBasisForMeasurementCode2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR1Manager - CR105 - Invalid UnitOrBasisForMeasurementCode2=" + objReturn.UnitOrBasisForMeasurementCode2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 6) objReturn.Quantity = Utils.ParseDecimal(elements[6]);

				if (elements.Length > 9) objReturn.Description1 = elements[9];
				if (elements.Length > 10) objReturn.Description2 = elements[10];
			}

			return objReturn;
		}
	}

	public static class CR2Manager
	{
		public static CR2 Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			CR2 objReturn = new CR2(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.Count = Utils.ParseInt(elements[1]);
				if (elements.Length > 2) objReturn.Quantity1 = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3 && !string.IsNullOrEmpty(elements[3]))
				{
					objReturn.SubluxationLevelCode1 = new IDENTIFIER(elements[3]);
					if (!CodeLookup.Validate(loopCode, "CR203", objReturn.SubluxationLevelCode1))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR2Manager - CR203 - Invalid SubluxationLevelCode1=" + objReturn.SubluxationLevelCode1 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


				if (elements.Length > 4 && !string.IsNullOrEmpty(elements[4]))
				{
					objReturn.SubluxationLevelCode2 = new IDENTIFIER(elements[4]);
					if (!CodeLookup.Validate(loopCode, "CR204", objReturn.SubluxationLevelCode2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR2Manager - CR204 - Invalid SubluxationLevelCode2=" + objReturn.SubluxationLevelCode2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}



				if (elements.Length > 6) objReturn.Quantity2 = Utils.ParseDecimal(elements[6]);
				if (elements.Length > 7) objReturn.Quantity3 = Utils.ParseDecimal(elements[7]);
				if (elements.Length > 8) objReturn.NatureOfConditionCode = new IDENTIFIER(elements[8]);
				if (!CodeLookup.Validate(loopCode, "CR208", objReturn.NatureOfConditionCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR2Manager - CR208 - Invalid NatureOfConditionCode=" + objReturn.NatureOfConditionCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 9) objReturn.YesNoConditionOrResponseCode1 = new IDENTIFIER(elements[9]);
				if (!CodeLookup.Validate(loopCode, "CR209", objReturn.YesNoConditionOrResponseCode1))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR2Manager - CR209 - Invalid YesNoConditionOrResponseCode1=" + objReturn.YesNoConditionOrResponseCode1 + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 10) objReturn.Description1 = elements[10];
				if (elements.Length > 11) objReturn.Description2 = elements[11];
				if (elements.Length > 12 && !string.IsNullOrEmpty(elements[12]))
				{
					objReturn.YesNoConditionOrResponseCode2 = new IDENTIFIER(elements[12]);
					if (!CodeLookup.Validate(loopCode, "CR212", objReturn.YesNoConditionOrResponseCode2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR2Manager - CR212 - Invalid YesNoConditionOrResponseCode2=" + objReturn.YesNoConditionOrResponseCode2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


			}

			return objReturn;
		}
	}

	public static class CR4Manager
	{
		public static CR4 Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			CR4 objReturn = new CR4(settings);

			objReturn.Segment = segment;

			return objReturn;
		}
	}

	public static class CR5Manager
	{
		public static CR5 Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			CR5 objReturn = new CR5(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1 && !string.IsNullOrEmpty(elements[1]))
				{
					objReturn.CertificationTypeCode = new IDENTIFIER(elements[1]);
					if (!CodeLookup.Validate(loopCode, "CR501", objReturn.CertificationTypeCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR501 - Invalid CertificationTypeCode=" + objReturn.CertificationTypeCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


				if (elements.Length > 2) objReturn.Quantity1 = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3) objReturn.OxygenEquipmentTypeCode1 = new IDENTIFIER(elements[3]);
				if (!CodeLookup.Validate(loopCode, "CR503", objReturn.OxygenEquipmentTypeCode1))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR503 - Invalid OxygenEquipmentTypeCode1=" + objReturn.OxygenEquipmentTypeCode1 + " Loop=" + loopCode + "  Segment=" + segment));
				}

				if (elements.Length > 4 && !string.IsNullOrEmpty(elements[4]))
				{
					objReturn.OxygenEquipmentTypeCode2 = new IDENTIFIER(elements[4]);
					if (!CodeLookup.Validate(loopCode, "CR504", objReturn.OxygenEquipmentTypeCode2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR504 - Invalid OxygenEquipmentTypeCode2=" + objReturn.OxygenEquipmentTypeCode2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


				if (elements.Length > 5) objReturn.Description1 = elements[5];
				if (elements.Length > 6) objReturn.Quantity2 = Utils.ParseDecimal(elements[6]);
				if (elements.Length > 7) objReturn.Quantity3 = Utils.ParseDecimal(elements[7]);
				if (elements.Length > 8) objReturn.Quantity4 = Utils.ParseDecimal(elements[8]);
				if (elements.Length > 9) objReturn.Description2 = elements[9];
				if (elements.Length > 10) objReturn.Quantity5 = Utils.ParseDecimal(elements[10]);
				if (elements.Length > 11) objReturn.Quantity6 = Utils.ParseDecimal(elements[11]);
				if (elements.Length > 12 && !string.IsNullOrEmpty(elements[12]))
				{
					objReturn.OxygenTestConditionCode1 = new IDENTIFIER(elements[12]);
					if (!CodeLookup.Validate(loopCode, "CR512", objReturn.OxygenTestConditionCode1))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR512 - Invalid OxygenTestConditionCode1=" + objReturn.OxygenTestConditionCode1 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 13 && !string.IsNullOrEmpty(elements[13]))
				{
					objReturn.OxygenTestConditionCode2 = new IDENTIFIER(elements[13]);
					if (!CodeLookup.Validate(loopCode, "CR513", objReturn.OxygenTestConditionCode2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR513 - Invalid OxygenTestConditionCode2=" + objReturn.OxygenTestConditionCode2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 14 && !string.IsNullOrEmpty(elements[14]))
				{
					objReturn.OxygenTestConditionCode3 = new IDENTIFIER(elements[14]);
					if (!CodeLookup.Validate(loopCode, "CR514", objReturn.OxygenTestConditionCode3))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR514 - Invalid OxygenTestConditionCode3=" + objReturn.OxygenTestConditionCode3 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 15 && !string.IsNullOrEmpty(elements[15]))
				{
					objReturn.OxygenTestConditionCode4 = new IDENTIFIER(elements[15]);
					if (!CodeLookup.Validate(loopCode, "CR515", objReturn.OxygenTestConditionCode4))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR515 - Invalid OxygenTestConditionCode4=" + objReturn.OxygenTestConditionCode4 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 16) objReturn.Quantity7 = Utils.ParseDecimal(elements[16]);
				if (elements.Length > 17) objReturn.OxygenDeliverySystemCode = new IDENTIFIER(elements[17]);
				if (!CodeLookup.Validate(loopCode, "CR517", objReturn.OxygenDeliverySystemCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR517 - Invalid OxygenDeliverySystemCode=" + objReturn.OxygenDeliverySystemCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 18 && !string.IsNullOrEmpty(elements[18]))
				{
					objReturn.OxygenEquipmentTypeCode3 = new IDENTIFIER(elements[18]);
					if (!CodeLookup.Validate(loopCode, "CR518", objReturn.OxygenEquipmentTypeCode3))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR5Manager - CR518 - Invalid OxygenEquipmentTypeCode3=" + objReturn.OxygenEquipmentTypeCode3 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

			}

			return objReturn;
		}
	}

	public static class CR6Manager
	{
		public static CR6 Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			CR6 objReturn = new CR6(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.PrognosisCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "CR601", objReturn.PrognosisCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR6Manager - CR601 - Invalid PrognosisCode=" + objReturn.PrognosisCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2 && !string.IsNullOrEmpty(elements[2]))
				{
					objReturn.Date1 = Utils.ParseDate(elements[2], "yyyyMMdd");
					objReturn.Date1String = elements[2];
				}
				if (elements.Length > 3 && !string.IsNullOrEmpty(elements[3]))
				{
					objReturn.DateTimePeriodFormatQualifier1 = new IDENTIFIER(elements[3]);
					if (!CodeLookup.Validate(loopCode, "CR603", objReturn.DateTimePeriodFormatQualifier1))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR6Manager - CR603 - Invalid DateTimePeriodFormatQualifier1=" + objReturn.DateTimePeriodFormatQualifier1 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 4) objReturn.DateTimePeriod1 = elements[4];
				if (elements.Length > 5 && !string.IsNullOrEmpty(elements[5]))
				{
					objReturn.Date2 = Utils.ParseDate(elements[5], "yyyyMMdd");
					objReturn.Date2String = elements[5];
				}
				if (elements.Length > 6 && !string.IsNullOrEmpty(elements[6]))
				{
					objReturn.YesNoConditionOrResponseCode1 = new IDENTIFIER(elements[6]);
					if (!CodeLookup.Validate(loopCode, "CR606", objReturn.YesNoConditionOrResponseCode1))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR6Manager - CR606 - Invalid YesNoConditionOrResponseCode1=" + objReturn.YesNoConditionOrResponseCode1 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 7) objReturn.YesNoConditionOrResponseCode2 = new IDENTIFIER(elements[7]);
				if (!CodeLookup.Validate(loopCode, "CR607", objReturn.YesNoConditionOrResponseCode2))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR6Manager - CR607 - Invalid YesNoConditionOrResponseCode2=" + objReturn.YesNoConditionOrResponseCode2 + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 8) objReturn.CertificationTypeCode = new IDENTIFIER(elements[8]);
				if (!CodeLookup.Validate(loopCode, "CR608", objReturn.CertificationTypeCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR6Manager - CR608 - Invalid CertificationTypeCode=" + objReturn.CertificationTypeCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 9 && !string.IsNullOrEmpty(elements[9]))
				{
					objReturn.Date3 = Utils.ParseDate(elements[9], "yyyyMMdd");
					objReturn.Date3String = elements[9];
				}
				if (elements.Length > 10 && !string.IsNullOrEmpty(elements[10]))
				{
					objReturn.ProductServiceIdQualifier = new IDENTIFIER(elements[10]);
					if (!CodeLookup.Validate(loopCode, "CR610", objReturn.ProductServiceIdQualifier))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR6Manager - CR610 - Invalid ProductServiceIdQualifier=" + objReturn.ProductServiceIdQualifier + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 11) objReturn.MedicalCodeValue = elements[11];
				if (elements.Length > 12 && !string.IsNullOrEmpty(elements[12]))
				{
					objReturn.Date4 = Utils.ParseDate(elements[12], "yyyyMMdd");
					objReturn.Date4String = elements[12];
				}
				if (elements.Length > 13 && !string.IsNullOrEmpty(elements[13]))
				{
					objReturn.Date5 = Utils.ParseDate(elements[13], "yyyyMMdd");
					objReturn.Date5String = elements[13];
				}
				if (elements.Length > 14 && !string.IsNullOrEmpty(elements[14]))
				{
					objReturn.Date6 = Utils.ParseDate(elements[14], "yyyyMMdd");
					objReturn.Date6String = elements[14];
				}
				if (elements.Length > 15 && !string.IsNullOrEmpty(elements[15]))
				{
					objReturn.DateTimePeriodFormatQualifier2 = new IDENTIFIER(elements[15]);
					if (!CodeLookup.Validate(loopCode, "CR615", objReturn.DateTimePeriodFormatQualifier2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR6Manager - CR615 - Invalid DateTimePeriodFormatQualifier2=" + objReturn.DateTimePeriodFormatQualifier2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}


				if (elements.Length > 16) objReturn.DateTimePeriod2 = elements[16];
				if (elements.Length > 17 && !string.IsNullOrEmpty(elements[17]))
				{
					objReturn.PatientLocationCode = new IDENTIFIER(elements[17]);
					if (!CodeLookup.Validate(loopCode, "CR617", objReturn.PatientLocationCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.CR6Manager - CR617 - Invalid PatientLocationCode=" + objReturn.PatientLocationCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

			}

			return objReturn;
		}
	}

	public static class CR7Manager
	{
		public static CR7 Parse(ref Settings settings, string segment, ref List<AAA> requestValidation)
		{
			CR7 objReturn = new CR7(settings);

			objReturn.Segment = segment;

			return objReturn;
		}
	}

	public static class CR8Manager
	{
		public static CR8 Parse(ref Settings settings, string segment, ref List<AAA> requestValidation)
		{
			CR8 objReturn = new CR8(settings);

			objReturn.Segment = segment;

			return objReturn;
		}
	}

	public static class PWKManager
	{
		public static PWK Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			PWK objReturn = new PWK(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ReportTypeCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "PWK01", objReturn.ReportTypeCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PWKManager - PWK01 - Invalid ReportTypeCode=" + objReturn.ReportTypeCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2) objReturn.ReportTransmissionCode = new IDENTIFIER(elements[2]);
				if (!CodeLookup.Validate(loopCode, "PWK02", objReturn.ReportTransmissionCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PWKManager - PWK02 - Invalid ReportTransmissionCode=" + objReturn.ReportTransmissionCode + " Loop=" + loopCode + "  Segment=" + segment));
				}

				if (elements.Length > 4 && !string.IsNullOrEmpty(elements[4]))
				{
					objReturn.EntityIdentifierCode = new IDENTIFIER(elements[4]);
					if (!CodeLookup.Validate(loopCode, "PWK04", objReturn.EntityIdentifierCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PWKManager - PWK04 - Invalid EntityIdentifierCode=" + objReturn.EntityIdentifierCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 5 && !string.IsNullOrEmpty(elements[5]))
				{
					objReturn.IdentificationCodeQualifier = new IDENTIFIER(elements[5]);
					if (!CodeLookup.Validate(loopCode, "PWK05", objReturn.IdentificationCodeQualifier))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PWKManager - PWK05 - Invalid IdentificationCodeQualifier=" + objReturn.IdentificationCodeQualifier + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 6) objReturn.IdentificationCode = elements[6];
				if (elements.Length > 7) objReturn.Description = elements[7];



				// Add SqlCommand
				string sql = "INSERT INTO App.PatientInformation(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, ReportTypeCode, ReportTransCode, ReportIdCodeQualifier, Identifier, Description, IDCodeQualifier) ";
				sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @ReportTypeCode, @ReportTransCode, @ReportIdCodeQualifier, @Identifier, @Description, '')";
				SqlCommand cmd = new SqlCommand(sql);

				cmd.Parameters.AddWithValue("@LoopCode", loopCode);
				cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
				cmd.Parameters.AddWithValue("@ReportTypeCode", (elements.Length > 1 && elements[1] != null) ? elements[1] : "");
				cmd.Parameters.AddWithValue("@ReportTransCode", (elements.Length > 2 && elements[2] != null) ? elements[2] : "");
				cmd.Parameters.AddWithValue("@ReportIdCodeQualifier", (elements.Length > 5 && elements[5] != null) ? elements[5] : "");
				cmd.Parameters.AddWithValue("@Identifier", (elements.Length > 6 && elements[6] != null) ? elements[6] : "");
				cmd.Parameters.AddWithValue("@Description", (elements.Length > 7 && elements[7] != null) ? elements[7] : "");
				//cmd.Parameters.AddWithValue("@IDCodeQualifier", (elements[6] != null) ? elements[6] : "");
				settings.SqlCommands.Add(cmd);

				settings.SqlCommandsResponse.Add(cmd);

			}

			return objReturn;
		}
	}

	public static class MSGManager
	{
		public static MSG Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			MSG objReturn = new MSG(settings,"");

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.FreeFormMessageText = elements[1];
				if (string.IsNullOrEmpty(objReturn.FreeFormMessageText))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, loopNumber, "N", "15", "C", "CommonManager.MSGManager - FreeFormMessageText is empty\r\nSegment=" + segment));
				}

				if (elements.Length > 2) objReturn.PrinterCarriageControlCode = new IDENTIFIER(elements[2]);

				if (elements.Length > 3) objReturn.Number = Utils.ParseInt(elements[3]);
			}

			return objReturn;
		}
	}

	public static class NM1Manager
	{
		public static NM1 Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			NM1 objReturn = new NM1(settings);
 
			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.EntityIdentifierCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "NM101", objReturn.EntityIdentifierCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.NM1Manager - NM101 - Invalid EntityIdentifierCode=" + objReturn.EntityIdentifierCode + " Loop = " + loopCode + "  Segment = " + segment));
				}
				if (elements.Length > 2) objReturn.EntityTypeQualifier = new IDENTIFIER(elements[2]);
				if (!CodeLookup.Validate(loopCode, "NM102", objReturn.EntityTypeQualifier))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.NM1Manager - NM102 - Invalid EntityTypeQualifier=" + objReturn.EntityTypeQualifier + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 3) objReturn.NameLastOrOrganizationName = elements[3];
				if (elements.Length > 4) objReturn.NameFirst = elements[4];
				if (elements.Length > 5) objReturn.NameMiddle = elements[5];
				if (elements.Length > 6) objReturn.NamePrefix = elements[6];
				if (elements.Length > 7) objReturn.NameSuffix = elements[7];
				if (elements.Length > 8) objReturn.IdentificationCodeQualifier = new IDENTIFIER(elements[8]);
				if (!CodeLookup.Validate(loopCode, "NM108", objReturn.IdentificationCodeQualifier))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.NM1Manager - NM108 - Invalid IdentificationCodeQualifier=" + objReturn.IdentificationCodeQualifier + " - Segment=" + segment));
				}
				if (elements.Length > 9) objReturn.IdentificationCode = elements[9];


				// Add to segment table
				string sql = "INSERT INTO App.EntityName(RequestId,IsResponse,LoopCode,LoopNumber,SequenceNumber,EntityIdCode,EntityType,NameLastOrganization,NameFirst,EntityIdCodeQualifier,EntityIdentificationCode" + ((loopCode == "2010C") ? ",DateOfBirth,Gender" : "") + ") ";
				sql += "VALUES([RequestId],[IsResponse],@LoopCode,@LoopNumber,1,@EntityIdCode,1,@NameLastOrganization,@NameFirst,'34',@EntityIdentificationCode" + ((loopCode == "2010C") ? ",[DateOfBirth],'[Gender]'" : "") + ")";
				SqlCommand cmd = new SqlCommand(sql);

				cmd.Parameters.AddWithValue("@LoopCode", loopCode);
				cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
				cmd.Parameters.AddWithValue("@EntityIdCode", (elements.Length > 1 && elements[1] != null) ? elements[1] : "");
				
				cmd.Parameters.AddWithValue("@NameLastOrganization", (elements.Length > 3 && elements[3] != null) ? elements[3] : "");
				cmd.Parameters.AddWithValue("@NameFirst", (elements.Length > 4 && elements[4] != null) ? elements[4] : "");
				cmd.Parameters.AddWithValue("@EntityIdCodeQualifier", (elements.Length > 5 && elements[8] != null) ? elements[8] : "");
				cmd.Parameters.AddWithValue("@EntityIdentificationCode", (elements.Length > 9 && elements[9] != null) ? elements[9] : "");
				settings.SqlCommands.Add(cmd);

				settings.SqlCommandsResponse.Add(cmd);
			}

			return objReturn;
		}
	}

	public static class N2Manager
	{
		public static N2 Parse(ref Settings settings, string segment, ref List<AAA> requestValidation)
		{
			N2 objReturn = new N2(settings);

			objReturn.Segment = segment;

			return objReturn;
		}
	}

	public static class N3Manager
	{
		public static N3 Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			N3 objReturn = new N3(settings);


			string addressInformation1 = "";
			string addressInformation2 = "";

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) addressInformation1 = elements[1];
				if (elements.Length > 2) addressInformation2 = elements[2];
			}

			objReturn.AddressInformation1 = addressInformation1;
			objReturn.AddressInformation2 = addressInformation2;


			// add to sqlCommand
			string sql = "IF NOT EXISTS (SELECT 1 FROM App.EntityAddress WHERE RequestId = [RequestId] AND LoopCode = '" + loopCode + "' AND LoopNumber = " + loopNumber + ") ";
			sql += "INSERT INTO App.EntityAddress(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, Address1, Address2, City, StateProvCode, PostalCode, CountryCode, CountrySubCode) ";
			sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @Address1, @Address2, '', '', '', '', ''); ";
			sql += "UPDATE App.EntityAddress SET Address1 = @Address1_Update, Address2 = @Address2_Update WHERE RequestId = [RequestId] AND LoopCode = '" + loopCode + "' AND LoopNumber = " + loopNumber ;

			SqlCommand cmd = new SqlCommand(sql);

			cmd.Parameters.AddWithValue("@LoopCode", loopCode);
			cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
			cmd.Parameters.AddWithValue("@Address1", addressInformation1);
			cmd.Parameters.AddWithValue("@Address2", addressInformation2);
			cmd.Parameters.AddWithValue("@Address1_Update", addressInformation1);
			cmd.Parameters.AddWithValue("@Address2_Update", addressInformation2);

			settings.SqlCommands.Add(cmd);

			settings.SqlCommandsResponse.Add(cmd);

			return objReturn;
		}
	}

	public static class N4Manager
	{
		public static N4 Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			N4 objReturn = new N4(settings);
			string cityName = "";
			string stateOrProvinceCode = "";
			string postalCode = "";
			string countryCode = "";

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) cityName = elements[1];
				if (elements.Length > 2) stateOrProvinceCode = elements[2];
				if (elements.Length > 3) postalCode = elements[3];
				if (elements.Length > 4) countryCode = elements[4];
			}

			objReturn.CityName = cityName;
			objReturn.StateOrProvinceCode = new IDENTIFIER(stateOrProvinceCode);
			objReturn.PostalCode = new IDENTIFIER(postalCode);
			objReturn.CountryCode = new IDENTIFIER(countryCode);


			// add to sqlCommand
			string sql = "IF NOT EXISTS (SELECT 1 FROM App.EntityAddress WHERE RequestId = [RequestId] AND LoopCode = '" + loopCode + "' AND LoopNumber = " + loopNumber + ") ";
			sql += "INSERT INTO App.EntityAddress(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, Address1, Address2, City, StateProvCode, PostalCode, CountryCode, CountrySubCode) ";
			sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, '', '', @City, @StateProvCode, @PostalCode, @CountryCode, ''); ";
			sql += "UPDATE App.EntityAddress SET City = @City_Update, StateProvCode = @StateProvCode_Update, PostalCode = @PostalCode_Update, CountryCode = @CountryCode_Update WHERE RequestId = [RequestId] AND LoopCode = '" + loopCode + "' AND LoopNumber = " + loopNumber;

			SqlCommand cmd = new SqlCommand(sql);

			cmd.Parameters.AddWithValue("@LoopCode", loopCode);
			cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
			cmd.Parameters.AddWithValue("@City", cityName);
			cmd.Parameters.AddWithValue("@StateProvCode", stateOrProvinceCode);
			cmd.Parameters.AddWithValue("@PostalCode", postalCode);
			cmd.Parameters.AddWithValue("@CountryCode", countryCode);
			cmd.Parameters.AddWithValue("@City_Update", cityName);
			cmd.Parameters.AddWithValue("@StateProvCode_Update", stateOrProvinceCode);
			cmd.Parameters.AddWithValue("@PostalCode_Update", postalCode);
			cmd.Parameters.AddWithValue("@CountryCode_Update", countryCode);
			settings.SqlCommands.Add(cmd);

			settings.SqlCommandsResponse.Add(cmd);

			return objReturn;
		}
	}

	public static class PERManager
	{
		public static PER Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			PER objReturn = new PER(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ContactFunctionCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "PER01", objReturn.ContactFunctionCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PERManager - Invalid ContactFunctionCode=" + objReturn.ContactFunctionCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2) objReturn.Name = elements[2];
				if (elements.Length > 3 && !string.IsNullOrEmpty(elements[3]))
				{
					objReturn.CommunicationNumberQualifier1 = new IDENTIFIER(elements[3]);
					if (!CodeLookup.Validate(loopCode, "PER03", objReturn.CommunicationNumberQualifier1))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PERManager - Invalid CommunicationNumberQualifier1=" + objReturn.CommunicationNumberQualifier1 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 4) objReturn.CommunicationNumber1 = elements[4];
				if (elements.Length > 5 && !string.IsNullOrEmpty(elements[5]))
				{
					objReturn.CommunicationNumberQualifier2 = new IDENTIFIER(elements[5]);
					if (!CodeLookup.Validate(loopCode, "PER05", objReturn.CommunicationNumberQualifier2))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PERManager - Invalid CommunicationNumberQualifier2=" + objReturn.CommunicationNumberQualifier2 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 6) objReturn.CommunicationNumber2 = elements[6];
				if (elements.Length > 7 && !string.IsNullOrEmpty(elements[7]))
				{
					objReturn.CommunicationNumberQualifier3 = new IDENTIFIER(elements[7]);
					if (!CodeLookup.Validate(loopCode, "PER07", objReturn.CommunicationNumberQualifier3))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PERManager - Invalid CommunicationNumberQualifier3=" + objReturn.CommunicationNumberQualifier3 + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				if (elements.Length > 8) objReturn.CommunicationNumber3 = elements[8];
				if (elements.Length > 9) objReturn.ContactInquiryReference = elements[9];


				// Add SqlCommand
				string sql = "INSERT INTO App.ContactInformation(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, ContactFunctionId, ContactName, CommNumberQualifier1, CommNumber1, CommNumberQualifier2, CommNumber2, CommNumberQualifier3, CommNumber3) ";
				sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @ContactFunctionId, @ContactName, @CommNumberQualifier1, @CommNumber1, @CommNumberQualifier2, @CommNumber2, @CommNumberQualifier3, @CommNumber3)";
				SqlCommand cmd = new SqlCommand(sql);

				cmd.Parameters.AddWithValue("@LoopCode", loopCode);
				cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
				cmd.Parameters.AddWithValue("@ContactFunctionId", (elements.Length > 1 && elements[1] != null) ? elements[1] : "");
				cmd.Parameters.AddWithValue("@ContactName", (elements.Length > 2 && elements[2] != null) ? elements[2] : "");
				cmd.Parameters.AddWithValue("@CommNumberQualifier1", (elements.Length > 3 && elements[3] != null) ? elements[3] : "");
				cmd.Parameters.AddWithValue("@CommNumber1", (elements.Length > 4 && elements[4] != null) ? elements[4] : "");
				cmd.Parameters.AddWithValue("@CommNumberQualifier2", (elements.Length > 5 && elements[5] != null) ? elements[5] : "");
				cmd.Parameters.AddWithValue("@CommNumber2", (elements.Length > 6 && elements[6] != null) ? elements[6] : "");
				cmd.Parameters.AddWithValue("@CommNumberQualifier3", (elements.Length > 7 && elements[7] != null) ? elements[7] : "");
				cmd.Parameters.AddWithValue("@CommNumber3", (elements.Length > 8 && elements[8] != null) ? elements[8] : "");

				settings.SqlCommands.Add(cmd);

				settings.SqlCommandsResponse.Add(cmd);
			}

			return objReturn;
		}

		public static string GetEmail(PER per)
		{
			try
			{
				if (per == null) return "";

				if (per.CommunicationNumberQualifier1.Code == "EM") return per.CommunicationNumber1;
				if (per.CommunicationNumberQualifier2.Code == "EM") return per.CommunicationNumber2;
				if (per.CommunicationNumberQualifier3.Code == "EM") return per.CommunicationNumber3;
				return "";
			}
			catch (Exception)
			{
				return "";
			}
		}

		public static string GetFax(PER per)
		{
			try
			{
				if (per == null) return "";

				if (per.CommunicationNumberQualifier1.Code == "FX") return per.CommunicationNumber1;
				if (per.CommunicationNumberQualifier2.Code == "FX") return per.CommunicationNumber2;
				if (per.CommunicationNumberQualifier3.Code == "FX") return per.CommunicationNumber3;
				return "";
			}
			catch (Exception)
			{
				return "";
			}
		}

		public static string GetPhone(PER per)
		{
			try
			{
				if (per == null) return "";

				if (per.CommunicationNumberQualifier1.Code == "TE") return per.CommunicationNumber1;
				if (per.CommunicationNumberQualifier2.Code == "TE") return per.CommunicationNumber2;
				if (per.CommunicationNumberQualifier3.Code == "TE") return per.CommunicationNumber3;
				return "";
			}
			catch (Exception)
			{
				return "";
			}
		}

		public static string GetUrl(PER per)
		{
			try
			{
				if (per == null) return "";

				if (per.CommunicationNumberQualifier1.Code == "UR") return per.CommunicationNumber1;
				if (per.CommunicationNumberQualifier2.Code == "UR") return per.CommunicationNumber2;
				if (per.CommunicationNumberQualifier3.Code == "UR") return per.CommunicationNumber3;
				return "";
			}
			catch (Exception)
			{
				return "";
			}
		}

	}

	public static class PRVManager
	{
		public static PRV Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			PRV objReturn = new PRV(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ProviderCode = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "PRV01", objReturn.ProviderCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PRVManager - PRV01 - Invalid ProviderCode=" + objReturn.ProviderCode + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2) objReturn.ReferenceIdentificationQualifier = new IDENTIFIER(elements[2]);
				if (!CodeLookup.Validate(loopCode, "PRV02", objReturn.ReferenceIdentificationQualifier))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PRVManager - PRV02 - Invalid ReferenceIdentificationQualifier=" + objReturn.ReferenceIdentificationQualifier + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 3) objReturn.ReferenceIdentification = elements[3];
				if (string.IsNullOrEmpty(objReturn.ReferenceIdentification))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.PRVManager - ReferenceIdentification is required\r\nSegment=" + segment));
				}


				// Add SqlCommand
				string sql = "INSERT INTO App.ProviderInformation(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, ProviderCode, ReferenceIdentificationQualifier, ReferenceIdent) ";
				sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @ProviderCode, @ReferenceIdentificationQualifier, @ReferenceIdent)";
				SqlCommand cmd = new SqlCommand(sql);

				cmd.Parameters.AddWithValue("@LoopCode", loopCode);
				cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
				cmd.Parameters.AddWithValue("@ProviderCode", (elements.Length > 1 && elements[1] != null) ? elements[1] : "");
				cmd.Parameters.AddWithValue("@ReferenceIdentificationQualifier", (elements.Length > 2 && elements[2] != null) ? elements[2] : "");
				cmd.Parameters.AddWithValue("@ReferenceIdent", (elements.Length > 3 && elements[3] != null) ? elements[3] : "");

				settings.SqlCommands.Add(cmd);

				settings.SqlCommandsResponse.Add(cmd);
			}

			return objReturn;
		}
	}

	public static class DMGManager
	{
		public static DMG Parse(ref Settings settings, string loopCode, int loopNumber, string segment, ref List<AAA> requestValidation)
		{
			DMG objReturn = new DMG(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.DateTimePeriodFormatQualifier = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "DMG01", objReturn.DateTimePeriodFormatQualifier))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.DMGManager - DMG01 - Invalid DateTimePeriodFormatQualifier=" + objReturn.DateTimePeriodFormatQualifier + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2) objReturn.DateTimePeriod = elements[2];
				if (string.IsNullOrEmpty(objReturn.DateTimePeriod))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.DMGManager - DateTimePeriod is required\r\nSegment=" + segment));
				}

				if (elements.Length > 3 && !string.IsNullOrEmpty(elements[3]))
				{
					objReturn.GenderCode = new IDENTIFIER(elements[3]);
					if (!CodeLookup.Validate(loopCode, "DMG03", objReturn.GenderCode))
					{
						requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "66", "C", "CommonManager.DMGManager - DMG03 - Invalid GenderCode([3] {F,M,U})=" + objReturn.GenderCode + " Loop=" + loopCode + "  Segment=" + segment));
					}
				}

				// Add SqlCommand
				string sql = "INSERT INTO App.DemographicInformation(RequestId, IsResponse, LoopCode, LoopNumber, SequenceNumber, DateTimeQualifier, DateTimePeriod, GenderCode)";
				sql += "VALUES([RequestId], [IsResponse], @LoopCode, @LoopNumber, 1, @DateTimeQualifier, @DateTimePeriod, @GenderCode)";
				SqlCommand cmd = new SqlCommand(sql);

				cmd.Parameters.AddWithValue("@LoopCode", loopCode);
				cmd.Parameters.AddWithValue("@LoopNumber", loopNumber);
				cmd.Parameters.AddWithValue("@DateTimeQualifier", (elements.Length > 1 && elements[1] != null) ? elements[1] : "");
				cmd.Parameters.AddWithValue("@DateTimePeriod", (elements.Length > 2 && elements[2] != null) ? elements[2] : "");
				cmd.Parameters.AddWithValue("@GenderCode", (elements.Length > 3 && elements[3] != null) ? elements[3] : "");

				settings.SqlCommands.Add(cmd);

				settings.SqlCommandsResponse.Add(cmd);
			}

			return objReturn;
		}
	}

	public static class INSManager
	{
		public static INS Parse(ref Settings settings, string loopCode, string segment, ref List<AAA> requestValidation)
		{
			INS objReturn = new INS(settings);

			string[] elements = segment.Split(settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.YesNoConditionOrResponseCode1 = new IDENTIFIER(elements[1]);
				if (!CodeLookup.Validate(loopCode, "INS01", objReturn.YesNoConditionOrResponseCode1))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.INSManager - INS01 - Invalid YesNoConditionOrResponseCode1=" + objReturn.YesNoConditionOrResponseCode1 + " Loop=" + loopCode + "  Segment=" + segment));
				}
				if (elements.Length > 2) objReturn.IndividualRelationshipCode = new IDENTIFIER(elements[2]);
				if (!CodeLookup.Validate(loopCode, "INS02", objReturn.IndividualRelationshipCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.INSManager - INS02 - Invalid IndividualRelationshipCode=" + objReturn.IndividualRelationshipCode + " Loop=" + loopCode + "  Segment=" + segment));
				}

				if (elements.Length > 8) objReturn.EmploymentStatusCode = new IDENTIFIER(elements[8]);
				if (!CodeLookup.Validate(loopCode, "INS08", objReturn.EmploymentStatusCode))
				{
					requestValidation.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", "15", "C", "CommonManager.INSManager - INS08 - Invalid EmploymentStatusCode=" + objReturn.EmploymentStatusCode + " Loop=" + loopCode + "  Segment=" + segment));
				}

			}

			return objReturn;
		}
	}

}


