﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;

namespace X12Engine.BLL.X12._278
{
	// --- ISA
   public static class ISAManager
   {
      public static ISA Parse(Settings settings, string segment, ref MessageBase messageBase)
      {
			try
			{
				ISA objReturn = new ISA(settings);

				string[] elements = segment.Split(settings.Data_Element_Separator);
				if (elements != null)
				{
					objReturn.AuthorizationInformationQualifier = new IDENTIFIER(elements[1]);

					objReturn.AuthorizationInformation = elements[2];

					objReturn.SecurityInformationQualifier = new IDENTIFIER(elements[3]);

					objReturn.SecurityInformation = elements[4];

					objReturn.InterchangeSenderIdQualifier = new IDENTIFIER(elements[5]);

					objReturn.InterchangeSenderId = elements[6];

					objReturn.InterchangeReceiverIdQualifier = new IDENTIFIER(elements[7]);

					objReturn.InterchangeReceiverId = elements[8];

					// set date
					objReturn.InterchangeDate = Utils.ParseDate(elements[9] + " " + elements[10], "yyMMdd HHmm");
					objReturn.InterchangeDateString = elements[9];

					// set time
					objReturn.InterchangeTime = Utils.ParseTime(elements[10], "HHmm", objReturn.InterchangeDate);
					objReturn.InterchangeTimeString = elements[10];

					objReturn.RepetitionSeperator = elements[11];

					objReturn.InterchangeControlVersionNumber = new IDENTIFIER(elements[12]);

					objReturn.InterchangeControlNumber = Utils.ParseInt(elements[13]);

					objReturn.AcknowledgmentRequested = new IDENTIFIER(elements[14]);

					objReturn.InterchangeUsageIndicator = new IDENTIFIER(elements[15]);

					objReturn.ComponentElementSeparator = elements[16];

				}

				return objReturn;
			}
			catch (Exception ex)
			{
				messageBase.ErrorFound = true;
				messageBase.ErrorMessage = messageBase.ErrorMessage + "\r\nControlSegmentsManager.ISAManager Parse error - " + ex.Message;
            return null;
			}

		}
	}

   public static class GSManager
   {
      public static GS Parse(Settings settings, string segment, ref MessageBase messageBase)
      {
			try
			{
				GS objReturn = new GS(settings);

				string[] elements = segment.Split(settings.Data_Element_Separator);
				if (elements != null)
				{
					objReturn.FunctionalIdentifierCode = new IDENTIFIER(elements[1]);

					objReturn.ApplicationSendersCode = elements[2];
					objReturn.ApplicationReceiversCode = elements[3];

					objReturn.Date = Utils.ParseDate(elements[4], "yyyyMMdd");
					objReturn.DateString = elements[4];

					objReturn.Time = Utils.ParseTime(elements[5], string.Empty, objReturn.Date);
					objReturn.TimeString = elements[5];

					objReturn.GroupControlNumber = Utils.ParseInt(elements[6]);
					messageBase.GroupControlNumber = objReturn.GroupControlNumber.ToString();

					objReturn.ResponsibleAgencyCode = new IDENTIFIER(elements[7]);
					objReturn.VersionReleaseIndustryIdentifierCode = elements[8];
				}

				return objReturn;
			}
			catch (Exception ex)
			{
				messageBase.ErrorFound = true;
				messageBase.ErrorMessage = messageBase.ErrorMessage + "\r\nControlSegmentsManager.GSManager Parse error - " + ex.Message;
				return null;
			}

		}
   }

   public static class GEManager
   {
      public static GE Parse(Settings settings, string segment, ref MessageBase messageBase)
      {
			try
			{
				GE objReturn = new GE(settings);

				string[] elements = segment.Split(settings.Data_Element_Separator);
				if (elements != null)
				{
					if (elements.Length > 1) objReturn.NumberOfTransactionSetsIncluded = Utils.ParseInt(elements[1]);
					if (elements.Length > 2) objReturn.GroupControlNumber = Utils.ParseInt(elements[2]);
				}

				return objReturn;
			}
			catch (Exception ex)
			{
				messageBase.ErrorFound = true;
				messageBase.ErrorMessage = messageBase.ErrorMessage + "\r\nControlSegmentsManager.GEManager Parse error - " + ex.Message;
				return null;
			}

		}
   }

   public static class IEAManager
   {
      public static IEA Parse(Settings settings, string segment, ref MessageBase messageBase)
      {
			try
			{
				IEA objReturn = new IEA(settings);

				string[] elements = segment.Split(settings.Data_Element_Separator);
				if (elements != null)
				{
					if (elements.Length > 1) objReturn.NumberOfIncludedFunctionalGroups = Utils.ParseInt(elements[1]);
					if (elements.Length > 2) objReturn.InterchangeControlNumber = Utils.ParseInt(elements[2]);
				}

				return objReturn;
			}
			catch (Exception ex)
			{
				messageBase.ErrorFound = true;
				messageBase.ErrorMessage = messageBase.ErrorMessage + "\r\nControlSegmentsManager.IEAManager Parse error - " + ex.Message;
				return null;
			}
		}
   }


   public static class STManager
   {
      public static ST Parse(Settings settings, string segment)
      {
         ST objReturn = new ST(settings);

         string[] elements = segment.Split(settings.Data_Element_Separator);
         if (elements != null)
         {
            if (elements.Length > 1) objReturn.TransactionSetIdentifierCode = new IDENTIFIER(elements[1]);
            if (elements.Length > 2) objReturn.TransactionSetControlNumber = elements[2];
            if (elements.Length > 3) objReturn.ImplementationConventionReference = elements[3];
         }

         return objReturn;
      }
   }

   public static class BHTManager
   {
      public static BHT Parse(Settings settings, string segment)
      {
         BHT objReturn = new BHT(settings);

         string[] elements = segment.Split(settings.Data_Element_Separator);
         if (elements != null)
         {
            if (elements.Length > 1) objReturn.HierarchicalStructureCode = new IDENTIFIER(elements[1]);
            if (elements.Length > 2) objReturn.TransactionSetPurposeCode = new IDENTIFIER(elements[2]);
            if (elements.Length > 3) objReturn.ReferenceIdentification = elements[3];
            if (elements.Length > 4)
            {
               objReturn.Date = Utils.ParseDate(elements[4], "yyyyMMdd"); 
               objReturn.DateString = elements[4];
            }
            if (elements.Length > 5)
            {
               objReturn.Time = Utils.ParseTime(elements[5], string.Empty, objReturn.Date); 
               objReturn.TimeString = elements[5];
            }
            if (elements.Length > 6) objReturn.TransactionTypeCode = new IDENTIFIER(elements[6]);
         }

         return objReturn;
      }
   }

   public static class SEManager
   {
      public static SE Parse(Settings settings, string segment)
      {
         SE objReturn = new SE(settings);

         string[] elements = segment.Split(settings.Data_Element_Separator);
         if (elements != null)
         {
            if (elements.Length > 1) objReturn.NumberOfIncludedSegments = Utils.ParseInt(elements[1]);
            if (elements.Length > 2) objReturn.TransactionSetControlNumber = elements[2];
         }

         return objReturn;
      }
   }
}
