﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;

namespace X12Engine.BLL.X12._278
{
   public static class DetailBaseManager
   {
      public static DetailBase Parse(ref Settings settings, string loopCode, int loopNumber, string segment, DetailBase detailBase)
      {
			DetailBase objReturn = new DetailBase();
			List<AAA> requestValidation = new List<AAA>();	
				
			if (detailBase != null) {
				objReturn = detailBase;
				foreach (var aaa in detailBase.RequestValidation)
				{
					requestValidation.Add(aaa);
				}
			}

			

			string[] elements = segment.Split(settings.Data_Element_Separator);
         if (elements != null)
         {

            string segmentId = elements[0].ToUpper();
            switch (segmentId)
            {
               case "HL":
                  objReturn.HierarchicalLevel = HLManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "TRN":
                  if (objReturn.Trace == null) objReturn.Trace = new List<TRN>();
                  objReturn.Trace.Add(TRNManager.Parse(ref settings, loopCode, segment, ref requestValidation));
                  break;
               case "UM":
                  objReturn.HealthCareServicesReviewInformation = UMManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation); 
                  break;
               case "HCR":
                  objReturn.HealthCareServicesReview = HCRManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "REF":
                  if (objReturn.ReferenceInformation == null) objReturn.ReferenceInformation = new List<REF>();
                  objReturn.ReferenceInformation.Add(REFManager.Parse(ref settings, loopCode, segment, ref requestValidation));
                  break;
               case "DTP":
                  if (objReturn.DateOrTimeOrPeriod == null) objReturn.DateOrTimeOrPeriod = new List<DTP>();
                  objReturn.DateOrTimeOrPeriod.Add(DTPManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation));
                  break;
               case "SV1":
                  objReturn.ProfessionalService = SV1Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "SV2":
                  objReturn.InstitutionalService = SV2Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "SV3":
                  objReturn.DentalService = SV3Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "TOO":
                  if (objReturn.ToothIdentification == null) objReturn.ToothIdentification = new List<TOO>();
                  objReturn.ToothIdentification.Add(TOOManager.Parse(ref settings, loopCode, segment, ref requestValidation));
                  break;
               case "HSD":
                  objReturn.HealthCareServicesDelivery = HSDManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "CRC":
                  if (objReturn.ConditionsIndicator == null) objReturn.ConditionsIndicator = new List<CRC>();
                  objReturn.ConditionsIndicator.Add(CRCManager.Parse(ref settings, loopCode, segment, ref requestValidation));
                  break;
               case "CL1":
                  objReturn.ClaimCodes = CL1Manager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
                  break;
               case "CR1":
                  objReturn.AmbulanceCertification = CR1Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "CR2":
                  objReturn.ChiropracticCertification = CR2Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "CR4":
                  objReturn.EnteralOrParenteralTherapyCertification = CR4Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "CR5":
                  objReturn.OxygenTherapyCertification = CR5Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "CR6":
                  objReturn.HomeHealthCareCertification = CR6Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "CR7":
                  objReturn.HomeHealthTreatmentPlanCertification = CR7Manager.Parse(ref settings, segment, ref requestValidation);
                  break;
               case "CR8":
                  objReturn.PacemakerCertification = CR8Manager.Parse(ref settings, segment, ref requestValidation);
                  break;
               case "PWK":
                  if (objReturn.Paperwork == null) objReturn.Paperwork = new List<PWK>();
                  objReturn.Paperwork.Add(PWKManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation));
                  break;
               case "MSG":
                  objReturn.MessageText = MSGManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
                  break;
					case "SE":
						break;

               default:
						//requestValidation.Add(AAAManager.GetAAA(settings, "N", "AAA", "C", "Parsing Error...segment has invalid format - " + segment));
                  break;
            }
				objReturn.RequestValidation = requestValidation;
			}

         return objReturn;
      }
   }
}
