﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;

namespace X12Engine.BLL.X12._278
{
   public static class EntityBaseManager
   {
		 
      public static EntityBase Parse(ref Settings settings, string loopCode, int loopNumber, string segment, EntityBase entityBase)
      {
         EntityBase objReturn = new EntityBase();
			List<AAA> requestValidation = new List<AAA>();
			if (entityBase != null)
			{
				objReturn = entityBase;
				requestValidation = entityBase.RequestValidation;
         }
			

			string[] elements = segment.Split(settings.Data_Element_Separator);
         if (elements != null)
         {
            string segmentId = elements[0].ToUpper();
            switch (segmentId)
            {
               case "NM1":
                  objReturn.IndividualOrOrganizationName = NM1Manager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
                  break;
               case "REF":
                  if (objReturn.ReferenceInformation == null) objReturn.ReferenceInformation = new List<REF>();
                  objReturn.ReferenceInformation.Add(REFManager.Parse(ref settings, loopCode, segment, ref requestValidation));
                  break;
               case "N2":
                  objReturn.AdditionalNameInformation = N2Manager.Parse(ref settings, segment, ref requestValidation);
                  break;
               case "N3":
                  objReturn.PartyLocation = N3Manager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
                  break;
               case "N4":
                  objReturn.GeographicLocation = N4Manager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
                  break;
               case "PER":
                  if (objReturn.AdministrativeCommunicationsContact == null) objReturn.AdministrativeCommunicationsContact = new List<PER>();
                  objReturn.AdministrativeCommunicationsContact.Add(PERManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation));
                  break;
               case "PRV":
                  objReturn.ProviderInformation = PRVManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
                  break;
               case "DMG":
                  objReturn.DemographicInformation = DMGManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
                  break;
               case "INS":
                  objReturn.InsuredBenefit = INSManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  break;
               case "DTP":
                  if (objReturn.DateOrTimeOrPeriod == null) objReturn.DateOrTimeOrPeriod = new List<DTP>();
                  objReturn.DateOrTimeOrPeriod.Add(DTPManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation));
                  break;
               default:
                  break;
            }
				objReturn.RequestValidation = requestValidation;
			}

         return objReturn;
      }



		public static List<AAA> ValidN3N4(Settings settings, string loopCode, N3 partyLocation, N4 geographicLocation, string errorCode)
		{
			List<AAA> aaa = new List<AAA>();


			//---- Validate N3/N4 Get country code first
			string countryCode = "";
			if (geographicLocation != null)
			{
				countryCode = geographicLocation.CountryCode.Code;
			}

			// Validate N3
			if (partyLocation != null)
			{
				if (countryCode == "" )
				{
					if (partyLocation.AddressInformation1.Trim() == "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N301 AddressInformation1 is required"));
					}
				}
				else
				{
					if (partyLocation.AddressInformation1.Trim() == "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N301 AddressInformation1 is required"));
					}
				}
			}

			// Validate  N4
			if (geographicLocation != null)
			{
				if (countryCode == "" )
				{
					if (geographicLocation.CityName.Trim() == "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N401 CityName is required"));
					}

					if (geographicLocation.StateOrProvinceCode.Code.Trim() == "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N402 StateOrProvinceCode is required"));
					}

					if (geographicLocation.PostalCode.Code.Trim() == "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N403 PostalCode is required"));
					}

				}
				else
				{
					if (geographicLocation.CityName.Trim() == "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N401 CityName is required"));
					}

					if (geographicLocation.StateOrProvinceCode.Code.Trim() != "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N402 StateOrProvinceCode should be empty"));
					}

					if (geographicLocation.PostalCode.Code.Trim() != "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N403 PostalCode should be empty"));
					}

					if (geographicLocation.CountryCode.Code.Trim() == "")
					{
						aaa.Add(AAAManager.GetAAA(ref settings, loopCode, 1, "N", errorCode, "C", loopCode + " - N404 CountryCode is required"));
					}
				}

			}

			return aaa;

		}
   }
}
