﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using X12Engine.BLL.X12._278.XParse;
using X12Engine.BO.X12._278;
using X12Engine.BO.X12._278.XModel;

namespace X12Engine.BLL.X12._278
{
	public class MessageBaseManager
	{
		public MessageBase ParseSegments(ref Settings settings, string requestMessage)
		{

			MessageBase messageBase = null;

			//try
			//{
				// Repetition_Separator 83 byte
				settings.Repetition_Separator = Convert.ToChar(requestMessage.Substring(82, 1));

				// Component_Element_Separator 105
				settings.Component_Element_Separator = Convert.ToChar(requestMessage.Substring(104, 1));

				// Segment_Terminator 106
				settings.Segment_Terminator = Convert.ToChar(requestMessage.Substring(105, 1));

				// set Data_Element_Separator default *
				settings.Data_Element_Separator = Convert.ToChar(requestMessage.Substring(3, 1));

			//}
			//catch (Exception ex)
			//{
			//	messageBase.ErrorFound = true;
			//	Utils.LogEvent(string.Format("Error setting separators: {0}. ", ex.Message) + "\r\n" + ex.StackTrace);
			//}


			//try
			//{
				bool gotST = false;
				bool gotSE = false;
				string transactionSet = string.Empty;
				string implementationConventionReference = "";
				string[] segments = requestMessage.Split(settings.Segment_Terminator);
				if (segments != null)
				{
					messageBase = new MessageBase();

					foreach (string segment in segments)
					{
						if (string.IsNullOrEmpty(segment)) continue;


						// segment must contain Data_Element_Separator
						if (!segment.Contains(settings.Data_Element_Separator.ToString()) || segment.Length < 3)
						{
							//messageBase.ErrorFound = true;
							//messageBase.ErrorMessage = messageBase.ErrorMessage + "\r\n Invalid message.  Data_Element_Separator(" + settings.Data_Element_Separator.ToString() + ") does not exist in segment=" + segment;
							//return messageBase;
							continue;
						}


						string[] elements = segment.Split(settings.Data_Element_Separator);
						if (elements != null)
						{
							string segmentId = elements[0].ToUpper();

							switch (segmentId)
							{
								case "ISA":  // Interchange Control Header
									messageBase.InterchangeControlHeader = ISAManager.Parse(settings, segment, ref messageBase);
									break;
								case "GS":   // Functional Group Header
									messageBase.FunctionalGroupHeader = GSManager.Parse(settings, segment, ref messageBase);
									break;
								case "GE":   //  Functional Group Trailer
									messageBase.FunctionalGroupTrailer = GEManager.Parse(settings, segment, ref messageBase);
									break;
								case "IEA":  // Interchange Control Trailer
									messageBase.InterchangeControlTrailer = IEAManager.Parse(settings, segment, ref messageBase);
									break;
								default:

									if (segmentId == "ST")  // Transaction Set Header
									{
										gotST = true;
										transactionSet = segment + settings.Segment_Terminator;
										if (elements.Length > 3) implementationConventionReference = elements[3].ToUpper();
										settings.TransType = implementationConventionReference;
									}
									else if (segmentId == "SE")  // Transaction Set Trailer
									{
										gotSE = true;
										transactionSet += segment + settings.Segment_Terminator;

										if (implementationConventionReference == "005010X215" || implementationConventionReference == "005010X217")
										{

											Message message = ParseMessage(ref settings, transactionSet, ref messageBase);
											messageBase.TransErrorFound = message.ErrorFound;
											messageBase.XMessage.Add(message);
										}
									}
									else
									{
										transactionSet += segment + settings.Segment_Terminator;
									}

									break;
							}
						}
					}

					if (!gotST)
					{
						messageBase.ErrorFound = true;
						messageBase.ErrorMessage = messageBase.ErrorMessage + "\r\n Unable to get ST segment.";
						return messageBase;
					}
					if (!gotSE)
					{
						messageBase.ErrorFound = true;
						messageBase.ErrorMessage = messageBase.ErrorMessage + "\r\n Unable to get SE segment.";
						return messageBase;
					}
				}

			//}
			//catch (Exception ex)
			//{
			//	messageBase.ErrorFound = true;
			//	Utils.LogEvent(string.Format("ParseSegments(): {0}. ", ex.Message) + "\r\n" + ex.StackTrace);
			//}

			return messageBase;
		}

		public Message ParseMessage(ref Settings settings, string message, ref MessageBase messageBase)
		{
			//try
			//{

				AuthorizationManager authorizationManager = new AuthorizationManager();
				string validCertificationTypeCode = "|1|2|3|4|I|N|R|S|";
				bool hasMultipleRecords = false;
				bool aaaInEventLoop = false;
				Message objReturn = new Message(settings);
				List<SFeeVendor> vendors = new List<SFeeVendor>();
				List<SPatient> patients = new List<SPatient>();
				string nameFirst = "", nameLast = "", ssn = "", idNumber = null, npi = null, medicareIDNumber = null;
				string[] formats = { "yyyyMMdd" };
				DateTime dTemp = new DateTime();


				if (settings.TransType == "005010X215")
				{
					validCertificationTypeCode = "| ||1|2|3|4|I|N|R|S|";
				}

				string[] loops = Utils.Split(message, "HL" + settings.Data_Element_Separator);
				if (loops != null)
				{
					string[] hierarchicalIdList = null;

					foreach (string loop in loops)
					{
						string[] segments = loop.Split(settings.Segment_Terminator);
						if (segments != null)
						{
							foreach (string segment in segments)
							{
								if (string.IsNullOrEmpty(segment)) continue;

								string[] elements = segment.Split(settings.Data_Element_Separator);
								if (elements != null)
								{
									string segmentId = elements[0].ToUpper();

									switch (segmentId)
									{
										case "ST":
											objReturn.TransactionSetHeader = STManager.Parse(settings, segment);
											break;

										case "BHT":
											objReturn.BeginningOfHierarchicalTransaction = BHTManager.Parse(settings, segment);
											break;

										case "SE":
											objReturn.TransactionSetTrailer = SEManager.Parse(settings, segment);
											break;

										case "HL":

											string hierarchicalIdNumber = string.Empty;
											string hierarchicalLevelCode = string.Empty;

											hierarchicalIdNumber = elements[1];
											hierarchicalLevelCode = elements[3].ToUpper();

											if (Utils.Contains(hierarchicalIdList, hierarchicalIdNumber))
											{
												objReturn.ErrorFound = true;
												continue;
											}
											else
											{
												Utils.AddToArray(ref hierarchicalIdList, hierarchicalIdNumber);
											}

											#region  HL1 - UMO
											if (hierarchicalLevelCode == "20")   //--------- HL1 - 2000A & 2010A - Utilization Management Organization
											{
												//Utils.LogEvent("Parse Header Segments");
												objReturn.UmoDetail = UmoDetailManager.Parse(ref settings, "2010A", loop);

												if (objReturn.ErrorFound) continue;

												//--- get values 
												string entityIdCode = (objReturn.UmoDetail.UMOName != null && objReturn.UmoDetail.UMOName.IndividualOrOrganizationName != null && objReturn.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode != null) ? objReturn.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode.Code : "";
												string idCode = (objReturn.UmoDetail.UMOName != null && objReturn.UmoDetail.UMOName.IndividualOrOrganizationName != null && objReturn.UmoDetail.UMOName.IndividualOrOrganizationName.IdentificationCode != null) ? objReturn.UmoDetail.UMOName.IndividualOrOrganizationName.IdentificationCode : "";
												string umoName = (objReturn.UmoDetail.UMOName != null && objReturn.UmoDetail.UMOName.IndividualOrOrganizationName != null && objReturn.UmoDetail.UMOName.IndividualOrOrganizationName.NameLastOrOrganizationName != null) ? objReturn.UmoDetail.UMOName.IndividualOrOrganizationName.NameLastOrOrganizationName : "";


												//--- validate 
												string transactionSetPurposeCode = (objReturn.BeginningOfHierarchicalTransaction != null && objReturn.BeginningOfHierarchicalTransaction.TransactionSetPurposeCode != null) ? objReturn.BeginningOfHierarchicalTransaction.TransactionSetPurposeCode.Code : "";
												if ((settings.TransType == "005010X217" && transactionSetPurposeCode != "13") || (settings.TransType == "005010X215" && transactionSetPurposeCode != "28"))
												{
													objReturn.UmoDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010A", 1, "N", "41", "C", "Authorization/Access Restrictions"));
												}

												if (settings.TransType == "005010X217")
												{
													objReturn.BeginningOfHierarchicalTransaction.TransactionSetPurposeCode = new IDENTIFIER("11");
													objReturn.BeginningOfHierarchicalTransaction.TransactionTypeCode = new IDENTIFIER("18");
												}
												else
												{
													objReturn.BeginningOfHierarchicalTransaction.TransactionSetPurposeCode = new IDENTIFIER("49");
												}

												if (umoName.Length == 0 || idCode.Length == 0)
												{
													objReturn.UmoDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010A", 1, "N", "T4", "C", "Payer Name or Identifier Missing"));
												}
												else if (entityIdCode == "" || entityIdCode == "1P" || entityIdCode == "2B" || entityIdCode == "36")
												{
													objReturn.UmoDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010A", 1, "N", "79", "C", "Invalid EntityIdCode should not be " + entityIdCode));
												}
												else if (idCode != X12Engine.Properties.Settings.Default.UMOIdCode)
												{
													objReturn.UmoDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010A", 1, "N", "79", "C", "Invalid Participant Identification <> " + X12Engine.Properties.Settings.Default.UMOIdCode));
												}

												int validationCount1 = (objReturn.UmoDetail != null) ? objReturn.UmoDetail.RequestValidation.Count : 0;
												int validationCount2 = (objReturn.UmoDetail.UMOName != null) ? objReturn.UmoDetail.UMOName.RequestValidation.Count : 0;
												int validationCount3 = (objReturn.UmoDetail.Other != null) ? objReturn.UmoDetail.Other.RequestValidation.Count : 0;
												if (validationCount1 > 0 || validationCount2 > 0 || validationCount3 > 0)
												{
													objReturn.UmoDetail.UMOLevel.HierarchicalChildCode = new IDENTIFIER("0");

													objReturn.ErrorFound = true;
													continue;
												}


											}
											#endregion

											#region HL2
											else if (hierarchicalLevelCode == "21") //--------- HL2 - 2010B - Requestor
											{
												//Utils.LogEvent("Parse UMO");
												RequesterDetail requesterDetail = RequesterDetailManager.Parse(ref settings, "2010B", loop);
												objReturn.RequesterDetail = requesterDetail;
												if (objReturn.ErrorFound) continue;

												// get vendor
												var vendorId = (requesterDetail.RequesterName != null && requesterDetail.RequesterName.IndividualOrOrganizationName != null && requesterDetail.RequesterName.IndividualOrOrganizationName.IdentificationCodeQualifier != null && !string.IsNullOrEmpty(requesterDetail.RequesterName.IndividualOrOrganizationName.IdentificationCode)) ? requesterDetail.RequesterName.IndividualOrOrganizationName.IdentificationCode : "";
												var codeQualifier = (requesterDetail.RequesterName != null && requesterDetail.RequesterName.IndividualOrOrganizationName != null && requesterDetail.RequesterName.IndividualOrOrganizationName.IdentificationCodeQualifier != null && !string.IsNullOrEmpty(requesterDetail.RequesterName.IndividualOrOrganizationName.IdentificationCodeQualifier.Code)) ? requesterDetail.RequesterName.IndividualOrOrganizationName.IdentificationCodeQualifier.Code : "";
												messageBase.VendorQualifier = codeQualifier;

												if (codeQualifier == "24" || codeQualifier == "34")
												{
													vendors = authorizationManager.GetVendorByIDNumber(vendorId, ref messageBase);
													idNumber = vendorId;
												}
												else if (codeQualifier == "XV")
												{
													vendors = authorizationManager.GetVendorByMedicareIDNumber(vendorId, ref messageBase);
													medicareIDNumber = vendorId;
												}
												else if (codeQualifier == "XX")   // NPI, do a crosswalk and search by IDNumber(s)
												{
													vendors = authorizationManager.GetVendorByNPI(vendorId, ref messageBase);
													npi = vendorId;
												}


												// check first db connection here
												if (!messageBase.ValidConnection)
												{
													return objReturn;
												}

												if (vendorId.Length == 0)
												{
													objReturn.RequesterDetail = requesterDetail;
													objReturn.RequesterDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010B", 1, "N", "43", "C", "HL2.RequesterDetail - Invalid/Missing Provider Identification"));
												}
												else if (vendors.Count == 0)
												{
													objReturn.RequesterDetail = requesterDetail;

													if (codeQualifier == "XX")
													{
														objReturn.RequesterDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010B", 1, "N", "43", "C", "HL2.RequesterDetail - Provider Not Found"));
													}
													else
													{
														objReturn.RequesterDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010B", 1, "N", "51", "C", "HL2.RequesterDetail - Provider Not on File"));
													}
												}
												else
												{
													messageBase.FeeVendors = vendors;
													messageBase.FeeVendorID = vendorId;
												}

												//---- Validate N3/N4 
												List<AAA> aaas = EntityBaseManager.ValidN3N4(settings, "2010B", objReturn.RequesterDetail.RequesterName.PartyLocation, objReturn.RequesterDetail.RequesterName.GeographicLocation, "97");
												if (aaas.Count > 0)
												{
													objReturn.RequesterDetail.RequesterName.PartyLocation = null;
													objReturn.RequesterDetail.RequesterName.GeographicLocation = null;
													foreach (var aaa in aaas)
													{
														objReturn.RequesterDetail.RequesterName.RequestValidation.Add(aaa);
													}
												}


												// do not return in response
												objReturn.RequesterDetail.RequesterName.PartyLocation = null;
												objReturn.RequesterDetail.RequesterName.GeographicLocation = null;


												// validation count
												int validationCount1 = (objReturn.RequesterDetail != null) ? objReturn.RequesterDetail.RequestValidation.Count : 0;

												var validationCount2 = (objReturn.RequesterDetail.RequesterName != null) ? objReturn.RequesterDetail.RequesterName.RequestValidation.Count : 0;

												int validationCount3 = (objReturn.RequesterDetail.Other != null) ? objReturn.RequesterDetail.Other.RequestValidation.Count : 0;


												//set AAA limit of 9 - no duplicates
												if (validationCount1 > 0 || validationCount2 > 0 || validationCount3 > 0)
												{

													List<AAA> mainValidation = new List<AAA>();
													string gotAAA = ":";
													var gotCount = 0;

													if (objReturn.RequesterDetail != null)
													{
														foreach (var aaa in objReturn.RequesterDetail.RequestValidation)
														{
															if (gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;

															mainValidation.Add(aaa);
															gotAAA += aaa.RejectReasonCode.Code + ":";
															gotCount++;
															if (gotCount >= 9) break;
														}

														objReturn.RequesterDetail.RequestValidation.Clear();
													}

													if (objReturn.RequesterDetail.RequesterName != null)
													{
														foreach (var aaa in objReturn.RequesterDetail.RequesterName.RequestValidation)
														{
															if (gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;

															mainValidation.Add(aaa);
															gotAAA += aaa.RejectReasonCode.Code + ":";
															gotCount++;
															if (gotCount >= 9) break;
														}

														objReturn.RequesterDetail.RequesterName.RequestValidation.Clear();
													}

													if (objReturn.RequesterDetail.Other != null)
													{
														foreach (var aaa in objReturn.RequesterDetail.Other.RequestValidation)
														{
															if(gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;
															mainValidation.Add(aaa);
															gotAAA += aaa.RejectReasonCode.Code + ":";
															gotCount++;
															if (gotCount >= 9) break;
														}

														objReturn.RequesterDetail.Other.RequestValidation.Clear();
													}

													objReturn.RequesterDetail.RequestValidation = mainValidation;

													objReturn.RequesterDetail.RequesterLevel.HierarchicalChildCode = new IDENTIFIER("0");
													objReturn.ErrorFound = true;

												}

											}
											#endregion

											#region HL3
											else if (hierarchicalLevelCode == "22")  //--------- HL3 -2010C - Subscriber The subscriber could also be the patient.   
											{
												if (objReturn.SubscriberDetail == null) objReturn.SubscriberDetail = new List<SubscriberDetail>();

												SubscriberDetail subscriberDetail = new SubscriberDetail(settings);

												subscriberDetail = SubscriberDetailManager.Parse(ref settings, "2010C", loop);
												if (objReturn.ErrorFound)
												{
													objReturn.SubscriberDetail.Add(subscriberDetail);

													foreach (var cmd in settings.SqlCommands)
													{
														if (cmd.CommandText.Contains("[DateOfBirth]"))
														{
															cmd.CommandText = cmd.CommandText.Replace("[DateOfBirth]", "null");
															cmd.CommandText = cmd.CommandText.Replace("[Gender]", "");
														}
													}
													continue;
												}
		
												// set Date-of-Birth / Gender Code
												string dateOfBirth = (subscriberDetail.SubscriberName.DemographicInformation != null && subscriberDetail.SubscriberName.DemographicInformation.DateTimePeriod != null) ? subscriberDetail.SubscriberName.DemographicInformation.DateTimePeriod : "";
												messageBase.DateOfBirth = dateOfBirth.Trim();
												string genderCode = (subscriberDetail.SubscriberName.DemographicInformation != null && subscriberDetail.SubscriberName.DemographicInformation.GenderCode != null) ? subscriberDetail.SubscriberName.DemographicInformation.GenderCode.Code : "";

												// set Patient Name  
												nameFirst = subscriberDetail.SubscriberName.IndividualOrOrganizationName != null && subscriberDetail.SubscriberName.IndividualOrOrganizationName.NameFirst != null ? subscriberDetail.SubscriberName.IndividualOrOrganizationName.NameFirst : "";
												nameLast = subscriberDetail.SubscriberName.IndividualOrOrganizationName != null && subscriberDetail.SubscriberName.IndividualOrOrganizationName.NameLastOrOrganizationName != null ? subscriberDetail.SubscriberName.IndividualOrOrganizationName.NameLastOrOrganizationName : "";


												// set patientId (ssn) / firstname / lastname
												ssn = (!string.IsNullOrEmpty(subscriberDetail.SubscriberName.IndividualOrOrganizationName.IdentificationCode)) ? subscriberDetail.SubscriberName.IndividualOrOrganizationName.IdentificationCode : "";
												messageBase.SSN = ssn;
												messageBase.NameFirst = nameFirst;
												messageBase.NameLast = nameLast;


												// if ssn is blank/missing throw  72 - Invalid/Missing subscriber
												if (ssn.Length == 0)
												{
													subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "64", "C", "Invalid/Missing Patient ID"));
													subscriberDetail.SubscriberLevel.HierarchicalChildCode = new IDENTIFIER("0");
													objReturn.SubscriberDetail.Add(subscriberDetail);
												
													objReturn.ErrorFound = true;
													foreach (var cmd in settings.SqlCommands)
													{
														if (cmd.CommandText.Contains("[DateOfBirth]"))
														{
															cmd.CommandText = cmd.CommandText.Replace("[DateOfBirth]", "null");
															cmd.CommandText = cmd.CommandText.Replace("[Gender]", "");
														}
													}
													continue;
												}


												// get patient ---- if any AAA, set the queue patient problem queue 	
												patients = authorizationManager.GetPatient(ssn);

												// get patient with vendors   --- 2010EA  2010F  vendors can come.   Crosswalk table?    NM109  --- NPI number 99%    supplemental   
												string chkPatientSID = ":";
												List<SPatient> patientWithVendors = new List<SPatient>();
												foreach (var vendor in vendors)
												{
													foreach (var patient in patients)
													{
														if (patient.Sta3n == vendor.Sta3n)
														{
															if (!chkPatientSID.Contains(":" + patient.PatientSID + ":"))
															{
																patientWithVendors.Add(patient);
																chkPatientSID += patient.PatientSID + ":";
															}

														}
													}
												}

												if (patients.Count == 0)
												{
													subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "67", "C", "Patient not found"));
												}
												else if (patientWithVendors.Count == 0)
												{
													// put in event loop 57 and AA
													aaaInEventLoop = true;
												}
												else
												{
													if (patientWithVendors.Count > 0 && patients.Count > 1 && patientWithVendors.Count != 1)
													{
														hasMultipleRecords = true;
													}

													SPatient sPatient = patientWithVendors[0];


													if (subscriberDetail.SubscriberName.DemographicInformation != null && messageBase.DateOfBirth.Length == 0)
													{
														subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "58", "C", "Invalid/Missing Date of Birth - " + messageBase.DateOfBirth));
													}
													else if (messageBase.DateOfBirth.Length > 0)
													{
														if (!DateTime.TryParseExact(messageBase.DateOfBirth, formats, new System.Globalization.CultureInfo("en-US"), System.Globalization.DateTimeStyles.None, out dTemp))
														{
															subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "58", "C", "Invalid/Missing Date of Birth - " + messageBase.DateOfBirth));
														}
													}

													if (ssn.Length == 0)
													{
														subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "64", "C", "Invalid/Missing Patient ID"));

														sPatient = null;
													}

													if (nameLast.Length == 0)
													{
														subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "73", "C", "Invalid/Missing Subscriber/Insured Name"));
													}

													if (sPatient == null)
													{
														if (nameLast.Length == 0)
														{
															subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "65", "C", "Invalid/Missing Patient Name"));
														}
														if (ssn.Length > 0)
														{
															subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "64", "C", "Invalid/Missing Patient ID"));
														}

														subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "67", "C", "Patient Not Found"));
													}
													else if (!hasMultipleRecords)
													{
														string patientName5 = (sPatient.PatientLastName.Length > 5) ? sPatient.PatientLastName.ToUpper().Substring(0, 5) : sPatient.PatientLastName.ToUpper();
														string name5 = (nameLast.Length > 5) ? nameLast.ToUpper().Substring(0, 5) : nameLast.ToUpper();

														if (patientName5 != name5 || nameLast.Length == 0)
														{
															subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "65", "C", "Mismatched Patient Last Name"));
														}

														// Invalid/Missing Date-of-Birth
														if (messageBase.DateOfBirth.Length > 0)
														{

															if (DateTime.TryParseExact(messageBase.DateOfBirth, formats, new System.Globalization.CultureInfo("en-US"), System.Globalization.DateTimeStyles.None, out dTemp))
															{
																string dob_ = (sPatient.DateOfBirth != null) ? Convert.ToDateTime(sPatient.DateOfBirth).ToString("yyyyMMdd") : "";
																if (messageBase.DateOfBirth != dob_)
																{
																	subscriberDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", 1, "N", "71", "C", "Invalid/Missing/Mismatch Date-of-Birth.  Onfile=" + dob_));
																}
															}
														}

														messageBase.Patient = sPatient;

														foreach (var cmd in settings.SqlCommands)
														{
															if (cmd.CommandText.Contains("[DateOfBirth]"))
															{
																cmd.CommandText = cmd.CommandText.Replace("[DateOfBirth]", (sPatient.DateOfBirth == null) ? "null" : "'" + sPatient.DateOfBirth + "'");
																cmd.CommandText = cmd.CommandText.Replace("[Gender]", sPatient.Gender);
															}
														}
													}

												}



												//---- Validate N3/N4 
												if (settings.TransType == "005010X217")
												{
													List<AAA> aaas = EntityBaseManager.ValidN3N4(settings, "2010C", subscriberDetail.SubscriberName.PartyLocation, subscriberDetail.SubscriberName.GeographicLocation, "79");
													if (aaas.Count > 0)
													{
														subscriberDetail.SubscriberName.PartyLocation = null;
														subscriberDetail.SubscriberName.GeographicLocation = null;
														foreach (var aaa in aaas)
														{
															subscriberDetail.SubscriberName.RequestValidation.Add(aaa);
														}
													}
												}


												// validation count
												int validationCount1 = (subscriberDetail != null) ? subscriberDetail.RequestValidation.Count : 0;
												int validationCount2 = (subscriberDetail.SubscriberName != null) ? subscriberDetail.SubscriberName.RequestValidation.Count : 0;
												int validationCount3 = (subscriberDetail.Other != null) ? subscriberDetail.Other.RequestValidation.Count : 0;

												//set AAA limit of 9 - no duplicates
												if (validationCount1 > 0 || validationCount2 > 0 || validationCount3 > 0)
												{

													List<AAA> mainValidation = new List<AAA>();
													string gotAAA = ":";
													var gotCount = 0;

													if (subscriberDetail != null)
													{
														foreach (var aaa in subscriberDetail.RequestValidation)
														{
															if (gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;
															mainValidation.Add(aaa);
															gotAAA += aaa.RejectReasonCode.Code + ":";
															gotCount++;
															if (gotCount >= 9) break;
														}

														subscriberDetail.RequestValidation.Clear();
													}


													if (subscriberDetail.SubscriberName != null)
													{
														foreach (var aaa in subscriberDetail.SubscriberName.RequestValidation)
														{
															if (gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;
															mainValidation.Add(aaa);
															gotAAA += aaa.RejectReasonCode.Code + ":";
															gotCount++;
															if (gotCount >= 9) break;
														}

														subscriberDetail.SubscriberName.RequestValidation.Clear();
													}

													if (subscriberDetail.Other != null)
													{
														foreach (var aaa in subscriberDetail.Other.RequestValidation)
														{
															if (gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;
															mainValidation.Add(aaa);
															gotAAA += aaa.RejectReasonCode.Code + ":";
															gotCount++;
															if (gotCount >= 9) break;
														}

														subscriberDetail.Other.RequestValidation.Clear();
													}

													subscriberDetail.RequestValidation = mainValidation;
													subscriberDetail.SubscriberLevel.HierarchicalChildCode = new IDENTIFIER("0");
													objReturn.SubscriberDetail.Add(subscriberDetail);

													objReturn.ErrorFound = true;

													foreach (var cmd in settings.SqlCommands)
													{
														if (cmd.CommandText.Contains("[DateOfBirth]"))
														{
															cmd.CommandText = cmd.CommandText.Replace("[DateOfBirth]", "null");
															cmd.CommandText = cmd.CommandText.Replace("[Gender]", "");
														}
													}
													continue;
												}

												objReturn.SubscriberDetail.Add(subscriberDetail);
											}
											#endregion

											#region DependentDetail
											else if (hierarchicalLevelCode == "23")  //---------  DependentDetail  2010D  
											{

												if (objReturn.DependentDetail == null) objReturn.DependentDetail = new List<DependentDetail>();
												DependentDetail dependentDetail = DependentDetailManager.Parse(ref settings, "2010D", loop);
												if (objReturn.ErrorFound)
												{
													objReturn.DependentDetail.Add(dependentDetail);
													continue;
												}

												//---- Validate N3/N4 
												if (settings.TransType == "005010X217")
												{
													List<AAA> aaas = EntityBaseManager.ValidN3N4(settings, "2010D", dependentDetail.DependentName.PartyLocation, dependentDetail.DependentName.GeographicLocation, "15");
													if (aaas.Count > 0)
													{
														dependentDetail.DependentName.PartyLocation = null;
														dependentDetail.DependentName.GeographicLocation = null;
														foreach (var aaa in aaas)
														{
															dependentDetail.DependentName.RequestValidation.Add(aaa);
														}
													}
												}


												int validationCount1 = (dependentDetail != null) ? dependentDetail.RequestValidation.Count : 0;
												int validationCount2 = (dependentDetail.DependentName != null) ? dependentDetail.DependentName.RequestValidation.Count : 0;
												int validationCount3 = (dependentDetail.Other != null) ? dependentDetail.Other.RequestValidation.Count : 0;

												// automatically return no 65 - Missing Patient
												dependentDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010D", 1, "N", "65", "C", "Missing Patient"));
												dependentDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010D", 1, "N", "67", "C", "Patient Not Found"));
												dependentDetail.DependentLevel.HierarchicalChildCode = new IDENTIFIER("0");
												
												objReturn.ErrorFound = true;

												objReturn.DependentDetail.Add(dependentDetail);
											}
											#endregion

											#region HL4
											else if (hierarchicalLevelCode == "EV")  //---------  HL4   2010E  --- patient event detail
											{

												if (objReturn.PatientEventDetail == null) objReturn.PatientEventDetail = new List<PatientEventDetail>();
												PatientEventDetail patientEventDetail = new PatientEventDetail(settings);
												patientEventDetail = PatientEventDetailManager.Parse(ref settings, "2000E", objReturn.PatientEventDetail.Count + 1, loop);
												if (objReturn.ErrorFound)
												{
													objReturn.PatientEventDetail.Add(patientEventDetail);
													continue;
												}


												// set EventTraceNumber
												if (patientEventDetail.PatientEventTraceNumber.Count > 0)
												{
													foreach (var patientEventTraceNumber in patientEventDetail.PatientEventTraceNumber)
													{
														messageBase.EventTraceNumber += patientEventTraceNumber.ReferenceIdentification1 + ",";
													}

													messageBase.EventTraceNumber = messageBase.EventTraceNumber.Substring(0, messageBase.EventTraceNumber.Length - 1);
												}

												// set dates
												DateTime? fromDate = null;
												DateTime? toDate = null;
												DateTime? admissionDateFrom = null;
												DateTime? admissionDateTo = null;
												DateTime? issueDateFrom = null;
												DateTime? issueDateTo = null;


												// Validate event dates
												bool validDates = true;
												if (patientEventDetail.EventDate != null)
												{
													patientEventDetail.EventDate.GetDates(ref fromDate, ref toDate);

													settings.LogMessages.AppendLine("Search fromDate/toDate " + fromDate + " - " + toDate);
													if (fromDate == null || toDate == null)
													{
														patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "57", "C", "Invalid/missing event date."));
													}
												}

												// Validate Admission dates
												if (patientEventDetail.AdmissionDate != null)
												{
													patientEventDetail.AdmissionDate.GetDates(ref admissionDateFrom, ref admissionDateTo);
													settings.LogMessages.AppendLine("Search admissionDateFrom/admissionDateTo " + admissionDateFrom + " - " + admissionDateTo);
													if (admissionDateFrom == null || admissionDateTo == null)
													{
														patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "AM", "C", "Invalid/missing admission date."));
														validDates = false;
													}
												}

												// Validate CertificationIssueDate dates
												if (patientEventDetail.CertificationIssueDate != null)
												{
													patientEventDetail.CertificationIssueDate.GetDates(ref issueDateFrom, ref issueDateTo);

													settings.LogMessages.AppendLine("Search CertificationIssueDate " + issueDateFrom + " - " + issueDateTo);
													if (issueDateFrom == null || issueDateTo == null)
													{
														patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "57", "C", "Invalid/missing event date."));
													}
												}


												if (fromDate != null || toDate != null)
												{
													if (fromDate != null && toDate == null)
													{
														toDate = fromDate;
													}
													if (fromDate == null && toDate != null)
													{
														fromDate = toDate;
													}
												}

												if (admissionDateFrom != null || admissionDateTo != null)
												{
													if (admissionDateFrom != null && admissionDateTo == null)
													{
														admissionDateTo = admissionDateFrom;
													}
													if (admissionDateFrom == null && admissionDateTo != null)
													{
														admissionDateFrom = admissionDateTo;
													}
												}

												if (issueDateFrom != null || issueDateTo != null)
												{
													if (issueDateFrom != null && issueDateTo == null)
													{
														issueDateTo = issueDateFrom;
													}
													if (issueDateFrom == null && issueDateTo != null)
													{
														issueDateFrom = issueDateTo;
													}
												}


												patientEventDetail.FromDate = fromDate;
												patientEventDetail.ToDate = toDate;
												patientEventDetail.IssueDateFrom = issueDateFrom;
												patientEventDetail.IssueDateTo = issueDateTo;
												patientEventDetail.AdmissionDateFrom = admissionDateFrom;
												patientEventDetail.AdmissionDateTo = admissionDateTo;


												// set requestCategoryCode
												string requestCategoryCode = (patientEventDetail.HealthCareServicesReviewInformation != null && patientEventDetail.HealthCareServicesReviewInformation.RequestCategoryCode != null) ? patientEventDetail.HealthCareServicesReviewInformation.RequestCategoryCode.Code : "";



												if (requestCategoryCode == "AR")
												{
													if (admissionDateFrom == null && validDates)  // 435
													{
														patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "AM", "C", "Invalid/missing admission date."));
														validDates = false;
													}
												}

												// Patient(s) and Vendor(s) no match on St3n
												if (aaaInEventLoop)
												{
													patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "AA", "C", "Authorization not found"));
												}

												if (hasMultipleRecords && validDates && !aaaInEventLoop)
												{

													List<PatientVendor> patientVendors = authorizationManager.GetPatientVendor(ssn, fromDate, toDate, admissionDateFrom, admissionDateTo, idNumber, npi, medicareIDNumber);
													//Utils.LogEvent("Datebase query for Patient/Vendor");
													if (patientVendors.Count > 0)
													{
														//GetPatientBySID	
														SPatient sPatient = authorizationManager.GetPatientBySID(patientVendors[0].PatientSID);

														string patientName5 = (sPatient.PatientLastName.Length > 5) ? sPatient.PatientLastName.ToUpper().Substring(0, 5) : sPatient.PatientLastName.ToUpper();
														string name5 = (nameLast.Length > 5) ? nameLast.ToUpper().Substring(0, 5) : nameLast.ToUpper();

														if (patientName5 != name5 || nameLast.Length == 0)
														{
															// put in subscriber loop 
															if (objReturn.SubscriberDetail.Count > 0)
															{
																objReturn.SubscriberDetail[0].RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "65", "C", "Mismatched Patient Last Name"));
															}
														}

														// Invalid/Missing Date-of-Birth
														if (messageBase.DateOfBirth.Length > 0)
														{
															if (DateTime.TryParseExact(messageBase.DateOfBirth, formats, new System.Globalization.CultureInfo("en-US"), System.Globalization.DateTimeStyles.None, out dTemp))
															{
																string dob_ = (sPatient.DateOfBirth != null) ? Convert.ToDateTime(sPatient.DateOfBirth).ToString("yyyyMMdd") : "";
																if (messageBase.DateOfBirth != dob_)
																{
																	patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "71", "C", "Invalid/Missing/Mismatch Date-of-Birth.  Onfile=" + dob_));

																	patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "AA", "C", "Authorization not found"));
																}
															}
														}

														messageBase.Patient = sPatient;

														List<SFeeVendor> vends = new List<SFeeVendor>();
														foreach (var patientVendor in patientVendors)
														{
															vends.Add(authorizationManager.GetVendorBySID(patientVendor.FeeVendorSID));
														}

														messageBase.FeeVendors = vends;
													}
													else
													{
														patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "AA", "C", "Authorization not found"));
													}

												}


												//Exclude IN - Individual and SC - Special care review  UM is required for 217
												if (settings.TransType == "005010X217")
												{
													if (requestCategoryCode.Length == 0 || requestCategoryCode == "IN" || requestCategoryCode == "SC")
													{
														patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "33", "C", "HL4.PatientEventDetail - Input Errors.  Special care review should not be blank, IN, or SC"));
													}
												}

												// Validate  certificationTypeCode - required for 217.  optional for 215 but in values if present
												string certificationTypeCode = (patientEventDetail.HealthCareServicesReviewInformation != null && patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode != null) ? patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code : "";
												if (!validCertificationTypeCode.Contains("|" + certificationTypeCode + "|"))
												{
													patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "T5", "C", "PatientEventDetail - Certification Information Missing"));
												}

												//  PatientEventProviderName loop 2010EA
												var validationCount2 = 0;
												if (patientEventDetail.PatientEventProviderName != null)
												{
													foreach (var providerName in patientEventDetail.PatientEventProviderName)
													{
														if (providerName.RequestValidation.Count > 0)
														{
															validationCount2++;
														}

														//---- Validate N3/N4 
														List<AAA> aaas = EntityBaseManager.ValidN3N4(settings, "2010EA", providerName.PartyLocation, providerName.GeographicLocation, "97");
														if (aaas.Count > 0)
														{
															providerName.PartyLocation = null;
															providerName.GeographicLocation = null;
															foreach (var aaa in aaas)
															{
																providerName.RequestValidation.Add(aaa);
																validationCount2++;
															}
														}
													}
												}


												// REQUEST ONLY Transport - Send RESPONSE in 2010EB Patient Event Transport Information
												var validationCount1 = 0;
												if (patientEventDetail.PatientEventTransportInformation != null)
												{
													if (settings.TransType == "005010X217")
													{
														foreach (var transport in patientEventDetail.PatientEventTransportInformation)
														{
															bool foundAAA = false;


															// N3
															if (transport.PartyLocation == null)
															{
																transport.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "97", "C", "PatientEventTransportInformation - N3 is required"));
																validationCount1++;
																foundAAA = true;
															}
															else
															{
																if (transport.PartyLocation.AddressInformation1.Trim() == "")
																{
																	transport.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "97", "C", "PatientEventTransportInformation - N301 is missing"));
																	validationCount1++;
																	foundAAA = true;
																}
															}

															// N4
															if (transport.GeographicLocation != null)
															{
																if (transport.GeographicLocation.PostalCode.Code.Trim() == "")
																{
																	if (transport.GeographicLocation.CityName.Trim() == "")
																	{
																		transport.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "97", "C", "PatientEventTransportInformation - N4 - PostalCode is null and CityName is required"));
																		validationCount1++;
																		foundAAA = true;
																	}
																	if (transport.GeographicLocation.StateOrProvinceCode.Code.Trim() == "")
																	{
																		transport.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "97", "C", "PatientEventTransportInformation - N4 - PostalCode is null and StateOrProvinceCode is required"));
																		validationCount1++;
																		foundAAA = true;
																	}
																}

																if (transport.GeographicLocation.CityName.Trim() == "" && transport.GeographicLocation.StateOrProvinceCode.Code.Trim() == "")
																{
																	if (transport.GeographicLocation.PostalCode.Code.Trim() == "")
																	{
																		transport.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "97", "C", "PatientEventTransportInformation - N4 - CityName and StateOrProvinceCode is null.  PostalCode is required "));
																		validationCount1++;
																		foundAAA = true;
																	}
																}
															}


															if (foundAAA)
															{
																transport.PartyLocation = null;
																transport.GeographicLocation = null;
															}


														}
													}


													if (patientEventDetail.PatientEventTransportInformation.Count < 2)
													{
														patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "15", "C", "PatientEventTransportInformation.Count < 2"));
														validationCount1++;
													}
												}


												// validation count
												int validationCount3 = (patientEventDetail.RequestValidation != null) ? patientEventDetail.RequestValidation.Count : 0;
												int validationCount4 = (patientEventDetail.AdditionalPatientInformationContactName != null) ? patientEventDetail.AdditionalPatientInformationContactName.RequestValidation.Count : 0;
												int validationCount5 = (patientEventDetail.Other != null) ? patientEventDetail.Other.RequestValidation.Count : 0;

												if (validationCount3 > 0 || validationCount4 > 0 || validationCount1 > 0 || validationCount2 > 0 || validationCount5 > 0)
												{
													patientEventDetail.ErrorFound = true;
													// clear any message segment
													foreach (var item in objReturn.PatientEventDetail)
													{
														item.MessageText = null;
													}
												}

												if (settings.TransType == "005010X217" && objReturn.PatientEventDetail.Count == 1)
												{
													patientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", 1, "N", "???", "C", "217 cannot have more than one EV loop"));
													return objReturn;
												}

												messageBase.PatientEventDetails.Add(patientEventDetail);

												objReturn.PatientEventDetail.Add(patientEventDetail);

												if (settings.TransType == "005010X217" && patientEventDetail.ErrorFound)
												{
													return objReturn;
												}

											}
											#endregion

											#region Service Detail


											else if (hierarchicalLevelCode == "SS")  //--------- Service Detail 2010F
											{
												if (objReturn.ServiceDetail == null) objReturn.ServiceDetail = new List<ServiceDetail>();

												ServiceDetail serviceDetail = new ServiceDetail(settings);
												serviceDetail = ServiceDetailManager.Parse(ref settings, "2000F", objReturn.ServiceDetail.Count + 1, loop);
												if (objReturn.ErrorFound)
												{
													objReturn.ServiceDetail.Add(serviceDetail);
													continue;
												}

												var validationCount = 0;
												if (serviceDetail.ServiceProviderName != null)
												{
													string requestCategoryCode = (serviceDetail.HealthCareServicesReviewInformation != null && serviceDetail.HealthCareServicesReviewInformation.RequestCategoryCode != null) ? serviceDetail.HealthCareServicesReviewInformation.RequestCategoryCode.Code : "";
													if (requestCategoryCode.Length == 0 || requestCategoryCode != "HS")
													{
														serviceDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000F", 1, "N", "33", "C", "217.NA.ServiceDetail - Input Errors - requestCategoryCode must be HS"));
													}

													// Validate  certificationTypeCode
													string certificationTypeCode = (serviceDetail.HealthCareServicesReviewInformation != null && serviceDetail.HealthCareServicesReviewInformation.CertificationTypeCode != null) ? serviceDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code : "";

													if (!validCertificationTypeCode.Contains("|" + certificationTypeCode + "|"))
													{
														serviceDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000F", 1, "N", "T5", "C", "PatientEventDetail - Certification Information Missing"));
													}

													foreach (var item in serviceDetail.ServiceProviderName)
													{
														if (item.RequestValidation.Count > 0)
														{
															validationCount++;
														}
													}


													//---- Validate N3/N4 
													List<AAA> aaas = EntityBaseManager.ValidN3N4(settings, "2010EA", serviceDetail.PartyLocation, serviceDetail.GeographicLocation, "97");
													if (aaas.Count > 0)
													{
														serviceDetail.PartyLocation = null;
														serviceDetail.GeographicLocation = null;

														foreach (var aaa in aaas)
														{
															serviceDetail.RequestValidation.Add(aaa);
															validationCount++;
														}
													}

													// validation count
													int validationCount2 = (serviceDetail.RequestValidation != null) ? serviceDetail.RequestValidation.Count : 0;
													int validationCount3 = (serviceDetail.AdditionalServiceInformationContactName != null) ? serviceDetail.AdditionalServiceInformationContactName.RequestValidation.Count : 0;
													int validationCount4 = (serviceDetail.Other != null) ? serviceDetail.Other.RequestValidation.Count : 0;

													if (validationCount2 > 0 || validationCount3 > 0 || validationCount > 0 || validationCount4 > 0)
													{

														serviceDetail.ServiceLevel.HierarchicalChildCode = new IDENTIFIER("0");
														objReturn.ServiceDetail.Add(serviceDetail);
													
														objReturn.ErrorFound = true;
													}

													objReturn.ServiceDetail.Add(serviceDetail);
												}
											}
											#endregion

											break;

										default:
											break;
									}
								}
							}
						}
					}
				}

				return objReturn;
			//}
			//catch (Exception ex)
			//{
			//	Utils.LogEvent(string.Format("ParseMessage(): {0}. ", ex.Message) + "\r\n" + ex.StackTrace);
			//	return null;
			//}


		}






	}
}
