﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;
using X12Engine.BO.X12._278.XModel;

namespace X12Engine.BLL.X12._278.XParse
{
   public static class PatientEventDetailManager
   {
      public static PatientEventDetail Parse(ref Settings settings, string loopCode, int loopNumber, string loop)
      {
         PatientEventDetail objReturn = new PatientEventDetail(settings);
			List<AAA> requestValidation = new List<AAA>();

			string[] segments = loop.Split(settings.Segment_Terminator);
         if (segments != null)
         {
            bool contactEntityFound = false;
            bool transportEntityFound = false;
				bool providerEntityFound = false;

				EntityBase patientEventProviderName = null;
            EntityBase patientEventTransportInformation = null;

            foreach (string segment in segments)
            {
               if (string.IsNullOrEmpty(segment)) continue;

               string[] elements = segment.Split(settings.Data_Element_Separator);
               if (elements != null && !string.IsNullOrEmpty(elements[0]))
               {
                  string segmentId = elements[0].ToUpper();
                  string qualifier = elements[1].ToUpper();

						if (segmentId == "HL")
							objReturn.PatientEventLevel = HLManager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "TRN")
						{
							if (objReturn.PatientEventTraceNumber == null) objReturn.PatientEventTraceNumber = new List<TRN>();
							objReturn.PatientEventTraceNumber.Add(TRNManager.Parse(ref settings, loopCode, segment, ref requestValidation));
						}
						else if (segmentId == "UM")
							objReturn.HealthCareServicesReviewInformation = UMManager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
						else if (segmentId == "HCR")
							objReturn.HealthCareServicesReview = HCRManager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "REF" && qualifier == "BB")
							objReturn.PreviousReviewAuthorizationNumber = REFManager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "REF" && qualifier == "NT")
							objReturn.PreviousReviewAdministrativeReferenceNumber = REFManager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "")
							requestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", loopNumber, "N", "15", "C", "DTP - bad qualifier in segment " + segment));
						else if (segmentId == "DTP" && qualifier == "439")
							objReturn.AccidentDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "484")
							objReturn.LastMenstrualPeriodDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "ABC")
							objReturn.EstimatedDateOfBirth = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "431")
							objReturn.OnsetOfCurrentSymptomsOrIllnessDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "AAH")        //----------------------------------- Event Date 
							objReturn.EventDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "435")
							objReturn.AdmissionDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "096")
							objReturn.DischargeDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "102")      //-------- Issue Date
							objReturn.CertificationIssueDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "036")
							objReturn.CertificationExpirationDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "007")
							objReturn.CertificationEffectiveDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "DTP" && qualifier == "881")
							objReturn.HealthCareServicesReviewRequestDate = DTPManager.Parse(ref settings, "2000E", loopNumber, segment, ref requestValidation);
						else if (segmentId == "HI")
							objReturn.PatientDiagnosis = HIManager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "HSD")
							objReturn.HealthCareServicesDelivery = HSDManager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "CRC")
							objReturn.AmbulanceCertificationInformation = CRCManager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "CL1")
							objReturn.InstitutionalClaimCode = CL1Manager.Parse(ref settings, loopCode, loopNumber, segment, ref requestValidation);
						else if (segmentId == "CR1")
							objReturn.AmbulanceTransportInformation = CR1Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "CR2")
							objReturn.SpinalManipulationServiceInformation = CR2Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "CR5")
							objReturn.HomeOxygenTherapyInformation = CR5Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "CR6")
							objReturn.HomeHealthCareInformation = CR6Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
						else if (segmentId == "PWK")
						{
							if (objReturn.AdditionalPatientInformation == null) objReturn.AdditionalPatientInformation = new List<PWK>();
							objReturn.AdditionalPatientInformation.Add(PWKManager.Parse(ref settings, loopCode,  loopNumber, segment, ref requestValidation));
						}
						else if (segmentId == "MSG")
							objReturn.MessageText = new MSG(settings, "");
						else if (Utils.IsEntitySegment(segmentId))
						{

							if (segmentId == "NM1")
							{
								if (Utils.IsTransportEntity(qualifier))
								{
									transportEntityFound = true;
								}
								else
								{
									transportEntityFound = false;
								}


								if (Utils.IsContactEntity(qualifier))
								{
									contactEntityFound = true;
								}
								else
								{
									contactEntityFound = false;
								}

								if (Utils.IsProviderEntity(qualifier))
								{
									providerEntityFound = true;
								}
								else
								{
									providerEntityFound = false;
								}

								if (!transportEntityFound && !contactEntityFound && !providerEntityFound)
								{
									objReturn.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", loopNumber, "N", "15", "C", "Invalid NM101 qualifier.  Unable to determine EV NM1 segment"));
									objReturn.ErrorFound = true;
									return objReturn;
								}
							}


							if (transportEntityFound)
							{
								contactEntityFound = false;
								providerEntityFound = false;

								if (objReturn.PatientEventTransportInformation == null) objReturn.PatientEventTransportInformation = new List<EntityBase>();

								if (segmentId == "NM1")
								{
									if (patientEventTransportInformation != null) objReturn.PatientEventTransportInformation.Add(patientEventTransportInformation);
									patientEventTransportInformation = new EntityBase();
								}

								patientEventTransportInformation = EntityBaseManager.Parse(ref settings, "2010EB", loopNumber, segment, patientEventTransportInformation);
							}
							else if (contactEntityFound)
							{
								providerEntityFound = false;
								transportEntityFound = false;

								// do not send.    objReturn.AdditionalPatientInformationContactName = EntityBaseManager.Parse(ref settings, segment, objReturn.AdditionalPatientInformationContactName);
							}
							else if (providerEntityFound)
							{
								contactEntityFound = false;
								transportEntityFound = false;

								if (objReturn.PatientEventProviderName == null) objReturn.PatientEventProviderName = new List<EntityBase>();

								if (segmentId == "NM1")
								{
									if (patientEventProviderName != null) objReturn.PatientEventProviderName.Add(patientEventProviderName);
									patientEventProviderName = new EntityBase();
								}

								patientEventProviderName = EntityBaseManager.Parse(ref settings, "2010EA", loopNumber, segment, patientEventProviderName);
							}
						}
						else
						{
							objReturn.Other = DetailBaseManager.Parse(ref settings, "2010EC", loopNumber, segment, objReturn.Other);
						}
							
               }
            }

            if (patientEventProviderName != null) objReturn.PatientEventProviderName.Add(patientEventProviderName);
            if (patientEventTransportInformation != null) objReturn.PatientEventTransportInformation.Add(patientEventTransportInformation);

				objReturn.RequestValidation = requestValidation;
			}

         return objReturn;
      }
   }
}
