﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;
using X12Engine.BO.X12._278.XModel;

namespace X12Engine.BLL.X12._278.XParse
{
   public static class ServiceDetailManager
   {
      public static ServiceDetail Parse(ref Settings settings, string loopCode, int loopNumber, string loop)
      {
         ServiceDetail objReturn = new ServiceDetail(settings);
			List<AAA> requestValidation = new List<AAA>();

			string[] segments = loop.Split(settings.Segment_Terminator);
         if (segments != null)
         {

            EntityBase serviceProviderName = null;

            foreach (string segment in segments)
            {
               if (string.IsNullOrEmpty(segment)) continue;

               string[] elements = segment.Split(settings.Data_Element_Separator);
               if (elements != null && !string.IsNullOrEmpty(elements[0]))
               {
                  string segmentId = elements[0].ToUpper();
                  string qualifier = elements[1].ToUpper();

                  if (segmentId == "HL")
                        objReturn.ServiceLevel = HLManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  else if (segmentId == "TRN")
                  {
                        if (objReturn.ServiceTraceNumber == null) objReturn.ServiceTraceNumber = new List<TRN>();
                        objReturn.ServiceTraceNumber.Add(TRNManager.Parse(ref settings, loopCode, segment, ref requestValidation));
                  }
                  else if (segmentId == "UM")
                        objReturn.HealthCareServicesReviewInformation = UMManager.Parse(ref settings, "2000F", loopNumber, segment, ref requestValidation);
                  else if (segmentId == "HCR")
                        objReturn.HealthCareServicesReview = HCRManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  else if (segmentId == "REF" && qualifier == "BB")
                        objReturn.PreviousReviewAuthorizationNumber = REFManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  else if (segmentId == "REF" && qualifier == "NT")
                        objReturn.PreviousReviewAuthorizationNumber = REFManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  else if (segmentId == "DTP" && qualifier == "472")
                        objReturn.ServiceDate = DTPManager.Parse(ref settings, "2000F", loopNumber, segment, ref requestValidation);
                  else if (segmentId == "DTP" && qualifier == "102")
                        objReturn.CertificationIssueDate = DTPManager.Parse(ref settings, "2000F", loopNumber, segment, ref requestValidation);
                  else if (segmentId == "DTP" && qualifier == "036")
                        objReturn.CertificationExpirationDate = DTPManager.Parse(ref settings, "2000F", loopNumber, segment, ref requestValidation);
                  else if (segmentId == "DTP" && qualifier == "007")
                        objReturn.CertificationEffectiveDate = DTPManager.Parse(ref settings, "2000F", loopNumber, segment, ref requestValidation);
                  else if (segmentId == "SV1")
                        objReturn.ProfessionalService = SV1Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  else if (segmentId == "SV2")
                        objReturn.InstitutionalService = SV2Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  else if (segmentId == "SV3")
                        objReturn.DentalService = SV3Manager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  else if (segmentId == "TOO")
                  {
                        if (objReturn.ToothInformation == null) objReturn.ToothInformation = new List<TOO>();
                        objReturn.ToothInformation.Add(TOOManager.Parse(ref settings, loopCode, segment, ref requestValidation));
                  }
                  else if (segmentId == "HSD")
                        objReturn.HealthCareServicesDelivery = HSDManager.Parse(ref settings, loopCode, segment, ref requestValidation);
                  else if (segmentId == "PWK")
                  {
                        if (objReturn.AdditionalServiceInformation == null) objReturn.AdditionalServiceInformation = new List<PWK>();
                        objReturn.AdditionalServiceInformation.Add(PWKManager.Parse(ref settings, "2000F", loopNumber, segment, ref requestValidation));
                  }
                  else if (segmentId == "MSG")
                        objReturn.MessageText = MSGManager.Parse(ref settings, "2000F", loopNumber, segment, ref requestValidation);  
						else if (Utils.IsEntitySegment(segmentId))
                  {

							if (objReturn.ServiceProviderName == null) objReturn.ServiceProviderName = new List<EntityBase>();
							if (segmentId == "NM1")
							{
								if (serviceProviderName != null) objReturn.ServiceProviderName.Add(serviceProviderName);
								serviceProviderName = new EntityBase();
							}
							serviceProviderName = EntityBaseManager.Parse(ref settings, "2010F", objReturn.ServiceProviderName.Count, segment, serviceProviderName);

						}
                  else if (segmentId == "SE")
                  {
                        //do nothing, will be parsed in the main loop
                  }
                  else
						{

						}
               }
            }

            if (serviceProviderName != null) objReturn.ServiceProviderName.Add(serviceProviderName);

				objReturn.ServiceRequestValidation = requestValidation;
			}

         return objReturn;
      }
   }
}
