﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace X12Engine.BO.X12._278
{


	public class DataRoutingCode
	{
		public int RuleNumber { get; set; }
		public int SequenceNumber { get; set; }
		public string Description { get; set; }
		public string LoopCode { get; set; }
		public string TR3Id { get; set; }
		public string TableName { get; set; }
		public string RouteType { get; set; }
		public string CodeName { get; set; }
		public string Condition { get; set; }
		public string RouteValue { get; set; }
		public int QueueId { get; set; }
	}


	[Serializable()]
	public class HL
	{
		private Settings settings;

		public HL(Settings s)
		{
			settings = s;
		}

		public HL(Settings s, IDENTIFIER hierarchicalLevelCode)
		{
			settings = s;
			_hierarchicalLevelCode = hierarchicalLevelCode;
		}

		public HL(Settings s, string hierarchicalIdNumber, string hierarchicalParentIdNumber, IDENTIFIER hierarchicalLevelCode, IDENTIFIER hierarchicalChildCode)
		{
			settings = s;
			_hierarchicalIdNumber = hierarchicalIdNumber;
			_hierarchicalParentIdNumber = hierarchicalParentIdNumber;
			_hierarchicalLevelCode = hierarchicalLevelCode;
			_hierarchicalChildCode = hierarchicalChildCode;

		}

		private string _hierarchicalIdNumber;
		private string _hierarchicalParentIdNumber;
		private IDENTIFIER _hierarchicalLevelCode;
		private IDENTIFIER _hierarchicalChildCode;

		public string HierarchicalIdNumber
		{
			get { return _hierarchicalIdNumber; }
			set { _hierarchicalIdNumber = value; }
		}

		public string HierarchicalParentIdNumber
		{
			get { return _hierarchicalParentIdNumber; }
			set { _hierarchicalParentIdNumber = value; }
		}

		public IDENTIFIER HierarchicalLevelCode
		{
			get { return _hierarchicalLevelCode; }
			set { _hierarchicalLevelCode = value; }
		}

		public IDENTIFIER HierarchicalChildCode
		{
			get { return _hierarchicalChildCode; }
			set { _hierarchicalChildCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "HL" + settings.Data_Element_Separator;

			strReturn += _hierarchicalIdNumber + settings.Data_Element_Separator;

			strReturn += _hierarchicalParentIdNumber + settings.Data_Element_Separator;

			if (_hierarchicalLevelCode != null)
				strReturn += _hierarchicalLevelCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_hierarchicalChildCode != null)
				strReturn += _hierarchicalChildCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class TRN
	{
		private Settings settings;

		public TRN(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _traceTypeCode;
		private string _referenceIdentification1;
		private string _originatingCompanyIdentifier;
		private string _referenceIdentification2;

		public IDENTIFIER TraceTypeCode
		{
			get { return _traceTypeCode; }
			set { _traceTypeCode = value; }
		}

		public string ReferenceIdentification1
		{
			get { return _referenceIdentification1; }
			set { _referenceIdentification1 = value; }
		}

		public string OriginatingCompanyIdentifier
		{
			get { return _originatingCompanyIdentifier; }
			set { _originatingCompanyIdentifier = value; }
		}

		public string ReferenceIdentification2
		{
			get { return _referenceIdentification2; }
			set { _referenceIdentification2 = value; }
		}


		public override string ToString()
		{
			string strReturn = "TRN" + settings.Data_Element_Separator;

			if (_traceTypeCode != null)
				strReturn += _traceTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _referenceIdentification1 + settings.Data_Element_Separator;

			strReturn += _originatingCompanyIdentifier + settings.Data_Element_Separator;

			strReturn += _referenceIdentification2 + settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class AAA
	{
		private Settings settings;
		public AAA(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _yesNoConditionOrResponseCode;
		private IDENTIFIER _agencyQualifierCode;
		private IDENTIFIER _rejectReasonCode;
		private IDENTIFIER _followUpActionCode;
		public string ErrorMessage = "";

		public IDENTIFIER YesNoConditionOrResponseCode
		{
			get { return _yesNoConditionOrResponseCode; }
			set { _yesNoConditionOrResponseCode = value; }
		}

		public IDENTIFIER AgencyQualifierCode
		{
			get { return _agencyQualifierCode; }
			set { _agencyQualifierCode = value; }
		}

		public IDENTIFIER RejectReasonCode
		{
			get { return _rejectReasonCode; }
			set { _rejectReasonCode = value; }
		}

		public IDENTIFIER FollowUpActionCode
		{
			get { return _followUpActionCode; }
			set { _followUpActionCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "AAA" + settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode != null)
				strReturn += _yesNoConditionOrResponseCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_agencyQualifierCode != null)
				strReturn += _agencyQualifierCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_rejectReasonCode != null)
				strReturn += _rejectReasonCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_followUpActionCode != null)
				strReturn += _followUpActionCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class UM
	{

		private Settings settings;
		public UM(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _requestCategoryCode;
		private IDENTIFIER _certificationTypeCode;
		private IDENTIFIER _serviceTypeCode;
		private HEALTH_CARE_SERVICE_LOCATION_INFORMATION _healthCareServiceLocationInformation;
		private RELATED_CAUSES_INFORMATION _relatedCausesInformation;
		private IDENTIFIER _levelOfServiceCode;
		private IDENTIFIER _currentHealthConditionCode;
		private IDENTIFIER _prognosisCode;
		private IDENTIFIER _releaseOfInformationCode;
		private IDENTIFIER _delayReasonCode;

		public IDENTIFIER RequestCategoryCode
		{
			get { return _requestCategoryCode; }
			set { _requestCategoryCode = value; }
		}

		public IDENTIFIER CertificationTypeCode
		{
			get { return _certificationTypeCode; }
			set { _certificationTypeCode = value; }
		}

		public IDENTIFIER ServiceTypeCode
		{
			get { return _serviceTypeCode; }
			set { _serviceTypeCode = value; }
		}

		public HEALTH_CARE_SERVICE_LOCATION_INFORMATION HealthCareServiceLocationInformation
		{
			get { return _healthCareServiceLocationInformation; }
			set { _healthCareServiceLocationInformation = value; }
		}

		public RELATED_CAUSES_INFORMATION RelatedCausesInformation
		{
			get { return _relatedCausesInformation; }
			set { _relatedCausesInformation = value; }
		}

		public IDENTIFIER LevelOfServiceCode
		{
			get { return _levelOfServiceCode; }
			set { _levelOfServiceCode = value; }
		}

		public IDENTIFIER CurrentHealthConditionCode
		{
			get { return _currentHealthConditionCode; }
			set { _currentHealthConditionCode = value; }
		}

		public IDENTIFIER PrognosisCode
		{
			get { return _prognosisCode; }
			set { _prognosisCode = value; }
		}

		public IDENTIFIER ReleaseOfInformationCode
		{
			get { return _releaseOfInformationCode; }
			set { _releaseOfInformationCode = value; }
		}

		public IDENTIFIER DelayReasonCode
		{
			get { return _delayReasonCode; }
			set { _delayReasonCode = value; }
		}


		public override string ToString()
		{
			string strReturn = "UM" + settings.Data_Element_Separator;

			if (_requestCategoryCode != null)
				strReturn += _requestCategoryCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_certificationTypeCode != null)
				strReturn += _certificationTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_serviceTypeCode != null)
				strReturn += _serviceTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareServiceLocationInformation != null)
				strReturn += _healthCareServiceLocationInformation.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_relatedCausesInformation != null)
				strReturn += _relatedCausesInformation.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_levelOfServiceCode != null)
				strReturn += _levelOfServiceCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_currentHealthConditionCode != null)
				strReturn += _currentHealthConditionCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_prognosisCode != null)
				strReturn += _prognosisCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_releaseOfInformationCode != null)
				strReturn += _releaseOfInformationCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_delayReasonCode != null)
				strReturn += _delayReasonCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class HCR
	{
		private Settings settings;
		public HCR(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _actionCode;
		private string _referenceIdentification;
		private string _industryCode;
		private IDENTIFIER _yesNoConditionOrResponseCode;

		public IDENTIFIER ActionCode
		{
			get { return _actionCode; }
			set { _actionCode = value; }
		}

		public string ReferenceIdentification
		{
			get { return _referenceIdentification; }
			set { _referenceIdentification = value; }
		}

		public string IndustryCode
		{
			get { return _industryCode; }
			set { _industryCode = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode
		{
			get { return _yesNoConditionOrResponseCode; }
			set { _yesNoConditionOrResponseCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "HCR" + settings.Data_Element_Separator;

			if (_actionCode != null)
				strReturn += _actionCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _referenceIdentification + settings.Data_Element_Separator;

			strReturn += _industryCode + settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode != null)
				strReturn += _yesNoConditionOrResponseCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class REF
	{
		private Settings settings;
		public REF(Settings s)
		{
			settings = s;
		}
		public REF(Settings s, string referenceIdentificationQualifier, string UAK)
		{
			settings = s;
			_referenceIdentificationQualifier = new IDENTIFIER(referenceIdentificationQualifier);
			_referenceIdentification = UAK;

		}

		private IDENTIFIER _referenceIdentificationQualifier;
		private string _referenceIdentification;
		private string _description;
		private REFERENCE_IDENTIFIER _referenceIdentifier;

		public IDENTIFIER ReferenceIdentificationQualifier
		{
			get { return _referenceIdentificationQualifier; }
			set { _referenceIdentificationQualifier = value; }
		}

		public string ReferenceIdentification
		{
			get { return _referenceIdentification; }
			set { _referenceIdentification = value; }
		}

		public string Description
		{
			get { return _description; }
			set { _description = value; }
		}

		public REFERENCE_IDENTIFIER ReferenceIdentifier
		{
			get { return _referenceIdentifier; }
			set { _referenceIdentifier = value; }
		}

		public override string ToString()
		{
			string strReturn = "REF" + settings.Data_Element_Separator;

			if (_referenceIdentificationQualifier != null)
				strReturn += _referenceIdentificationQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _referenceIdentification + settings.Data_Element_Separator;

			strReturn += _description + settings.Data_Element_Separator;

			if (_referenceIdentifier != null)
				strReturn += _referenceIdentifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class DTP
	{
		private Settings settings;
		public DTP(Settings s)
		{
			settings = s;
		}

		public DTP(Settings s, string dateTimeQualifier, DateTime date1)
		{
			settings = s;
			_dateTimeQualifier = new IDENTIFIER(dateTimeQualifier);
			_dateTimePeriodFormatQualifier = new IDENTIFIER("D8");
			_dateTimePeriod = date1.ToString("yyyyMMdd");

		}

		public DTP(Settings s, string dateTimeQualifier, DateTime date1, DateTime date2)
		{
			settings = s;
			_dateTimeQualifier = new IDENTIFIER(dateTimeQualifier);
			if (date2 > date1)
			{
				_dateTimePeriodFormatQualifier = new IDENTIFIER("RD8");
				_dateTimePeriod = date1.ToString("yyyyMMdd") + "-" + date2.ToString("yyyyMMdd");
			}
			else
			{
				_dateTimePeriodFormatQualifier = new IDENTIFIER("D8");
				_dateTimePeriod = date1.ToString("yyyyMMdd");
			}

		}

		private IDENTIFIER _dateTimeQualifier;
		private IDENTIFIER _dateTimePeriodFormatQualifier;
		private string _dateTimePeriod;

		public IDENTIFIER DateTimeQualifier
		{
			get { return _dateTimeQualifier; }
			set { _dateTimeQualifier = value; }
		}

		public IDENTIFIER DateTimePeriodFormatQualifier
		{
			get { return _dateTimePeriodFormatQualifier; }
			set { _dateTimePeriodFormatQualifier = value; }
		}

		public string DateTimePeriod
		{
			get { return _dateTimePeriod; }
			set { _dateTimePeriod = value; }
		}

		public void GetDate(ref DateTime? date1)
		{
			if (_dateTimePeriodFormatQualifier != null)
			{
				if (_dateTimePeriodFormatQualifier.Code == "D8")
				{
					date1 = X12Engine.Utils.ParseDate(_dateTimePeriod, "yyyyMMdd");
				}
				else if (_dateTimePeriodFormatQualifier.Code == "RD8")
				{
					string[] dates = _dateTimePeriod.Split('-');
					if (dates != null)
					{
						if (dates.Length > 0) date1 = X12Engine.Utils.ParseDate(dates[0], "yyyyMMdd");
					}
				}
			}
		}

		public void GetDates(ref DateTime? date1, ref DateTime? date2)
		{
			if (_dateTimePeriodFormatQualifier != null)
			{
				if (_dateTimePeriodFormatQualifier.Code == "D8")
				{
					date1 = X12Engine.Utils.ParseDate(_dateTimePeriod, "yyyyMMdd");
					date2 = date1;
				}
				else if (_dateTimePeriodFormatQualifier.Code == "RD8")
				{
					string[] dates = _dateTimePeriod.Split('-');
					if (dates != null)
					{
						if (dates.Length > 0) date1 = X12Engine.Utils.ParseDate(dates[0], "yyyyMMdd");
						if (dates.Length > 1) date2 = X12Engine.Utils.ParseDate(dates[1], "yyyyMMdd");
					}
				}
			}
		}

		public override string ToString()
		{
			string strReturn = "DTP" + settings.Data_Element_Separator;

			if (_dateTimeQualifier != null)
				strReturn += _dateTimeQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_dateTimePeriodFormatQualifier != null)
				strReturn += _dateTimePeriodFormatQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _dateTimePeriod + settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class HI
	{
		private Settings settings;
		public HI(Settings s)
		{
			settings = s;
		}

		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation1;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation2;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation3;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation4;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation5;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation6;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation7;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation8;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation9;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation10;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation11;
		private HEALTH_CARE_CODE_INFORMATION _healthCareCodeInformation12;

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation1
		{
			get { return _healthCareCodeInformation1; }
			set { _healthCareCodeInformation1 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation2
		{
			get { return _healthCareCodeInformation2; }
			set { _healthCareCodeInformation2 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation3
		{
			get { return _healthCareCodeInformation3; }
			set { _healthCareCodeInformation3 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation4
		{
			get { return _healthCareCodeInformation4; }
			set { _healthCareCodeInformation4 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation5
		{
			get { return _healthCareCodeInformation5; }
			set { _healthCareCodeInformation5 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation6
		{
			get { return _healthCareCodeInformation6; }
			set { _healthCareCodeInformation6 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation7
		{
			get { return _healthCareCodeInformation7; }
			set { _healthCareCodeInformation7 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation8
		{
			get { return _healthCareCodeInformation8; }
			set { _healthCareCodeInformation8 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation9
		{
			get { return _healthCareCodeInformation9; }
			set { _healthCareCodeInformation9 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation10
		{
			get { return _healthCareCodeInformation10; }
			set { _healthCareCodeInformation10 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation11
		{
			get { return _healthCareCodeInformation11; }
			set { _healthCareCodeInformation11 = value; }
		}

		public HEALTH_CARE_CODE_INFORMATION HealthCareCodeInformation12
		{
			get { return _healthCareCodeInformation12; }
			set { _healthCareCodeInformation12 = value; }
		}

		public override string ToString()
		{
			string strReturn = "HI" + settings.Data_Element_Separator;

			if (_healthCareCodeInformation1 != null)
				strReturn += _healthCareCodeInformation1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation2 != null)
				strReturn += _healthCareCodeInformation2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation3 != null)
				strReturn += _healthCareCodeInformation3.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation4 != null)
				strReturn += _healthCareCodeInformation4.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation5 != null)
				strReturn += _healthCareCodeInformation5.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation6 != null)
				strReturn += _healthCareCodeInformation6.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation7 != null)
				strReturn += _healthCareCodeInformation7.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation8 != null)
				strReturn += _healthCareCodeInformation8.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation9 != null)
				strReturn += _healthCareCodeInformation9.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation10 != null)
				strReturn += _healthCareCodeInformation10.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation11 != null)
				strReturn += _healthCareCodeInformation11.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareCodeInformation12 != null)
				strReturn += _healthCareCodeInformation12.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class SV1
	{
		private Settings settings;
		public SV1(Settings s)
		{
			settings = s;
		}

		private COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER _compositeMedicalProcedureIdentifier;
		private decimal? _monetaryAmount1;
		private IDENTIFIER _unitOrBasisForMeasurementCode;
		private decimal? _quantity;
		private string _facilityCodeValue;
		private IDENTIFIER _serviceTypeCode;
		private COMPOSITE_DIAGNOSIS_CODE_POINTER _compositeDiagnosisCodePointer;
		private decimal? _monetaryAmount2;
		private IDENTIFIER _yesNoConditionOrResponseCode1;
		private IDENTIFIER _multipleProcedureCode;
		private IDENTIFIER _yesNoConditionOrResponseCode2;
		private IDENTIFIER _yesNoConditionOrResponseCode3;
		private IDENTIFIER _reviewCode;
		private string _nationalOrLocalAssignedReviewValue;
		private IDENTIFIER _copayStatusCode;
		private IDENTIFIER _healthCareProfessionalShortageAreaCode;
		private string _referenceIdentification;
		private IDENTIFIER _postalCode;
		private decimal? _monetaryAmount3;
		private IDENTIFIER _levelOfCareCode;
		private IDENTIFIER _providerAgreementCode;

		public COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER CompositeMedicalProcedureIdentifier
		{
			get { return _compositeMedicalProcedureIdentifier; }
			set { _compositeMedicalProcedureIdentifier = value; }
		}

		public decimal? MonetaryAmount1
		{
			get { return _monetaryAmount1; }
			set { _monetaryAmount1 = value; }
		}

		public IDENTIFIER UnitOrBasisForMeasurementCode
		{
			get { return _unitOrBasisForMeasurementCode; }
			set { _unitOrBasisForMeasurementCode = value; }
		}

		public decimal? Quantity
		{
			get { return _quantity; }
			set { _quantity = value; }
		}

		public string FacilityCodeValue
		{
			get { return _facilityCodeValue; }
			set { _facilityCodeValue = value; }
		}

		public IDENTIFIER ServiceTypeCode
		{
			get { return _serviceTypeCode; }
			set { _serviceTypeCode = value; }
		}

		public COMPOSITE_DIAGNOSIS_CODE_POINTER CompositeDiagnosisCodePointer
		{
			get { return _compositeDiagnosisCodePointer; }
			set { _compositeDiagnosisCodePointer = value; }
		}

		public decimal? MonetaryAmount2
		{
			get { return _monetaryAmount2; }
			set { _monetaryAmount2 = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode1
		{
			get { return _yesNoConditionOrResponseCode1; }
			set { _yesNoConditionOrResponseCode1 = value; }
		}

		public IDENTIFIER MultipleProcedureCode
		{
			get { return _multipleProcedureCode; }
			set { _multipleProcedureCode = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode2
		{
			get { return _yesNoConditionOrResponseCode2; }
			set { _yesNoConditionOrResponseCode2 = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode3
		{
			get { return _yesNoConditionOrResponseCode3; }
			set { _yesNoConditionOrResponseCode3 = value; }
		}

		public IDENTIFIER ReviewCode
		{
			get { return _reviewCode; }
			set { _reviewCode = value; }
		}

		public string NationalOrLocalAssignedReviewValue
		{
			get { return _nationalOrLocalAssignedReviewValue; }
			set { _nationalOrLocalAssignedReviewValue = value; }
		}

		public IDENTIFIER CopayStatusCode
		{
			get { return _copayStatusCode; }
			set { _copayStatusCode = value; }
		}

		public IDENTIFIER HealthCareProfessionalShortageAreaCode
		{
			get { return _healthCareProfessionalShortageAreaCode; }
			set { _healthCareProfessionalShortageAreaCode = value; }
		}

		public string ReferenceIdentification
		{
			get { return _referenceIdentification; }
			set { _referenceIdentification = value; }
		}

		public IDENTIFIER PostalCode
		{
			get { return _postalCode; }
			set { _postalCode = value; }
		}

		public decimal? MonetaryAmount3
		{
			get { return _monetaryAmount3; }
			set { _monetaryAmount3 = value; }
		}

		public IDENTIFIER LevelOfCareCode
		{
			get { return _levelOfCareCode; }
			set { _levelOfCareCode = value; }
		}

		public IDENTIFIER ProviderAgreementCode
		{
			get { return _providerAgreementCode; }
			set { _providerAgreementCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "SV1" + settings.Data_Element_Separator;

			if (_compositeMedicalProcedureIdentifier != null)
				strReturn += _compositeMedicalProcedureIdentifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_monetaryAmount1 != null)
				strReturn += _monetaryAmount1.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_unitOrBasisForMeasurementCode != null)
				strReturn += _unitOrBasisForMeasurementCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity != null)
				strReturn += _quantity.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _facilityCodeValue + settings.Data_Element_Separator;

			if (_serviceTypeCode != null)
				strReturn += _serviceTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_compositeDiagnosisCodePointer != null)
				strReturn += _compositeDiagnosisCodePointer.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_monetaryAmount2 != null)
				strReturn += _monetaryAmount2.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode1 != null)
				strReturn += _yesNoConditionOrResponseCode1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_multipleProcedureCode != null)
				strReturn += _multipleProcedureCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode2 != null)
				strReturn += _yesNoConditionOrResponseCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode3 != null)
				strReturn += _yesNoConditionOrResponseCode3.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_reviewCode != null)
				strReturn += _reviewCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _nationalOrLocalAssignedReviewValue + settings.Data_Element_Separator;

			if (_copayStatusCode != null)
				strReturn += _copayStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_healthCareProfessionalShortageAreaCode != null)
				strReturn += _healthCareProfessionalShortageAreaCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _referenceIdentification + settings.Data_Element_Separator;

			if (_postalCode != null)
				strReturn += _postalCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_monetaryAmount3 != null)
				strReturn += _monetaryAmount3.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_levelOfCareCode != null)
				strReturn += _levelOfCareCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_providerAgreementCode != null)
				strReturn += _providerAgreementCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class SV2
	{
		private Settings settings;
		public SV2(Settings s)
		{
			settings = s;
		}

		private string _productServiceId;
		private COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER _compositeMedicalProcedureIdentifier;
		private decimal? _monetaryAmount1;
		private IDENTIFIER _unitOrBasisForMeasurementCode;
		private decimal? _quantity;
		private decimal? _unitRate;
		private decimal? _monetaryAmount2;
		private IDENTIFIER _yesNoConditionOrResponseCode;
		private IDENTIFIER _nursingHomeResidentialStatusCode;
		private IDENTIFIER _levelOfCareCode;

		public string ProductServiceId
		{
			get { return _productServiceId; }
			set { _productServiceId = value; }
		}

		public COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER CompositeMedicalProcedureIdentifier
		{
			get { return _compositeMedicalProcedureIdentifier; }
			set { _compositeMedicalProcedureIdentifier = value; }
		}

		public decimal? MonetaryAmount1
		{
			get { return _monetaryAmount1; }
			set { _monetaryAmount1 = value; }
		}

		public IDENTIFIER UnitOrBasisForMeasurementCode
		{
			get { return _unitOrBasisForMeasurementCode; }
			set { _unitOrBasisForMeasurementCode = value; }
		}

		public decimal? Quantity
		{
			get { return _quantity; }
			set { _quantity = value; }
		}

		public decimal? UnitRate
		{
			get { return _unitRate; }
			set { _unitRate = value; }
		}

		public decimal? MonetaryAmount2
		{
			get { return _monetaryAmount2; }
			set { _monetaryAmount2 = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode
		{
			get { return _yesNoConditionOrResponseCode; }
			set { _yesNoConditionOrResponseCode = value; }
		}

		public IDENTIFIER NursingHomeResidentialStatusCode
		{
			get { return _nursingHomeResidentialStatusCode; }
			set { _nursingHomeResidentialStatusCode = value; }
		}

		public IDENTIFIER LevelOfCareCode
		{
			get { return _levelOfCareCode; }
			set { _levelOfCareCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "SV2" + settings.Data_Element_Separator;

			strReturn += _productServiceId + settings.Data_Element_Separator;

			if (_compositeMedicalProcedureIdentifier != null)
				strReturn += _compositeMedicalProcedureIdentifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_monetaryAmount1 != null)
				strReturn += _monetaryAmount1.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_unitOrBasisForMeasurementCode != null)
				strReturn += _unitOrBasisForMeasurementCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity != null)
				strReturn += _quantity.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_unitRate != null)
				strReturn += _unitRate.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_monetaryAmount2 != null)
				strReturn += _monetaryAmount2.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode != null)
				strReturn += _yesNoConditionOrResponseCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_nursingHomeResidentialStatusCode != null)
				strReturn += _nursingHomeResidentialStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_levelOfCareCode != null)
				strReturn += _levelOfCareCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class SV3
	{
		private Settings settings;
		public SV3(Settings s)
		{
			settings = s;
		}

		private COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER _compositeMedicalProcedureIdentifier;
		private decimal? _monetaryAmount;
		private string _facilityCodeValue;
		private ORAL_CAVITY_DESIGNATION _oralCavityDesignation;
		private IDENTIFIER _prosthesisCrownOrInlayCode;
		private decimal? _quantity;
		private string _description;
		private IDENTIFIER _copayStatusCode;
		private IDENTIFIER _providerAgreementCode;
		private IDENTIFIER _yesNoConditionOrResponseCode;
		private COMPOSITE_DIAGNOSIS_CODE_POINTER _compositeDiagnosisCodePointer;

		public COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER CompositeMedicalProcedureIdentifier
		{
			get { return _compositeMedicalProcedureIdentifier; }
			set { _compositeMedicalProcedureIdentifier = value; }
		}

		public decimal? MonetaryAmount
		{
			get { return _monetaryAmount; }
			set { _monetaryAmount = value; }
		}

		public string FacilityCodeValue
		{
			get { return _facilityCodeValue; }
			set { _facilityCodeValue = value; }
		}

		public ORAL_CAVITY_DESIGNATION OralCavityDesignation
		{
			get { return _oralCavityDesignation; }
			set { _oralCavityDesignation = value; }
		}

		public IDENTIFIER ProsthesisCrownOrInlayCode
		{
			get { return _prosthesisCrownOrInlayCode; }
			set { _prosthesisCrownOrInlayCode = value; }
		}

		public decimal? Quantity
		{
			get { return _quantity; }
			set { _quantity = value; }
		}

		public string Description
		{
			get { return _description; }
			set { _description = value; }
		}

		public IDENTIFIER CopayStatusCode
		{
			get { return _copayStatusCode; }
			set { _copayStatusCode = value; }
		}

		public IDENTIFIER ProviderAgreementCode
		{
			get { return _providerAgreementCode; }
			set { _providerAgreementCode = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode
		{
			get { return _yesNoConditionOrResponseCode; }
			set { _yesNoConditionOrResponseCode = value; }
		}

		public COMPOSITE_DIAGNOSIS_CODE_POINTER CompositeDiagnosisCodePointer
		{
			get { return _compositeDiagnosisCodePointer; }
			set { _compositeDiagnosisCodePointer = value; }
		}

		public override string ToString()
		{
			string strReturn = "SV3" + settings.Data_Element_Separator;

			if (_compositeMedicalProcedureIdentifier != null)
				strReturn += _compositeMedicalProcedureIdentifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_monetaryAmount != null)
				strReturn += _monetaryAmount.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _facilityCodeValue + settings.Data_Element_Separator;

			if (_oralCavityDesignation != null)
				strReturn += _oralCavityDesignation.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_prosthesisCrownOrInlayCode != null)
				strReturn += _prosthesisCrownOrInlayCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity != null)
				strReturn += _quantity.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _description + settings.Data_Element_Separator;

			if (_copayStatusCode != null)
				strReturn += _copayStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_providerAgreementCode != null)
				strReturn += _providerAgreementCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode != null)
				strReturn += _yesNoConditionOrResponseCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_compositeDiagnosisCodePointer != null)
				strReturn += _compositeDiagnosisCodePointer.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class TOO
	{
		private Settings settings;
		public TOO(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _codeListQualifierCode;
		private string _industryCode;
		private TOOTH_SURFACE _toothSurface;

		public IDENTIFIER CodeListQualifierCode
		{
			get { return _codeListQualifierCode; }
			set { _codeListQualifierCode = value; }
		}

		public string IndustryCode
		{
			get { return _industryCode; }
			set { _industryCode = value; }
		}

		public TOOTH_SURFACE ToothSurface
		{
			get { return _toothSurface; }
			set { _toothSurface = value; }
		}

		public override string ToString()
		{
			string strReturn = "TOO" + settings.Data_Element_Separator;

			if (_codeListQualifierCode != null)
				strReturn += _codeListQualifierCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _industryCode + settings.Data_Element_Separator;

			if (_toothSurface != null)
				strReturn += _toothSurface.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class HSD
	{
		private Settings settings;
		public HSD(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _quantityQualifier;
		private decimal? _quantity;
		private IDENTIFIER _unitOrBasisForMeasurementCode;
		private decimal? _sampleSelectionModulus;
		private IDENTIFIER _timePeriodQualifier;
		private int? _numberOfPeriods;
		private IDENTIFIER _shipDeliveryOrCalendarPatternCode;
		private IDENTIFIER _shipDeliveryPatternTimeCode;

		public IDENTIFIER QuantityQualifier
		{
			get { return _quantityQualifier; }
			set { _quantityQualifier = value; }
		}

		public decimal? Quantity
		{
			get { return _quantity; }
			set { _quantity = value; }
		}

		public IDENTIFIER UnitOrBasisForMeasurementCode
		{
			get { return _unitOrBasisForMeasurementCode; }
			set { _unitOrBasisForMeasurementCode = value; }
		}

		public decimal? SampleSelectionModulus
		{
			get { return _sampleSelectionModulus; }
			set { _sampleSelectionModulus = value; }
		}

		public IDENTIFIER TimePeriodQualifier
		{
			get { return _timePeriodQualifier; }
			set { _timePeriodQualifier = value; }
		}

		public int? NumberOfPeriods
		{
			get { return _numberOfPeriods; }
			set { _numberOfPeriods = value; }
		}

		public IDENTIFIER ShipDeliveryOrCalendarPatternCode
		{
			get { return _shipDeliveryOrCalendarPatternCode; }
			set { _shipDeliveryOrCalendarPatternCode = value; }
		}

		public IDENTIFIER ShipDeliveryPatternTimeCode
		{
			get { return _shipDeliveryPatternTimeCode; }
			set { _shipDeliveryPatternTimeCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "HSD" + settings.Data_Element_Separator;

			if (_quantityQualifier != null)
				strReturn += _quantityQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity != null)
				strReturn += _quantity.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_unitOrBasisForMeasurementCode != null)
				strReturn += _unitOrBasisForMeasurementCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_sampleSelectionModulus != null)
				strReturn += _sampleSelectionModulus.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_timePeriodQualifier != null)
				strReturn += _timePeriodQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_numberOfPeriods != null)
				strReturn += _numberOfPeriods.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_shipDeliveryOrCalendarPatternCode != null)
				strReturn += _shipDeliveryOrCalendarPatternCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_shipDeliveryPatternTimeCode != null)
				strReturn += _shipDeliveryPatternTimeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CRC
	{
		private Settings settings;
		public CRC(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _codeCategory;
		private IDENTIFIER _yesNoConditionOrResponseCode;
		private IDENTIFIER _conditionIndicator1;
		private IDENTIFIER _conditionIndicator2;
		private IDENTIFIER _conditionIndicator3;
		private IDENTIFIER _conditionIndicator4;
		private IDENTIFIER _conditionIndicator5;

		public IDENTIFIER CodeCategory
		{
			get { return _codeCategory; }
			set { _codeCategory = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode
		{
			get { return _yesNoConditionOrResponseCode; }
			set { _yesNoConditionOrResponseCode = value; }
		}

		public IDENTIFIER ConditionIndicator1
		{
			get { return _conditionIndicator1; }
			set { _conditionIndicator1 = value; }
		}

		public IDENTIFIER ConditionIndicator2
		{
			get { return _conditionIndicator2; }
			set { _conditionIndicator2 = value; }
		}

		public IDENTIFIER ConditionIndicator3
		{
			get { return _conditionIndicator3; }
			set { _conditionIndicator3 = value; }
		}

		public IDENTIFIER ConditionIndicator4
		{
			get { return _conditionIndicator4; }
			set { _conditionIndicator4 = value; }
		}

		public IDENTIFIER ConditionIndicator5
		{
			get { return _conditionIndicator5; }
			set { _conditionIndicator5 = value; }
		}

		public override string ToString()
		{
			string strReturn = "CRC" + settings.Data_Element_Separator;

			if (_codeCategory != null)
				strReturn += _codeCategory.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode != null)
				strReturn += _yesNoConditionOrResponseCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_conditionIndicator1 != null)
				strReturn += _conditionIndicator1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_conditionIndicator2 != null)
				strReturn += _conditionIndicator2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_conditionIndicator3 != null)
				strReturn += _conditionIndicator3.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_conditionIndicator4 != null)
				strReturn += _conditionIndicator4.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_conditionIndicator5 != null)
				strReturn += _conditionIndicator5.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CL1
	{
		private Settings settings;
		public CL1(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _admissionTypeCode;
		private IDENTIFIER _admissionSourceCode;
		private IDENTIFIER _patientStatusCode;
		private IDENTIFIER _nursingHomeResidentialStatusCode;

		public IDENTIFIER AdmissionTypeCode
		{
			get { return _admissionTypeCode; }
			set { _admissionTypeCode = value; }
		}

		public IDENTIFIER AdmissionSourceCode
		{
			get { return _admissionSourceCode; }
			set { _admissionSourceCode = value; }
		}

		public IDENTIFIER PatientStatusCode
		{
			get { return _patientStatusCode; }
			set { _patientStatusCode = value; }
		}

		public IDENTIFIER NursingHomeResidentialStatusCode
		{
			get { return _nursingHomeResidentialStatusCode; }
			set { _nursingHomeResidentialStatusCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "CL1" + settings.Data_Element_Separator;

			if (_admissionTypeCode != null)
				strReturn += _admissionTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_admissionSourceCode != null)
				strReturn += _admissionSourceCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_patientStatusCode != null)
				strReturn += _patientStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_nursingHomeResidentialStatusCode != null)
				strReturn += _nursingHomeResidentialStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CR1
	{
		private Settings settings;
		public CR1(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _unitOrBasisForMeasurementCode1;
		private decimal? _weight;
		private IDENTIFIER _ambulanceTransportCode;
		private IDENTIFIER _ambulanceTransportReasonCode;
		private IDENTIFIER _unitOrBasisForMeasurementCode2;
		private decimal? _quantity;
		private string _addressInformation1;
		private string _addressInformation2;
		private string _description1;
		private string _description2;

		public IDENTIFIER UnitOrBasisForMeasurementCode1
		{
			get { return _unitOrBasisForMeasurementCode1; }
			set { _unitOrBasisForMeasurementCode1 = value; }
		}

		public decimal? Weight
		{
			get { return _weight; }
			set { _weight = value; }
		}

		public IDENTIFIER AmbulanceTransportCode
		{
			get { return _ambulanceTransportCode; }
			set { _ambulanceTransportCode = value; }
		}

		public IDENTIFIER AmbulanceTransportReasonCode
		{
			get { return _ambulanceTransportReasonCode; }
			set { _ambulanceTransportReasonCode = value; }
		}

		public IDENTIFIER UnitOrBasisForMeasurementCode2
		{
			get { return _unitOrBasisForMeasurementCode2; }
			set { _unitOrBasisForMeasurementCode2 = value; }
		}

		public decimal? Quantity
		{
			get { return _quantity; }
			set { _quantity = value; }
		}

		public string AddressInformation1
		{
			get { return _addressInformation1; }
			set { _addressInformation1 = value; }
		}

		public string AddressInformation2
		{
			get { return _addressInformation2; }
			set { _addressInformation2 = value; }
		}

		public string Description1
		{
			get { return _description1; }
			set { _description1 = value; }
		}

		public string Description2
		{
			get { return _description2; }
			set { _description2 = value; }
		}

		public override string ToString()
		{
			string strReturn = "CR1" + settings.Data_Element_Separator;

			if (_unitOrBasisForMeasurementCode1 != null)
				strReturn += _unitOrBasisForMeasurementCode1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_weight != null)
				strReturn += _weight.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_ambulanceTransportCode != null)
				strReturn += _ambulanceTransportCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_ambulanceTransportReasonCode != null)
				strReturn += _ambulanceTransportReasonCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_unitOrBasisForMeasurementCode2 != null)
				strReturn += _unitOrBasisForMeasurementCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity != null)
				strReturn += _quantity.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _addressInformation1 + settings.Data_Element_Separator;

			strReturn += _addressInformation2 + settings.Data_Element_Separator;

			strReturn += _description1 + settings.Data_Element_Separator;

			strReturn += _description2 + settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CR2
	{
		private Settings settings;
		public CR2(Settings s)
		{
			settings = s;
		}

		private int? _count;
		private decimal? _quantity1;
		private IDENTIFIER _subluxationLevelCode1;
		private IDENTIFIER _subluxationLevelCode2;
		private IDENTIFIER _unitOrBasisForMeasurementCode;
		private decimal? _quantity2;
		private decimal? _quantity3;
		private IDENTIFIER _natureOfConditionCode;
		private IDENTIFIER _yesNoConditionOrResponseCode1;
		private string _description1;
		private string _description2;
		private IDENTIFIER _yesNoConditionOrResponseCode2;

		public int? Count
		{
			get { return _count; }
			set { _count = value; }
		}

		public decimal? Quantity1
		{
			get { return _quantity1; }
			set { _quantity1 = value; }
		}

		public IDENTIFIER SubluxationLevelCode1
		{
			get { return _subluxationLevelCode1; }
			set { _subluxationLevelCode1 = value; }
		}

		public IDENTIFIER SubluxationLevelCode2
		{
			get { return _subluxationLevelCode2; }
			set { _subluxationLevelCode2 = value; }
		}

		public IDENTIFIER UnitOrBasisForMeasurementCode
		{
			get { return _unitOrBasisForMeasurementCode; }
			set { _unitOrBasisForMeasurementCode = value; }
		}

		public decimal? Quantity2
		{
			get { return _quantity2; }
			set { _quantity2 = value; }
		}

		public decimal? Quantity3
		{
			get { return _quantity3; }
			set { _quantity3 = value; }
		}

		public IDENTIFIER NatureOfConditionCode
		{
			get { return _natureOfConditionCode; }
			set { _natureOfConditionCode = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode1
		{
			get { return _yesNoConditionOrResponseCode1; }
			set { _yesNoConditionOrResponseCode1 = value; }
		}

		public string Description1
		{
			get { return _description1; }
			set { _description1 = value; }
		}

		public string Description2
		{
			get { return _description2; }
			set { _description2 = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode2
		{
			get { return _yesNoConditionOrResponseCode2; }
			set { _yesNoConditionOrResponseCode2 = value; }
		}

		public override string ToString()
		{
			string strReturn = "CR2" + settings.Data_Element_Separator;

			if (_count != null)
				strReturn += _count.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity1 != null)
				strReturn += _quantity1.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_subluxationLevelCode1 != null)
				strReturn += _subluxationLevelCode1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_subluxationLevelCode2 != null)
				strReturn += _subluxationLevelCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_unitOrBasisForMeasurementCode != null)
				strReturn += _unitOrBasisForMeasurementCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity2 != null)
				strReturn += _quantity2.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity3 != null)
				strReturn += _quantity3.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_natureOfConditionCode != null)
				strReturn += _natureOfConditionCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode1 != null)
				strReturn += _yesNoConditionOrResponseCode1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _description1 + settings.Data_Element_Separator;

			strReturn += _description2 + settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode2 != null)
				strReturn += _yesNoConditionOrResponseCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CR4
	{
		private Settings settings;
		public CR4(Settings s)
		{
			settings = s;
		}

		private string _segment;

		public string Segment
		{
			get { return _segment; }
			set { _segment = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty; //"CR4" + settings.Data_Element_Separator;

			strReturn += _segment;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CR5
	{
		private Settings settings;
		public CR5(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _certificationTypeCode;
		private decimal? _quantity1;
		private IDENTIFIER _oxygenEquipmentTypeCode1;
		private IDENTIFIER _oxygenEquipmentTypeCode2;
		private string _description1;
		private decimal? _quantity2;
		private decimal? _quantity3;
		private decimal? _quantity4;
		private string _description2;
		private decimal? _quantity5;
		private decimal? _quantity6;
		private IDENTIFIER _oxygenTestConditionCode1;
		private IDENTIFIER _oxygenTestConditionCode2;
		private IDENTIFIER _oxygenTestConditionCode3;
		private IDENTIFIER _oxygenTestConditionCode4;
		private decimal? _quantity7;
		private IDENTIFIER _oxygenDeliverySystemCode;
		private IDENTIFIER _oxygenEquipmentTypeCode3;

		public IDENTIFIER CertificationTypeCode
		{
			get { return _certificationTypeCode; }
			set { _certificationTypeCode = value; }
		}

		public decimal? Quantity1
		{
			get { return _quantity1; }
			set { _quantity1 = value; }
		}

		public IDENTIFIER OxygenEquipmentTypeCode1
		{
			get { return _oxygenEquipmentTypeCode1; }
			set { _oxygenEquipmentTypeCode1 = value; }
		}

		public IDENTIFIER OxygenEquipmentTypeCode2
		{
			get { return _oxygenEquipmentTypeCode2; }
			set { _oxygenEquipmentTypeCode2 = value; }
		}

		public string Description1
		{
			get { return _description1; }
			set { _description1 = value; }
		}

		public decimal? Quantity2
		{
			get { return _quantity2; }
			set { _quantity2 = value; }
		}

		public decimal? Quantity3
		{
			get { return _quantity3; }
			set { _quantity3 = value; }
		}

		public decimal? Quantity4
		{
			get { return _quantity4; }
			set { _quantity4 = value; }
		}

		public string Description2
		{
			get { return _description2; }
			set { _description2 = value; }
		}

		public decimal? Quantity5
		{
			get { return _quantity5; }
			set { _quantity5 = value; }
		}

		public decimal? Quantity6
		{
			get { return _quantity6; }
			set { _quantity6 = value; }
		}

		public IDENTIFIER OxygenTestConditionCode1
		{
			get { return _oxygenTestConditionCode1; }
			set { _oxygenTestConditionCode1 = value; }
		}

		public IDENTIFIER OxygenTestConditionCode2
		{
			get { return _oxygenTestConditionCode2; }
			set { _oxygenTestConditionCode2 = value; }
		}

		public IDENTIFIER OxygenTestConditionCode3
		{
			get { return _oxygenTestConditionCode3; }
			set { _oxygenTestConditionCode3 = value; }
		}

		public IDENTIFIER OxygenTestConditionCode4
		{
			get { return _oxygenTestConditionCode4; }
			set { _oxygenTestConditionCode4 = value; }
		}

		public decimal? Quantity7
		{
			get { return _quantity7; }
			set { _quantity7 = value; }
		}

		public IDENTIFIER OxygenDeliverySystemCode
		{
			get { return _oxygenDeliverySystemCode; }
			set { _oxygenDeliverySystemCode = value; }
		}

		public IDENTIFIER OxygenEquipmentTypeCode3
		{
			get { return _oxygenEquipmentTypeCode3; }
			set { _oxygenEquipmentTypeCode3 = value; }
		}

		public override string ToString()
		{
			string strReturn = "CR5" + settings.Data_Element_Separator;

			if (_certificationTypeCode != null)
				strReturn += _certificationTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity1 != null)
				strReturn += _quantity1.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_oxygenEquipmentTypeCode1 != null)
				strReturn += _oxygenEquipmentTypeCode1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_oxygenEquipmentTypeCode2 != null)
				strReturn += _oxygenEquipmentTypeCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _description1 + settings.Data_Element_Separator;

			if (_quantity2 != null)
				strReturn += _quantity2.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity3 != null)
				strReturn += _quantity3.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity4 != null)
				strReturn += _quantity4.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _description2 + settings.Data_Element_Separator;

			if (_quantity5 != null)
				strReturn += _quantity5.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity6 != null)
				strReturn += _quantity6.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_oxygenTestConditionCode1 != null)
				strReturn += _oxygenTestConditionCode1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_oxygenTestConditionCode2 != null)
				strReturn += _oxygenTestConditionCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_oxygenTestConditionCode3 != null)
				strReturn += _oxygenTestConditionCode3.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_oxygenTestConditionCode4 != null)
				strReturn += _oxygenTestConditionCode4.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity7 != null)
				strReturn += _quantity7.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_oxygenDeliverySystemCode != null)
				strReturn += _oxygenDeliverySystemCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_oxygenEquipmentTypeCode3 != null)
				strReturn += _oxygenEquipmentTypeCode3.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CR6
	{
		private Settings settings;
		public CR6(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _prognosisCode;
		private DateTime? _date1;
		private string _date1String;
		private IDENTIFIER _dateTimePeriodFormatQualifier1;
		private string _dateTimePeriod1;
		private DateTime? _date2;
		private string _date2String;
		private IDENTIFIER _yesNoConditionOrResponseCode1;
		private IDENTIFIER _yesNoConditionOrResponseCode2;
		private IDENTIFIER _certificationTypeCode;
		private DateTime? _date3;
		private string _date3String;
		private IDENTIFIER _productServiceIdQualifier;
		private string _medicalCodeValue;
		private DateTime? _date4;
		private string _date4String;
		private DateTime? _date5;
		private string _date5String;
		private DateTime? _date6;
		private string _date6String;
		private IDENTIFIER _dateTimePeriodFormatQualifier2;
		private string _dateTimePeriod2;
		private IDENTIFIER _patientLocationCode;
		private DateTime? _date7;
		private string _date7String;
		private DateTime? _date8;
		private string _date8String;
		private DateTime? _date9;
		private string _date9String;
		private DateTime? _date10;
		private string _date10String;

		public IDENTIFIER PrognosisCode
		{
			get { return _prognosisCode; }
			set { _prognosisCode = value; }
		}

		public DateTime? Date1
		{
			get { return _date1; }
			set { _date1 = value; }
		}

		public string Date1String
		{
			get { return _date1String; }
			set { _date1String = value; }
		}

		public IDENTIFIER DateTimePeriodFormatQualifier1
		{
			get { return _dateTimePeriodFormatQualifier1; }
			set { _dateTimePeriodFormatQualifier1 = value; }
		}

		public string DateTimePeriod1
		{
			get { return _dateTimePeriod1; }
			set { _dateTimePeriod1 = value; }
		}

		public DateTime? Date2
		{
			get { return _date2; }
			set { _date2 = value; }
		}

		public string Date2String
		{
			get { return _date2String; }
			set { _date2String = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode1
		{
			get { return _yesNoConditionOrResponseCode1; }
			set { _yesNoConditionOrResponseCode1 = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode2
		{
			get { return _yesNoConditionOrResponseCode2; }
			set { _yesNoConditionOrResponseCode2 = value; }
		}

		public IDENTIFIER CertificationTypeCode
		{
			get { return _certificationTypeCode; }
			set { _certificationTypeCode = value; }
		}

		public DateTime? Date3
		{
			get { return _date3; }
			set { _date3 = value; }
		}

		public string Date3String
		{
			get { return _date3String; }
			set { _date3String = value; }
		}

		public IDENTIFIER ProductServiceIdQualifier
		{
			get { return _productServiceIdQualifier; }
			set { _productServiceIdQualifier = value; }
		}

		public string MedicalCodeValue
		{
			get { return _medicalCodeValue; }
			set { _medicalCodeValue = value; }
		}

		public DateTime? Date4
		{
			get { return _date4; }
			set { _date4 = value; }
		}

		public string Date4String
		{
			get { return _date4String; }
			set { _date4String = value; }
		}

		public DateTime? Date5
		{
			get { return _date5; }
			set { _date5 = value; }
		}

		public string Date5String
		{
			get { return _date5String; }
			set { _date5String = value; }
		}

		public DateTime? Date6
		{
			get { return _date6; }
			set { _date6 = value; }
		}

		public string Date6String
		{
			get { return _date6String; }
			set { _date6String = value; }
		}

		public IDENTIFIER DateTimePeriodFormatQualifier2
		{
			get { return _dateTimePeriodFormatQualifier2; }
			set { _dateTimePeriodFormatQualifier2 = value; }
		}

		public string DateTimePeriod2
		{
			get { return _dateTimePeriod2; }
			set { _dateTimePeriod2 = value; }
		}

		public IDENTIFIER PatientLocationCode
		{
			get { return _patientLocationCode; }
			set { _patientLocationCode = value; }
		}

		public DateTime? Date7
		{
			get { return _date7; }
			set { _date7 = value; }
		}

		public string Date7String
		{
			get { return _date7String; }
			set { _date7String = value; }
		}

		public DateTime? Date8
		{
			get { return _date8; }
			set { _date8 = value; }
		}

		public string Date8String
		{
			get { return _date8String; }
			set { _date8String = value; }
		}

		public DateTime? Date9
		{
			get { return _date9; }
			set { _date9 = value; }
		}

		public string Date9String
		{
			get { return _date9String; }
			set { _date9String = value; }
		}

		public DateTime? Date10
		{
			get { return _date10; }
			set { _date10 = value; }
		}

		public string Date10String
		{
			get { return _date10String; }
			set { _date10String = value; }
		}

		public override string ToString()
		{
			string strReturn = "CR6" + settings.Data_Element_Separator;

			if (_prognosisCode != null)
				strReturn += _prognosisCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _date1String + settings.Data_Element_Separator;

			if (_dateTimePeriodFormatQualifier1 != null)
				strReturn += _dateTimePeriodFormatQualifier1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _dateTimePeriod1 + settings.Data_Element_Separator;

			strReturn += _date2String + settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode1 != null)
				strReturn += _yesNoConditionOrResponseCode1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode2 != null)
				strReturn += _yesNoConditionOrResponseCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_certificationTypeCode != null)
				strReturn += _certificationTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _date3String + settings.Data_Element_Separator;

			if (_productServiceIdQualifier != null)
				strReturn += _productServiceIdQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _medicalCodeValue + settings.Data_Element_Separator;

			strReturn += _date4String + settings.Data_Element_Separator;

			strReturn += _date5String + settings.Data_Element_Separator;

			strReturn += _date6String + settings.Data_Element_Separator;

			if (_dateTimePeriodFormatQualifier2 != null)
				strReturn += _dateTimePeriodFormatQualifier2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _dateTimePeriod2 + settings.Data_Element_Separator;

			if (_patientLocationCode != null)
				strReturn += _patientLocationCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _date7String + settings.Data_Element_Separator;

			strReturn += _date8String + settings.Data_Element_Separator;

			strReturn += _date9String + settings.Data_Element_Separator;

			strReturn += _date10String + settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CR7
	{
		private Settings settings;
		public CR7(Settings s)
		{
			settings = s;
		}

		private string _segment;

		public string Segment
		{
			get { return _segment; }
			set { _segment = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty; //"CR7" + settings.Data_Element_Separator;

			strReturn += _segment;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class CR8
	{
		private Settings settings;
		public CR8(Settings s)
		{
			settings = s;
		}

		private string _segment;

		public string Segment
		{
			get { return _segment; }
			set { _segment = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty; //"CR8" + settings.Data_Element_Separator;

			strReturn += _segment;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class PWK
	{
		private Settings settings;
		public PWK(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _reportTypeCode;
		private IDENTIFIER _reportTransmissionCode;
		private int? _reportCopiesNeeded;
		private IDENTIFIER _entityIdentifierCode;
		private IDENTIFIER _identificationCodeQualifier;
		private string _identificationCode;
		private string _description;
		private ACTIONS_INDICATED _actionsIndicated;
		private IDENTIFIER _requestCategoryCode;

		public IDENTIFIER ReportTypeCode
		{
			get { return _reportTypeCode; }
			set { _reportTypeCode = value; }
		}

		public IDENTIFIER ReportTransmissionCode
		{
			get { return _reportTransmissionCode; }
			set { _reportTransmissionCode = value; }
		}

		public int? ReportCopiesNeeded
		{
			get { return _reportCopiesNeeded; }
			set { _reportCopiesNeeded = value; }
		}

		public IDENTIFIER EntityIdentifierCode
		{
			get { return _entityIdentifierCode; }
			set { _entityIdentifierCode = value; }
		}

		public IDENTIFIER IdentificationCodeQualifier
		{
			get { return _identificationCodeQualifier; }
			set { _identificationCodeQualifier = value; }
		}

		public string IdentificationCode
		{
			get { return _identificationCode; }
			set { _identificationCode = value; }
		}

		public string Description
		{
			get { return _description; }
			set { _description = value; }
		}

		public ACTIONS_INDICATED ActionsIndicated
		{
			get { return _actionsIndicated; }
			set { _actionsIndicated = value; }
		}

		public IDENTIFIER RequestCategoryCode
		{
			get { return _requestCategoryCode; }
			set { _requestCategoryCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "PWK" + settings.Data_Element_Separator;

			if (_reportTypeCode != null)
				strReturn += _reportTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_reportTransmissionCode != null)
				strReturn += _reportTransmissionCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_reportCopiesNeeded != null)
				strReturn += _reportCopiesNeeded.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_entityIdentifierCode != null)
				strReturn += _entityIdentifierCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_identificationCodeQualifier != null)
				strReturn += _identificationCodeQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _identificationCode + settings.Data_Element_Separator;

			strReturn += _description + settings.Data_Element_Separator;

			if (_actionsIndicated != null)
				strReturn += _actionsIndicated.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_requestCategoryCode != null)
				strReturn += _requestCategoryCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class MSG
	{
		private Settings settings;
		//public MSG(Settings s)
		//{
		//	settings = s;
		//}

		public MSG(Settings s, string messageText)
		{
			try
			{
				settings = s;
				if (string.IsNullOrEmpty(messageText))
				{
					_freeFormMessageText = "";
				}
				else if (messageText.Length > 264)
				{
					_freeFormMessageText = messageText.Substring(0, 263).Replace(settings.Data_Element_Separator.ToString(), " ").Replace(settings.Repetition_Separator.ToString(), " ").Replace(settings.Component_Element_Separator.ToString(), " ").Replace(settings.Segment_Terminator.ToString(), " ");
				}
				else
				{
					_freeFormMessageText = messageText.Replace(settings.Data_Element_Separator.ToString(), " ").Replace(settings.Repetition_Separator.ToString(), " ").Replace(settings.Component_Element_Separator.ToString(), " ").Replace(settings.Segment_Terminator.ToString(), " ");
				}

			}
			catch (Exception ex)
			{
				Utils.LogEvent("Error MSG() - " + ex.Message);
			}

		}

		private string _freeFormMessageText;
		private IDENTIFIER _printerCarriageControlCode;
		private int? _number;

		public string FreeFormMessageText
		{
			get { return _freeFormMessageText; }
			set { _freeFormMessageText = value; }
		}

		public IDENTIFIER PrinterCarriageControlCode
		{
			get { return _printerCarriageControlCode; }
			set { _printerCarriageControlCode = value; }
		}

		public int? Number
		{
			get { return _number; }
			set { _number = value; }
		}

		public override string ToString()
		{
			string strReturn = "MSG" + settings.Data_Element_Separator;

			strReturn += _freeFormMessageText + settings.Data_Element_Separator;

			if (_printerCarriageControlCode != null)
				strReturn += _printerCarriageControlCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_number != null)
				strReturn += _number.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class NM1
	{
		private Settings settings;
		public NM1(Settings s)
		{
			settings = s;
		}

		private IDENTIFIER _entityIdentifierCode;
		private IDENTIFIER _entityTypeQualifier;
		private string _nameLastOrOrganizationName;
		private string _nameFirst;
		private string _nameMiddle;
		private string _namePrefix;
		private string _nameSuffix;
		private IDENTIFIER _identificationCodeQualifier;
		private string _identificationCode;
		private IDENTIFIER _entityRelationshipCode;
		private IDENTIFIER _entityIdentifierCode2;
		private string _nameLastOrOrganizationName2;

		public IDENTIFIER EntityIdentifierCode
		{
			get { return _entityIdentifierCode; }
			set { _entityIdentifierCode = value; }
		}

		public IDENTIFIER EntityTypeQualifier
		{
			get { return _entityTypeQualifier; }
			set { _entityTypeQualifier = value; }
		}

		public string NameLastOrOrganizationName
		{
			get { return _nameLastOrOrganizationName; }
			set { _nameLastOrOrganizationName = value; }
		}

		public string NameFirst
		{
			get { return _nameFirst; }
			set { _nameFirst = value; }
		}

		public string NameMiddle
		{
			get { return _nameMiddle; }
			set { _nameMiddle = value; }
		}

		public string NamePrefix
		{
			get { return _namePrefix; }
			set { _namePrefix = value; }
		}

		public string NameSuffix
		{
			get { return _nameSuffix; }
			set { _nameSuffix = value; }
		}

		public IDENTIFIER IdentificationCodeQualifier
		{
			get { return _identificationCodeQualifier; }
			set { _identificationCodeQualifier = value; }
		}

		public string IdentificationCode
		{
			get { return _identificationCode; }
			set { _identificationCode = value; }
		}

		public IDENTIFIER EntityRelationshipCode
		{
			get { return _entityRelationshipCode; }
			set { _entityRelationshipCode = value; }
		}

		public IDENTIFIER EntityIdentifierCode2
		{
			get { return _entityIdentifierCode2; }
			set { _entityIdentifierCode2 = value; }
		}

		public string NameLastOrOrganizationName2
		{
			get { return _nameLastOrOrganizationName2; }
			set { _nameLastOrOrganizationName2 = value; }
		}

		public void GetVendorInfo(ref string vendorTin, ref string vendorName)
		{

			vendorTin = _identificationCode;
			vendorTin = vendorTin.Replace("-", "").Replace(" ", "");

			vendorName = _nameLastOrOrganizationName;
		}

		public override string ToString()
		{
			string strReturn = "NM1" + settings.Data_Element_Separator;

			if (_entityIdentifierCode != null)
				strReturn += _entityIdentifierCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_entityTypeQualifier != null)
				strReturn += _entityTypeQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _nameLastOrOrganizationName + settings.Data_Element_Separator;

			strReturn += _nameFirst + settings.Data_Element_Separator;

			strReturn += _nameMiddle + settings.Data_Element_Separator;

			strReturn += _namePrefix + settings.Data_Element_Separator;

			strReturn += _nameSuffix + settings.Data_Element_Separator;

			if (_identificationCodeQualifier != null)
				strReturn += _identificationCodeQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _identificationCode + settings.Data_Element_Separator;

			if (_entityRelationshipCode != null)
				strReturn += _entityRelationshipCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_entityIdentifierCode2 != null)
				strReturn += _entityIdentifierCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _nameLastOrOrganizationName2 + settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class N2
	{
		private Settings settings;
		public N2(Settings s)
		{
			settings = s;
		}


		private string _segment;

		public string Segment
		{
			get { return _segment; }
			set { _segment = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty; //"N2" + settings.Data_Element_Separator;

			strReturn += _segment;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class N3
	{
		private Settings settings;
		public N3(Settings s)
		{
			settings = s;
		}


		private string _addressInformation1 = "";
		private string _addressInformation2 = "";

		public string AddressInformation1
		{
			get { return _addressInformation1; }
			set { _addressInformation1 = value; }
		}

		public string AddressInformation2
		{
			get { return _addressInformation2; }
			set { _addressInformation2 = value; }
		}

		public override string ToString()
		{
			string strReturn = "N3" + settings.Data_Element_Separator;

			strReturn += ((_addressInformation1 == null) ? "" : _addressInformation1) + settings.Data_Element_Separator;

			strReturn += ((_addressInformation2 == null) ? "" : _addressInformation2) + settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class N4
	{
		private Settings settings;
		public N4(Settings s)
		{
			settings = s;
		}


		private string _cityName;
		private IDENTIFIER _stateOrProvinceCode;
		private IDENTIFIER _postalCode;
		private IDENTIFIER _countryCode;
		private IDENTIFIER _locationQualifier;
		private string _locationIdentifier;
		private IDENTIFIER _countrySubdivisionCode;

		public string CityName
		{
			get { return _cityName; }
			set { _cityName = value; }
		}

		public IDENTIFIER StateOrProvinceCode
		{
			get { return _stateOrProvinceCode; }
			set { _stateOrProvinceCode = value; }
		}

		public IDENTIFIER PostalCode
		{
			get { return _postalCode; }
			set { _postalCode = value; }
		}

		public IDENTIFIER CountryCode
		{
			get { return _countryCode; }
			set { _countryCode = value; }
		}

		public IDENTIFIER LocationQualifier
		{
			get { return _locationQualifier; }
			set { _locationQualifier = value; }
		}

		public string LocationIdentifier
		{
			get { return _locationIdentifier; }
			set { _locationIdentifier = value; }
		}

		public IDENTIFIER CountrySubdivisionCode
		{
			get { return _countrySubdivisionCode; }
			set { _countrySubdivisionCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "N4" + settings.Data_Element_Separator;

			strReturn += _cityName + settings.Data_Element_Separator;

			if (_stateOrProvinceCode != null)
				strReturn += _stateOrProvinceCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_postalCode != null)
				strReturn += _postalCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_countryCode != null)
				strReturn += _countryCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_locationQualifier != null)
				strReturn += _locationQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _locationIdentifier + settings.Data_Element_Separator;

			if (_countrySubdivisionCode != null)
				strReturn += _countrySubdivisionCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class PER
	{
		private Settings settings;
		public PER(Settings s)
		{
			settings = s;
		}


		private IDENTIFIER _contactFunctionCode;
		private string _name;
		private IDENTIFIER _communicationNumberQualifier1;
		private string _communicationNumber1;
		private IDENTIFIER _communicationNumberQualifier2;
		private string _communicationNumber2;
		private IDENTIFIER _communicationNumberQualifier3;
		private string _communicationNumber3;
		private string _contactInquiryReference;

		public IDENTIFIER ContactFunctionCode
		{
			get { return _contactFunctionCode; }
			set { _contactFunctionCode = value; }
		}

		public string Name
		{
			get { return _name; }
			set { _name = value; }
		}

		public IDENTIFIER CommunicationNumberQualifier1
		{
			get { return _communicationNumberQualifier1; }
			set { _communicationNumberQualifier1 = value; }
		}

		public string CommunicationNumber1
		{
			get { return _communicationNumber1; }
			set { _communicationNumber1 = value; }
		}

		public IDENTIFIER CommunicationNumberQualifier2
		{
			get { return _communicationNumberQualifier2; }
			set { _communicationNumberQualifier2 = value; }
		}

		public string CommunicationNumber2
		{
			get { return _communicationNumber2; }
			set { _communicationNumber2 = value; }
		}

		public IDENTIFIER CommunicationNumberQualifier3
		{
			get { return _communicationNumberQualifier3; }
			set { _communicationNumberQualifier3 = value; }
		}

		public string CommunicationNumber3
		{
			get { return _communicationNumber3; }
			set { _communicationNumber3 = value; }
		}

		public string ContactInquiryReference
		{
			get { return _contactInquiryReference; }
			set { _contactInquiryReference = value; }
		}

		public override string ToString()
		{
			string strReturn = "PER" + settings.Data_Element_Separator;

			if (_contactFunctionCode != null)
				strReturn += _contactFunctionCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _name + settings.Data_Element_Separator;

			if (_communicationNumberQualifier1 != null)
				strReturn += _communicationNumberQualifier1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _communicationNumber1 + settings.Data_Element_Separator;

			if (_communicationNumberQualifier2 != null)
				strReturn += _communicationNumberQualifier2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _communicationNumber2 + settings.Data_Element_Separator;

			if (_communicationNumberQualifier3 != null)
				strReturn += _communicationNumberQualifier3.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _communicationNumber3 + settings.Data_Element_Separator;

			strReturn += _contactInquiryReference + settings.Data_Element_Separator;


			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class PRV
	{
		private Settings settings;
		public PRV(Settings s)
		{
			settings = s;
		}


		private IDENTIFIER _providerCode;
		private IDENTIFIER _referenceIdentificationQualifier;
		private string _referenceIdentification;
		private IDENTIFIER _stateOrProvinceCode;
		private PROVIDER_SPECIALTY_INFORMATION _providerSpecialtyInformation;
		private IDENTIFIER _providerOrganizationCode;

		public IDENTIFIER ProviderCode
		{
			get { return _providerCode; }
			set { _providerCode = value; }
		}

		public IDENTIFIER ReferenceIdentificationQualifier
		{
			get { return _referenceIdentificationQualifier; }
			set { _referenceIdentificationQualifier = value; }
		}

		public string ReferenceIdentification
		{
			get { return _referenceIdentification; }
			set { _referenceIdentification = value; }
		}

		public IDENTIFIER StateOrProvinceCode
		{
			get { return _stateOrProvinceCode; }
			set { _stateOrProvinceCode = value; }
		}

		public PROVIDER_SPECIALTY_INFORMATION ProviderSpecialtyInformation
		{
			get { return _providerSpecialtyInformation; }
			set { _providerSpecialtyInformation = value; }
		}

		public IDENTIFIER ProviderOrganizationCode
		{
			get { return _providerOrganizationCode; }
			set { _providerOrganizationCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "PRV" + settings.Data_Element_Separator;

			if (_providerCode != null)
				strReturn += _providerCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_referenceIdentificationQualifier != null)
				strReturn += _referenceIdentificationQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _referenceIdentification + settings.Data_Element_Separator;

			if (_stateOrProvinceCode != null)
				strReturn += _stateOrProvinceCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_providerSpecialtyInformation != null)
				strReturn += _providerSpecialtyInformation.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_providerOrganizationCode != null)
				strReturn += _providerOrganizationCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class DMG
	{
		private Settings settings;
		public DMG(Settings s)
		{
			settings = s;
		}


		private IDENTIFIER _dateTimePeriodFormatQualifier;
		private string _dateTimePeriod;
		private IDENTIFIER _genderCode;
		private IDENTIFIER _maritalStatusCode;
		private COMPOSITE_RACE_OR_ETHNICITY_INFORMATION _compositeRaceOrEthnicityInformation;
		private IDENTIFIER _citizenshipStatusCode;
		private IDENTIFIER _countryCode;
		private IDENTIFIER _basisOfVerificationCode;
		private decimal? _quantity;
		private IDENTIFIER _codeListQualifierCode;
		private string _industryCode;

		public IDENTIFIER DateTimePeriodFormatQualifier
		{
			get { return _dateTimePeriodFormatQualifier; }
			set { _dateTimePeriodFormatQualifier = value; }
		}

		public string DateTimePeriod
		{
			get { return _dateTimePeriod; }
			set { _dateTimePeriod = value; }
		}

		public IDENTIFIER GenderCode
		{
			get { return _genderCode; }
			set { _genderCode = value; }
		}

		public IDENTIFIER MaritalStatusCode
		{
			get { return _maritalStatusCode; }
			set { _maritalStatusCode = value; }
		}

		public COMPOSITE_RACE_OR_ETHNICITY_INFORMATION CompositeRaceOrEthnicityInformation
		{
			get { return _compositeRaceOrEthnicityInformation; }
			set { _compositeRaceOrEthnicityInformation = value; }
		}

		public IDENTIFIER CitizenshipStatusCode
		{
			get { return _citizenshipStatusCode; }
			set { _citizenshipStatusCode = value; }
		}

		public IDENTIFIER CountryCode
		{
			get { return _countryCode; }
			set { _countryCode = value; }
		}

		public IDENTIFIER BasisOfVerificationCode
		{
			get { return _basisOfVerificationCode; }
			set { _basisOfVerificationCode = value; }
		}

		public decimal? Quantity
		{
			get { return _quantity; }
			set { _quantity = value; }
		}

		public IDENTIFIER CodeListQualifierCode
		{
			get { return _codeListQualifierCode; }
			set { _codeListQualifierCode = value; }
		}

		public string IndustryCode
		{
			get { return _industryCode; }
			set { _industryCode = value; }
		}

		public override string ToString()
		{
			string strReturn = "DMG" + settings.Data_Element_Separator;

			if (_dateTimePeriodFormatQualifier != null)
				strReturn += _dateTimePeriodFormatQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _dateTimePeriod + settings.Data_Element_Separator;

			if (_genderCode != null)
				strReturn += _genderCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_maritalStatusCode != null)
				strReturn += _maritalStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_compositeRaceOrEthnicityInformation != null)
				strReturn += _compositeRaceOrEthnicityInformation.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_citizenshipStatusCode != null)
				strReturn += _citizenshipStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_countryCode != null)
				strReturn += _countryCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_basisOfVerificationCode != null)
				strReturn += _basisOfVerificationCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_quantity != null)
				strReturn += _quantity.Value.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_codeListQualifierCode != null)
				strReturn += _codeListQualifierCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _industryCode + settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class INS
	{
		private Settings settings;
		public INS(Settings s)
		{
			settings = s;
		}


		private IDENTIFIER _yesNoConditionOrResponseCode1;
		private IDENTIFIER _individualRelationshipCode;
		private IDENTIFIER _maintenanceTypeCode;
		private IDENTIFIER _maintenanceReasonCode;
		private IDENTIFIER _benefitStatusCode;
		private MEDICARE_STATUS_CODE _medicareStatusCode;
		private IDENTIFIER _consolidatedOmnibusBudgetReconciliationActQualifying;
		private IDENTIFIER _employmentStatusCode;
		private IDENTIFIER _studentStatusCode;
		private IDENTIFIER _yesNoConditionOrResponseCode2;
		private IDENTIFIER _dateTimePeriodFormatQualifier;
		private string _dateTimePeriod;
		private IDENTIFIER _confidentialityCode;
		private string _cityName;
		private IDENTIFIER _stateOrProvinceCode;
		private IDENTIFIER _contryCode;
		private int? _number;

		public IDENTIFIER YesNoConditionOrResponseCode1
		{
			get { return _yesNoConditionOrResponseCode1; }
			set { _yesNoConditionOrResponseCode1 = value; }
		}

		public IDENTIFIER IndividualRelationshipCode
		{
			get { return _individualRelationshipCode; }
			set { _individualRelationshipCode = value; }
		}

		public IDENTIFIER MaintenanceTypeCode
		{
			get { return _maintenanceTypeCode; }
			set { _maintenanceTypeCode = value; }
		}

		public IDENTIFIER MaintenanceReasonCode
		{
			get { return _maintenanceReasonCode; }
			set { _maintenanceReasonCode = value; }
		}

		public IDENTIFIER BenefitStatusCode
		{
			get { return _benefitStatusCode; }
			set { _benefitStatusCode = value; }
		}

		public MEDICARE_STATUS_CODE MedicareStatusCode
		{
			get { return _medicareStatusCode; }
			set { _medicareStatusCode = value; }
		}

		public IDENTIFIER ConsolidatedOmnibusBudgetReconciliationActQualifying
		{
			get { return _consolidatedOmnibusBudgetReconciliationActQualifying; }
			set { _consolidatedOmnibusBudgetReconciliationActQualifying = value; }
		}

		public IDENTIFIER EmploymentStatusCode
		{
			get { return _employmentStatusCode; }
			set { _employmentStatusCode = value; }
		}

		public IDENTIFIER StudentStatusCode
		{
			get { return _studentStatusCode; }
			set { _studentStatusCode = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode2
		{
			get { return _yesNoConditionOrResponseCode2; }
			set { _yesNoConditionOrResponseCode2 = value; }
		}

		public IDENTIFIER DateTimePeriodFormatQualifier
		{
			get { return _dateTimePeriodFormatQualifier; }
			set { _dateTimePeriodFormatQualifier = value; }
		}

		public string DateTimePeriod
		{
			get { return _dateTimePeriod; }
			set { _dateTimePeriod = value; }
		}

		public IDENTIFIER ConfidentialityCode
		{
			get { return _confidentialityCode; }
			set { _confidentialityCode = value; }
		}

		public string CityName
		{
			get { return _cityName; }
			set { _cityName = value; }
		}

		public IDENTIFIER StateOrProvinceCode
		{
			get { return _stateOrProvinceCode; }
			set { _stateOrProvinceCode = value; }
		}

		public IDENTIFIER ContryCode
		{
			get { return _contryCode; }
			set { _contryCode = value; }
		}

		public int? Number
		{
			get { return _number; }
			set { _number = value; }
		}

		public override string ToString()
		{
			string strReturn = "INS" + settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode1 != null)
				strReturn += _yesNoConditionOrResponseCode1.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_individualRelationshipCode != null)
				strReturn += _individualRelationshipCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_maintenanceTypeCode != null)
				strReturn += _maintenanceTypeCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_maintenanceReasonCode != null)
				strReturn += _maintenanceReasonCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_benefitStatusCode != null)
				strReturn += _benefitStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_medicareStatusCode != null)
				strReturn += _medicareStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_consolidatedOmnibusBudgetReconciliationActQualifying != null)
				strReturn += _consolidatedOmnibusBudgetReconciliationActQualifying.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_employmentStatusCode != null)
				strReturn += _employmentStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_studentStatusCode != null)
				strReturn += _studentStatusCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_yesNoConditionOrResponseCode2 != null)
				strReturn += _yesNoConditionOrResponseCode2.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_dateTimePeriodFormatQualifier != null)
				strReturn += _dateTimePeriodFormatQualifier.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _dateTimePeriod + settings.Data_Element_Separator;

			if (_confidentialityCode != null)
				strReturn += _confidentialityCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			strReturn += _cityName + settings.Data_Element_Separator;

			if (_stateOrProvinceCode != null)
				strReturn += _stateOrProvinceCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_contryCode != null)
				strReturn += _contryCode.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			if (_number != null)
				strReturn += _number.ToString() + settings.Data_Element_Separator;
			else
				strReturn += settings.Data_Element_Separator;

			return Utils.TerminateSegment(settings, strReturn);
		}
	}

	[Serializable()]
	public class REFERENCE_IDENTIFIER
	{
		public REFERENCE_IDENTIFIER(string element) { _value = element; }

		private string _value;

		public string Value
		{
			get { return _value; }
			set { _value = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _value;

			return strReturn;
		}
	}

	[Serializable()]
	public class PROVIDER_SPECIALTY_INFORMATION
	{
		public PROVIDER_SPECIALTY_INFORMATION(string element) { _value = element; }

		private string _value;

		public string Value
		{
			get { return _value; }
			set { _value = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _value;

			return strReturn;
		}
	}

	[Serializable()]
	public class COMPOSITE_RACE_OR_ETHNICITY_INFORMATION
	{
		public COMPOSITE_RACE_OR_ETHNICITY_INFORMATION(string element) { _value = element; }

		private string _value;

		public string Value
		{
			get { return _value; }
			set { _value = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _value;

			return strReturn;
		}
	}

	[Serializable()]
	public class HEALTH_CARE_SERVICE_LOCATION_INFORMATION
	{
		private Settings settings;

		public HEALTH_CARE_SERVICE_LOCATION_INFORMATION(Settings s, string element)
		{
			settings = s;
			string[] subs = element.Split(settings.Component_Element_Separator);
			if (subs != null)
			{
				if (subs.Length > 0) _facilityCodeValue = subs[0];
				if (subs.Length > 1) _facilityCodeQualifier = new IDENTIFIER(subs[1]);
			}
		}

		private string _facilityCodeValue;
		private IDENTIFIER _facilityCodeQualifier;

		public string FacilityCodeValue
		{
			get { return _facilityCodeValue; }
			set { _facilityCodeValue = value; }
		}

		public IDENTIFIER FacilityCodeQualifier
		{
			get { return _facilityCodeQualifier; }
			set { _facilityCodeQualifier = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _facilityCodeValue + settings.Component_Element_Separator;

			if (_facilityCodeQualifier != null)
				strReturn += _facilityCodeQualifier.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			return Utils.TerminateComponent(settings, strReturn);
		}
	}

	[Serializable()]
	public class RELATED_CAUSES_INFORMATION
	{
		public RELATED_CAUSES_INFORMATION(string element) { _value = element; }

		private string _value;

		public string Value
		{
			get { return _value; }
			set { _value = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _value;

			return strReturn;
		}
	}

	[Serializable()]
	public class HEALTH_CARE_CODE_INFORMATION
	{
		private Settings settings;

		public HEALTH_CARE_CODE_INFORMATION(Settings s, string element)
		{
			settings = s;
			string[] comps = element.Split(settings.Component_Element_Separator);
			if (comps != null)
			{
				if (comps.Length > 0) _codeListQualifierCode = new IDENTIFIER(comps[0]);
				if (comps.Length > 1) _industryCode1 = comps[1];
				if (comps.Length > 2) _dateTimePeriodFormatQualifier = new IDENTIFIER(comps[2]);
				if (comps.Length > 3) _dateTimePeriod = comps[3];
				if (comps.Length > 4) _monetaryAmount = Utils.ParseDecimal(comps[4]);
				if (comps.Length > 5) _quantity = Utils.ParseDecimal(comps[5]);
				if (comps.Length > 6) _versionIdentifier = comps[6];
				if (comps.Length > 7) _industryCode2 = comps[7];
				if (comps.Length > 8) _yesNoConditionOrResponseCode = new IDENTIFIER(comps[8]);
			}
		}

		private IDENTIFIER _codeListQualifierCode;
		private string _industryCode1;
		private IDENTIFIER _dateTimePeriodFormatQualifier;
		private string _dateTimePeriod;
		private decimal? _monetaryAmount;
		private decimal? _quantity;
		private string _versionIdentifier;
		private string _industryCode2;
		private IDENTIFIER _yesNoConditionOrResponseCode;

		public IDENTIFIER CodeListQualifierCode
		{
			get { return _codeListQualifierCode; }
			set { _codeListQualifierCode = value; }
		}

		public string IndustryCode1
		{
			get { return _industryCode1; }
			set { _industryCode1 = value; }
		}

		public IDENTIFIER DateTimePeriodFormatQualifier
		{
			get { return _dateTimePeriodFormatQualifier; }
			set { _dateTimePeriodFormatQualifier = value; }
		}

		public string DateTimePeriod
		{
			get { return _dateTimePeriod; }
			set { _dateTimePeriod = value; }
		}

		public decimal? MonetaryAmount
		{
			get { return _monetaryAmount; }
			set { _monetaryAmount = value; }
		}

		public decimal? Quantity
		{
			get { return _quantity; }
			set { _quantity = value; }
		}

		public string VersionIdentifier
		{
			get { return _versionIdentifier; }
			set { _versionIdentifier = value; }
		}

		public string IndustryCode2
		{
			get { return _industryCode2; }
			set { _industryCode2 = value; }
		}

		public IDENTIFIER YesNoConditionOrResponseCode
		{
			get { return _yesNoConditionOrResponseCode; }
			set { _yesNoConditionOrResponseCode = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			if (_codeListQualifierCode != null)
				strReturn += _codeListQualifierCode.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			strReturn += _industryCode1 + settings.Component_Element_Separator;

			if (_dateTimePeriodFormatQualifier != null)
				strReturn += _dateTimePeriodFormatQualifier.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			strReturn += _dateTimePeriod + settings.Component_Element_Separator;

			if (_monetaryAmount != null)
				strReturn += _monetaryAmount.Value.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_quantity != null)
				strReturn += _quantity.Value.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			strReturn += _versionIdentifier + settings.Component_Element_Separator;

			strReturn += _industryCode2 + settings.Component_Element_Separator;

			if (_yesNoConditionOrResponseCode != null)
				strReturn += _yesNoConditionOrResponseCode.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			return Utils.TerminateComponent(settings, strReturn);
		}
	}

	[Serializable()]
	public class COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER
	{
		private Settings settings;

		public COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER(Settings s, string element)
		{
			settings = s;
			string[] subs = element.Split(settings.Component_Element_Separator);
			if (subs != null)
			{
				if (subs.Length > 0) _productServiceIdQualifier = new IDENTIFIER(subs[0]);
				if (subs.Length > 1) _productServiceId1 = subs[1];
				if (subs.Length > 2) _procedureModifier1 = subs[2];
				if (subs.Length > 3) _procedureModifier2 = subs[3];
				if (subs.Length > 4) _procedureModifier3 = subs[4];
				if (subs.Length > 5) _procedureModifier4 = subs[5];
				if (subs.Length > 6) _description = subs[6];
				if (subs.Length > 7) _productServiceId2 = subs[7];
			}
		}

		private IDENTIFIER _productServiceIdQualifier;
		private string _productServiceId1;
		private string _procedureModifier1;
		private string _procedureModifier2;
		private string _procedureModifier3;
		private string _procedureModifier4;
		private string _description;
		private string _productServiceId2;

		public IDENTIFIER ProductServiceIdQualifier
		{
			get { return _productServiceIdQualifier; }
			set { _productServiceIdQualifier = value; }
		}

		public string ProductServiceId1
		{
			get { return _productServiceId1; }
			set { _productServiceId1 = value; }
		}

		public string ProcedureModifier1
		{
			get { return _procedureModifier1; }
			set { _procedureModifier1 = value; }
		}

		public string ProcedureModifier2
		{
			get { return _procedureModifier2; }
			set { _procedureModifier2 = value; }
		}

		public string ProcedureModifier3
		{
			get { return _procedureModifier3; }
			set { _procedureModifier3 = value; }
		}

		public string ProcedureModifier4
		{
			get { return _procedureModifier4; }
			set { _procedureModifier4 = value; }
		}

		public string Description
		{
			get { return _description; }
			set { _description = value; }
		}

		public string ProductServiceId2
		{
			get { return _productServiceId2; }
			set { _productServiceId2 = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			if (_productServiceIdQualifier != null)
				strReturn += _productServiceIdQualifier.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			strReturn += _productServiceId1 + settings.Component_Element_Separator;

			strReturn += _procedureModifier1 + settings.Component_Element_Separator;

			strReturn += _procedureModifier2 + settings.Component_Element_Separator;

			strReturn += _procedureModifier3 + settings.Component_Element_Separator;

			strReturn += _procedureModifier4 + settings.Component_Element_Separator;

			strReturn += _description + settings.Component_Element_Separator;

			strReturn += _productServiceId2 + settings.Component_Element_Separator;

			return Utils.TerminateComponent(settings, strReturn);
		}
	}

	[Serializable()]
	public class COMPOSITE_DIAGNOSIS_CODE_POINTER
	{
		private Settings settings;

		public COMPOSITE_DIAGNOSIS_CODE_POINTER(Settings s, string element)
		{
			_value = element;
			settings = s;
		}

		private string _value;

		public string Value
		{
			get { return _value; }
			set { _value = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _value;

			return Utils.TerminateComponent(settings, strReturn);
		}
	}

	[Serializable()]
	public class ORAL_CAVITY_DESIGNATION
	{
		private Settings settings;

		public ORAL_CAVITY_DESIGNATION(Settings s, string element)
		{
			settings = s;

			string[] subs = element.Split(settings.Component_Element_Separator);
			if (subs != null)
			{
				if (subs.Length > 0) _oralCavityDesignationCode1 = new IDENTIFIER(subs[0]);
				if (subs.Length > 1) _oralCavityDesignationCode2 = new IDENTIFIER(subs[1]);
				if (subs.Length > 2) _oralCavityDesignationCode3 = new IDENTIFIER(subs[2]);
				if (subs.Length > 3) _oralCavityDesignationCode4 = new IDENTIFIER(subs[3]);
				if (subs.Length > 4) _oralCavityDesignationCode5 = new IDENTIFIER(subs[4]);
			}
		}

		private IDENTIFIER _oralCavityDesignationCode1;
		private IDENTIFIER _oralCavityDesignationCode2;
		private IDENTIFIER _oralCavityDesignationCode3;
		private IDENTIFIER _oralCavityDesignationCode4;
		private IDENTIFIER _oralCavityDesignationCode5;

		public IDENTIFIER OralCavityDesignationCode1
		{
			get { return _oralCavityDesignationCode1; }
			set { _oralCavityDesignationCode1 = value; }
		}

		public IDENTIFIER OralCavityDesignationCode2
		{
			get { return _oralCavityDesignationCode2; }
			set { _oralCavityDesignationCode2 = value; }
		}

		public IDENTIFIER OralCavityDesignationCode3
		{
			get { return _oralCavityDesignationCode3; }
			set { _oralCavityDesignationCode3 = value; }
		}

		public IDENTIFIER OralCavityDesignationCode4
		{
			get { return _oralCavityDesignationCode4; }
			set { _oralCavityDesignationCode4 = value; }
		}

		public IDENTIFIER OralCavityDesignationCode5
		{
			get { return _oralCavityDesignationCode5; }
			set { _oralCavityDesignationCode5 = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			if (_oralCavityDesignationCode1 != null)
				strReturn += _oralCavityDesignationCode1.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_oralCavityDesignationCode2 != null)
				strReturn += _oralCavityDesignationCode2.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_oralCavityDesignationCode3 != null)
				strReturn += _oralCavityDesignationCode3.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_oralCavityDesignationCode4 != null)
				strReturn += _oralCavityDesignationCode4.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_oralCavityDesignationCode5 != null)
				strReturn += _oralCavityDesignationCode5.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			return Utils.TerminateComponent(settings, strReturn);
		}
	}

	[Serializable()]
	public class TOOTH_SURFACE
	{
		private Settings settings;

		public TOOTH_SURFACE(Settings s, string element)
		{
			settings = s;
			string[] subs = element.Split(settings.Component_Element_Separator);
			if (subs != null)
			{
				if (subs.Length > 0) _toothSurfaceCode1 = new IDENTIFIER(subs[0]);
				if (subs.Length > 1) _toothSurfaceCode2 = new IDENTIFIER(subs[1]);
				if (subs.Length > 2) _toothSurfaceCode3 = new IDENTIFIER(subs[2]);
				if (subs.Length > 3) _toothSurfaceCode4 = new IDENTIFIER(subs[3]);
				if (subs.Length > 4) _toothSurfaceCode5 = new IDENTIFIER(subs[4]);
			}
		}

		private IDENTIFIER _toothSurfaceCode1;
		private IDENTIFIER _toothSurfaceCode2;
		private IDENTIFIER _toothSurfaceCode3;
		private IDENTIFIER _toothSurfaceCode4;
		private IDENTIFIER _toothSurfaceCode5;

		public IDENTIFIER ToothSurfaceCode1
		{
			get { return _toothSurfaceCode1; }
			set { _toothSurfaceCode1 = value; }
		}

		public IDENTIFIER ToothSurfaceCode2
		{
			get { return _toothSurfaceCode2; }
			set { _toothSurfaceCode2 = value; }
		}

		public IDENTIFIER ToothSurfaceCode3
		{
			get { return _toothSurfaceCode3; }
			set { _toothSurfaceCode3 = value; }
		}

		public IDENTIFIER ToothSurfaceCode4
		{
			get { return _toothSurfaceCode4; }
			set { _toothSurfaceCode4 = value; }
		}

		public IDENTIFIER ToothSurfaceCode5
		{
			get { return _toothSurfaceCode5; }
			set { _toothSurfaceCode5 = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			if (_toothSurfaceCode1 != null)
				strReturn += _toothSurfaceCode1.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_toothSurfaceCode2 != null)
				strReturn += _toothSurfaceCode2.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_toothSurfaceCode3 != null)
				strReturn += _toothSurfaceCode3.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_toothSurfaceCode4 != null)
				strReturn += _toothSurfaceCode4.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			if (_toothSurfaceCode5 != null)
				strReturn += _toothSurfaceCode5.ToString() + settings.Component_Element_Separator;
			else
				strReturn += settings.Component_Element_Separator;

			return Utils.TerminateComponent(settings, strReturn);
		}
	}

	[Serializable()]
	public class MEDICARE_STATUS_CODE
	{
		private Settings settings;

		public MEDICARE_STATUS_CODE(Settings s, string element)
		{
			settings = s;
			_value = element;
		}

		private string _value;

		public string Value
		{
			get { return _value; }
			set { _value = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _value;

			return Utils.TerminateComponent(settings, strReturn);
		}
	}

	[Serializable()]
	public class ACTIONS_INDICATED
	{
		private Settings settings;

		public ACTIONS_INDICATED(Settings s, string element)
		{
			settings = s;
			_value = element;
		}

		private string _value;

		public string Value
		{
			get { return _value; }
			set { _value = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _value;

			return Utils.TerminateComponent(settings, strReturn);
		}
	}

	[Serializable()]
	public class IDENTIFIER
	{
		public IDENTIFIER() { }
		public IDENTIFIER(string code) { _code = code; }
		public IDENTIFIER(string code, string definition) { _code = code; _definition = definition; }

		private string _code;
		private string _definition;

		public string Code
		{
			get { return _code; }
			set { _code = value; }
		}

		public string Definition
		{
			get { return _definition; }
			set { _definition = value; }
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			strReturn += _code;

			return strReturn;
		}
	}

	[Serializable()]
	public class IDENTIFIER_LIST
	{
		protected List<IDENTIFIER> _identifiers;

		public IDENTIFIER_LIST()
		{
			_identifiers = new List<IDENTIFIER>();
		}

		public List<IDENTIFIER> Identifiers
		{
			get { return _identifiers; }
		}

		public IDENTIFIER GetIdentifier(string code)
		{
			IDENTIFIER identifier = null;

			if (Identifiers != null && !string.IsNullOrEmpty(code))
			{
				foreach (IDENTIFIER i in Identifiers)
				{
					if (i.Code == code)
					{
						identifier = new IDENTIFIER(i.Code, i.Definition);
					}
				}
			}

			return identifier;
		}

		public bool Validate(string code)
		{
			if (_identifiers != null && !string.IsNullOrEmpty(code))
			{
				foreach (IDENTIFIER i in _identifiers)
				{
					if (i.Code == code)
					{
						return true;
					}
				}
			}

			return false;
		}
	}
}
