﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace X12Engine.BO.X12._278
{
   [Serializable()]
   public class EntityBase
   {

      private NM1 _individualOrOrganizationName;
      private List<REF> _referenceInformation;
      private N2 _additionalNameInformation;
      private N3 _partyLocation;
      private N4 _geographicLocation;
      private List<PER> _administrativeCommunicationsContact = new List<PER>();
      private List<AAA> _requestValidation = new List<AAA>();
      private PRV _providerInformation;
      private DMG _demographicInformation;
      private INS _insuredBenefit;
      private List<DTP> _dateOrTimeOrPeriod;

      public EntityBase() { }

      public NM1 IndividualOrOrganizationName
      {
         get { return _individualOrOrganizationName; }
         set { _individualOrOrganizationName = value; }
      }

      public List<REF> ReferenceInformation
      {
         get { return _referenceInformation; }
         set { _referenceInformation = value; }
      }

      public N2 AdditionalNameInformation
      {
         get { return _additionalNameInformation; }
         set { _additionalNameInformation = value; }
      }

      public N3 PartyLocation
      {
         get { return _partyLocation; }
         set { _partyLocation = value; }
      }

      public N4 GeographicLocation
      {
         get { return _geographicLocation; }
         set { _geographicLocation = value; }
      }

      public List<PER> AdministrativeCommunicationsContact
      {
         get { return _administrativeCommunicationsContact; }
         set { _administrativeCommunicationsContact = value; }
      }

      public List<AAA> RequestValidation
      {
         get { return _requestValidation; }
         set { _requestValidation = value; }
      }

      public PRV ProviderInformation
      {
         get { return _providerInformation; }
         set { _providerInformation = value; }
      }

      public DMG DemographicInformation
      {
         get { return _demographicInformation; }
         set { _demographicInformation = value; }
      }

      public INS InsuredBenefit
      {
         get { return _insuredBenefit; }
         set { _insuredBenefit = value; }
      }

      public List<DTP> DateOrTimeOrPeriod
      {
         get { return _dateOrTimeOrPeriod; }
         set { _dateOrTimeOrPeriod = value; }
      }


      public void GetPatientInfo(ref string lastName, ref string firstName, ref string ssn, ref DateTime? dateOfBirth)
      {
         if (_individualOrOrganizationName != null)
         {
            lastName = _individualOrOrganizationName.NameLastOrOrganizationName;
            firstName = _individualOrOrganizationName.NameFirst;
            if (_individualOrOrganizationName.IdentificationCodeQualifier.Code == "MI")
            {
               ssn = _individualOrOrganizationName.IdentificationCode;
            }
         }

         if (_demographicInformation != null)
         {
            if (_demographicInformation.DateTimePeriodFormatQualifier != null && _demographicInformation.DateTimePeriodFormatQualifier.Code == "D8") //D8 = CCYYMMDD or yyyyMMdd
            {
               dateOfBirth = X12Engine.Utils.ParseDate(_demographicInformation.DateTimePeriod, "yyyyMMdd");
            }
         }

      }

		public string EntityBaseToString(string HL, List<AAA> requestValidations)
		{
			string strReturn = string.Empty;
			string gotAAA = ":";
			int aaaCount = 0;

			// NM1
			if (HL != "")
			{
				if (_individualOrOrganizationName != null) strReturn += _individualOrOrganizationName.ToString();
			}
				
			// REF  
			if (HL != "EVB" && HL != "EVC" && HL != "HL1" && HL != "HL2" && _referenceInformation != null)
			{
				string refQualifier = "";
				foreach (REF refInfo in _referenceInformation)
				{
					refQualifier = (refInfo.ReferenceIdentificationQualifier != null) ? refInfo.ReferenceIdentificationQualifier.Code : "";

					if (   ((HL == "HL3" || HL == "HL4") && refQualifier == "EJ") || (HL == "EVA") || (HL == "SS"))
					{
						strReturn += refInfo.ToString();
					}

					
				}

			}

			// N2
			if (HL != "EVA" && HL != "EVB" && HL != "EVC" && HL != "HL1" && HL != "HL2" && HL != "HL3" && HL != "HL4" && HL != "SS")
			{
				if (_additionalNameInformation != null) strReturn += _additionalNameInformation.ToString();
			}

			// N3
			if (HL == "EVB" || HL == "EVC" || HL == "HL4" || HL == "SS")
			{
				if (_partyLocation != null) strReturn += _partyLocation.ToString();
			}

			// N4
			if (HL == "EVB" || HL == "EVC" || HL == "HL4" || HL == "SS")
			{
				if (_geographicLocation != null) strReturn += _geographicLocation.ToString();
			}

			// AAA
			if (HL != "EVA" && HL != "EVB" && HL != "EVC" && HL != "HL1" && HL != "SS")
			{

				if (requestValidations != null)
				{
					foreach (AAA aaa in requestValidations)
					{
						if (gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;
						gotAAA += aaa.RejectReasonCode.Code + ":";
						aaaCount++; if (aaaCount > 9) break;
						strReturn += aaa.ToString();
					}
				}

				if (_requestValidation != null)
				{
					foreach (AAA aaa in _requestValidation)
					{
						if (gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;
						gotAAA += aaa.RejectReasonCode.Code + ":";
						aaaCount++; if (aaaCount > 9) break;
						strReturn += aaa.ToString();
					}
				}
			}

			// PER
			if (HL != "EVC" && HL != "HL2" && HL != "HL4")
			{
				if (_administrativeCommunicationsContact != null)
				{
					foreach (PER p in _administrativeCommunicationsContact)
					{
						strReturn += p.ToString();
					}
				}
			}

			// AAA
			if (HL == "EVA" || HL == "EVB" || HL == "HL1" || HL == "SS")
			{
				if (_requestValidation != null)
				{
					foreach (AAA aaa in _requestValidation)
					{
						if (gotAAA.Contains(":" + aaa.RejectReasonCode.Code + ":") && !":15:79:97:".Contains(":" + aaa.RejectReasonCode.Code + ":")) continue;
						gotAAA += aaa.RejectReasonCode.Code + ":";
						aaaCount++; if (aaaCount > 9) break;
						strReturn += aaa.ToString();
					}
				}
			}

			// PRV
			if (HL != "EVB" && HL != "EVC" && HL != "HL1" && HL != "HL3" && HL != "HL4")
			{
				if (_providerInformation != null) strReturn += _providerInformation.ToString();
			}

			// DMG
			if (HL != "EVA" && HL != "EVB" && HL != "EVC" && HL != "HL1" && HL != "HL2" && HL != "SS")
			{
				if (_demographicInformation != null) strReturn += _demographicInformation.ToString();
			}

			// INS
			if (HL != "EVA" && HL != "EVB" && HL != "EVC" && HL != "HL1" && HL != "HL2" && HL != "SS")
			{
				if (_insuredBenefit != null) strReturn += _insuredBenefit.ToString();
			}

			// DTP
			if (HL != "EVA" && HL != "EVB" && HL != "EVC" && HL != "HL1" && HL != "HL2" && HL != "HL3" && HL != "HL4" && HL != "SS")
			{
				
				if (_dateOrTimeOrPeriod != null)
				{
					foreach (DTP d in _dateOrTimeOrPeriod)
					{
						strReturn += d.ToString();
					}
				}
			}

			return strReturn;
		}


   }
}
