﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;

namespace X12Engine.BO.X12._278.IdentifierCodes
{
	public static class CodeLookup
	{
		private static List<DataRoutingCode> _DataRoutingCode;

		private static Dictionary<string, string> _CodeDictionary;

		public static Dictionary<string, string> CodeDictionary
		{
			get
			{
				if (_CodeDictionary == null)
				{
					_CodeDictionary = new Dictionary<string, string>();
					string key = "";
					using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
					{
						conn.Open();

						using (SqlCommand cmd = new SqlCommand("SELECT DISTINCT LoopCode, SegId, Code FROM App.DataCodeLookUp", conn))
						{
							cmd.CommandTimeout = 300;


							using (SqlDataReader reader = cmd.ExecuteReader())
							{
								while (reader.Read())
								{
									key = reader["LoopCode"].ToString() + reader["SegId"].ToString() + reader["Code"].ToString();
									_CodeDictionary.Add(key, "T");
								}
							}
						}
					}

				}

				return _CodeDictionary;
			}
		}


		public static List<DataRoutingCode> DataRoutingCode
		{
			get
			{
				if (_DataRoutingCode == null) {

					_DataRoutingCode = new List<DataRoutingCode>();

					using (SqlConnection conn = new SqlConnection(Globals.DBConnectionString))
					{
						conn.Open();

						using (SqlCommand cmd = new SqlCommand("SELECT DISTINCT RuleNumber, SequenceNumber, Description, LoopCode, TR3Id, TableName, RouteType, CodeName, Condition, RouteValue, QueueId FROM App.DataRoutingCode", conn))
						{
							cmd.CommandTimeout = 300;

							using (SqlDataReader reader = cmd.ExecuteReader())
							{
								while (reader.Read())
								{
									DataRoutingCode dataRoutingCode = new DataRoutingCode();
									dataRoutingCode.RuleNumber = Convert.ToInt32(reader["RuleNumber"]);
									dataRoutingCode.SequenceNumber = Convert.ToInt32(reader["SequenceNumber"]);
									dataRoutingCode.Description = reader["Description"].ToString();
									dataRoutingCode.LoopCode = reader["LoopCode"].ToString();
									dataRoutingCode.TR3Id = reader["TR3Id"].ToString();
									dataRoutingCode.TableName = reader["TableName"].ToString();
									dataRoutingCode.RouteType = reader["RouteType"].ToString();
									dataRoutingCode.CodeName = reader["CodeName"].ToString();
									dataRoutingCode.Condition = reader["Condition"].ToString();
									dataRoutingCode.RouteValue = reader["RouteValue"].ToString();
									dataRoutingCode.QueueId = Convert.ToInt32(reader["QueueId"]);

									_DataRoutingCode.Add(dataRoutingCode);

								}
							}
						}
					}

				}

				return _DataRoutingCode;
			}
		}



		public static bool Validate(string loopCode, string segmentId, IDENTIFIER identifier)
		{
			//return true; //xxx

			if (CodeDictionary.ContainsKey(loopCode + segmentId + identifier.Code))
			{
				return true;
			}
			return false;
		}

		public static void ValidateRoute(ref Settings settings, string loopCode, string segment, string routeValue)
		{
			if (settings.TransType != "005010X217")
			{
				return;
			}

			settings.LogMessages.Append("ValidateRoute() loop: " + loopCode + "   segment: " + segment + "   value: " + routeValue + "\r\n");
			var equalRec = DataRoutingCode.Where(w => w.LoopCode == loopCode && w.TR3Id == segment && w.Condition == "=" && w.RouteValue == routeValue).OrderBy(o => o.RuleNumber).ThenBy(t => t.SequenceNumber).FirstOrDefault();
			if (equalRec != null)
			{
				settings.DataRoutingCode.Add(equalRec);
			} else
			{
				var recs = DataRoutingCode.Where(w => w.LoopCode == loopCode && w.TR3Id == segment && w.Condition != "=").OrderBy(o => o.RuleNumber).ThenBy(t => t.SequenceNumber).ToList();
				if (recs != null)
				{
					int intValue = 0;
					int intRouteValue = 0;
					foreach (var rec in recs)
					{
						if (rec.Condition == ">" || rec.Condition == "<" && int.TryParse(rec.RouteValue, out intValue))
						{
							int.TryParse(rec.RouteValue, out intValue);
							int.TryParse(routeValue, out intRouteValue);
							if (rec.Condition == ">" && intRouteValue > intValue)
							{
								settings.DataRoutingCode.Add(rec);
								break;
							}
							else if (rec.Condition == "<" && intRouteValue < intValue)
							{
								settings.DataRoutingCode.Add(rec);
								break;
							}
						}
						else if (rec.Condition == "Like")
						{
							if (routeValue.StartsWith(rec.RouteValue, StringComparison.OrdinalIgnoreCase))
							{
								settings.DataRoutingCode.Add(rec);
								break;
							}
						}
						else if (rec.Condition == "NOT NULL")
						{
							if (rec.RouteValue.Length == 0)
							{
								settings.DataRoutingCode.Add(rec);
								break;
							}
						}
						else if (rec.Condition == "EXIST")
						{
							settings.DataRoutingCode.Add(rec);
							break;
						}
					}
				}
			}

		}

	}
}