﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BLL.X12._278;

namespace X12Engine.BO.X12._278.XModel
{
	[Serializable()]
	public class MessageGroup
	{

		private List<Message> _messageList;

		public MessageGroup() { }

		public List<Message> MessageList
		{
			get { return _messageList; }
			set { _messageList = value; }
		}

		public void Add(Message message)
		{
			if (_messageList == null) _messageList = new List<Message>();
			_messageList.Add(message);
		}

		public bool ValidateMessage()
		{
			try
			{
				bool validMessage = true;

				foreach (Message m in _messageList)
				{
					if (m.TransactionSetHeader == null)
					{
						//m.UmoDetail.RequestValidation.Add(AAAManager.GetAAA(settings, "N", "42", "C", "Invalid message request"));
						Utils.LogEvent("X217.MessageGroup.ValidateMessage - m.TransactionSetHeader == null"); validMessage = false;
					}

					if (m.TransactionSetTrailer == null)
					{
						//m.UmoDetail.RequestValidation.Add(AAAManager.GetAAA(settings, "N", "42", "C", "Invalid message request"));
						Utils.LogEvent("X217.MessageGroup.ValidateMessage - m.TransactionSetTrailer == null"); validMessage = false;
					}

					if (m.TransactionSetHeader != null && m.TransactionSetTrailer != null)
					{
						if (m.TransactionSetHeader.TransactionSetControlNumber != m.TransactionSetTrailer.TransactionSetControlNumber)
						{
							//m.UmoDetail.RequestValidation.Add(AAAManager.GetAAA(settings, "N", "42", "C", "Invalid message request"));
							Utils.LogEvent("X217.MessageGroup.ValidateMessage - m.TransactionSetHeader.TransactionSetControlNumber != m.TransactionSetTrailer.TransactionSetControlNumber");
							Utils.LogEvent(m.TransactionSetHeader.TransactionSetControlNumber + " != " + m.TransactionSetTrailer.TransactionSetControlNumber); validMessage = false;
						}
					}

				}

				return validMessage;
			}
			catch (Exception ex)
			{
				Utils.LogEvent("Invalid X12 Message - " + ex.Message);
				return false;
			}


		}


		public void GetPatientInfo(ref string lastName, ref string firstName, ref string ssn, ref DateTime? dateOfBirth)
		{

			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.DependentDetail != null)
					{
						foreach (DependentDetail dd in m.DependentDetail)
						{
							if (dd.DependentName != null)
							{
								dd.DependentName.GetPatientInfo(ref lastName, ref firstName, ref ssn, ref dateOfBirth);
								return;  //return first dependent record found???
							}
						}
					}

					if (m.SubscriberDetail != null)
					{
						foreach (SubscriberDetail sd in m.SubscriberDetail)
						{
							if (sd.SubscriberName != null)
							{

								sd.SubscriberName.GetPatientInfo(ref lastName, ref firstName, ref ssn, ref dateOfBirth);
								return; //return first subscriber record found???
							}
						}
					}
				}
			}
		}

		public void GetVendorInfo(ref string vendorTin, ref string vendorName)
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.RequesterDetail != null)
					{
						if (m.RequesterDetail.RequesterName != null)
						{
							if (m.RequesterDetail.RequesterName.IndividualOrOrganizationName != null) m.RequesterDetail.RequesterName.IndividualOrOrganizationName.GetVendorInfo(ref vendorTin, ref vendorName);
						}
					}
				}
			}
		}

		public void GetLookupDates(ref DateTime? issueDate, ref DateTime? fromDate, ref DateTime? toDate)
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.PatientEventDetail != null)
					{
						foreach (PatientEventDetail ped in m.PatientEventDetail)
						{
							if (ped.EventDate != null) ped.EventDate.GetDates(ref fromDate, ref toDate);
							if (ped.CertificationIssueDate != null) ped.CertificationIssueDate.GetDate(ref issueDate);
						}
					}
				}
			}
		}

		public void UpdateUmoDetail(Institution institution)
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.UmoDetail != null)
					{
						m.UmoDetail.UMOName.IndividualOrOrganizationName.NameLastOrOrganizationName = institution.Sta3n.ToString() + "-" + institution.InstitutionName;
					}
				}
			}
		}

		public void UpdateRequesterDetail()
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.RequesterDetail != null)
					{
						if (m.RequesterDetail.RequesterName != null)
						{
							m.RequesterDetail.RequesterName.AdministrativeCommunicationsContact = null;
							m.RequesterDetail.RequesterName.ProviderInformation = null;
						}
					}
				}
			}
		}

		public void UpdateSubscriberDetail(Settings settings, SPatient sPatient)
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{

					//FS-278-3.0-098

					SubscriberDetail subscriberDetail = new SubscriberDetail(settings);
					subscriberDetail.SubscriberLevel = new HL(settings, "3", "2", new IDENTIFIER("22"), new IDENTIFIER("1"));

					subscriberDetail.SubscriberName = new EntityBase();
					subscriberDetail.SubscriberName.IndividualOrOrganizationName = new NM1(settings);


					subscriberDetail.SubscriberName.IndividualOrOrganizationName.EntityIdentifierCode = new IDENTIFIER("IL");
					subscriberDetail.SubscriberName.IndividualOrOrganizationName.EntityTypeQualifier = new IDENTIFIER("1");
					subscriberDetail.SubscriberName.IndividualOrOrganizationName.IdentificationCodeQualifier = new IDENTIFIER("MI");

					subscriberDetail.SubscriberName.IndividualOrOrganizationName.IdentificationCode = sPatient.PatientSSN.ToString();
					subscriberDetail.SubscriberName.IndividualOrOrganizationName.NameLastOrOrganizationName = sPatient.PatientLastName;
					subscriberDetail.SubscriberName.IndividualOrOrganizationName.NameFirst = sPatient.PatientFirstName;

					//if (m.SubscriberDetail.Count > 0)
					//{
					//	if (m.SubscriberDetail[0].SubscriberName.PartyLocation != null)
					//	{
					//		subscriberDetail.SubscriberName.PartyLocation = new N3();
					//		subscriberDetail.SubscriberName.PartyLocation.AddressInformation1 = sPatient.StreetAddress1;
					//		subscriberDetail.SubscriberName.PartyLocation.AddressInformation2 = sPatient.StreetAddress2;
					//	}

					//	if (m.SubscriberDetail[0].SubscriberName.GeographicLocation != null)
					//	{
					//		subscriberDetail.SubscriberName.GeographicLocation = new N4();
					//		subscriberDetail.SubscriberName.GeographicLocation.CityName = sPatient.City;
					//		subscriberDetail.SubscriberName.GeographicLocation.StateOrProvinceCode = new IDENTIFIER(sPatient.State);
					//		subscriberDetail.SubscriberName.GeographicLocation.PostalCode = new IDENTIFIER(sPatient.Zip);
					//		subscriberDetail.SubscriberName.GeographicLocation.CountryCode = new IDENTIFIER(sPatient.Country);
					//	}

					//}


					// TRN
					if (m.SubscriberDetail[0].SubscriberTraceNumber != null)
					{
						subscriberDetail.SubscriberTraceNumber = new List<TRN>();
						foreach (var trn in m.SubscriberDetail[0].SubscriberTraceNumber)
						{
							subscriberDetail.SubscriberTraceNumber.Add(trn);
						}
					}


					// return DMG if present
					if (m.SubscriberDetail.Count > 0)
					{
						if (m.SubscriberDetail[0].SubscriberName.DemographicInformation != null)
						{
							DateTime birthDate = Convert.ToDateTime(sPatient.DateOfBirth);
							DMG dmg = new DMG(settings);
							dmg.DateTimePeriod = birthDate.Year.ToString() + birthDate.Month.ToString() + birthDate.Day.ToString();
							dmg.DateTimePeriodFormatQualifier = new IDENTIFIER("D8");
							dmg.GenderCode = new IDENTIFIER(sPatient.Gender);
							subscriberDetail.SubscriberName.DemographicInformation = dmg;
						}
					}

					m.SubscriberDetail = new List<SubscriberDetail>();
					m.SubscriberDetail.Add(subscriberDetail);


				}
			}
		}

		public void UpdateDependentDetail()
		{
			//if (_messageList != null)
			//{
			//   foreach (Message m in _messageList)
			//   {

			//   }
			//}
		}

		public void UpdatePatientEventDetail(Settings settings, List<PatientEventDetail> patientEventDetails)
		{
			try
			{

				if (_messageList != null)
				{
					foreach (Message m in _messageList)
					{
						int parentIdNumber = GetPatientEventParentId(m);
						int hierarchicalIdNumber = parentIdNumber + 1;

						List<PatientEventDetail> newPatientEventDetails = new List<PatientEventDetail>();
						string id = "1";

						// loop thru patientEventDetails 
						for (int i = 0; i <= patientEventDetails.Count - 1; i++)
						{

							// EV loop has authorization
							if (patientEventDetails[i].FeeAuthorizations.Count > 0)
							{
								for (int f = 0; f <= patientEventDetails[i].FeeAuthorizations.Count - 1; f++)
								{
									PatientEventDetail newPatientEventDetail = new PatientEventDetail(settings);


									// set level
									id = "1";
									if (i == (patientEventDetails.Count - 1) && f == (patientEventDetails[i].FeeAuthorizations.Count - 1))
									{
										id = "0";
									}
									newPatientEventDetail.PatientEventLevel = new HL(settings, hierarchicalIdNumber.ToString(), parentIdNumber.ToString(), new IDENTIFIER("EV"), new IDENTIFIER(id));

									parentIdNumber++;


									// set TRN
									if (patientEventDetails[i].PatientEventTraceNumber != null)
									{
										foreach (var trn in patientEventDetails[i].PatientEventTraceNumber)
										{
											newPatientEventDetail.PatientEventTraceNumber.Add(trn);
										}
									}

									// set UM
									if (patientEventDetails[i].HealthCareServicesReviewInformation != null)
									{
										newPatientEventDetail.HealthCareServicesReviewInformation = patientEventDetails[i].HealthCareServicesReviewInformation;
									}

									// set HCR and UAK
									HCR hcr = new HCR(settings);
									hcr.ActionCode = new IDENTIFIER("A1");
									hcr.ReferenceIdentification = patientEventDetails[i].FeeAuthorizations[f].Sta3n + "-" + patientEventDetails[i].FeeAuthorizations[f].PatientIEN + "-" + patientEventDetails[i].FeeAuthorizations[f].FeeAuthorizationIEN;   //St3n - PatientIEN - AuthEIN 
									newPatientEventDetail.HealthCareServicesReview = hcr;


									// set PatientEventProviderName
									if (patientEventDetails[i].PatientEventProviderName != null)
									{
										newPatientEventDetail.PatientEventProviderName = patientEventDetails[i].PatientEventProviderName;
									}


									// set PatientEventTransportInformation
									if (patientEventDetails[i].PatientEventTransportInformation != null)
									{
										newPatientEventDetail.PatientEventTransportInformation = patientEventDetails[i].PatientEventTransportInformation;
									}


									// set requestCategoryCode
									string requestCategoryCode = (patientEventDetails[i].HealthCareServicesReviewInformation != null && patientEventDetails[i].HealthCareServicesReviewInformation.RequestCategoryCode != null) ? patientEventDetails[i].HealthCareServicesReviewInformation.RequestCategoryCode.Code : "";


									// set dates
									if (m.ErrorFound)
									{
										newPatientEventDetail.AdmissionDate = null;
										newPatientEventDetail.EventDate = null;
									}
									else
									{
										if (requestCategoryCode == "AR" && patientEventDetails[i].FeeAuthorizations[f].AdmissionDate != null)
										{

											newPatientEventDetail.AdmissionDate = new DTP(settings, "435", Convert.ToDateTime(patientEventDetails[i].FeeAuthorizations[f].AdmissionDate));

											// if 215, add HCR*A1~
											if (patientEventDetails[i].FeeAuthorizations[f].FromDate != null && patientEventDetails[i].FeeAuthorizations[f].ToDate != null)
											{
												newPatientEventDetail.EventDate = new DTP(settings, "AAH", Convert.ToDateTime(patientEventDetails[i].FeeAuthorizations[f].FromDate), Convert.ToDateTime(patientEventDetails[i].FeeAuthorizations[f].ToDate));
											}
											else if (patientEventDetails[i].FeeAuthorizations[f].FromDate != null)
											{
												newPatientEventDetail.EventDate = new DTP(settings, "AAH", Convert.ToDateTime(patientEventDetails[i].FeeAuthorizations[f].FromDate));
											}

										}
										else
										{
											if (patientEventDetails[i].FeeAuthorizations[f].FromDate != null && patientEventDetails[i].FeeAuthorizations[f].ToDate != null)
											{
												newPatientEventDetail.EventDate = new DTP(settings, "AAH", Convert.ToDateTime(patientEventDetails[i].FeeAuthorizations[f].FromDate), Convert.ToDateTime(patientEventDetails[i].FeeAuthorizations[f].ToDate));
											}
											else if (patientEventDetails[i].FeeAuthorizations[f].FromDate != null)
											{
												newPatientEventDetail.EventDate = new DTP(settings, "AAH", Convert.ToDateTime(patientEventDetails[i].FeeAuthorizations[f].FromDate));
											}
										}
									}


									// message text
									string messageText = patientEventDetails[i].FeeAuthorizations[f].FeeAuthorizationRemark != null ? patientEventDetails[i].FeeAuthorizations[f].FeeAuthorizationRemark : "";

									if (messageText.Length == 0)
									{
										newPatientEventDetail.MessageText = null;
									}
									else
									{
										newPatientEventDetail.MessageText = new MSG(settings, messageText);
									}

									newPatientEventDetails.Add(newPatientEventDetail);
								}

							}
							else
							{
								PatientEventDetail newPatientEventDetail = new PatientEventDetail(settings);

								newPatientEventDetail = patientEventDetails[i];
								newPatientEventDetail.MessageText = null;

								// set level
								id = "1";
								if (i == (patientEventDetails.Count - 1))
								{
									id = "0";
								}
								newPatientEventDetail.PatientEventLevel = new HL(settings, hierarchicalIdNumber.ToString(), parentIdNumber.ToString(), new IDENTIFIER("EV"), new IDENTIFIER(id));


								// add AAA - AA
								newPatientEventDetail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", i, "N", "AA", "C", "Authorization not found"));
								parentIdNumber++;

								newPatientEventDetails.Add(newPatientEventDetail);
							}

						}

						m.PatientEventDetail = newPatientEventDetails;

					}
				}
			}
			catch (Exception)
			{
			}

		}



		public void UpdateServiceDetail()
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					m.ServiceDetail = null;
				}
			}
		}

		public void AddErrorToPatientEvent(ref Settings settings, string responseCode, string rejectCode, string followUp, string errorMessage)
		{
			if (_messageList != null)
			{
				int i = 0;
				foreach (Message m in _messageList)
				{
					List<PatientEventDetail> details = m.PatientEventDetail;
					m.PatientEventDetail = new List<PatientEventDetail>();
					foreach (var detail in details)
					{
						i++;
						detail.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2000E", i, responseCode, rejectCode, followUp, errorMessage));
						m.PatientEventDetail.Add(detail);
					}
				}
			}
		}


		public void AddErrorToSubscriber(ref Settings settings, string responseCode, string rejectCode, string followUp, string errorMessage)
		{
			if (_messageList != null)
			{
				int i = 0;
				foreach (Message m in _messageList)
				{
					List<SubscriberDetail> recs = m.SubscriberDetail;
					m.SubscriberDetail = new List<SubscriberDetail>();
					foreach (var rec in recs)
					{
						i++;
						rec.RequestValidation.Add(AAAManager.GetAAA(ref settings, "2010C", i, responseCode, rejectCode, followUp, errorMessage));
						m.SubscriberDetail.Add(rec);
					}
				}
			}
		}


		public void UpdateSegmentIds()
		{
			try
			{
				int idNumber = 0;

				if (_messageList != null)
				{
					foreach (Message m in _messageList)
					{
						if (m.UmoDetail != null && m.UmoDetail.UMOLevel != null)
						{
							idNumber++;

							m.UmoDetail.UMOLevel.HierarchicalIdNumber = idNumber.ToString();
						}

						if (m.RequesterDetail != null && m.RequesterDetail.RequesterLevel != null)
						{
							idNumber++;

							m.RequesterDetail.RequesterLevel.HierarchicalIdNumber = idNumber.ToString();
						}

						if (m.SubscriberDetail != null)
						{
							foreach (SubscriberDetail sd in m.SubscriberDetail)
							{
								idNumber++;

								if (sd.SubscriberLevel != null) sd.SubscriberLevel.HierarchicalIdNumber = idNumber.ToString();
							}
						}

						if (m.DependentDetail != null)
						{
							foreach (DependentDetail dd in m.DependentDetail)
							{
								idNumber++;

								if (dd.DependentLevel != null) dd.DependentLevel.HierarchicalIdNumber = idNumber.ToString();
							}
						}

						if (m.PatientEventDetail != null)
						{
							foreach (PatientEventDetail ped in m.PatientEventDetail)
							{
								idNumber++;

								if (ped.PatientEventLevel != null) ped.PatientEventLevel.HierarchicalIdNumber = idNumber.ToString();
							}
						}

						if (m.ServiceDetail != null)
						{
							foreach (ServiceDetail sd in m.ServiceDetail)
							{
								idNumber++;

								if (sd.ServiceLevel != null) sd.ServiceLevel.HierarchicalIdNumber = idNumber.ToString();
							}
						}
					}
				}
			}
			catch (Exception)
			{

			}
		}

		public void UpdateSegmentCount(Settings settings)
		{
			try
			{
				if (_messageList != null)
				{
					foreach (Message m in _messageList)
					{
						int segmentCount = Utils.CountSegments(settings, m.ToString());

						if (m.TransactionSetHeader != null) m.TransactionSetTrailer.TransactionSetControlNumber = m.TransactionSetHeader.TransactionSetControlNumber;

						m.TransactionSetTrailer.NumberOfIncludedSegments = segmentCount;
					}
				}
			}
			catch (Exception)
			{
			}

		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m != null) strReturn += m.ToString();
				}
			}

			return strReturn;
		}


		private int GetPatientEventParentId(Message m)
		{
			int tempId = 0;
			int parentIdNumber = 0;

			if (m.DependentDetail != null)
			{
				foreach (DependentDetail dd in m.DependentDetail)
				{
					if (dd.DependentLevel != null)
					{
						if (int.TryParse(dd.DependentLevel.HierarchicalIdNumber, out tempId))
						{
							if (tempId > parentIdNumber) parentIdNumber = tempId;
						}
					}
				}
			}

			if (parentIdNumber > 0) return parentIdNumber;

			if (m.SubscriberDetail != null)
			{
				foreach (SubscriberDetail sd in m.SubscriberDetail)
				{
					if (sd.SubscriberLevel != null)
					{
						if (int.TryParse(sd.SubscriberLevel.HierarchicalIdNumber, out tempId))
						{
							if (tempId > parentIdNumber) parentIdNumber = tempId;
						}
					}
				}
			}

			return parentIdNumber;
		}

		private List<TRN> GetPatientEventTrnList(Message m)
		{
			List<TRN> trnList = new List<TRN>();

			try
			{
				if (m == null) return trnList;

				if (m.PatientEventDetail != null)
				{

					foreach (PatientEventDetail ped in m.PatientEventDetail)
					{
						if (ped.PatientEventTraceNumber.Count > 0)
						{
							foreach (TRN t in ped.PatientEventTraceNumber)
							{
								trnList.Add(t);
							}
						}
					}
				}

				return trnList;
			}
			catch (Exception)
			{
				return trnList;
			}

		}

		private List<UM> GetPatientEventUmList(Message m)
		{
			List<UM> umList = new List<UM>();

			try
			{
				if (m == null) return umList;

				if (m.PatientEventDetail != null)
				{
					foreach (PatientEventDetail ped in m.PatientEventDetail)
					{
						if (ped.HealthCareServicesReviewInformation != null)
						{
							umList.Add(ped.HealthCareServicesReviewInformation);
						}
					}
				}

				return umList;
			}
			catch (Exception)
			{
				return umList;
			}


		}

	}
}
