﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BLL.X12._278;

namespace X12Engine.BO.X12._278.XModel
{
   [Serializable()]
   public class ServiceDetail : EntityBase
   {
		private Settings settings;

		public ServiceDetail(Settings s)
		{
			settings = s;
		}

		private HL _serviceLevel;
      private List<TRN> _serviceTraceNumber;
		private List<AAA> _serviceRequestValidation = new List<AAA>();
		private UM _healthCareServicesReviewInformation;
      private HCR _healthCareServicesReview;
      private REF _previousReviewAuthorizationNumber;
      private REF _previousReviewAdministrativeReferenceNumber;
      private DTP _serviceDate;
      private DTP _certificationIssueDate;
      private DTP _certificationExpirationDate;
      private DTP _certificationEffectiveDate;
      private HI _requestForAdditionalInformation;
      private SV1 _professionalService;
      private SV2 _institutionalService;
      private SV3 _dentalService;
      private List<TOO> _toothInformation;
      private HSD _healthCareServicesDelivery;
      private List<PWK> _additionalServiceInformation;
      private MSG _messageText;
      private DetailBase _other;
      private List<EntityBase> _serviceProviderName;
      private EntityBase _additionalServiceInformationContactName;

		public HL ServiceLevel
      {
         get { return _serviceLevel; }
         set { _serviceLevel = value; }
      }

      public List<TRN> ServiceTraceNumber
      {
         get { return _serviceTraceNumber; }
         set { _serviceTraceNumber = value; }
      }

		public List<AAA> ServiceRequestValidation
		{
			get { return _serviceRequestValidation; }
			set { _serviceRequestValidation = value; }
		}

		public UM HealthCareServicesReviewInformation
      {
         get { return _healthCareServicesReviewInformation; }
         set { _healthCareServicesReviewInformation = value; }
      }

      public HCR HealthCareServicesReview
      {
         get { return _healthCareServicesReview; }
         set { _healthCareServicesReview = value; }
      }

      public REF PreviousReviewAuthorizationNumber
      {
         get { return _previousReviewAuthorizationNumber; }
         set { _previousReviewAuthorizationNumber = value; }
      }

      public REF PreviousReviewAdministrativeReferenceNumber
      {
         get { return _previousReviewAdministrativeReferenceNumber; }
         set { _previousReviewAdministrativeReferenceNumber = value; }
      }

      public DTP ServiceDate
      {
         get { return _serviceDate; }
         set { _serviceDate = value; }
      }

      public DTP CertificationIssueDate
      {
         get { return _certificationIssueDate; }
         set { _certificationIssueDate = value; }
      }

      public DTP CertificationExpirationDate
      {
         get { return _certificationExpirationDate; }
         set { _certificationExpirationDate = value; }
      }

      public DTP CertificationEffectiveDate
      {
         get { return _certificationEffectiveDate; }
         set { _certificationEffectiveDate = value; }
      }

      public HI RequestForAdditionalInformation
      {
         get { return _requestForAdditionalInformation; }
         set { _requestForAdditionalInformation = value; }
      }

      public SV1 ProfessionalService
      {
         get { return _professionalService; }
         set { _professionalService = value; }
      }

      public SV2 InstitutionalService
      {
         get { return _institutionalService; }
         set { _institutionalService = value; }
      }

      public SV3 DentalService
      {
         get { return _dentalService; }
         set { _dentalService = value; }
      }

      public List<TOO> ToothInformation
      {
         get { return _toothInformation; }
         set { _toothInformation = value; }
      }

      public HSD HealthCareServicesDelivery
      {
         get { return _healthCareServicesDelivery; }
         set { _healthCareServicesDelivery = value; }
      }

      public List<PWK> AdditionalServiceInformation
      {
         get { return _additionalServiceInformation; }
         set { _additionalServiceInformation = value; }
      }

      public MSG MessageText
      {
         get { return _messageText; }
         set { _messageText = value; }
      }

      public DetailBase Other
      {
         get { return _other; }
         set { _other = value; }
      }

      public List<EntityBase> ServiceProviderName
      {
         get { return _serviceProviderName; }
         set { _serviceProviderName = value; }
      }

      public EntityBase AdditionalServiceInformationContactName
      {
         get { return _additionalServiceInformationContactName; }
         set { _additionalServiceInformationContactName = value; }
      }


      public override string ToString()
      {
         string strReturn = string.Empty;

         if (_serviceLevel != null) strReturn += _serviceLevel.ToString();

         if (_serviceTraceNumber != null)
         {
            foreach (TRN t in _serviceTraceNumber)
            {
               strReturn += t.ToString();
            }
         }

			if (_serviceRequestValidation != null)
			{
				int aaaCount = 0;
				foreach (AAA a in _serviceRequestValidation)
				{
					aaaCount++; if (aaaCount > 9) break;
					strReturn += a.ToString();
				}
			}


			if (_healthCareServicesReviewInformation != null) strReturn += _healthCareServicesReviewInformation.ToString();

         if (_healthCareServicesReview != null) strReturn += _healthCareServicesReview.ToString();

			// REF
			// do not return --- if (_previousReviewAuthorizationNumber != null) strReturn += _previousReviewAuthorizationNumber.ToString();


			// return AdministrativeReferenceNumber only if no authorization found.  set ReferenceIdentification to ResponseId after creating response record
			if (settings.AuthorizationCount == 0)
			{
				REF refInfo = new REF(settings);
				refInfo.ReferenceIdentificationQualifier = new IDENTIFIER("NT");
				refInfo.ReferenceIdentification = "";
				strReturn += refInfo.ToString();
			}


			if (_other != null) _other.CheckSegment("REF", ref strReturn);


         if (_serviceDate != null) strReturn += _serviceDate.ToString();

         if (_certificationIssueDate != null) strReturn += _certificationIssueDate.ToString();

         if (_certificationExpirationDate != null) strReturn += _certificationExpirationDate.ToString();

         if (_certificationEffectiveDate != null) strReturn += _certificationEffectiveDate.ToString();

         if (_other != null) _other.CheckSegment("DTP", ref strReturn);

         if (_requestForAdditionalInformation != null) strReturn += _requestForAdditionalInformation.ToString();

			if (_other != null) _other.CheckSegment("HI", ref strReturn);  // 215 only

			if (_professionalService != null) strReturn += _professionalService.ToString();

         if (_institutionalService != null) strReturn += _institutionalService.ToString();

         if (_dentalService != null) strReturn += _dentalService.ToString();

         if (_toothInformation != null)
         {
            foreach (TOO t in _toothInformation)
            {
               strReturn += t.ToString();
            }
         }

         if (_healthCareServicesDelivery != null) strReturn += _healthCareServicesDelivery.ToString();

         if (_other != null) _other.CheckSegment("CRC", ref strReturn);

         if (_other != null) _other.CheckSegment("CL1", ref strReturn);

         if (_other != null) _other.CheckSegment("CR1", ref strReturn);

         if (_other != null) _other.CheckSegment("CR2", ref strReturn);

         if (_other != null) _other.CheckSegment("CR4", ref strReturn);

         if (_other != null) _other.CheckSegment("CR5", ref strReturn);

         if (_other != null) _other.CheckSegment("CR6", ref strReturn);

			if (_other != null) _other.CheckSegment("PWK", ref strReturn);  //215 only

			if (_other != null) _other.CheckSegment("CR7", ref strReturn);

         if (_other != null) _other.CheckSegment("CR8", ref strReturn);

         if (_additionalServiceInformation != null)
         {
            foreach (PWK p in _additionalServiceInformation)
            {
               strReturn += p.ToString();
            }
         }

         if (_messageText != null) strReturn += _messageText.ToString();

         if (_serviceProviderName != null)
         {
            foreach (EntityBase serviceProviderName in _serviceProviderName)
            {
               strReturn += serviceProviderName.EntityBaseToString("SS", serviceProviderName.RequestValidation);
            }
         }

         if (_additionalServiceInformationContactName != null) strReturn += _additionalServiceInformationContactName.ToString();




			return strReturn;
      }


   }
}
