package va.gov.controller;

import java.io.IOException;
import java.net.URLEncoder;
import java.security.Principal;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.joda.time.Duration;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;

import va.gov.domain.ARPSQueueAdapter;
import va.gov.domain.QueueListAdapter;
import va.gov.jpa.dto.ARPSQueueCodes;
import va.gov.jpa.dto.EntityName;
import va.gov.jpa.dto.HealthServicesReview;
import va.gov.jpa.dto.TransactionLog;
import va.gov.jpa.dto.UserAuthorization;
import va.gov.jpa.service.GeneralService;
import va.gov.model.QueueList;
import va.gov.security.AESEncryption;
import va.gov.service.PatientServiceIntf;
import va.gov.util.Utility;

/**
 * 
 * @author 
 *
 */
@Controller
public class HomeController  {

	private static final Logger LOGGER = Logger.getLogger(HomeController.class);
	
	@Autowired
	private PatientServiceIntf patService;

	public void setPatService(PatientServiceIntf patService) {
		this.patService = patService;
	}

   
    ModelMap modelMap = new ModelMap();
    
     
 	
	public HomeController() {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in HomeController:-HomeController ");						
		}
	    	
	}
	
	/**
	 * Injected from security.properties
	 */
	@Value("${login.aspCheck}")
	private String aspCheck;
	
	/**
	 * Injected from security.properties
	 */
	@Value("${login.capturedUrl}")
	private String capturedUrl;
	
	/**
	 * Injected from security.properties
	 */
	@Value("${login.forwardUrl}")
	private String forwardUrl;
	
	/**
	 * This view initiates the authentication and authorization process.
	 * Upon acceptance of the Warning by the user, the view will be
	 * posted to the authentication service provider.
	 * 
	 * @param model	Object returned to the view
	 * @param req	HttpServletRequest
	 * @param res	HttpServletResponse
	 * @return		The view to be displayed
	 */
	@RequestMapping(value={"/", "/warning"})
	public String warning(Model model, HttpServletRequest request, HttpServletResponse response) {
				
		String encryptedCapturedUrl = "";
		String encryptedForwardUrl = "";
		String encodedCapturedUrl = "";
		String encodedForwardUrl = "";
        HttpSession session = request.getSession();
		String host = request.getLocalName();
		
		int port = request.getLocalPort();
		host = new StringBuffer(host).append(':').append(port).toString();

        

		LOGGER.trace("Encrypt and encode certain parameters to be posted");
    	try {    		
    		encryptedCapturedUrl = AESEncryption.encrypt(MessageFormat.format(capturedUrl, host));
			encodedCapturedUrl = URLEncoder.encode(encryptedCapturedUrl,"UTF-8");
			
			encryptedForwardUrl = AESEncryption.encrypt(MessageFormat.format(forwardUrl, host));
            encodedForwardUrl = URLEncoder.encode(MessageFormat.format(encryptedForwardUrl, host), "UTF-8");
		} catch (Exception e) {
			LOGGER.error(e);
		}

    	LOGGER.trace("Add the attributes to the model");
		model.addAttribute("aspCheck",aspCheck);
		model.addAttribute("capturedUrl", encodedCapturedUrl);
		model.addAttribute("forwardUrl", encodedForwardUrl);
		model.addAttribute("session",session.getId());
		
		return "/warning";
	}
	
	public String currentUserNameSimple(HttpServletRequest request) {
        Principal principal = request.getUserPrincipal();
        return principal.getName();
    }
	
	/**
	 * The view returned if the login failed.
	 * 
	 * @return		"/warning?login=failed" view
	 */
	@RequestMapping(value = "/login/failed")
	public String login() {		
		return "/warning?login=failed";
	}
	
	/**
	 * The view returned if the user attempted to access a restricted area
	 * 
	 * @return		"/error/403" view
	 */
	@RequestMapping(value = "/error/403")
	public String accessDenied() {		
		return "/error/403";
	}

	/**
	 * This method view the front page contains from database table StaticPageInfo
	 * @param request
	 * @param response
	 * @param command
	 * @param errors
	 * @return ModelAndView
	 * @throws ServletException
	 * @throws IOException
	 */
	
	@RequestMapping(value = "/allpatPortal", method = {RequestMethod.POST, RequestMethod.GET})
	
	public ModelAndView patPortal(
			HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in HomeController:-patPortal");
		}
		
		
		UserDetails  udetail= null;
		
	//	User user = (User)SecurityContextHolder.getContext().getAuthentication().getPrincipal();
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		if (!(auth instanceof AnonymousAuthenticationToken)) {
			udetail = (UserDetails)auth.getPrincipal();
		}
		if(udetail.getUsername()!=null && udetail.getUsername().length()>0){
		String user =udetail.getUsername().split("::")[1];
		modelMap.addAttribute("user", user);
		 
		UserAuthorization uath = patService.getUserAuth(user);
		
		if (uath!=null){
			modelMap.addAttribute("stationNo", uath.getStation_number()); 
		}
		if( uath == null)
		{
			String msg="User " + udetail.getUsername() + " does not have authorization for ARPS";
			modelMap.addAttribute("uerNotAuthorizedFlag", "n");
				modelMap.addAttribute("uerNotAuthorized", msg);
		}	
		
		else
		{
		
		 List<ARPSQueueCodes> qList = patService.getQueueList(uath.getStation_number());			
		
		String queueId=(String)request.getParameter("queue");
		 List<TransactionLog> pList=null;
			if (queueId==null){
				//default to incoming request
				queueId="4";
				 pList = patService.getAllPatientByQueue(new Integer(queueId), uath.getStation_number());
			} else{
				  pList = patService.getAllPatientByQueue(new Integer(queueId), uath.getStation_number());
				  //queue name
				  
			}
			
			JsonArray jarray = new JsonArray();
			if (pList.size()>0){
			
				for (int i=0; i<pList.size(); i++){
					
					JsonObject json = new JsonObject();
					TransactionLog quRow = (TransactionLog)pList.get(i);
					int reqId= quRow.getRequestId();
					json.addProperty("reqId", reqId);
					String reqDate= quRow.getRequestDate().toString();
					try {
						json.addProperty("requestDate", Utility.convertMMDDYYDate(reqDate));
					}
					catch (Exception e){
						json.addProperty("requestDate", "");
					}
					//now entity
					List<EntityName> ent= patService.getEntityInfo(reqId , "2010B"); //Loop code						
					if (ent.size()>0){
						EntityName entnm = ent.get(0);
						json.addProperty("requester",Utility.buildName(entnm));
					}
					
					List<EntityName> entad= patService.getEntityInfo(reqId , "2010C"); //Loop code
					if (entad.size()>0){					
						EntityName entN= entad.get(0);
						if (entN.getNameMiddle()!=null){
							json.addProperty("name",entN.getNameFirst()+' '+entN.getNameMiddle() +' '+ entN.getNameLastOrganization());
						}
						else{
							json.addProperty("name",entN.getNameFirst()+' '+ entN.getNameLastOrganization());
						}
						json.addProperty("id",entN.getEntityIdentificationCode());					
					//dob
						try {
							json.addProperty("dob",Utility.convertMMDDYYFromyyyyMMDD(entN.getDateOfBirth()));
						}
						catch (Exception e){
							json.addProperty("dob","");
						}
					}
//					DemographicInformation demo= patService.getDOBInfo(reqId, "2010C" ); 
//					if (demo!=null){
//						try {
//							json.addProperty("dob",Utility.convertMMDDYYDate(demo.getDateTimePeriod()));
//						}
//						catch (Exception e){
//							json.addProperty("dob","");
//						}
//					
//					}
//					else {
//						json.addProperty("dob","");
//					}
					
					//service
					HealthServicesReview hrev = patService.getHealthServicesInfo(reqId,"2000E","1" ); 
					if (hrev!=null){
					if(null !=hrev.getRequestCategCode()) {
						String strUM01 = patService.getDecodedDescription("2000E", "UM01", hrev.getRequestCategCode() );
						String strUM02 = patService.getDecodedDescription("2000E", "UM02", hrev.getCertificationType());
						String strUM03 = patService.getDecodedDescription("2000E", "UM03", hrev.getServicesTypeCode());
						String scodedesc =strUM01 + "-" +strUM02 +"-"+strUM03;
						
						json.addProperty("service",scodedesc );
						
					}
					}else{
						json.addProperty("service","");
					}
					//queue 
					 DateTimeFormatter dformat = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss.SSSSSS");
				   try {
					 DateTime reqd = dformat.parseDateTime(reqDate);
				     DateTime today = new DateTime();
				       
				     Duration duration = new Duration(reqd, today);
				     Long qDays= duration.getStandardDays();
				        
				    // Days.daysBetween(new LocalDate(start), new LocalDate(end)).getDays() ;
				     json.addProperty("queueDays", qDays.toString());
				     }
				     catch (Exception e) {
				    	 e.printStackTrace();
				     }
				        
					jarray.add(json);
					
				}
				modelMap.addAttribute("PatientList", jarray);
			}
			else {
				modelMap.addAttribute("PatientList", "");
			}
			String serviceList = null;
			List<ARPSQueueCodes> serviceCount = patService.getQueueList(uath.getStation_number());
			GsonBuilder gsonserviceList = new GsonBuilder();
			Gson gsonServ = gsonserviceList.registerTypeAdapter(TransactionLog.class, new ARPSQueueAdapter()).create();
			serviceList = gsonServ.toJson(serviceCount);
			serviceList= " {\"servicecount\": "+ serviceList +"}";
			modelMap.addAttribute("serviceList", serviceList);
						
			if (queueId!=null){
		        String qCode= queueId.toString();
		        String qdescr = GeneralService.getInstance().findQDescrebyQueueId(qCode);
		        modelMap.addAttribute("QueueName", qdescr);
	        }
			
		}
		}
			
		return new ModelAndView("welcome", modelMap);
	}

private void  groupQueues( List<ARPSQueueCodes> qList  ){
	
	JsonArray ojarray = new JsonArray();
	
	Map<String, List<ARPSQueueCodes>> groupedStudents = new HashMap<String, List<ARPSQueueCodes>>();
//	for (ARPSQueueCodes queue: qList) {
//	    String key = queue.getQueueName();
//	    if (groupedStudents.get(key) == null) {
//	        groupedStudents.put(key.toString(), new ArrayList<ARPSQueueCodes>());
//	    }
//	    groupedStudents.get(key).add(queue);
//	}
	
	
	// Now let's group all person by city in pre Java 8 world        
	 // Let's see how we can group objects in Java 8
	 Map<String,List<ARPSQueueCodes>> personByCity = new HashMap<>();
	 ArrayList<ARPSQueueCodes> hfque= new ArrayList<ARPSQueueCodes>();
	 ArrayList<ARPSQueueCodes> psque= new ArrayList<ARPSQueueCodes>();
	 ArrayList<ARPSQueueCodes> refque= new ArrayList<ARPSQueueCodes>();
	 
	 for(ARPSQueueCodes p : qList){
         if(!personByCity.containsKey(p.getMenuOrder())){
        	 if (p.getMenuOrder()==1){	 
        		 hfque.add(p);
        	 }
        	 else  if (p.getMenuOrder()==2){	 
        		 psque.add(p);
        	 } else  if (p.getMenuOrder()==3){
        		 refque.add(p);
        		 }
         }
       //  personByCity.get(p.getQueueName()).add(p);
     }
	 
	 QueueList l1hfqList = new QueueList();
	 l1hfqList.setL1QueueName("High Priority Queue");
	 l1hfqList.setQueueList(hfque);
	 QueueList l1PList = new QueueList();
	 l1PList.setL1QueueName("Processing Queue");
	 l1PList.setQueueList(psque);
	 QueueList l1rList = new QueueList();
	 l1rList.setL1QueueName("Archive");
	 l1rList.setQueueList(refque);
	 
	 ArrayList <QueueList> dnList = new ArrayList<QueueList>();
	 dnList.add(l1hfqList);
	 dnList.add(l1PList);
	 dnList.add(l1rList);
	 
	 
	 
	 GsonBuilder gsonBuilder = new GsonBuilder();
		Gson gsondr = gsonBuilder.registerTypeAdapter(QueueList.class, new QueueListAdapter()).create();
		String qjson = gsondr.toJson(dnList);
		qjson= " {\"Queues\": "+ qjson +"}";
		modelMap.addAttribute("queueList", qjson);
 
	 
//	 
//	 JsonArray ijarray = new JsonArray();
//		
//	 for (int i=0; i<hfque.size(); i++){
//		 JsonObject qjson = new JsonObject();
//		 qjson.addProperty("queuename", hfque.get(i).getQueueName().toString());
////		 qjson.addProperty("queueId", String.valueOf(hfque.get(i).getQueueId()));
////		 qjson.addProperty("qCount", String.valueOf(hfque.get(i).getqCount()));
//		 ijarray.add(qjson);
//	 }
//	 JsonObject oqjson = new JsonObject();
//	 oqjson.addProperty("QueueGrp", "High Frequency Queue");
//	 oqjson.addProperty("QueueList", ijarray.toString());
//	 ojarray.add(oqjson);
//	 
//	 JsonArray ipjarray = new JsonArray();
//		
//	 for (int i=0; i<psque.size(); i++){
//		 JsonObject qjson = new JsonObject();
//		 qjson.addProperty("queuename", psque.get(i).getQueueName());
////		 qjson.addProperty("queueId", String.valueOf(psque.get(i).getQueueId()));
////		 qjson.addProperty("qCount", String.valueOf(psque.get(i).getqCount()));
//		 ipjarray.add(qjson);
//	 }
//	 JsonObject opjson = new JsonObject();
//	 opjson.addProperty("QueueGrp", "Processing Queue");
//	 opjson.addProperty("QueueList", ipjarray.toString());
//	 ojarray.add(opjson);
//	 
//	 JsonArray irjarray = new JsonArray();
//	 for (int i=0; i<refque.size(); i++){
//		 JsonObject qjson = new JsonObject();
//		 qjson.addProperty("queuename", refque.get(i).getQueueName());
////		 qjson.addProperty("queueId", String.valueOf(refque.get(i).getQueueId()));
////		 qjson.addProperty("qCount", String.valueOf(refque.get(i).getqCount()));
//		 
//		 irjarray.add(qjson);
//	 }
//	 JsonObject orjson = new JsonObject();
//	 orjson.addProperty("QueueGrp", "Reference Queue");
//	 orjson.addProperty("QueueList", irjarray.toString());
//	 ojarray.add(orjson);
//	 
//	
//
//	    
//	 modelMap.addAttribute("queueList", ojarray);
//	 
//	 String feed =  ojarray.toString().replace("\\","");
//     
//     String feeds= " {\"Queues\": "+ feed +"}";

   // return feeds;

}

}

