package va.gov.controller;

import java.security.Principal;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.json.JSONArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonObject;

import va.gov.domain.AATGUIQueueAdapter;
import va.gov.domain.AcceptDenialReasonAdapter;
import va.gov.domain.InstitutionAdapter;
import va.gov.jpa.dto.ARPSQueueCodes;
import va.gov.jpa.dto.DateInformation;
import va.gov.jpa.dto.DenialReason;
import va.gov.jpa.dto.EntityAddress;
import va.gov.jpa.dto.EntityName;
import va.gov.jpa.dto.EventMenuItem;
import va.gov.jpa.dto.HealthServicesReview;
import va.gov.jpa.dto.Institution;
import va.gov.jpa.dto.TransactionLog;
import va.gov.jpa.dto.ServicesMenuItem;
import va.gov.jpa.dto.UserAuthorization;
import va.gov.model.AcceptDenial;
import va.gov.service.PatientServiceIntf;
import va.gov.util.Utility;

@Controller
public class NavController {

	private static final Logger LOGGER = Logger.getLogger(NavController.class);

	@Autowired
	private PatientServiceIntf patService;

	public void setPatService(PatientServiceIntf patService) {
		this.patService = patService;
	}



	ModelMap modelMap = new ModelMap();

	public NavController() {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in NavSpController:-NavSpController");
		}
	}


	/**
	 * This method open the checkLogin form
	 * @param request
	 * @param response
	 * @param command
	 * @return ModelAndView
	 * @throws Exception
	 */
	
	@RequestMapping(value = "/patIndPortal", method = RequestMethod.GET)
	
	public ModelAndView navspIndPortal(HttpServletRequest request, HttpServletResponse response, @RequestParam("sno") String sno, @RequestParam("qname") String qname) throws Exception {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in NavController:patIndPortal");
		}
		
		String rid= (String)request.getParameter("id");	
		
		String depFlag= getDependentInfo(rid );
		if (depFlag.equals("1")){
			String depMenu=getDepMenuByReq(rid);
			modelMap.addAttribute("depMenu", depMenu);		
		}
		String lmenu = getLeftMenuItem2000EByReq(rid);		
		modelMap.addAttribute("l2000EMenu", lmenu);
		
		String l2000Fmenu = getLeft2000FMenuItemByReq(rid);		
		modelMap.addAttribute("l2000FMenu", l2000Fmenu);

		getQueueList(rid, sno);
		getStationList(rid, sno);
		getDenialReasonList(rid);
		
		patientIndHeaderPart(request,rid);
		modelMap.addAttribute("patid", request.getParameter("id"));
		modelMap.addAttribute("qname", qname);
		
		String loginName="";
			Principal userCredentials  = request.getUserPrincipal();
			if (userCredentials != null)  
				loginName = userCredentials.getName();
			
			writeToAuditLog(  rid );
			
			UserDetails  udetail= null;
			Authentication auth = SecurityContextHolder.getContext().getAuthentication();
			if (!(auth instanceof AnonymousAuthenticationToken)) {
				udetail = (UserDetails)auth.getPrincipal();
			}
			if(udetail!=null && udetail.getUsername()!=null && udetail.getUsername().length()>0){
				String user =udetail.getUsername().split("::")[1];
				modelMap.addAttribute("user", user);
				 
				UserAuthorization uath = patService.getUserAuth(user);
				
				if (uath!=null){
					modelMap.addAttribute("stationNo", uath.getStation_number()); 
				} else{
					String msg="User " + udetail.getUsername() + " does not have authorization for ARPS";
						modelMap.addAttribute("uerNotAuthorizedFlag", "n");
						modelMap.addAttribute("uerNotAuthorized", msg);
				}	
			}
		return new ModelAndView("spIndPortal", modelMap);
	}
	
	private String getDependentInfo( String rid ){
		
		List<EntityName> dent= patService.getEntityInfo(new Integer(rid) , "2010D");

		if (dent.size()>0){
			modelMap.addAttribute("depFlag", "1");
			return "1";
		} else {
			modelMap.addAttribute("depFlag", "0");
			return "0";
		}
			
		}
	
//	private void getTracking_Care( String rid ){
//	//trace
//		List<TrackingNumber> dinfo= patService.getTrackList(new Integer(rid) , "2000E", "1");
//		JsonObject djson = new JsonObject();
//		if (dinfo!=null){
//			djson.addProperty("trackFlag","1");
//		} else {
//			djson.addProperty("trackFlag","0");
//		}
//			modelMap.addAttribute("trackFlag", djson);
//		
//	}
	
//	private void getServiceReview_Care( String id ){
//		//service Review
//		Integer	rId = new Integer(id);
//		HealthServicesReview hrev = patService.getHealthServicesInfo(rId ); 
//		JsonObject djson = new JsonObject();
//			if (hrev!=null){
//				djson.addProperty("sReviewFlag","1");
//			} else {
//				djson.addProperty("sReviewFlag","0");
//			}
//				modelMap.addAttribute("sReviewFlag", djson);
//			
//		}

	private void patientIndHeaderPart( HttpServletRequest request,String rid ){
		
		if (rid!=null){
			Integer	rId = new Integer((rid));
		
		JsonObject patientjson = new JsonObject();
		
		//name
		List<EntityName> entN= patService.getEntityInfo(rId , "2010B"); //Loop code
		EntityName entm = entN.get(0);
		
		patientjson.addProperty("rname",Utility.buildName(entm));
		patientjson.addProperty("rId", entm.getEntityIdentificationCode());
		
		//authorization
		DateInformation dinfo= patService.getDateInfo(rId , "AAH", "1");
		if (dinfo!=null){
			String aFromto= dinfo.getDateTimePeriod();
			String dateFormatStr = dinfo.getDateTimePeriodFormat();
			String[] atokens = aFromto.split("-");
			patientjson.addProperty("authFrom", Utility.conStrToMMDDYY("D8", atokens[0]));
			patientjson.addProperty("authTo", Utility.conStrToMMDDYY("D8" , atokens[1]));
		}
		else{
			patientjson.addProperty("authFrom", " ");
			patientjson.addProperty("authTo", " ");
		}
		//admission
		DateInformation adinfo= patService.getDateInfo(rId , "435", "1");
		if (adinfo!=null){
			patientjson.addProperty("admissionDat", Utility.conStrToMMDDYY("D8" ,adinfo.getDateTimePeriod()));
		}
		//278 request
		TransactionLog quRow = patService.getARPSQueue(rId );;
		if (quRow!=null &&  quRow.getRequestDate()!=null) {
			String reqDate= quRow.getRequestDate().toString();
			try {
			patientjson.addProperty("requestDate", Utility.convertMMDDYYDate(reqDate));
			}
			catch (Exception e){
				patientjson.addProperty("requestDate", "");
			}
			patientjson.addProperty("queueName", quRow.getQueueId());
		}
		else{
			patientjson.addProperty("requestDate", "");
			patientjson.addProperty("queueName", "");
		}
		
	//Patient -Subscriber information 
		List<EntityName> ent= patService.getEntityInfo(new Integer(rid) , "2010C"); //Loop code		
		EntityName et =(EntityName) ent.get(0);
		patientjson.addProperty("pname",Utility.buildName(et));
		patientjson.addProperty("pId",et.getEntityIdentificationCode());
		try {
			patientjson.addProperty("pdob",Utility.convertMMDDYYFromyyyyMMDD(et.getDateOfBirth()));
		}
		catch (Exception e){
			patientjson.addProperty("pdob","");
		}

		patientjson.addProperty("pgender",et.getGender());
		
		EntityAddress add= patService.getAddress(rId , "2010C");
		String reqAdd;
		if (add != null) {
			if (add.getAddress2()!=null && !add.getAddress2().equals("")){
			 reqAdd= add.getAddress1() + ", "+add.getAddress2()+", " +add.getCity()+", "+add.getStateProvCode()+" "+add.getPostalCode();
			}
			else {
				reqAdd= add.getAddress1() +", " +add.getCity()+","+add.getStateProvCode()+" "+add.getPostalCode();
			}
			patientjson.addProperty("paddress", reqAdd);
		}
		else{
			patientjson.addProperty("paddress", "");
		}
		patientjson.addProperty("pId",et.getEntityIdentificationCode());
		//service
		HealthServicesReview hrev = patService.getHealthServicesInfo(rId,"2000E","1" ); 
		if (hrev!=null){
		if(null !=hrev.getRequestCategCode()) {
			String strUM01 = patService.getDecodedDescription("2000E", "UM01", hrev.getRequestCategCode() );
			String strUM02 = patService.getDecodedDescription("2000E", "UM02", hrev.getCertificationType());
			String strUM03 = patService.getDecodedDescription("2000E", "UM03", hrev.getServicesTypeCode());
			String scodedesc =strUM01 + "-" +strUM02 +"-"+strUM03;
			
			patientjson.addProperty("service",scodedesc );
		}
		}else{
			patientjson.addProperty("service","");
		}
		List<EntityName> entNR= patService.getEntityInfo(new Integer(rid) , "2010C"); //Loop code
		EntityName etnm =(EntityName) ent.get(0);
		if (etnm.getNameMiddle()!=null){
			patientjson.addProperty("name",etnm.getNameFirst()+' '+etnm.getNameMiddle() +' '+ etnm.getNameLastOrganization());
		}
		else{
			patientjson.addProperty("name",etnm.getNameFirst()+' '+ etnm.getNameLastOrganization());
		}
		patientjson.addProperty("qname", request.getParameter("qname"));
		
		modelMap.addAttribute("Patient", patientjson);
		modelMap.addAttribute("reqId", rId);
	
		}
		
	}
	
//	private String buildName(EntityName  entm){
//		String name=null;
//		if (entm.getNameLastOrganization()!=null){
//			name = entm.getNameLastOrganization();
//		}
//		if (entm.getNameFirst()!=null){
//			name = entm.getNameFirst() +" " +entm.getNameLastOrganization();
//		}
//		if (entm.getNameFirst()!=null && entm.getNameMiddle()!=null ){
//			name = entm.getNameFirst() +" " +entm.getNameMiddle()+ " "+entm.getNameLastOrganization();
//		}
//		if (entm.getNameFirst()!=null &&  entm.getNameSuffix()!=null ){
//			name = entm.getNameFirst() + " "+entm.getNameLastOrganization() +" "+ entm.getNameSuffix();
//		}
//		if (entm.getNameFirst()!=null && entm.getNameMiddle()!=null && entm.getNameSuffix()!=null ){
//			name = entm.getNameFirst() +" " +entm.getNameMiddle()+ " "+entm.getNameLastOrganization() +" "+ entm.getNameSuffix();
//		}
//		if (entm.getNameFirst()!=null && entm.getNameMiddle()!=null && entm.getNameSuffix()!=null && entm.getNamePrefix()!=null ){
//			name = entm.getNamePrefix() + " "+entm.getNameFirst() +" " +entm.getNameMiddle()+ " "+entm.getNameLastOrganization() +" "+ entm.getNameSuffix();
//		}
//
//		return name;
//	}
	
	private void writeToAuditLog( String rid ){
		//add auditlog 
		UserDetails udetail =null;
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		if (!(auth instanceof AnonymousAuthenticationToken)) {
			 udetail = (UserDetails)auth.getPrincipal();
		
		if (udetail.getUsername()!=null){
			String user =udetail.getUsername().split("::")[1];
			modelMap.addAttribute("user", user);
			boolean addAudit = patService.postAuditLog(user, "accessed by", "", "", rid);
		}
		
		}
	}
	
	private void getDenialReasonList( String rid ){
		//get denial reason
		GsonBuilder gsonBuilder = new GsonBuilder();
		
		List<DenialReason> dnList = patService.getAllDenialReason();
		Gson gsondr = gsonBuilder.registerTypeAdapter(AcceptDenial.class, new AcceptDenialReasonAdapter()).create();
		if (dnList!=null){
			String drjson = gsondr.toJson(dnList);
			drjson= " {\"dr\": "+ drjson +"}";
			modelMap.addAttribute("DenialRList", drjson);
		}
	}
	
	private void getQueueList( String rid, String sno ){
		
		String feeds = null;
	
		 List<ARPSQueueCodes> qList = patService.getQueueList(sno);
		 if (qList!=null){
		GsonBuilder gsonaaque = new GsonBuilder();
		Gson gsonq = gsonaaque.registerTypeAdapter(ARPSQueueCodes.class, new AATGUIQueueAdapter()).create();
		 feeds = gsonq.toJson(qList);
		 feeds= " {\"queue\": "+ feeds +"}";
			modelMap.addAttribute("queueList", feeds);		
		 }
	}
	
	private void getStationList( String rid, String sno ){
		
		String feeds = null;
		 List<Institution> qList = patService.getStationList(new Integer(sno));
		 if (qList!=null){
			 
			 
		//GsonBuilder gsonaaque = new GsonBuilder();
		//Gson gsonq = gsonaaque.registerTypeAdapter(Institution.class, new InstitutionAdapter()).create();
		// feeds = gsonq.toJson(qList.toArray());
		 JSONArray JSONArray = new JSONArray(qList); 
		 feeds= " {\"station\": "+ JSONArray +"}";
			modelMap.addAttribute("stationList", feeds);		
		 }
	}
	
	private String getLeft2000FMenuItemByReq( String rid ) {
		ServicesMenuItem dMenu = patService.getServiceMenuItemByreqId(new Integer(rid));
		//get the highest number
		int maxval = getHighestFromColumns(dMenu);
		String out="";
		String endst="";
		int i=0;
		
		
		for (i=maxval; i>0; i--){
			endst = out +endst;
			out=   "<div class=\"accordion-group\">"
					+ "<div class=\"accordion-heading servaccorddiv\" data-loopNo=\""+i+"\" id=\"servaccorddiv\"> <a class=\"accordion-toggle incolor\" data-toggle=\"collapse\" data-parent=\"#leftMenu1\" href=\"#servicesDiv"+i+"\">Service Request- "+i+" </a>  </div>";
			out=out+ "<div id=\"servicesDiv"+i+"\" class=\"accordion-body \" > "
					+ "<div class=\"accordion-inner\"> "
					+ "<ul>";
            
			if (!dMenu.getLtncount().equals("0") && i<= new Integer(dMenu.getLtncount() ))
				 out=out +" <li id='tracesli' ><a  id='traceS' data-loopNo=\""+i+"\">Trace No</a></li>";
			if (!dMenu.getLhsrcount().equals("0") && i<= new Integer(dMenu.getLhsrcount()) )
				 out=out +" <li id='sreviewSli'><a  id='sReviewS' data-loopNo=\""+i+"\">Service Review </a></li>";
			if (!dMenu.getLrefcount().equals("0") && i<= new Integer(dMenu.getLrefcount() ))
				 out=out +" <li id='srefSli'><a  id='referenceS' data-loopNo=\""+i+"\"> Reference </a></li>";
			if (!dMenu.getLdatecount().equals("0")  && i<= new Integer(dMenu.getLdatecount() ))
				 out=out + "\n <li id=\"dateSli\"><a  id='datesS'>Dates</a></li>";
			
		 if (!dMenu.getLspcount().equals("0") && i<= new Integer(dMenu.getLspcount()) )
			 out= out+"<li id='profSli'><a  id='profService' data-loopNo=\""+i+"\">Professional Service </a></li>" ;
		  if (!dMenu.getLiclaimcount().equals("0")  && i<= new Integer(dMenu.getLiclaimcount()) )
			 out=out +"\n <li id='instSli'><a  id='instServS' data-loopNo=\""+i+"\">Institutional Service </a></li>";
		 if (!dMenu.getLsdcount().equals("0") && i<= new Integer(dMenu.getLsdcount()) )
			 out=out +" <li id='dentSli'><a  id='dental' data-loopNo=\""+i+"\">Dental Service</a></li>";
		 if (!dMenu.getLstcount().equals("0") && i<= new Integer(dMenu.getLstcount()) )
			 out=out +" <li id='tootSli'><a  id='toothInfo' data-loopNo=\""+i+"\">Tooth Information</a></li>";
		  if (!dMenu.getLiclaimcount().equals("0") && i<= new Integer(dMenu.getLiclaimcount()) )
			 out=out +" <li id='serdli'><a  id='serDelivery' data-loopNo=\""+i+"\">HealthCare Service Delivery </a></li>";
		 
		// if (!dMenu.getLpInfocount().equals("0") && i<= new Integer(dMenu.getLpInfocount() ))
		//	 out=out +"  <li id=\"addIli\"><a  id=\"addInfo\">Additional Services</a></li>";
		 if (!dMenu.getLmsgcount().equals("0") && i<= new Integer(dMenu.getLmsgcount() ))
			 out=out +"  <li id=\"msgSli\"><a  id='msgTxtS' data-loopNo=\""+i+"\">Message Text</a></li>";
		 if (!dMenu.getLentcount().equals("0") && i<= new Integer(dMenu.getLentcount() ))
			 out=out +"  <li id=\"serProvSli\"><a  id='sProvsvcS' data-loopNo=\""+i+"\">Service Provider Name</a></li>";
		// if (!dMenu.getLpInfocount().equals("0") && i<= new Integer(dMenu.getLpInfocount() ))
			//	 out=out +"  <li id=\"contSli\"><a  id=\"aContactS\">Additional Contact</a></li>";
		 
		 out = out+"</ul></div></div></div>";
		}
		// out = out+"</div></div>";
		endst = out +endst;
		//System.out.println("getLeft2000FMenuItemByReq :-"+ endst);
		return endst;
	}
	
	private int getHighestFromColumns(ServicesMenuItem dMenu){
		int i=0;
		int retvalu=0;
		 if (new Integer(dMenu.getLdatecount()) >retvalu)
			 retvalu=new Integer(dMenu.getLdatecount());
		 if (new Integer(dMenu.getLhsrcount()) >retvalu)
			 retvalu=new Integer(dMenu.getLhsrcount());
		 if (new Integer(dMenu.getLhcsdcount()) >retvalu )
			 retvalu=new Integer(dMenu.getLhcsdcount());
		 if ( new Integer(dMenu.getLrefcount()) >retvalu)
			 retvalu=new Integer(dMenu.getLrefcount());
		 if ( new Integer(dMenu.getLiclaimcount()) >retvalu)
			 retvalu=new Integer(dMenu.getLiclaimcount());
		 if ( new Integer(dMenu.getLstcount()) >retvalu)
			 retvalu=new Integer(dMenu.getLstcount());
		 if ( new Integer(dMenu.getLsdcount()) >retvalu)
			 retvalu=new Integer(dMenu.getLsdcount());
		 if ( new Integer(dMenu.getLspcount()) >retvalu)
			 retvalu=new Integer(dMenu.getLspcount());
		 if ( new Integer(dMenu.getLmsgcount()) >retvalu)
			 retvalu=new Integer(dMenu.getLmsgcount());
		 if ( new Integer(dMenu.getLtncount()) >retvalu)
			 retvalu=new Integer(dMenu.getLtncount());
		
		 //System.out.println(retvalu);
	
		return retvalu;
	}
	
	private String  getDepMenuByReq( String rid ) {
		String out ="<div class=\"accordion-group\"><div class=\"accordion-heading\">"
				+ " <a class=\"accordion-toggle\" data-toggle=\"collapse\" data-parent=\"#leftMenu\" href=\"#collapseThree\">"
						+ "<i class=\"icon-th-list\"></i> Dependent Information</a></div>"
								+ "<div id=\"collapseThree\" class=\"accordion-body\" >"
										+ "<div class=\"accordion-inner\">"
												+ "<ul><li><a  id=\"dependent\">Dependent</a></li></ul>"
														+ "</div></div></div>";
		
		return out;
	}

	private String  getLeftMenuItem2000EByReq( String rid ) {
		EventMenuItem dMenu = patService.getEventMenuItemByreqId(new Integer(rid));
		//get the highest number
	//	int maxval = getHighestFromColumns(dMenu);
		String out="";
		String endst="";
		int i=0;
		
		
		//for (i=maxval; i>0; i--){
			endst = out +endst;
			out=   "<div class=\"accordion-group\">";
	//				+ "<div class=\"accordion-heading\" id=\"serviceAcordDiv"+i+"> <a class=\"accordion-toggle care\" data-toggle=\"collapse\" data-parent=\"#leftMenu1\" href=\"#serviceDiv"+i+"\">Episode - "+i+" </a>  </div>";
			out=out+ "<div id=\"serviceDiv"+i+"\" class=\"accordion-body \" > "
					+ "<div class=\"accordion-inner\"> "
					+ "<ul>";
               
		 if (!dMenu.getLcertcount().equals("0") && i<= new Integer(dMenu.getLcertcount()) )
			 out= out+"<li id='certli'><a  id='certificate'>Certificate Info </a></li>" ;
		 if (!dMenu.getLdatecount().equals("0")  && i<= new Integer(dMenu.getLdatecount() ))
			 out=out + "\n <li id=\"datesli\"><a  id=\"dates\">Dates</a></li>";
		 if (!dMenu.getLdiagcount().equals("0")  && i<= new Integer(dMenu.getLdiagcount()) )
			 out=out +"\n <li id='diagnosisli'><a  id='diagnosis'>Diagnosis </a></li>";
		 if (!dMenu.getLhcarecount().equals("0") && i<= new Integer(dMenu.getLhcarecount()) )
			 out=out +" <li id='hcareli'><a  id='homeCare'>Home Health Care</a></li>";
		 if (!dMenu.getLhcsdcount().equals("0") && i<= new Integer(dMenu.getLhcsdcount()) )
			 out=out +" <li id='SerDli'><a  id='careSerDelivery'>Health Services Delivery</a></li>";
		 if (!dMenu.getLhsrcount().equals("0") && i<= new Integer(dMenu.getLhsrcount()) )
			 out=out +" <li id='sReviewli'><a  id='sReview'>Service Review </a></li>";
		 if (!dMenu.getLiclaimcount().equals("0") && i<= new Integer(dMenu.getLiclaimcount()) )
			 out=out +" <li id='instli'><a  id='institution'>Institution Claim </a></li>";
		 if (!dMenu.getLoxycount().equals("0") && i<= new Integer(dMenu.getLoxycount() ))
			 out=out +"  <li id='oxyli'><a  id='oxygen'>Home Oxygen </a></li>";
		 if (!dMenu.getLrefcount().equals("0") && i<= new Integer(dMenu.getLrefcount() ))
			 out=out +" <li id='refli'><a  id='reference'>Reference </a></li>";
		 if (!dMenu.getLspinalcount().equals("0") && i<= new Integer(dMenu.getLspinalcount() ))
			 out=out +" <li id='spinalli'><a  id='spinal'>Spinal Info</a></li>";
		 if (!dMenu.getLtncount().equals("0") && i<= new Integer(dMenu.getLtncount() ))
			 out=out +" <li id='trackli' ><a  id='traceN'>Trace No</a></li>";
		 if (!dMenu.getLtransportcount().equals("0") && i<= new Integer(dMenu.getLtransportcount() ))
			 out=out +" <li id='transtli'><a  id='transport'>Transportation</a></li>";
		 if (!dMenu.getLpInfocount().equals("0") && i<= new Integer(dMenu.getLpInfocount() ))
			 out=out +"  <li id=\"addli\"><a  id=\"addCareInfo\">Additional Info</a></li>";
		 if (!dMenu.getLmsgcount().equals("0") && i<= new Integer(dMenu.getLmsgcount() ))
			 out=out +"  <li id=\"msgli\"><a  id=\"msgCareTxt\">Message Text</a></li>";
		 if (!dMenu.getLentcount().equals("0") && i<= new Integer(dMenu.getLentcount() ))
			 out=out +"  <li id=\"eProvli\"><a  id=\"evtProvider\">Event Provider Name</a></li>";
		 if (!dMenu.getLumocount().equals("0") && i<= new Integer(dMenu.getLumocount() ))
			 out=out +"  <li id=\"umoli\"><a  id=\"umo\">Other UMO</a></li>";
		 if (!dMenu.getLevtTranscount().equals("0") && i<= new Integer(dMenu.getLevtTranscount() ))
			 out=out +"  <li id=\"transpli\"><a  id=\"evtTransport\">Event Transport</a></li>";
		 out = out+"</ul></div></div></div>";
		//}
		// out = out+"</div></div>";
		endst = out +endst;
	//	System.out.println("final :-"+ endst);
		return endst;
	}	
}