
package va.gov.jpa.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

import org.apache.log4j.Logger;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import va.gov.jpa.dto.ARPSQueueCodes;
import va.gov.jpa.dto.AuditLog;
import va.gov.jpa.dto.CertificationInformation;
import va.gov.jpa.dto.CodeDataCodeLookUp;
import va.gov.jpa.dto.ContactInformation;
import va.gov.jpa.dto.DateInformation;
import va.gov.jpa.dto.DemographicInformation;
import va.gov.jpa.dto.DenialReason;
import va.gov.jpa.dto.DetailMenuItemList;
import va.gov.jpa.dto.DiagnosisInformation;
import va.gov.jpa.dto.EntityAddress;
import va.gov.jpa.dto.EntityName;
import va.gov.jpa.dto.EventMenuItem;
import va.gov.jpa.dto.HealthCareServicesDelivery;
import va.gov.jpa.dto.HealthServicesReview;
import va.gov.jpa.dto.HomeHealthCare;
import va.gov.jpa.dto.HomeOxygenTherapy;
import va.gov.jpa.dto.Institution;
import va.gov.jpa.dto.InstitutionClaim;
import va.gov.jpa.dto.InsuranceInformation;
import va.gov.jpa.dto.MessageText;
import va.gov.jpa.dto.PatientInformation;
import va.gov.jpa.dto.ProviderInformation;
import va.gov.jpa.dto.References;
import va.gov.jpa.dto.TransactionLog;
import va.gov.jpa.dto.SearchPatientById;
import va.gov.jpa.dto.ServicesDental;
import va.gov.jpa.dto.ServicesInstitution;
import va.gov.jpa.dto.ServicesMenuItem;
import va.gov.jpa.dto.ServicesProfessional;
import va.gov.jpa.dto.ServicesTooth;
import va.gov.jpa.dto.SpinalInformation;
import va.gov.jpa.dto.TrackingNumber;
import va.gov.jpa.dto.TranportInformation;
import va.gov.jpa.dto.UserAuthorization;

public class PatientService {

	@PersistenceUnit(unitName = "vaPU")
	EntityManager em = CommonEntityManagerFactory.entityManagerFactory();

	private static final Logger LOGGER = Logger.getLogger(PatientService.class);

	private static PatientService instance = null;

	public static synchronized PatientService getInstance() {
		/*
		 * Creates the Singleton instance, if needed.
		 */
		if (instance == null) {
			instance = new PatientService();
		}
		return instance;
	}
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<TransactionLog> findAllPatientByQueueId(Integer qid, String stationno) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findAllPatientByQueueId");
		}
		List<TransactionLog> userList = new ArrayList<TransactionLog>();
		
		try {
			// use entity manager to retrieve named query
			TypedQuery<TransactionLog> queryCollectByMail = em.createNamedQuery("TransactionLog.findQueueNameById",	TransactionLog.class);
			queryCollectByMail.setParameter("Id", qid);
			queryCollectByMail.setParameter("sid", stationno);

			
			// execute query and get results
			userList = queryCollectByMail.getResultList();

			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatientByQueueId:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<SearchPatientById> findPatientById(String qid, String stationno) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findPatientById");
		}
		List<SearchPatientById> userList = new ArrayList<SearchPatientById>();
		
		try {
			// use entity manager to retrieve named query
			TypedQuery<SearchPatientById> queryCollectByMail = em.createNamedQuery("AllRequestById.findbyid",	SearchPatientById.class);
			queryCollectByMail.setParameter("id", qid);
			queryCollectByMail.setParameter("selectedValues", Arrays.asList("1", "2","3"));
		
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();

			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findPatientById:==="+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public EventMenuItem findEventMenuByRid(Integer rid) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findEventMenuByRid");
		}
		List<EventMenuItem> userList = new ArrayList<EventMenuItem>();
		EventMenuItem user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<EventMenuItem> queryCollectByMail = em.createNamedQuery("DetailServiceMenuItem.findbyRid",	EventMenuItem.class);
			queryCollectByMail.setParameter("rid", rid);
	
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			if (userList.size() == 1) {
				user = (EventMenuItem) userList.get(0);
			}
			
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findEventMenuByRid:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public ServicesMenuItem findServiceMenuByRid(Integer rid) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findServiceMenuByRid");
		}
		List<ServicesMenuItem> userList = new ArrayList<ServicesMenuItem>();
		ServicesMenuItem user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<ServicesMenuItem> queryCollectByMail = em.createNamedQuery("ServicesMenuItem.findbyRid",	ServicesMenuItem.class);
			queryCollectByMail.setParameter("rid", rid);
	
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			if (userList.size() == 1) {
				user = (ServicesMenuItem) userList.get(0);
			}
			
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findServiceMenuByRid:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public DetailMenuItemList findLeftMenuByRidLoopCodeLoopNo(Integer reqId, String loopCode, Integer loopNo) {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findLeftMenuByRidLoopCodeLoopNo");
		}
		List<DetailMenuItemList> userList = new ArrayList<DetailMenuItemList>();
		DetailMenuItemList user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<DetailMenuItemList> queryCollectByMail = em.createNamedQuery("DetailMenuView.findbyRidLCodeLNo",	DetailMenuItemList.class);
			queryCollectByMail.setParameter("rid", reqId);
	
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			if (userList.size() == 1) {
				user = (DetailMenuItemList) userList.get(0);
			}
			
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findLeftMenuByRidLoopCodeLoopNo:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	

	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<ARPSQueueCodes> getCountQueue(String stationNo) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getCountQueue");
		}
		List<Object[]> userList = new ArrayList<Object[]>();
		List<ARPSQueueCodes> qList = new ArrayList<ARPSQueueCodes>();
	
		try {
			// use entity manager to retrieve named query
			
			String q = "SELECT  AQC.queueId, AQC.queueName, AQC.MenuOrder, count(AQ.queueId) count FROM App.ARPSQueueCodes AQC  FULL OUTER JOIN App.TransactionLog AQ on AQC.queueId=AQ.queueId and aq.Sta3n=\'"+stationNo+"\' where AQC.queueName is not NULL"
			 +"  Group By AQC.queueId, AQC.queueName, AQC.MenuOrder order by AQC.MenuOrder";
			
			Query queryCollectByMail = em.createNativeQuery(q);
  
			// execute query and get results
			userList   = queryCollectByMail.getResultList();
			
			for (int i=0; i<userList.size(); i++){
				ARPSQueueCodes aque= new ARPSQueueCodes();
				aque.setQueueId(i+1);
				Object[] strList = userList.get(i);
				aque.setQueueId((Integer)strList[0]);
				aque.setQueueName((String)strList[1]);
				aque.setMenuOrder((Integer)(strList[2]));
				aque.setqCount((Integer)(strList[3]));
				qList.add(aque);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getCountQueue:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return qList;
	}
	
	

	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<DenialReason> findAllDenialReason() {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findSPById");
		}
		List<DenialReason> userList = new ArrayList<DenialReason>();
	
		try {
			// use entity manager to retrieve named query
			TypedQuery<DenialReason> queryCollectByMail = em.createNamedQuery("DenialReason.findAllDenialReason",	DenialReason.class);

			
			// execute query and get results
			userList = queryCollectByMail.getResultList();

			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<Institution> findAllStationList(int sta) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findAllStationList");
		}
		List<Institution> userList = new ArrayList<Institution>();
	
		try {
			// use entity manager to retrieve named query
			Query queryCollectByMail = em.createNamedQuery("Institution.findAllStation");
			queryCollectByMail.setParameter("station", new Integer(sta));
			
			List<Object[]> userObjList = queryCollectByMail.getResultList();
			
			for (int i=0; i<userObjList.size(); i++){
				
				Object[] strList = userObjList.get(i);
				
				Institution inst= new Institution();
				inst.setInstitutionCode((String)strList[0]);
				inst.setInstitutionName((String)strList[1]);
				inst.setStation((Integer)(strList[2]));

				userList.add(inst);
			}

				// execute query and get results
			
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllStationList:===" + e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
		
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<AuditLog> findAuditLog(Long id) {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findAuditLog");
		}

		List<AuditLog> mailList = null;
		AuditLog userMail = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<AuditLog> queryBymail = em.createNamedQuery(
					"AuditLog.findAuditLogByPatID", AuditLog.class);
			// set dynamic data for query
			queryBymail.setParameter("rid", new Integer(id.toString()));

			// execute query and get results
			mailList = queryBymail.getResultList();

			
		}

		catch (Exception e) {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("\n === Exception in PatientService:-findAuditLog:=== "
						+ e.getMessage()+ "\n" +e.getCause());
			}
		} finally {

		}

		return mailList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<UserAuthorization> findUserList() {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findUserList");
		}

		List<UserAuthorization> mailList = null;
	
		try {
			// use entity manager to retrieve named query
			TypedQuery<UserAuthorization> queryBymail = em.createNamedQuery("UserAuthorization.findUserList", UserAuthorization.class);
			// set dynamic data for query
		//	queryBymail.setParameter("rid", new Integer(id.toString()));

			// execute query and get results
			mailList = queryBymail.getResultList();

			
		}

		catch (Exception e) {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("\n === Exception in PatientService:-findAuditLog:=== "
						+ e.getMessage()+ "\n" +e.getCause());
			}
		} finally {

		}

		return mailList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public boolean postAuditLog(AuditLog alog) {
		boolean succ=false;
		
		try{
			if (!em.getTransaction().isActive()){
				em.getTransaction().begin();
				
			}
			em.merge(alog);
			em.getTransaction().commit();
			succ=true;
		}
		catch (Exception ex){
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("\n === Exception in PatientService:-postAuditLog:=== " + ex.getMessage()+ "\n" +ex.getCause());
			}
		}
		return succ;
	}
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public boolean updateResponseLog(TransactionLog alog) {
		boolean succ=false;
		
		try{
			TransactionLog aqueue = findQueueByReqId(alog.getRequestId());
			aqueue.setQueueId(alog.getQueueId());
					
			if (!em.getTransaction().isActive()){
				em.getTransaction().begin();
			}
			em.persist(aqueue);
			em.getTransaction().commit();
			succ=true;
		}
		catch (Exception ex){
			if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("\n === Exception in PatientService:-updateResponseLog:=== " + ex.getMessage()+ "\n" +ex.getCause());
			}
		}
		return succ;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public boolean updateResponseLogWithStation(TransactionLog alog) {
		boolean succ=false;
		
		try{
			TransactionLog aqueue = findQueueByReqId(alog.getRequestId());
	//		aqueue.setAssignedStationInstCode(alog.getAssignedStationInstCode());
					
			if (!em.getTransaction().isActive()){
				em.getTransaction().begin();
			}
			em.persist(aqueue);
			em.getTransaction().commit();
			succ=true;
		}
		catch (Exception ex){
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("\n === Exception in PatientService:-updateResponseLogWithStation:=== " + ex.getMessage()+ "\n" +ex.getCause());
				}
		}
		return succ;
	}
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public boolean updateUser(String flag, String userId, String domain, String status, String station, String admin) {
		boolean succ=false;
		
		UserAuthorization usernew =null;
		try{
			
			if (flag.equalsIgnoreCase("u")){
				usernew = findUserByuId (userId);
			}
			else{
				usernew = new UserAuthorization();
			}
			usernew.setAdmin(admin);
			usernew.setDomain(domain);
			usernew.setStation_number(station);
			usernew.setStatus(status);
			usernew.setAdmin(admin);
			usernew.setUserId(userId);
			
			if (!em.getTransaction().isActive()){
				em.getTransaction().begin();
				
			}
			em.persist(usernew);
			em.getTransaction().commit();
			succ=true;
		}
		catch (Exception ex){
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("\n === Exception in PatientService:-updateUser:=== " + ex.getMessage()+ "\n" +ex.getCause());
				}
		}
		
		
		return succ;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public UserAuthorization findUserByuId (String uId ){

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findUserByuId");
		}
		List<UserAuthorization> depList = new ArrayList<UserAuthorization>();
		UserAuthorization user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<UserAuthorization> queryCollect = em.createNamedQuery("UserAuthorization.findUserByuId",	UserAuthorization.class);

			// set dynamic data for query
			queryCollect.setParameter("uid", uId);
						
			// execute query and get results
			depList = queryCollect.getResultList();

			if (depList.size() == 1) {
				user = (UserAuthorization) depList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findUserByuId:==="+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public TransactionLog findQueueByReqId (int reqId ){

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findQueueByReqId");
		}
		List<TransactionLog> depList = new ArrayList<TransactionLog>();
		TransactionLog user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<TransactionLog> queryCollect = em.createNamedQuery("TransactionLog.findARPSQueueByReqId",	TransactionLog.class);

			// set dynamic data for query
			queryCollect.setParameter("rId", reqId);
						
			// execute query and get results
			depList = queryCollect.getResultList();

			if (depList.size() == 1) {
				user = (TransactionLog) depList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findQueueByReqId:==="+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<ARPSQueueCodes> findAATQueue() {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-findAATQueue");
		}

		List<ARPSQueueCodes> mailList = null;
		
		try {
			// use entity manager to retrieve named query
			TypedQuery<ARPSQueueCodes> queryBymail = em.createNamedQuery(
					"AATQueue.findAATQueue", ARPSQueueCodes.class);
			// set dynamic data for query
		
			// execute query and get results
			mailList = queryBymail.getResultList();

			
		}

		catch (Exception e) {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("\n === Exception in PatientService:-findAuditLog:=== "
						+ e.getMessage()+ "\n" +e.getCause());
			}
		} finally {

		}

		return mailList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<EntityName> getEntityInfo(int reqId, String loopcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<EntityName> userList = new ArrayList<EntityName>();
		EntityName user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<EntityName> queryCollectByMail = em.createNamedQuery("EntityName.findEntityByReqIDLoopCode",	EntityName.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", loopcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public EntityAddress getEntityAddress(int reqId, String loopcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<EntityAddress> userList = new ArrayList<EntityAddress>();
		EntityAddress user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<EntityAddress> queryCollectByMail = em.createNamedQuery("EntityAddress.findDAddByReqIdLoopCode",	EntityAddress.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", loopcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (EntityAddress) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public DemographicInformation getDemoInfo(int reqId, String loopcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<DemographicInformation> userList = new ArrayList<DemographicInformation>();
		DemographicInformation user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<DemographicInformation> queryCollectByMail = em.createNamedQuery("DemographicInformation.findDemoByReqId",	DemographicInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", loopcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (DemographicInformation) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public InsuranceInformation getInsuranceInfo(int reqId, String loopcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getInsuranceInfo");
		}
		List<InsuranceInformation> userList = new ArrayList<InsuranceInformation>();
		InsuranceInformation user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<InsuranceInformation> queryCollectByMail = em.createNamedQuery("InsuranceInformation.findInsByReqId",	InsuranceInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", loopcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (InsuranceInformation) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getInsuranceInfo:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public HealthServicesReview getHealthServicesInfo(Integer reqId, String loopcode, String lnumber) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<HealthServicesReview> userList = new ArrayList<HealthServicesReview>();
		HealthServicesReview user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<HealthServicesReview> queryCollectByMail = em.createNamedQuery("HealthServicesReview.findHealthReviewByReqId",	HealthServicesReview.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", loopcode);
			queryCollectByMail.setParameter("lno", lnumber);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (HealthServicesReview) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-HealthServicesReview:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}

	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public EntityAddress getAddressInfo(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<EntityAddress> userList = new ArrayList<EntityAddress>();
		EntityAddress user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<EntityAddress> queryCollectByMail = em.createNamedQuery("EntityAddress.findDAddByReqIdLoopCode",	EntityAddress.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (EntityAddress) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public ProviderInformation getProviderInfo(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getProviderInfo");
		}
		List<ProviderInformation> userList = new ArrayList<ProviderInformation>();
		ProviderInformation user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<ProviderInformation> queryCollectByMail = em.createNamedQuery("ProviderInformation.findProviderByReqIdLoopCode",	ProviderInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (ProviderInformation) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<References> getReferences(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getProviderInfo");
		}
		List<References> userList = new ArrayList<References>();

		try {
			// use entity manager to retrieve named query
			TypedQuery<References> queryCollectByMail = em.createNamedQuery("References.findReferencesByReqIdLoopCode",	References.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("loopCode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
		
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public ContactInformation getContactInfo(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-ContactInformation");
		}
		List<ContactInformation> userList = new ArrayList<ContactInformation>();
		ContactInformation user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<ContactInformation> queryCollectByMail = em.createNamedQuery("ContactInformation.findDAddByReqIdLoopCode",	ContactInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (ContactInformation) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-ContactInformation:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public DateInformation getDateInfo(int reqId, String dateQual, String loopnumber) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<DateInformation> userList = new ArrayList<DateInformation>();
		DateInformation user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<DateInformation> queryCollectByMail = em.createNamedQuery("DateInformation.findDateInforByReqIDDqteQual",	DateInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("dateQual", dateQual);
			queryCollectByMail.setParameter("lno", loopnumber);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (DateInformation) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public int getLoopDateInfo(int reqId, String dateQual) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
			int user = 0;

		try {
			
			String q = "select max(alog.loopNumber) from App.DateInformation alog where alog.requestId=:rid and alog.loopCode=:lcode";			
			Query queryCollectByMail = em.createNativeQuery(q);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", dateQual);
			
			
			// execute query and get results
			user = (Integer) queryCollectByMail.getSingleResult();
			
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<DateInformation> getDateList(int reqId, String lcode, String lno) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<DateInformation> userList = new ArrayList<DateInformation>();

		try {
			// use entity manager to retrieve named query
			TypedQuery<DateInformation> queryCollectByMail = em.createNamedQuery("DateInformation.findDateInforByReqIdlcode",	DateInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			queryCollectByMail.setParameter("lno", lno);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-findAllPatient:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<ServicesTooth> getToothList(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getToothList");
		}
		List<ServicesTooth> userList = new ArrayList<ServicesTooth>();

		try {
			// use entity manager to retrieve named query
			TypedQuery<ServicesTooth> queryCollectByMail = em.createNamedQuery("ServicesTooth.findToothByReqIdlcode",	ServicesTooth.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getToothList:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public HealthCareServicesDelivery getHealthCareServicesDelivery(int reqId, String lcode ,  String loopnumber) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getToothList");
		}
		List<HealthCareServicesDelivery> userList = new ArrayList<HealthCareServicesDelivery>();
		HealthCareServicesDelivery user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<HealthCareServicesDelivery> queryCollectByMail = em.createNamedQuery("HealthCareServicesDelivery.findToothByReqIdlcode",	HealthCareServicesDelivery.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			queryCollectByMail.setParameter("lnumber", loopnumber);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (HealthCareServicesDelivery) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getToothList:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public ServicesDental getDentalServices(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getDentalServices");
		}
		List<ServicesDental> userList = new ArrayList<ServicesDental>();
		ServicesDental user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<ServicesDental> queryCollectByMail = em.createNamedQuery("ServicesDental.findToothByReqIdlcode",	ServicesDental.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (ServicesDental) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getDentalServices:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public ServicesProfessional getProfesnalServices(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getProfesnalServices");
		}
		List<ServicesProfessional> userList = new ArrayList<ServicesProfessional>();
		ServicesProfessional user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<ServicesProfessional> queryCollectByMail = em.createNamedQuery("ServicesProfessional.findToothByReqIdlcode",	ServicesProfessional.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (ServicesProfessional) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getDentalServices:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public ServicesInstitution getServicesInstituion(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getProfesnalServices");
		}
		List<ServicesInstitution> userList = new ArrayList<ServicesInstitution>();
		ServicesInstitution user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<ServicesInstitution> queryCollectByMail = em.createNamedQuery("ServicesInstitution.findInstInfoByReqIdlcode",	ServicesInstitution.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (ServicesInstitution) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getDentalServices:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public UserAuthorization getUserAuthbyUid( String uid) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getUserAuthbyUid");
		}
		List<UserAuthorization> userList = new ArrayList<UserAuthorization>();
		UserAuthorization user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<UserAuthorization> queryCollectByMail = em.createNamedQuery("UserAuthorization.findUUid",	UserAuthorization.class);
			
			queryCollectByMail.setParameter("uid", uid+'%');
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (UserAuthorization) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getUserAuthbyUid:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public PatientInformation getAdditionalServices( Integer rid, String loop) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getAdditionalServices");
		}
		List<PatientInformation> userList = new ArrayList<PatientInformation>();
		PatientInformation user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<PatientInformation> queryCollectByMail = em.createNamedQuery("PatientInformation.findAdditionalByRidLcode",	PatientInformation.class);
			queryCollectByMail.setParameter("rid", rid);
			queryCollectByMail.setParameter("lcode", loop);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (PatientInformation) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getAdditionalServices:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public MessageText getMessages( Integer rid, String loop) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getUserAuthbyUid");
		}
		List<MessageText> userList = new ArrayList<MessageText>();
		MessageText user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<MessageText> queryCollectByMail = em.createNamedQuery("MessageText.findAuditLogByrIDLcode",	MessageText.class);
			queryCollectByMail.setParameter("rid", rid);
			queryCollectByMail.setParameter("lcode", loop);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (MessageText) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-MessageText:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<DiagnosisInformation> getDiagnosis( Integer rid, String loop, String loopnumber) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getUserAuthbyUid");
		}
		List<DiagnosisInformation> userList = new ArrayList<DiagnosisInformation>();
		DiagnosisInformation user=null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<DiagnosisInformation> queryCollectByMail = em.createNamedQuery("DiagnosisInformation.findByrIDLcode",	DiagnosisInformation.class);
			queryCollectByMail.setParameter("rid", rid);
			queryCollectByMail.setParameter("lcode", loop);
			queryCollectByMail.setParameter("lnumber", loopnumber);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-MessageText:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<TrackingNumber> getTrackList(int reqId, String lcode, String lnumber) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getTrackList");
		}
		List<TrackingNumber> userList = new ArrayList<TrackingNumber>();

		try {
			// use entity manager to retrieve named query
			TypedQuery<TrackingNumber> queryCollectByMail = em.createNamedQuery("TrackingNumber.findTrackInforByReqIdlcode",	TrackingNumber.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			queryCollectByMail.setParameter("lno", lnumber);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getTrackList:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public List<CertificationInformation> getCertificateList(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getCertificateList");
		}
		List<CertificationInformation> userList = new ArrayList<CertificationInformation>();

		try {
			// use entity manager to retrieve named query
			TypedQuery<CertificationInformation> queryCollectByMail = em.createNamedQuery("CertificationInformation.findDateInforByReqIdlcode",	CertificationInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getCertificateList:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return userList;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public InstitutionClaim getInstitutionClaim(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getCertificateList");
		}
		List<InstitutionClaim> userList = new ArrayList<InstitutionClaim>();
		InstitutionClaim user = null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<InstitutionClaim> queryCollectByMail = em.createNamedQuery("InstitutionClaim.findInstitutionClaimByReqIdlcode",	InstitutionClaim.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (InstitutionClaim) userList.get(0);
			}
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getCertificateList:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public TranportInformation getTransport(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getCertificateList");
		}
		List<TranportInformation> userList = new ArrayList<TranportInformation>();
		TranportInformation user = null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<TranportInformation> queryCollectByMail = em.createNamedQuery("TranportInformation.findTranportInformationByReqIdlcode",	TranportInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (TranportInformation) userList.get(0);
			}
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getCertificateList:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public TransactionLog getARPSQueue(int reqId) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<TransactionLog> userList = new ArrayList<TransactionLog>();
		TransactionLog user = null;

		try {
			// use entity manager to retrieve named query
			TypedQuery<TransactionLog> queryCollectByMail = em.createNamedQuery("TransactionLog.findARPSQueueByReqId",	TransactionLog.class);
			queryCollectByMail.setParameter("rId", reqId);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (TransactionLog) userList.get(0);
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-HealthServicesReview:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public String getDecodedDescr(String loopcode, String segId, String code) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getEntityInfo");
		}
		List<CodeDataCodeLookUp> userList = new ArrayList<CodeDataCodeLookUp>();
		CodeDataCodeLookUp user = null;
		String descr="";

		try {
			// use entity manager to retrieve named query
			TypedQuery<CodeDataCodeLookUp> queryCollectByMail = em.createNamedQuery("CodeDataCodeLookUp.findByCodeDescr",	CodeDataCodeLookUp.class);
			queryCollectByMail.setParameter("lcode", loopcode);
			queryCollectByMail.setParameter("sid", segId);
			queryCollectByMail.setParameter("cde", code);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (CodeDataCodeLookUp) userList.get(0);
				descr=user.getDescription();
				
			}
			
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getDecodedDescr:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return descr;
	}
	
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public SpinalInformation getSpinalInfo(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getSpinalInfo");
		}
		List<SpinalInformation> userList = new ArrayList<SpinalInformation>();
		SpinalInformation user = null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<SpinalInformation> queryCollectByMail = em.createNamedQuery("SpinalInformation.findDateInforByReqIdlcode",	SpinalInformation.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (SpinalInformation) userList.get(0);
			}
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getSpinalInfo:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public HomeOxygenTherapy getHomeOxygenTherapyInfo(int reqId, String lcode) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getHomeOxygenTherapyInfo");
		}
		List<HomeOxygenTherapy> userList = new ArrayList<HomeOxygenTherapy>();
		HomeOxygenTherapy user = null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<HomeOxygenTherapy> queryCollectByMail = em.createNamedQuery("HomeOxygenTherapy.findHomeOxygenByReqIdlcode",	HomeOxygenTherapy.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (HomeOxygenTherapy) userList.get(0);
			}
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getHomeOxygenTherapyInfo:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
	@Transactional(propagation = Propagation.SUPPORTS, rollbackFor = Exception.class)
	public HomeHealthCare getHomeHealthCareInfo(int reqId, String lcode, String loopnumber) {

		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientService:-getHomeOxygenTherapyInfo");
		}
		List<HomeHealthCare> userList = new ArrayList<HomeHealthCare>();
		HomeHealthCare user = null;
		try {
			// use entity manager to retrieve named query
			TypedQuery<HomeHealthCare> queryCollectByMail = em.createNamedQuery("HomeHealthCare.findHomeHealthCareByReqIdlcode",	HomeHealthCare.class);
			queryCollectByMail.setParameter("rid", reqId);
			queryCollectByMail.setParameter("lcode", lcode);
			queryCollectByMail.setParameter("lnumber", loopnumber);
			
			// execute query and get results
			userList = queryCollectByMail.getResultList();
			
			if (userList.size() == 1) {
				user = (HomeHealthCare) userList.get(0);
			}
		} catch (Exception e) {
			//if (LOGGER.isDebugEnabled()) {
				LOGGER.info("\n === Exception in PatientService:-getHomeOxygenTherapyInfo:==="
						+ e.getMessage()+ "\n" +e.getCause());
			//}
		} finally {
		}
		return user;
	}
	
}