package va.gov.service;

import java.util.List;

import org.springframework.stereotype.Component;

import va.gov.jpa.dto.ARPSQueueCodes;
import va.gov.jpa.dto.AuditLog;
import va.gov.jpa.dto.CertificationInformation;
import va.gov.jpa.dto.ContactInformation;
import va.gov.jpa.dto.DateInformation;
import va.gov.jpa.dto.DemographicInformation;
import va.gov.jpa.dto.DenialReason;
import va.gov.jpa.dto.DetailMenuItemList;
import va.gov.jpa.dto.DiagnosisInformation;
import va.gov.jpa.dto.EntityAddress;
import va.gov.jpa.dto.EntityName;
import va.gov.jpa.dto.EventMenuItem;
import va.gov.jpa.dto.HealthCareServicesDelivery;
import va.gov.jpa.dto.HealthServicesReview;
import va.gov.jpa.dto.HomeHealthCare;
import va.gov.jpa.dto.HomeOxygenTherapy;
import va.gov.jpa.dto.Institution;
import va.gov.jpa.dto.InstitutionClaim;
import va.gov.jpa.dto.InsuranceInformation;
import va.gov.jpa.dto.MessageText;
import va.gov.jpa.dto.PatientInformation;
import va.gov.jpa.dto.ProviderInformation;
import va.gov.jpa.dto.References;
import va.gov.jpa.dto.TransactionLog;
import va.gov.jpa.dto.SearchPatientById;
import va.gov.jpa.dto.ServicesDental;
import va.gov.jpa.dto.ServicesInstitution;
import va.gov.jpa.dto.ServicesMenuItem;
import va.gov.jpa.dto.ServicesProfessional;
import va.gov.jpa.dto.ServicesTooth;
import va.gov.jpa.dto.SpinalInformation;
import va.gov.jpa.dto.TrackingNumber;
import va.gov.jpa.dto.TranportInformation;
import va.gov.jpa.dto.UserAuthorization;

@Component
public interface PatientServiceIntf {
 
 
	public List<AuditLog> getAuditLog(Long id);
	public List<UserAuthorization> getUserList();
	
	public List<TransactionLog> getAllPatientByQueue(Integer id, String stationNo);
	public List<SearchPatientById> getPatientById(String id, String stationNo);
	
	public UserAuthorization getUserAuth(String uid);
	public List<DenialReason> getAllDenialReason();
	public List<ARPSQueueCodes> getQueueList(String stationNo);
	public List<Institution> getStationList(int sta);
	
	public EventMenuItem getEventMenuItemByreqId(Integer reqId);
	public ServicesMenuItem getServiceMenuItemByreqId(Integer reqId);
	
	public DetailMenuItemList getDetailLeftMenuByreqId(Integer reqId, String loopCode, Integer loopNo); 
	
	public List<EntityName> getEntityInfo(Integer reqId, String loopcode);
	public EntityAddress getEntityAddress(Integer reqId, String loopcode);
	public String getDecodedDescription(String loopcode, String segId, String code);
	public EntityAddress getAddress(Integer reqId, String dateQualifier);
	public ContactInformation getContactnfoI(Integer reqId, String dateQualifier);
	public ProviderInformation getProviderInfo(Integer reqId, String lcode);
	public List<References> getReferences(Integer reqId, String lcode);
	public List<ServicesTooth> getToothInfo(Integer reqId, String lcode);
	public ServicesDental getDentalServicesInfo(Integer reqId, String loopcode);
	public PatientInformation getAdditionalServices(Integer reqId, String lcode);
	
	
	
	public List<DateInformation> getDateList(Integer reqId, String loopcode, String loopnumber);
	public List<DiagnosisInformation> getDiagnosisServices(Integer reqId, String lcode, String loopnumber);
	public HomeHealthCare  getHomeHealthCareInfo (Integer reqId, String dateQualifier,  String loopnumber);
	public HealthCareServicesDelivery getHealthCareServicesDelivery(Integer reqId, String lcode,  String loopnumber);
	
	
	public MessageText getMessges(Integer reqId, String lcode);
	
	
	public List<TrackingNumber> getTrackList(Integer reqId, String loopcode,  String loopnumber);
	public HealthServicesReview getHealthServicesInfo(Integer reqId, String loopcode,  String loopnumber);
	public DateInformation getDateInfo(Integer reqId, String dateQualifier, String loopnumber);
	
	public List<CertificationInformation> getCertificateList(Integer reqId, String loopcode);
	public ServicesProfessional  getProfessionalServicesInfo (Integer reqId, String loopcode);
	public ServicesInstitution  getServiceInstitutionInfo (Integer reqId, String loopcode);
	
	public HomeOxygenTherapy  getHomeOxygenTherapyInfo (Integer reqId, String dateQualifier);
	public SpinalInformation getSpinalInfo(Integer reqId, String dateQualifier);
	public TranportInformation getTransport(Integer reqId, String dateQualifier);
	public InstitutionClaim getInstitutionClaim(Integer reqId, String dateQualifier);
	public TransactionLog getARPSQueue(Integer reqId);
	public DemographicInformation getDOBInfo(int reqId, String loopcode);
	public InsuranceInformation getInsuranceInfo(int reqId, String loopcode);
	
	
	public boolean postAuditLog(String user, String decision, String code,String queue, String spId);
	
	public boolean updateQueue(String queue, String spId);
	public boolean updateUserAuth(String flag, String userId, String domain, String status, String station, String admin);
	public boolean updateResponsewithStation(String stationCode, String spId);
   
	public int getLoopDateList(Integer reqId, String loopcode);
	
	
}
