package va.gov.service.impl;

import java.util.List;

import org.apache.log4j.Logger;

import va.gov.jpa.dto.ARPSQueueCodes;
import va.gov.jpa.dto.AuditLog;
import va.gov.jpa.dto.CertificationInformation;
import va.gov.jpa.dto.ContactInformation;
import va.gov.jpa.dto.DateInformation;
import va.gov.jpa.dto.DemographicInformation;
import va.gov.jpa.dto.DenialReason;
import va.gov.jpa.dto.DetailMenuItemList;
import va.gov.jpa.dto.DiagnosisInformation;
import va.gov.jpa.dto.EntityAddress;
import va.gov.jpa.dto.EntityName;
import va.gov.jpa.dto.EventMenuItem;
import va.gov.jpa.dto.HealthCareServicesDelivery;
import va.gov.jpa.dto.HealthServicesReview;
import va.gov.jpa.dto.HomeHealthCare;
import va.gov.jpa.dto.HomeOxygenTherapy;
import va.gov.jpa.dto.Institution;
import va.gov.jpa.dto.InstitutionClaim;
import va.gov.jpa.dto.InsuranceInformation;
import va.gov.jpa.dto.MessageText;
import va.gov.jpa.dto.PatientInformation;
import va.gov.jpa.dto.ProviderInformation;
import va.gov.jpa.dto.References;
import va.gov.jpa.dto.TransactionLog;
import va.gov.jpa.dto.SearchPatientById;
import va.gov.jpa.dto.ServicesDental;
import va.gov.jpa.dto.ServicesInstitution;
import va.gov.jpa.dto.ServicesMenuItem;
import va.gov.jpa.dto.ServicesProfessional;
import va.gov.jpa.dto.ServicesTooth;
import va.gov.jpa.dto.SpinalInformation;
import va.gov.jpa.dto.TrackingNumber;
import va.gov.jpa.dto.TranportInformation;
import va.gov.jpa.dto.UserAuthorization;
import va.gov.jpa.service.PatientService;
import va.gov.service.PatientServiceIntf;
import va.gov.util.Utility;



public class PatientServiceImpl implements PatientServiceIntf {

	private static final Logger LOGGER = Logger.getLogger(PatientServiceImpl.class);
	

	public PatientServiceImpl() {
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:- ");
		}
	}
	public static Logger getLogger() {
		return LOGGER;
	}

	
	

	
	public List<TransactionLog> getAllPatientByQueue(Integer Id, String stationNo){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getAllPatientByQueue");
		}
		
		List<TransactionLog> pall = PatientService.getInstance().findAllPatientByQueueId(Id, stationNo);
		
		return pall;
	}

	public List<SearchPatientById> getPatientById(String id, String stationNo){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getPatientById");
		}
		
		List<SearchPatientById> pall = PatientService.getInstance().findPatientById(id, stationNo);
		
		return pall;
	}
	
	public EventMenuItem getEventMenuItemByreqId(Integer reqId){
	if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getEventMenuItemByreqId");
		}
		EventMenuItem pall = PatientService.getInstance().findEventMenuByRid(reqId);
		return pall;
	}
	
	public ServicesMenuItem getServiceMenuItemByreqId(Integer reqId){
		if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("Enter  in PatientServiceImpl:-getEventMenuItemByreqId");
			}
		ServicesMenuItem pall = PatientService.getInstance().findServiceMenuByRid(reqId);
			return pall;
		}

	public DetailMenuItemList getDetailLeftMenuByreqId(Integer reqId, String loopCode, Integer loopNo){
	if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDetailLeftMenuByreqId");
		}
		DetailMenuItemList pall = PatientService.getInstance().findLeftMenuByRidLoopCodeLoopNo(reqId, loopCode, loopNo);
		return pall;
	}
	
	
	public List<DenialReason> getAllDenialReason(){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getAllDenialReason");
		}
		
		List<DenialReason> pall = PatientService.getInstance().findAllDenialReason();
		
		return pall;
	}
	

	public List<UserAuthorization> getUserList(){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getAuditLog");
		}
		
		List<UserAuthorization> dept = PatientService.getInstance().findUserList();		
		return dept;
	}
	
	
	public List<AuditLog> getAuditLog(Long id){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getAuditLog");
		}
		
		List<AuditLog> dept = PatientService.getInstance().findAuditLog(id);		
		return dept;
	}
	
	public boolean postAuditLog(String user, String decision, String code,String queue, String rId){
		boolean result  =false;
		
		AuditLog alog =new AuditLog();
		
		alog.setUserId(user);
		alog.setReasonCode(code);
		alog.setAuditAction(decision);
		alog.setQueue(queue);
		alog.setRequestId(new Integer(rId));
		alog.setDateTimeStamp(Utility.getCurrentTimestamp().toString());
		
		if (rId!=null && user!=null && decision!=null){
			result = PatientService.getInstance().postAuditLog(alog);	
		}
		return result;
	}
	
	public boolean updateQueue(String queue, String rId){
		boolean result  =false;
		
		TransactionLog aqueue =new TransactionLog();
		
		aqueue.setQueueId(new Integer(queue));
		aqueue.setRequestId(new Integer(rId));
		//aqueue.setRoutingInformation("Changed by ARPS");
		try {
		aqueue.setRequestDate(Utility.getCurrentDate());
		}
		catch (Exception e){
			e.printStackTrace();
		}
		result = PatientService.getInstance().updateResponseLog( aqueue);	
		return result;
	}
	
	public boolean updateResponsewithStation(String stationCode, String spId){
		boolean result  =false;
		
		TransactionLog rlog =new TransactionLog();
		
	//	rlog.setAssignedStationInstCode(stationCode );
		rlog.setRequestId(new Integer(spId));
	
		result = PatientService.getInstance().updateResponseLogWithStation(rlog);	
		return result;
	}
	
	public List<ARPSQueueCodes> getQueueList(String stationNo){
		
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getQueueList");
		}		
		List<ARPSQueueCodes> que = PatientService.getInstance().getCountQueue(stationNo);		
		return que;
	}
	
	
	public List<Institution> getStationList(int sta){
		
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getStationList");
		}		
		List<Institution> que = PatientService.getInstance().findAllStationList(sta);		
		return que;
	}

	
	public List<EntityName> getEntityInfo(Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getEntityInfo");
		}		
		List<EntityName> scount = PatientService.getInstance().getEntityInfo(reqId, loopcode);		
		return scount;
	}
	
	public EntityAddress getEntityAddress(Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getEntityInfo");
		}		
		EntityAddress scount = PatientService.getInstance().getEntityAddress(reqId, loopcode);		
		return scount;
	}
	
	public DemographicInformation getDOBInfo(int reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDOBInfo");
		}		
		DemographicInformation scount = PatientService.getInstance().getDemoInfo(reqId, loopcode);		
		return scount;
	}
	
	public InsuranceInformation getInsuranceInfo(int reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getInsuranceInfo");
		}		
		InsuranceInformation scount = PatientService.getInstance().getInsuranceInfo(reqId, loopcode);		
		return scount;
	}
	
	public DateInformation getDateInfo(Integer reqId, String dateQualifier, String loopnumber){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDateInfo");
		}		
		DateInformation scount = PatientService.getInstance().getDateInfo(reqId, dateQualifier, loopnumber);		
		return scount;
	}
	
	public int getLoopDateList(Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getLoopDateList");
		}		
		int scount = PatientService.getInstance().getLoopDateInfo(reqId, loopcode);		
		return scount;
	}
	
	public EntityAddress getAddress(Integer reqId, String loopCode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getAddress");
		}		
		EntityAddress scount = PatientService.getInstance().getAddressInfo(reqId, loopCode);		
		return scount;
	}
	
	
	
	public ProviderInformation getProviderInfo(Integer reqId, String lcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getProviderInfo");
		}		
		ProviderInformation scount = PatientService.getInstance().getProviderInfo(reqId, lcode);		
		return scount;
	}
	
	public List<References> getReferences(Integer reqId, String lcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getReferences");
		}		
		List<References> scount = PatientService.getInstance().getReferences(reqId, lcode);		
		return scount;
	}
	
	public ContactInformation getContactnfoI(Integer reqId, String loopCode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getContactnfoI");
		}		
		ContactInformation scount = PatientService.getInstance().getContactInfo(reqId, loopCode);		
		return scount;
	}
	public TransactionLog getARPSQueue(Integer reqId){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getARPSQueue");
		}		
		TransactionLog scount = PatientService.getInstance().getARPSQueue(reqId);		
		return scount;		
	}
	public String getDecodedDescription(String loopcode, String segId, String code){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDecodedDescription");
		}		
		String scount = PatientService.getInstance().getDecodedDescr(loopcode, segId, code);		
		return scount;		
	}
	
	public List<TrackingNumber> getTrackList(Integer reqId, String loopcode, String lnumber){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getTrackList");
		}		
		List<TrackingNumber> scount = PatientService.getInstance().getTrackList(reqId, loopcode, lnumber);		
		return scount;		
	}
	public HealthServicesReview getHealthServicesInfo(Integer reqId, String loopcode, String lnumber){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getHealthServicesInfo");
		}		
		HealthServicesReview scount = PatientService.getInstance().getHealthServicesInfo(reqId, loopcode, lnumber);		
		return scount;
	}
	
	public List<DateInformation> getDateList(Integer reqId, String loopcode, String lnumber){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDateList");
		}		
		List<DateInformation> scount = PatientService.getInstance().getDateList(reqId, loopcode, lnumber);		
		return scount;		
	}
	public List<CertificationInformation> getCertificateList(Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDateInfo");
		}		
		List<CertificationInformation> scount = PatientService.getInstance().getCertificateList(reqId, loopcode);		
		return scount;		
	}
	
	public InstitutionClaim getInstitutionClaim(Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getInstitutionClaim");
		}		
		InstitutionClaim scount = PatientService.getInstance().getInstitutionClaim(reqId, loopcode);		
		return scount;		
	}
	
	public TranportInformation getTransport(Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getTransport");
		}		
		TranportInformation scount = PatientService.getInstance().getTransport(reqId, loopcode);		
		return scount;		
	}
	
	public SpinalInformation getSpinalInfo(Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getSpinalInfo");
		}		
		SpinalInformation scount = PatientService.getInstance().getSpinalInfo(reqId, loopcode);		
		return scount;		
	}
	
	public HomeOxygenTherapy  getHomeOxygenTherapyInfo (Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getHomeOxygenTherapyInfo");
		}		
		HomeOxygenTherapy scount = PatientService.getInstance().getHomeOxygenTherapyInfo(reqId, loopcode);		
		return scount;		
	}

	public HomeHealthCare  getHomeHealthCareInfo (Integer reqId, String loopcode, String loopnumber){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getHomeHealthCareInfo");
		}		
		HomeHealthCare scount = PatientService.getInstance().getHomeHealthCareInfo(reqId, loopcode, loopnumber);		
		return scount;		
	}
	
	public List<ServicesTooth> getToothInfo(Integer reqId, String lcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDateInfo");
		}		
		List<ServicesTooth> scount = PatientService.getInstance().getToothList(reqId, lcode);		
		return scount;		
	}
	
	public HealthCareServicesDelivery getHealthCareServicesDelivery(Integer reqId, String lcode,  String loopnumber){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDateInfo");
		}		
		HealthCareServicesDelivery scount = PatientService.getInstance().getHealthCareServicesDelivery(reqId, lcode, loopnumber);		
		return scount;		
	}
	
	public ServicesDental getDentalServicesInfo(Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDentalServicesInfo");
		}		
		ServicesDental scount = PatientService.getInstance().getDentalServices(reqId, loopcode);		
		return scount;		
	}
	
	public ServicesProfessional  getProfessionalServicesInfo (Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDentalServicesInfo");
		}		
		ServicesProfessional scount = PatientService.getInstance().getProfesnalServices(reqId, loopcode);		
		return scount;		
	}
	
	public ServicesInstitution  getServiceInstitutionInfo (Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDentalServicesInfo");
		}		
		ServicesInstitution scount = PatientService.getInstance().getServicesInstituion(reqId, loopcode);		
		return scount;		
	}
	
	
	public PatientInformation  getAdditionalServices (Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getAdditionalServices");
		}		
		PatientInformation scount = PatientService.getInstance().getAdditionalServices(reqId, loopcode);		
		return scount;		
	}
	
	public MessageText  getMessges (Integer reqId, String loopcode){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getMessges");
		}		
		MessageText scount = PatientService.getInstance().getMessages(reqId, loopcode);		
		return scount;		
	}

	public UserAuthorization getUserAuth(String  uid){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDentalServicesInfo");
		}		
		UserAuthorization scount = PatientService.getInstance().getUserAuthbyUid(uid);		
		return scount;
		
	}
	
	public List<DiagnosisInformation> getDiagnosisServices(Integer reqId, String lcode, String loopnumber){
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Enter  in PatientServiceImpl:-getDateInfo");
		}		
		List<DiagnosisInformation> scount = PatientService.getInstance().getDiagnosis(reqId, lcode,  loopnumber);		
		return scount;		
	}
	
	
	public boolean updateUserAuth(String flag, String userId, String domain, String status, String station, String admin){
		
		boolean result  =false;
		
		UserAuthorization user =new UserAuthorization();
		
		result = PatientService.getInstance().updateUser(flag, userId, domain, status, station,  admin);	
		return result;
	}
}