
package va.gov.ws;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;

import org.apache.log4j.Logger;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.ui.ModelMap;

import com.google.gson.JsonArray;
import com.google.gson.JsonObject;

import va.gov.jpa.dto.CertificationInformation;
import va.gov.jpa.dto.DateInformation;
import va.gov.jpa.dto.DiagnosisInformation;
import va.gov.jpa.dto.EntityName;
import va.gov.jpa.dto.HealthCareServicesDelivery;
import va.gov.jpa.dto.HealthServicesReview;
import va.gov.jpa.dto.HomeHealthCare;
import va.gov.jpa.dto.HomeOxygenTherapy;
import va.gov.jpa.dto.InstitutionClaim;
import va.gov.jpa.dto.MessageText;
import va.gov.jpa.dto.PatientInformation;
import va.gov.jpa.dto.References;
import va.gov.jpa.dto.SpinalInformation;
import va.gov.jpa.dto.TrackingNumber;
import va.gov.jpa.dto.TranportInformation;
import va.gov.service.PatientServiceIntf;
import va.gov.util.Utility;



@Path("/ws/sp")
public class CareReqService {
		
	private static final Logger LOGGER = Logger.getLogger(CareReqService.class);
	  @Context
	  UriInfo uriInfo;
	  @Context
	  HttpServletRequest req;
	  
	  @Context
	  HttpServletResponse  res;
	  
	  ModelMap modelMap = new ModelMap();
	  
	  @Autowired
		private PatientServiceIntf patService;

		public void setPatService(PatientServiceIntf patService) {
			this.patService = patService;
		}

	  @Autowired  
	  protected MessageSource messages;


	  public void setMessages(MessageSource messages) {
	  	this.messages = messages;
	  }
	  

@GET
@Path("/getracking")
@Produces("application/json")
public String getracking(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getracking");
	}
	Integer	rId = new Integer(id);
	JsonArray jarray = new JsonArray();
	JsonObject carejson = new JsonObject();
	//trace
	List<TrackingNumber> dinfo= patService.getTrackList(rId , "2000E","1");
	if (dinfo!=null && dinfo.size()>0){
		carejson.addProperty("trackFlag","1");
		carejson.addProperty("trackCount",dinfo.size());
		for (int i=0; i<dinfo.size();i++){
			JsonObject careTrackson = new JsonObject();
			TrackingNumber dd = (TrackingNumber) dinfo.get(i);
			careTrackson.addProperty("traceCode", patService.getDecodedDescription("2000E", "TRN01",dd.getTraceTypeCode()));
			careTrackson.addProperty("orgComp",   dd.getOriginatingCompanyID() );
			careTrackson.addProperty("refer1",   dd.getReferenceId1() );
			careTrackson.addProperty("refer2",   dd.getReferenceId2() );
			jarray.add(careTrackson);
		}
			carejson.addProperty("trackInfo",jarray.toString());
	} else {
		carejson.addProperty("trackFlag","0");
		carejson.addProperty("trackCount","0");
	}
	return carejson.toString(); 	
}

@GET
@Path("/getSReview")
@Produces("application/json")
public String getSReview(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getSReview");
	}
	Integer	rId = new Integer(id);
	JsonObject carejson = new JsonObject();
	//service
	HealthServicesReview hrev = patService.getHealthServicesInfo(rId,"2000E" ,"1"); 
	if (hrev!=null){
		carejson.addProperty("sReviewFlag","1");
		carejson.addProperty("sReviewCount","1");
		String strUM01 = patService.getDecodedDescription("2000E", "UM01", hrev.getRequestCategCode() );
		String strUM02 = patService.getDecodedDescription("2000E", "UM02", hrev.getCertificationType());
		String strUM03 = patService.getDecodedDescription("2000E", "UM03", hrev.getServicesTypeCode());
		String scodedesc =strUM01 + "-" +strUM02 +"-"+strUM03;
		
		carejson.addProperty("service",scodedesc );
		
		carejson.addProperty("facility",   hrev.getFacility() +"-"+patService.getDecodedDescription("2000E", "UM042", hrev.getFacilityQualifier()));
	
		String strum= patService.getDecodedDescription("2000E", "UM05", hrev.getRelatedCauseCode1()) +", "
					+ patService.getDecodedDescription("2000E", "UM05", hrev.getRelatedCauseCode2()) +", "
					+ patService.getDecodedDescription("2000E", "UM05", hrev.getRelatedCauseCode3())+", "
					+ hrev.getStateProvCode()+", "
					+ hrev.getCountryCode();
				
		carejson.addProperty("relCauseCode1",  strum);
		carejson.addProperty("state", hrev.getStateProvCode());
		carejson.addProperty("country",  hrev.getCountryCode());
		carejson.addProperty("lofService",  patService.getDecodedDescription("2000E", "UM06", hrev.getLevelOfServiceCode()));
		carejson.addProperty("curHealthcode", patService.getDecodedDescription("2000E", "UM07", hrev.getCurrentHealthConditionCode()));
		carejson.addProperty("progcode",  patService.getDecodedDescription("2000E", "UM08", hrev.getPrognosisCode()));
		carejson.addProperty("relInfoCode", patService.getDecodedDescription("2000E", "UM09", hrev.getReleaseOfInformationCode()));
		carejson.addProperty("delayReasonCode",  patService.getDecodedDescription("2000E", "UM10", hrev.getDelayReasonCode()));
	}
	else {
		carejson.addProperty("sReviewFlag","0");
		carejson.addProperty("sReviewCount","0");
	}
	return carejson.toString(); 
	
	
}
@GET
@Path("/getDates")
@Produces("application/json")
public String getDates(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getDates");
	}
	Integer	rId = new Integer(id);
	JsonArray jarray = new JsonArray();
	JsonObject carejson = new JsonObject();
	//JsonArray ojson = new JsonArray();
	
	//dates
	 int dloop = patService.getLoopDateList(rId , "2000E");
	
	 List<DateInformation> dinfo= patService.getDateList(rId , "2000E","1");
	if (dinfo!=null && dinfo.size()>0){
		carejson.addProperty("dateFlag","1");
		carejson.addProperty("dateCount",dinfo.size());
		carejson.addProperty("dloop",dloop);
	
	for (int i=0; i<dinfo.size();i++){
		JsonObject dtjson = new JsonObject();
		DateInformation dd = (DateInformation) dinfo.get(i);
		String strdtp01= patService.getDecodedDescription("2000E", "DTP01",dd.getDateTimeQualifier());
		String strdtp03="";
		if (dd.getDateTimePeriodFormat().equals("RD8")){
			String[] atokens = dd.getDateTimePeriod().split("-");
			String strStrg= Utility.conStrToMMDDYY("D8" , atokens[0]);
			String endStrg= Utility.conStrToMMDDYY("D8" ,atokens[1]);
			strdtp03=strStrg +"-" +endStrg;
			dtjson.addProperty("dateTimeP",   strStrg +"-" +endStrg);
		}else
		{
			strdtp03=Utility.conStrToMMDDYY(dd.getDateTimePeriodFormat() , dd.getDateTimePeriod() );
			dtjson.addProperty("dateTimeP",   Utility.conStrToMMDDYY(dd.getDateTimePeriodFormat(), dd.getDateTimePeriod() ) );
		}
		
		dtjson.addProperty("dateQual", strdtp01 +" -- " +strdtp03);
		
			jarray.add(dtjson);
		} 
	carejson.addProperty("dateInfo",jarray.toString());
	 }
	else {
		carejson.addProperty("dateFlag","0");
	}
	
	
	return carejson.toString(); 
}


@GET
@Path("/getReferences")
@Produces("application/json")
public String getReferences(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getCertificateInfo");
	}
	Integer	rId = new Integer(id);
	JsonArray jarray = new JsonArray();
	JsonObject carejson = new JsonObject();
	
	//References
	List<References> refs= patService.getReferences(rId , "2000E");
	if (refs!=null && refs.size()>0){
		
		carejson.addProperty("refFlag","1");
		carejson.addProperty("refCount",refs.size());

		for (int i=0; i<refs.size();i++){
			JsonObject refjson = new JsonObject();
			References indref = (References) refs.get(i);
			refjson.addProperty("refIdQual", patService.getDecodedDescription("2000E", "REF01",indref.getReferenceIdQualifier()));
			refjson.addProperty("refId",   indref.getReferenceId() );
		
			jarray.add(refjson);
		
		}
		carejson.addProperty("refInfo",jarray.toString());
	}
		 else {
		carejson.addProperty("refFlag","0");
		}
	return carejson.toString(); 
}


@GET
@Path("/getCertificateInfo")
@Produces("application/json")
public String getCertificateInfo(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getCertificateInfo");
	}
	Integer	rId = new Integer(id);
	JsonArray jarray = new JsonArray();
	JsonObject carejson = new JsonObject();

			List<CertificationInformation> cinfo= patService.getCertificateList(rId , "2000E");
				
				if (cinfo!=null &&  cinfo.size()>0){
					
					carejson.addProperty("certFlag","1");
					carejson.addProperty("certCount",cinfo.size());
					
					for (int i=0; i<cinfo.size();i++){
						CertificationInformation cdd = (CertificationInformation) cinfo.get(i);
						JsonObject certjson = new JsonObject();
	
						certjson.addProperty("categoryCode", patService.getDecodedDescription("2000E", "CRC01",cdd.getCategoryCode()));
						//certjson.addProperty("respCode",  patService.getDecodedDescription("2000E", "CRC02", cdd.getConditionIndicatorCode2() ));
						certjson.addProperty("condnInd1",  patService.getDecodedDescription("2000E", "CRC03", cdd.getConditionIndicatorCode1() ));		
						certjson.addProperty("condnInd2",  patService.getDecodedDescription("2000E", "CRC04", cdd.getConditionIndicatorCode2() ));
						certjson.addProperty("condnInd3",  patService.getDecodedDescription("2000E", "CRC05", cdd.getConditionIndicatorCode3() ));
						certjson.addProperty("condnInd4",  patService.getDecodedDescription("2000E", "CRC06", cdd.getConditionIndicatorCode4() ));
						certjson.addProperty("condnInd5",  patService.getDecodedDescription("2000E", "CRC07", cdd.getConditionIndicatorCode5() ));
						certjson.addProperty("condnInd6",  patService.getDecodedDescription("2000E", "CRC08", cdd.getConditionIndicatorCode6() ));
						jarray.add(certjson);
					}
					carejson.addProperty("certInfo",jarray.toString());
				}
				else {
					carejson.addProperty("certFlag","0");
				}

 return carejson.toString(); 
 
}

@GET
@Path("/getCareSerDelivery")
@Produces("application/json")
public String getSvcSerDelivery(@QueryParam("id") String id) throws Exception {
	  	if (LOGGER.isDebugEnabled()) {
	  		LOGGER.debug("Enter  in getCareSerDelivery");
	  	}
	  	Integer	rId = new Integer(id);
	  	JsonObject carejson = new JsonObject();
	  	//service
	  	HealthCareServicesDelivery hrev = patService.getHealthCareServicesDelivery(rId , "2000E","1"); 
	  	if (hrev!=null){
	  		
	  		carejson.addProperty("hsdevFlag","1");
			carejson.addProperty("hsdevCount","1");
			
		  	carejson.addProperty("delcode",  patService.getDecodedDescription("2000E", "HSD08", hrev.getDeliveryCode()));
		  	carejson.addProperty("delpatCode", patService.getDecodedDescription("2000E", "HSD07", hrev.getDeliveryPatternCode()));
		  	carejson.addProperty("freq",  hrev.getFrequency());
		  	carejson.addProperty("noOfPeriod", hrev.getNumberofPeriods());
		  	carejson.addProperty("timePeriodCode", hrev.getTimePeriodCode());
		  	carejson.addProperty("quantity",  hrev.getQuantity());
		  	carejson.addProperty("unit",  hrev.getUnitCode());
		  	carejson.addProperty("quanqual",  patService.getDecodedDescription("2000E", "HSD01", hrev.getQuantityQualifierCode()));
	  
	  	}
	  	else {
			carejson.addProperty("hsdevFlag","0");
			carejson.addProperty("hsdevCount","0");
		}
		return carejson.toString(); 
	  	
	  }


@GET
@Path("/getHomeHealthCare")
public String getHomeHealthCare(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getTransportInfo");
	}
	Integer	rId = new Integer(id);
	JSONObject carejson = new JSONObject();
	//HomeHealthCare 	
	HomeHealthCare hhea = patService.getHomeHealthCareInfo(rId , "2000E", "1"); 
	if (hhea!=null){
		carejson.put("hcareFlag","1");
		carejson.put("hcareCount","1");
		
		carejson.put("progcode", patService.getDecodedDescription("2000E", "CR601", hhea.getPrognosisCode()));
		carejson.put("progDate",  Utility.convertMMDDYYFromyyyyMMDD(hhea.getPrognoisisDate()));
		carejson.put("servPeriod",  Utility.conStrToMMDDYY(hhea.getDateTimeQualifier1(), hhea.getDateTimePeriod1()));
		
		carejson.put("yesNo", patService.getDecodedDescription("2000E", "CR607", hhea.getYesNo() ));
		carejson.put("certTypeCode", patService.getDecodedDescription("2000E", "CR608", hhea.getCertificationTypeCode()));
		carejson.put("surgDate", Utility.convertMMDDYYFromyyyyMMDD(hhea.getSurgeryDate()));
		carejson.put("psIDQual", patService.getDecodedDescription("2000E", "CR610", hhea.getProductServiceIDQualifier()));
		carejson.put("medicareVal",  hhea.getMedicalCareValue());
		carejson.put("orderRecvDate", Utility.convertMMDDYYFromyyyyMMDD(hhea.getOrdersReceivedDate()));
		carejson.put("laestVisitDate", Utility.convertMMDDYYFromyyyyMMDD(hhea.getLastVisitDate()));
		carejson.put("phyContactDate", Utility.convertMMDDYYFromyyyyMMDD(hhea.getPhysicianContactDate()));
		carejson.put("lastInpatient", Utility.conStrToMMDDYY(hhea.getDateTimeQualifier2(), hhea.getDateTimePeriod2()));
	}
	
	return carejson.toString(); 
	
}

@GET
@Path("/getSpinalInfo")
public String getSpinalInfo(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getTransportInfo");
	}
	Integer	rId = new Integer(id);
	JsonObject carejson = new JsonObject();
		
	//Spinal be
	SpinalInformation spev = patService.getSpinalInfo(rId , "2000E"); 
	
	if (spev!=null){
		carejson.addProperty("spinalFlag","1");
		carejson.addProperty("spinalCount","1");
		carejson.addProperty("condition1",spev.getCondition1());
		carejson.addProperty("condition2",spev.getCondition2());
		carejson.addProperty("count",spev.getCount());
		carejson.addProperty("quantity", spev.getQuantity());
		String natureCo = patService.getDecodedDescription("2000E", "CR208", spev.getNatureofConditionCode() );
		carejson.addProperty("natureofCondn",natureCo );
		carejson.addProperty("descr1",spev.getDescription1());
		carejson.addProperty("descr2",spev.getDescription2());
		String spinalsublxl1 = patService.getDecodedDescription("2000E", "CR204", spev.getSubluxAtionLevelCode1() );
		carejson.addProperty("subluxLevel1",spinalsublxl1 );
		String spinalsublxl2 = patService.getDecodedDescription("2000E", "CR204", spev.getSubluxAtionLevelCode2() );
		carejson.addProperty("subluxLevel2",spinalsublxl2 );
	}
	else {
		carejson.addProperty("spinalFlag","0");
	}

return carejson.toString(); 
}

@GET
@Path("/getTransportInfo")
public String getTransportInfo(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getTransportInfo");
	}
	Integer	rId = new Integer(id);
	JsonObject carejson = new JsonObject();
		
	//Tranport
	TranportInformation trev = patService.getTransport(rId , "2000E"); 
	if (trev!=null) {
		carejson.addProperty("transFlag","1");
		carejson.addProperty("transCount","1");
		//String transcodedesc = patService.getDecodedDescription("2000E", "CR1",  );
		
		carejson.addProperty("quantity", trev.getQuantity());
		carejson.addProperty("unit1", trev.getUnitBasisCode1() );	
		carejson.addProperty("wt", trev.getWeight());
		String transcodedesc = patService.getDecodedDescription("2000E", "CR103", trev.getTransportCode() );	
		carejson.addProperty("transcode", transcodedesc);
		String reasoncodedesc = patService.getDecodedDescription("2000E", "CR104", trev.getReasonCode() );	
		carejson.addProperty("reasoncode", reasoncodedesc );
		String ucodedesc = patService.getDecodedDescription("2000E", "CR105", trev.getUnitBasisCode2() );	
		carejson.addProperty("unit2", ucodedesc );
		carejson.addProperty("transDescr", trev.getDescription1());
		carejson.addProperty("transDescr2", trev.getDescription2());
	}
	else {
		carejson.addProperty("transFlag","0");
	}
	return carejson.toString(); 
}

@GET
@Path("/getHomeOxygen")
public String getHomeOxygen(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getHomeOxygen");
	}
	Integer	rId = new Integer(id);
	JsonObject carejson = new JsonObject();
		
	//HomeOxygen
	//HomeOxygenTherapy 
	HomeOxygenTherapy hoxev = patService.getHomeOxygenTherapyInfo(rId , "2000E");
	
	if (hoxev!=null) {
		
		carejson.addProperty("oxyFlag","1");
		carejson.addProperty("oxyCount","1");
		String hoxcodedesc = patService.getDecodedDescription("2000E", "UM01", hoxev.getOxygenEquimentCode1() );
		carejson.addProperty("homeOxygenTherapy ",hoxcodedesc);
		
		carejson.addProperty("descr1", hoxev.getDescription1());
		carejson.addProperty("descr2", hoxev.getDescription2());
		String equcodedesc1 = patService.getDecodedDescription("2000E", "CR503", hoxev.getOxygenEquimentCode() );
		String equcodedesc2 = patService.getDecodedDescription("2000E", "CR504", hoxev.getOxygenEquimentCode1() );
		String equcodedesc = equcodedesc1 + "-"+equcodedesc2;
		carejson.addProperty("oxyEq",equcodedesc );
		carejson.addProperty("oxyTest", patService.getDecodedDescription("2000E", "CR512", hoxev.getOxygenTestCondCode()));
		
		String oxyTestFindCode=patService.getDecodedDescription("2000E", "CR513", hoxev.getOxygenTestFindCode1()) 
								+patService.getDecodedDescription("2000E", "CR514", hoxev.getOxygenTestFindCode2())
								+patService.getDecodedDescription("2000E", "CR515", hoxev.getOxygenTestFindCode3());
		
		carejson.addProperty("oxyTestFind", oxyTestFindCode);
		
		carejson.addProperty("quant", hoxev.getQuantity1()+ " liters per minute "+ hoxev.getQuantity2() + " times per day" + hoxev.getQuantity3() + " number of hours per period");
		carejson.addProperty("quant45", hoxev.getQuantity4()+  " arterial blood gas " + hoxev.getQuantity5() + " Oxygen Saturation");
		
		carejson.addProperty("quant6", hoxev.getQuantity6());
		String odel = patService.getDecodedDescription("2000E", "CR517", hoxev.getOxygenDeliverySystemCode());
		carejson.addProperty("oxyDelSys",odel );
		carejson.addProperty("oxyEq2", patService.getDecodedDescription("2000E", "CR518", hoxev.getOxygenEquimentCode2()));
		
	}

	else {
		carejson.addProperty("oxyFlag","0");
		carejson.addProperty("oxyCount","0");
	}

return carejson.toString(); 
}

@GET
@Path("/getInstituionClaim")
public String getInstituionClaim(@QueryParam("id") String id) throws Exception {
	if (LOGGER.isDebugEnabled()) {
		LOGGER.debug("Enter  in getInstituionClaim");
	}
	Integer	rId = new Integer(id);
	JsonObject carejson = new JsonObject();
		
	InstitutionClaim irev = patService.getInstitutionClaim(rId , "2000E"); 
	if (irev!=null) {
		carejson.addProperty("instFlag","1");
		carejson.addProperty("instCount","1");
		
		carejson.addProperty("AdminTypeCode", patService.getDecodedDescription("2000E", "CL101",irev.getAddmissionTypeCode()) );
		carejson.addProperty("AdminSourceCode", patService.getDecodedDescription("2000E", "CL102",irev.getAddmissionSourceCode()) );
		carejson.addProperty("PatStatusCode",  patService.getDecodedDescription("2000E", "CL103",irev.getPatientStatusCode() ));
		carejson.addProperty("ResStatusCode",patService.getDecodedDescription("2000E", "CL104", irev.getResidentialStatusCode()) );
	}

	else {
		carejson.addProperty("instFlag","0");
		carejson.addProperty("instCount","0");
	}
	return carejson.toString(); 
}

@GET
@Path("/getAddCareSer")
@Produces("application/json")
public String getAddCareSer(@QueryParam("id") String id) throws Exception {
	  	if (LOGGER.isDebugEnabled()) {
	  		LOGGER.debug("Enter  in getAddCareSer");
	  	}
	  	Integer	rId = new Integer(id);
	  	JsonObject carejson = new JsonObject();
	  	//service
	  	PatientInformation hrev = patService.getAdditionalServices(rId , "2000E"); 
	  	
	  	if (hrev!=null){
	  		carejson.addProperty("addFlag","1");
			carejson.addProperty("addCount","1");
		  	carejson.addProperty("rTypeCode",  patService.getDecodedDescription("2000E", "PWK01", hrev.getReportTypeCode()));
		  	carejson.addProperty("rTransCode", patService.getDecodedDescription("2000E", "PWK02", hrev.getReportTransmCode()));
		  	carejson.addProperty("idCodeQual", patService.getDecodedDescription("2000E", "PWK05", hrev.getiDCodeQualifier()));
		   	carejson.addProperty("id",    hrev.getIdentifier());
		  	carejson.addProperty("descr",  hrev.getDescription());
	  	} 
	  	else {
			carejson.addProperty("addFlag","0");
			carejson.addProperty("addCount","0");
		}
	  	return carejson.toString();
	  }

@GET
@Path("/getDiagnosis")
@Produces("application/json")
public String getDiagnosis(@QueryParam("id") String id) throws Exception {
	  	if (LOGGER.isDebugEnabled()) {
	  		LOGGER.debug("Enter  in getDiagnosis");
	  	}
	  	Integer	rId = new Integer(id);
	  	JsonArray jarray = new JsonArray();
	  	JsonObject carejson = new JsonObject();
	  	
	 	//service
	  	List<DiagnosisInformation> hrev = patService.getDiagnosisServices(rId , "2000E","1"); 
	  	if (hrev!=null && hrev.size()>0){
			
			carejson.addProperty("diagFlag","1");
			carejson.addProperty("diagCount",hrev.size());

	  	
		  	for (int i=0; i<hrev.size();i++){
		  		DiagnosisInformation cdd = (DiagnosisInformation) hrev.get(i);
				JsonObject diagjson = new JsonObject();
	
				diagjson.addProperty("industryCode",  cdd.getIndustryCode());
				if (cdd.getDateTimePeriod()!=null && !cdd.getDateTimePeriod().equals("")){
					diagjson.addProperty("timePeriod", Utility.conStrToMMDDYY( cdd.getDateTimePeriodQualifier() , cdd.getDateTimePeriod()));
				}
				diagjson.addProperty("codeListQual",    patService.getDecodedDescription("2000E", "HI01-1",  cdd.getCodeListQuaifierCode()));  
			 		
			   	jarray.add(diagjson);			
		  	}
		  	carejson.addProperty("diagInfo",jarray.toString());
		}
			 else {
			carejson.addProperty("diagFlag","0");
			}
		return carejson.toString(); 
	}

@GET
@Path("/getMessageCare")
@Produces("application/json")
public String getMessageCare(@QueryParam("id") String id) throws Exception {
	  	if (LOGGER.isDebugEnabled()) {
	  		LOGGER.debug("Enter  in getMessageCare");
	  	}
	  	Integer	rId = new Integer(id);
	  	JsonObject carejson = new JsonObject();
	  	//service
	  	MessageText hrev = patService.getMessges(rId , "2000E"); 
	  	if (hrev!=null){
	  		carejson.addProperty("msgFlag","1");
			carejson.addProperty("msgCount","1");
	  		carejson.addProperty("freMesg", hrev.getMessage());
	  	}	  
	  	 else {
				carejson.addProperty("msgFlag","0");
				}
	  	
	  	return carejson.toString();
}

//2010EA
@GET
@Path("/getPatEvtProviderName")
	  
public String getPatEvtProviderName(@QueryParam("id") String id) throws Exception {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("Enter  in getPatEvtProviderName");
			}
			Integer	rId = new Integer(id);
			JsonArray jarray = new JsonArray();
			JsonObject carejson = new JsonObject();
			
			//provider
			List<EntityName> entlo= patService.getEntityInfo(rId , "2010EA"); //Loop code		
			if (entlo!=null && entlo.size()>0){
			 	for (int i=0; i<entlo.size();i++){
			 	
			 		carejson.addProperty("evtFlag","1");
					carejson.addProperty("evtCount",entlo.size());
			 		EntityName ent = (EntityName) entlo.get(i);
				JsonObject djson = new JsonObject();
				djson.addProperty("reqLnameOrg",Utility.buildNameorOrg(ent));
				
				djson.addProperty("reqEntIdQual",patService.getDecodedDescription("2010D", "NM102", ent.getEntityIdCodeQualifier()) );
				
				djson.addProperty("reqEntIdCode",ent.getEntityIdCode() );
				
				String strNM108= patService.getDecodedDescription("2010D", "NM108", ent.getEntityIdCodeQualifier());
				String strNM109= ent.getEntityIdentificationCode();
				String strNM = strNM108 +" " +strNM109;
				djson.addProperty("IdQual", strNM );
				//djson.addProperty("IdCode",ent.getEntityIdentificationCode() );
				jarray.add(djson);
			 	}	 
			 	carejson.addProperty("provInfo",jarray.toString());
			}
			
			else {
				carejson.addProperty("evtFlag","0");
				carejson.addProperty("evtCount","0");
			}
				 return carejson.toString(); 	
}

//2010EB
@GET
@Path("/getOtherUMO")
	  
public String getOtherUMO(@QueryParam("id") String id) throws Exception {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("Enter  in getOtherUMO");
			}
			
			Integer	rId = new Integer(id);
			JsonArray jarray = new JsonArray();
			JsonObject carejson = new JsonObject();
			
			//provider
			List<EntityName> entlo= patService.getEntityInfo(rId , "2010EB"); //Loop code		
			if (entlo!=null && entlo.size()>0){
			 	for (int i=0; i<entlo.size();i++){
			 		carejson.addProperty("umoFlag","1");
					carejson.addProperty("umoCount",entlo.size());
					
			 	EntityName ent = (EntityName) entlo.get(i);
				JsonObject djson = new JsonObject();
				djson.addProperty("reqLnameOrg",Utility.buildNameorOrg(ent));
				djson.addProperty("reqEntIdCode",ent.getEntityIdCode() );
				
				String strNM108= patService.getDecodedDescription("2010D", "NM108", ent.getEntityIdCodeQualifier());
				String strNM109= ent.getEntityIdentificationCode();
				String strNM = strNM108 +" " +strNM109;
				djson.addProperty("IdQual", strNM );
				
				djson.addProperty("firstname",ent.getNameFirst());
				djson.addProperty("middleName",ent.getNameMiddle());
				djson.addProperty("prefix",ent.getNamePrefix() );
				djson.addProperty("suffix",ent.getNameSuffix() );
				djson.addProperty("IdQual",patService.getDecodedDescription("2010D", "NM108", ent.getEntityIdCodeQualifier()) );
				djson.addProperty("IdCode",ent.getEntityIdCode() );
				jarray.add(djson);
			 	}	 
			 	carejson.addProperty("umoInfo",jarray.toString());
			}
			else {
				carejson.addProperty("umoFlag","0");
				carejson.addProperty("umoCount","0");
			}
				modelMap.addAttribute("dependentjson", jarray);
return carejson.toString();	 
}

//2010Ec
@GET
@Path("/getPatEvtTransport")
	  
public String getPatEvtTransport(@QueryParam("id") String id) throws Exception {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("Enter  in getPatEvtProviderName");
			}
			
			
			Integer	rId = new Integer(id);
			JsonArray jarray = new JsonArray();
			JsonObject carejson = new JsonObject();
			
			
			//provider
			List<EntityName> entlo= patService.getEntityInfo(rId , "2010EC"); //Loop code		
			if (entlo!=null && entlo.size()>0){
			 	for (int i=0; i<entlo.size();i++){
			 		carejson.addProperty("evtTFlag","1");
					carejson.addProperty("evtTCount",entlo.size());
					
			 		EntityName ent = (EntityName) entlo.get(i);
				JsonObject djson = new JsonObject();
				djson.addProperty("reqEntIdCode",patService.getDecodedDescription("2010EC", "NM101", ent.getEntityIdCode()) );
				djson.addProperty("IdQual",patService.getDecodedDescription("2010EC", "NM102", ent.getEntityIdCodeQualifier()) );
				
				
				
				djson.addProperty("reqLnameOrg",Utility.buildNameorOrg(ent));
				jarray.add(djson);
			 	}	 
			 	carejson.addProperty("evtTranInfo",jarray.toString());
			}
			else {
				carejson.addProperty("evtTFlag","0");
				carejson.addProperty("evtTCount","0");
			}
				modelMap.addAttribute("dependentjson", jarray);
return carejson.toString();	 	
}




}